   !!======================================================================

  SUBROUTINE dyn3d_height_mask(xe,tfg,tfd,tfb,tfh,test)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn2d_height_mask  ***
   !&E
   !&E ** Purpose : Evaluation of water column height, of the kind
   !&E              of cell we deal with and of the type of bound
   !&E              for each row and column
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dxy.F90,dyn3dyx.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hx,hy,xe,dhx,goulet,govlet,id,ig,jb,jh
   !&E
   !&E ** Modified variables : htx,hex,igdu,tfg,tfd,mn,ms,me,mo,
   !&E                         hty,hey,igdv,tfb,tfh,whx,why,
   !&E                         hzutx,hzwtx,hzuty,hzwty,hzex,hzey
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-07  (T. Duhaut)  Original code
   !&E       !  2008-07  (V. Garnier)  Introduction of generalized sigma
   !&E       !  2008-08  (T. Odaka) MPI
   !&E       !  2009-03  (V. Garnier) masks advection terms
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2009-04  (F. Dumas) generalized sigma : wet & drying management
   !&E       !  2011-03  (F. Dumas)  bottom stress parametrisation + remove case 2 and 3
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : htx,hty,hx,hy,hey,hex,goulet,govlet,    &
                         h0fond,jb,jh,ig,id,l_obc_cycl_x,l_obc_cycl_y, &
                         dhx,dhy,hm,dhxy,h0,fwet,fwetmin,fwetp
   USE comcolig2d,ONLY : igdu,igdv,mn,ms,me,mo,whx,why
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : dcusds,dcwsds,hzutx,hzwtx,hzuty,hzwty,hzex,hzey
   USE toolgeom,  ONLY : f_hzu_x,f_hzu_y,f_hzw_x,f_hzw_y
#endif
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,ex_i_int,ex_j_int

   IMPLICIT NONE

   !! * Arguments
   INTEGER, INTENT(in)                                        :: test
   REAL(KIND=rsh), DIMENSION(limin:limax), INTENT(out)        :: tfb,tfh
   REAL(KIND=rsh), DIMENSION(ljmin:ljmax), INTENT(out)        :: tfg,tfd
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: xe

   !! * Local declarations
   INTEGER                                                     :: i,j,k,i1,i2,j1,j2

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter DYN3D_HEIGHT_MASK routine'

   !-----------------------------------------------------------
   ! Initialize htx and hty :
   ! water depth at time n  (1st call from ligne : xev)
   ! water depth at time nprime (2nd call from ligne : xeint)
   ! water depth at time n+1/2  (1st call from colonne : xeu)
   ! water depth at time (n+1/2)prime (2nd call from colonne : xeint)
   ! htx and hty are used in momentum equations
   !-----------------------------------------------------------

   IF (fwetmin/=1.0_rsh) THEN
#ifdef key_MPI_2D
!$OMP DO schedule(runtime)
#else
!$OMP DO schedule(static)
#endif

   DO j=ljminp2,ljmaxm1
     fwetp(:,j)=fwet(:,j) ! fraction of wet surface of previous time step
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       ! test useful ?
       IF (dhxy(i,j) == 0.0_rsh) THEN
         fwet(i,j)=1.0_rsh
       ELSE
         fwet(i,j)=MIN(MAX(fwetmin,(xe(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
       END IF
     END DO
     fwetp(:,j)=fwet(:,j)   ! previous time step inoreoaration for the future
     ! as this solution is unstable we do not use it but the model is not
     ! conservtive anymore above tidal flat if fwetmin/=1
   END DO
!$OMP END DO
   END IF

#ifdef key_MPI_2D
!$OMP DO schedule(runtime)
#else
!$OMP DO schedule(static)
#endif

   DO j=ljmin,ljmaxm1
     DO i=limin,limaxm1
       htx(i,j)=hx(i,j)+0.5_rsh*(xe(i,j)+xe(i+1,j))
       hty(i,j)=hy(i,j)+0.5_rsh*(xe(i,j)+xe(i,j+1))
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hx (i+1/2) and depth u or w
       IF (test == 1) THEN  ! if calculation in line
         DO k=1,kmax
           hzutx(k,i,j)=f_hzu_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)
         END DO
         DO k=1,kmax-1
           hzwtx(k,i,j)=f_hzw_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)
         END DO
         IF ( htx(i,j)<=0.0_rsh .AND. ( hx(i,j)+xe(i,j) > 0.0_rsh .OR. hx(i,j)+xe(i+1,j) > 0.0_rsh) ) THEN
           DO k=1,kmax
             hzutx(k,i,j)=f_hzu_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)
           END DO
           DO k=1,kmax-1
             hzwtx(k,i,j)=f_hzw_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)
           END DO
         END IF
       END IF
   ! dz/ds at location hy (j+1/2) and depth of v or w
       IF (test == 2) THEN  ! if calculation in column
         DO k=1,kmax
           hzuty(k,i,j)=f_hzu_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)
         END DO
         DO k=1,kmax-1
           hzwty(k,i,j)=f_hzw_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)
         END DO
         IF ( hty(i,j)<=0.0_rsh .AND. ( hy(i,j)+xe(i,j) > 0.0_rsh .OR. hy(i,j)+xe(i,j+1) > 0.0_rsh )) THEN
           DO k=1,kmax
             hzuty(k,i,j)=f_hzu_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)
           END DO
           DO k=1,kmax-1
             hzwty(k,i,j)=f_hzw_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)
           END DO
         END IF
       END IF
#endif
!       IF ( htx(i,j)<=0.0 ) With such a test mn and ms are set to 1 more often as htx is smaller when negative
       IF ( htx(i,j)<=0.0_rsh .AND. ( hx(i,j)+xe(i,j) > 0.0_rsh .OR. hx(i,j)+xe(i+1,j) > 0.0_rsh) ) &
         htx(i,j)=hx(i,j)+MAX(xe(i,j),xe(i+1,j))
       IF ( hty(i,j)<=0.0_rsh .AND. ( hy(i,j)+xe(i,j) > 0.0_rsh .OR. hy(i,j)+xe(i,j+1) > 0.0_rsh )) &
         hty(i,j)=hy(i,j)+MAX(xe(i,j),xe(i,j+1))
     END DO

     IF_MPI (limax==imax) THEN
     IF (hy(imax,j) /= -valmanq-h0fond) THEN
       hty(imax,j)=hy(imax,j)+0.5_rsh*(xe(imax,j)+xe(imax,j+1))
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hy (j+1/2) and depth of v or w
       IF (test == 2) THEN  ! if calculation in column
         DO k=1,kmax
           hzuty(k,imax,j)=f_hzu_y(hy(imax,j),0.5_rsh*(xe(imax,j)+xe(imax,j+1)),k,imax,j)
         END DO
         DO k=1,kmax-1
           hzwty(k,imax,j)=f_hzw_y(hy(imax,j),0.5_rsh*(xe(imax,j)+xe(imax,j+1)),k,imax,j)
         END DO
         IF ( hty(imax,j)<=0.0_rsh .AND. ( hy(imax,j)+xe(imax,j) > 0.0_rsh .OR. hy(imax,j)+xe(imax,j+1) > 0.0_rsh )) THEN
           DO k=1,kmax
             hzuty(k,imax,j)=f_hzu_y(hy(imax,j),MAX(xe(imax,j),xe(imax,j+1)),k,imax,j)
           END DO
           DO k=1,kmax-1
             hzwty(k,imax,j)=f_hzw_y(hy(imax,j),MAX(xe(imax,j),xe(imax,j+1)),k,imax,j)
           END DO
         END IF
       END IF
#endif
       IF ( hty(imax,j)<=0.0_rsh .AND. ( hy(imax,j)+xe(imax,j) > 0.0_rsh .OR. hy(imax,j)+xe(imax,j+1) > 0.0_rsh )) &
         hty(imax,j)=hy(imax,j)+MAX(xe(imax,j),xe(imax,j+1))
     END IF
     ENDIF_MPI

   END DO ! end do for j loop
!$OMP END DO

!$OMP SINGLE
   IF_MPI (ljmax==jmax) THEN
     DO i=limin,limaxm1
       IF (hx(i,jmax) /= -valmanq-h0fond) THEN
         htx(i,jmax)=hx(i,jmax)+0.5_rsh*(xe(i,jmax)+xe(i+1,jmax))
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hx (i+1/2) and depth u or w
         IF (test == 1) THEN  ! if calculation in line
           DO k=1,kmax
             hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),0.5_rsh*(xe(i,jmax)+xe(i+1,jmax)),k,i,jmax)
           END DO
           DO k=1,kmax-1
             hzwtx(k,i,jmax)=f_hzw_x(hx(i,jmax),0.5_rsh*(xe(i,jmax)+xe(i+1,jmax)),k,i,jmax)
           END DO
           IF ( htx(i,jmax)<=0.0_rsh .AND. ( hx(i,jmax)+xe(i,jmax) > 0.0_rsh .OR. hx(i,jmax)+xe(i+1,jmax) > 0.0_rsh) ) THEN
             DO k=1,kmax
               hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),MAX(xe(i,jmax),xe(i+1,jmax)),k,i,jmax)
             END DO
             DO k=1,kmax-1
               hzwtx(k,i,jmax)=f_hzw_x(hx(i,jmax),MAX(xe(i,jmax),xe(i+1,jmax)),k,i,jmax)
             END DO
           END IF
         END IF
#endif
         IF ( htx(i,jmax)<=0.0_rsh .AND. ( hx(i,jmax)+xe(i,jmax) > 0.0_rsh .OR. hx(i,jmax)+xe(i+1,jmax) > 0.0_rsh) ) &
           htx(i,jmax)=hx(i,jmax)+MAX(xe(i,jmax),xe(i+1,jmax))
       END IF
     ENDDO
   ENDIF_MPI
!$OMP END SINGLE

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,htx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,htx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hty(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hty(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(htx,hty)

#ifdef key_MPI_2D
!$OMP DO schedule(runtime)
#else
!$OMP DO schedule(dynamic,3)
#endif
!   DO j=jmin+2,jmax-1
   ! FVDM pour definir ms au sud j=1 et ameliorer operateur d OBC
   ! et redescendre le calcul des termes termes NL des schemas quick et quickest
   ! mais pas compatible avec obc_mars
   !DO j=ljminp1,ljmaxm1
   DO j=ljminp2,ljmaxm1
!   DO j=ljminm2,ljmaxp1
!   DO j=jmin,jmax


   !-----------------------------------------------------------
   ! Evaluation of igdu which determines the nature
   ! of the cell in the estimate of u
   ! igdu = 1 water at both sides
   ! igdu = 5 water at both sides but very shallow depths (2d estimate)
   ! igdu = 2 water on the left
   ! igdu = 3 water on the right
   ! igdu = 4 no flow inside this cell
   ! Initialize hex :
   ! water depth at time n  (1st call from ligne : xev)
   ! water depth at time nprime (2nd call from ligne : xeint)
   ! water depth at time n+1/2  (1st call from colonne : xeu)
   ! water depth at time (n+1/2)prime (2nd call from colonne : xeint)
   ! hex and hey are used in the continuity and for tracers advection
   !-----------------------------------------------------------

     IF ( (j<=jmin+1) .OR. (j>=jmax) ) THEN
       igdu(limin:limax,j)=4
       mn(limin:limax,j)=0
       ms(limin:limax,j)=0
     ELSE
       i1=ig(j)
       i2=id(j)
       DO i=limin,limax
         IF ((i>=i1) .AND. (i<=i2-1)) THEN

   !-----------------------------------------------------------
   ! Evaluation of whx, variable which determines the location
   ! of dry points along the x-direction (wall at location hx)
   !-----------------------------------------------------------

           IF (htx(i,j) > 0.0_rsh) THEN
             whx(i,j)=1
           ELSE
             whx(i,j)=0
           END IF


           IF (htx(i,j) > 0.0_rsh) THEN
             igdu(i,j)=1
             IF (htx(i,j) < hm) igdu(i,j)=5
             IF (htx(i,j) < MIN(epsilon,hm)) igdu(i,j)=4
#ifdef key_dyn_adv_linear
             hex(i,j)=MAX(0.0_rsh,hx(i,j))*goulet(i,j)
#if defined key_siggen || defined key_gencoord
             DO k=1,kmax
               hzex(k,i,j)=(f_hzu_x(hx(i,j),0.0_rsh,k,i,j)+dhx(i,j)*dcusds(k))*goulet(i,j)
             END DO
#endif
#else
             hex(i,j)=(htx(i,j)+dhx(i,j))*goulet(i,j)
#if defined key_siggen || defined key_gencoord
             IF ( hx(i,j)+0.5_rsh*(xe(i,j)+xe(i+1,j)) < 0.0_rsh ) THEN
               DO k=1,kmax
                 hzex(k,i,j)=(f_hzu_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)+dhx(i,j)*dcusds(k))*goulet(i,j)
               END DO
             ELSE
               DO k=1,kmax
                 hzex(k,i,j)=(f_hzu_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)+dhx(i,j)*dcusds(k))*goulet(i,j)
               END DO
             END IF
#endif
#endif
           ELSE
             igdu(i,j)=4
             hex(i,j)=0.0_rsh
#if defined key_siggen || defined key_gencoord
             hzex(:,i,j)=0.0_rsh
#endif
           END IF

   !-------------------------------------------
   ! recherche de la presence de mur au noeud
   ! phi necessaire au calcul des gradients
   ! perpendiculaire a la direction de calcul
   ! look for the presence of a wall at the north and south
   ! used for estimate of viscosity term
   !-------------------------------------------

           IF (test == 1) THEN  ! if calculation in line
             IF (hty(i,j  )+hty(i+1,j  ) > 0.0_rsh .and. htx(i,j+1) > 0.0_rsh) THEN
               mn(i,j)=1
             ELSE
               mn(i,j)=0
             END IF
             IF (hty(i,j-1)+hty(i+1,j-1) > 0.0_rsh .AND. htx(i,j-1) > 0.0_rsh) THEN
               ms(i,j)=1
             ELSE
               ms(i,j)=0
             END IF
           END IF ! ends test on direction of calculation

         ELSE   ! out of domain
           igdu(i,j)=4
           whx(i,j)=0
           mn(i,j)=0
           ms(i,j)=0
         END IF

       END DO  ! ends loop on i

   !-------------------------------------------
   ! type of beginning and end of the line for boundary conditions
   ! tfg = 1 water at both sides = flow at open boundary (westwards or eastwards
   ! tfg = 0 no flow from the boundary (open or shore)
   ! tfg = 2 flow towards the east only
   ! tfd = 1 water at both sides = flow at open boundary
   ! tfd = 0 no flow from the boundary (open or shore)
   ! tfd = 2 flow towards the west only
   !-------------------------------------------

       IF (test == 1) THEN
         IF (.NOT.l_obc_cycl_x) THEN
           IF_MPI ( limin < i1 .AND. i1 <= limax ) THEN
             IF (igdu(i1,j) == 1 .or. igdu(i1,j) == 5) tfg(j) = 1.0_rsh
             IF (igdu(i1,j) == 4)                      tfg(j) = 0.0_rsh
           ENDIF_MPI
           IF_MPI ( limin <= i2-1 .AND. i2-1 <= limax ) THEN
             IF (igdu(i2-1,j) == 1 .or. igdu(i2-1,j) == 5) tfd(j) = 1.0_rsh
             IF (igdu(i2-1,j) == 4)                        tfd(j) = 0.0_rsh
           ENDIF_MPI
         END IF
       END IF

     END IF
   END DO
!$OMP END DO

#ifdef key_MPI_2D
!$OMP DO SCHEDULE(runtime)
#else
!$OMP DO SCHEDULE(dynamic,3)
#endif
   DO j=ljmin,ljmax

     IF (j==jmax .AND. ljmax==jmax) THEN
       igdv(limin:limax,j)=4
       why(limin:limax,j)=0
       DO i=limin,limax
         mo(i,j)=0
         me(i,j)=0
       END DO
     ELSE

       DO i=limin,limax    ! boucle sur toutes les colonnes
         j1=jb(i)
         j2=jh(i)
         ! FVDM pour definir mo a ouest i=1 et ameliorer operateur d OBC
         ! et redescendre le calcul des termes termes NL des schemas quick et quickest
         ! mais pas compatible avec obc_mars
         !IF ( (i>=imin+1 .AND. i<=imax-1) .AND. (j>=j1 .AND. j<=j2-1) ) THEN
         IF ( (i>=imin+2 .AND. i<=imax-1) .AND. (j>=j1 .AND. j<=j2-1) ) THEN

   !-----------------------------------------------------------
   ! Evaluation of why variable which determines the location
   ! of dry points along the y-direction (wall at location hy)
   !-----------------------------------------------------------

           IF (hty(i,j) > 0.0_rsh) THEN
             why(i,j)=1
           ELSE
             why(i,j)=0
           END IF

   !-----------------------------------------------------------
   ! Evaluation of igdv which determines the nature
   ! of the cell in the estimate of v
   ! igdv = 1 water at both sides
   ! igdv = 5 water at both sides but very shallow depths (2d estimate)
   ! igdv = 2 water at the south
   ! igdv = 3 water at the north
   ! igdv = 4 no flow inside this cell
   ! Initialize hey :
   ! water depth at time n  (1st call from ligne : xev)
   ! water depth at time nprime (2nd call from ligne : xeint)
   ! water depth at time n+1/2  (1st call from colonne : xeu)
   ! water depth at time (n+1/2)prime (2nd call from colonne : xeint)
   ! hex and hey are used in the continuity and for tracers advection
   !-----------------------------------------------------------


           IF (hty(i,j) > 0.0_rsh) THEN
             igdv(i,j)=1
             IF (hty(i,j) < hm) igdv(i,j)=5
             IF (hty(i,j) < MIN(epsilon,hm)) igdv(i,j)=4
#ifdef key_dyn_adv_linear
             hey(i,j)=MAX(0.0_rsh,hy(i,j))*govlet(i,j)
#if defined key_siggen || defined key_gencoord
             DO k=1,kmax
               hzey(k,i,j)=(f_hzu_y(hy(i,j),0.0_rsh,k,i,j)+dhy(i,j)*dcusds(k))*govlet(i,j)
             END DO
#endif
#else
             hey(i,j)=(hty(i,j)+dhy(i,j))*govlet(i,j)
#if defined key_siggen || defined key_gencoord
             IF ( hy(i,j)+0.5_rsh*(xe(i,j)+xe(i,j+1)) < 0.0_rsh ) THEN
               DO k=1,kmax
                 hzey(k,i,j)=(f_hzu_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)+dhy(i,j)*dcusds(k))*govlet(i,j)
               END DO
             ELSE
               DO k=1,kmax
                 hzey(k,i,j)=(f_hzu_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)+dhy(i,j)*dcusds(k))*govlet(i,j)
               END DO
             END IF
#endif
#endif
           ELSE
             igdv(i,j)=4
             hey(i,j)=0.0_rsh
#if defined key_siggen || defined key_gencoord
             hzey(:,i,j)=0.0_rsh
#endif
           END IF

   !---------------------------------------------------------
   !  detection des murs aux noeuds phi pour l evaluation des
   !  gradients perpendiculaires a la direction de
   !  calcul (ici les gradients selon Ox)
   ! look for the presence of a wall at the west and east
   ! used for estimate of viscosity term
   !-------------------------------------------

           IF (test == 2) THEN  ! if calculation in column
             IF (htx(i  ,j)+htx(i  ,j+1) > 0.0_rsh .and. hty(i+1,j) > 0.0_rsh) THEN
               me(i,j)=1
             ELSE
               me(i,j)=0
             END IF
             IF (htx(i-1,j)+htx(i-1,j+1) > 0.0_rsh .AND. hty(i-1,j) > 0.0_rsh) THEN
               mo(i,j)=1
             ELSE
               mo(i,j)=0
             END IF
           END IF ! ends test on direction of calculation

   !-------------------------------------------
   ! type of beginning and end of the line for boundary conditions
   ! tfb = 1 water at both sides = flow at open boundary (northwards or southwards)
   ! tfb = 0 no flow from the boundary (open or shore)
   ! tfb = 2 flow towards the north only
   ! tfh = 1 water at both sides = flow at open boundary
   ! tfh = 0 no flow from the boundary (open or shore)
   ! tfh = 2 flow towards the south only
   !-------------------------------------------

           IF (test == 2) THEN
             IF (.NOT.l_obc_cycl_y) THEN
               IF (j.EQ. j1) THEN
                 IF (igdv(i,j1) == 1 .OR. igdv(i,j1) == 5) tfb(i) = 1.0_rsh
                 IF (igdv(i,j1) == 4)                      tfb(i) = 0.0_rsh
               END IF

               IF (j.EQ. j2-1) THEN
                 IF (igdv(i,j2-1) == 1 .OR. igdv(i,j2-1) == 5) tfh(i) = 1.0_rsh
                 IF (igdv(i,j2-1) == 4)                        tfh(i) = 0.0_rsh
               END IF
             END IF
           END IF

         ELSE   ! out of domain

           igdv(i,j)=4
           why(i,j)=0
           mo(i,j)=0
           me(i,j)=0

         END IF

       END DO
     END IF
   END DO
!$OMP END DO

OMPMPI barrier
OMPMPI master
!-2 is needed due to loop on j=jmin-1,jmax and i=imin-1,imax for scheme ultimate quickest macho
!previous remark not correct   CALL ex_i_rsh(-2,1,1,liminm2,limaxp1,ljmin,ljmax,hex(liminm2:limaxp1,ljmin:ljmax))
!previous remark not correct   CALL ex_j_rsh(-2,1,1,limin,limax,ljminm2,ljmaxp1,hey(limin:limax,ljminm2:ljmaxp1))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljmin,ljmax,hex(liminm2:limaxp2,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-2,2,1,limin,limax,ljminm2,ljmaxp2,hey(limin:limax,ljminm2:ljmaxp2))
#if defined key_siggen || defined key_gencoord
   CALL_MPI ex_i_rsh(-1,0,kmax,liminm1,limax,ljmin,ljmax,hzex(:,liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,kmax,limin,limax,ljminm1,ljmax,hzey(:,limin:limax,ljminm1:ljmax))
#endif
 IF_MPI (test == 1) THEN
   CALL_MPI ex_i_int(-1,1,1,liminm1,limaxp1,ljmin,ljmax,whx(liminm1:limaxp1,ljmin:ljmax))
   CALL_MPI ex_i_int(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,igdu(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_int(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,igdu(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_int(-1,0,1,limin,limax,ljminm1,ljmax,ms(limin:limax,ljminm1:ljmax))
   CALL_MPI ex_j_int( 0,1,1,limin,limax,ljmin,ljmaxp1,mn(limin:limax,ljmin:ljmaxp1))
 ENDIF_MPI
 IF_MPI (test == 2) THEN
   CALL_MPI ex_j_int(-1,1,1,limin,limax,ljminm1,ljmaxp1,why(limin:limax,ljminm1:ljmaxp1))
   CALL_MPI ex_i_int(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,igdv(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_int(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,igdv(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_int(-1,0,1,liminm1,limax,ljmin,ljmax,mo(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_i_int( 0,1,1,limin,limaxp1,ljmin,ljmax,me(limin:limaxp1,ljmin:ljmax))
 ENDIF_MPI
OMPMPI end master
OMPMPI barrier
OMPMPI flush(hex,hey,whx,why,igdu,igdv,mn,ms,mo,me)
#if defined key_siggen || defined key_gencoord
OMPMPI flush(hzex,hzey)
#endif

   PRINT_DBG*, 'exit DYN3D_HEIGHT_MASK routine'
 
  END SUBROUTINE dyn3d_height_mask
