!**************    including dyn3dx_adv_quick.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dx_adv_quick.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the x-direction
   !&E
   !&E ** Description : explicit - QUICK scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,tfiusr,wz,yz_exp,dx_u,dx,dy_f,dy_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-02 (F. Dumas) Upwind scheme along open boundaries + QUICK at imin+2 and jmin+2
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

   IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN

   ! QUICK scheme
   ! ------------

     DO k=1,kmax

     ! u * du/dx

       uc=uz(k,i,j)
       up=0.5_rsh*(uc+ABS(uc))
       um=0.5_rsh*(uc-ABS(uc))
       ududx=(up*uz(k,i-2,j)*whx(i-1,j)+(-7.0_rsh*up-3.0_rsh*um)*uz(k,i-1,j)+  &
             (3.0_rsh*up-3.0_rsh*um)*uc+(3.0_rsh*up+7.0_rsh*um)*uz(k,i+1,j) &
             -um*uz(k,i+2,j)*whx(i+1,j))/(8.0_rsh*dx_u(i,j))

     ! v * du/dy

       vp=0.5_rsh*(vz_u(k)+ABS(vz_u(k)))
       vm=0.5_rsh*(vz_u(k)-ABS(vz_u(k)))
       vdudy=(vp*uz(k,i,j-2)*ms(i,j-1)*ms(i,j)+(-7.0_rsh*vp-3.0_rsh*vm)*uz(k,i,j-1)+  &
             (3.0_rsh*vp-3.0_rsh*vm)*uc+(3.0_rsh*vp+7.0_rsh*vm)*uz(k,i,j+1) &
             -vm*uz(k,i,j+2)*mn(i,j+1)*mn(i,j))/(8.0_rsh*dy_u(i,j))

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k)*tfiusr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! u * du/dx

       uc=uz(k,i,j)
       up=0.5_rsh*(uc+ABS(uc))
       um=0.5_rsh*(uc-ABS(uc))
       ududx=up*(uz(k,i,j)-uz(k,i-1,j))/dx(i,j)+um*(uz(k,i+1,j)-uz(k,i,j))/dx(i+1,j)

     ! v * du/dy

       vp=0.5_rsh*(vz_u(k)+ABS(vz_u(k)))
       vm=0.5_rsh*(vz_u(k)-ABS(vz_u(k)))
       vdudy=vp*(uz(k,i,j)-uz(k,i,j-1))/dy_f(i,j-1)+vm*(uz(k,i,j+1)-uz(k,i,j))/dy_f(i,j)

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k)*tfiusr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * du/dz
   ! -------------------------

   k=1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   k=2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k) &
          +wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   DO k=3,kmax-2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=(wp*uz(k-2,i,j)+(-7.0_rsh*wp-3.0_rsh*wm)*uz(k-1,i,j)+         &
           (3.0_rsh*wp-3.0_rsh*wm)*uz(k,i,j)+(3.0_rsh*wp+7.0_rsh*wm)*uz(k+1,i,j) &
           -wm*uz(k+2,i,j))/(8.0_rsh*dsigw(k))
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz
   END DO

   k=kmax-1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k) &
          +wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   k=kmax
     wmo=0.25_rsh*(wz(k-1,i+1,j)+wz(k-1,i,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdudz=wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

!************** end  including dyn3dx_adv_quick.h    ****************************
