!**************    including dyn3dx_adv_quickest.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dx_adv_quickest.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the x-direction
   !&E
   !&E ** Description : explicit - QUICKEST scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dx_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz,vz_u,tfiusr,wz,yz_exp,dx_u,dx,dy_f,dy_u
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2009-02 (F. Dumas) Original code
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quickest scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

   IF (i >= imin+2 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+3) THEN

   ! QUICKEST scheme
   ! ---------------

     DO k=1,kmax

     ! u * du/dx

       uc=uz(k,i,j)
       up=0.5_rsh*(uc+ABS(uc))
       um=0.5_rsh*(uc-ABS(uc))
       ududx=( (up+um)*( 0.5_rsh*(uz(k,i+1,j)-uz(k,i-1,j)) -0.5_rsh*(up+um)*(uz(k,i+1,j)+uz(k,i-1,j)-2.0_rsh*uz(k,i,j)) ) &
               +up/6.0_rsh*(1.0_rsh-up*up)*(uz(k,i-2,j)*whx(i-1,j)-3.0_rsh*uz(k,i-1,j)+3.0_rsh*uz(k,i  ,j)-uz(k,i+1,j))   &
               +um/6.0_rsh*(1.0_rsh-um*um)*(uz(k,i-1,j)-3.0_rsh*uz(k,i  ,j)+3.0_rsh*uz(k,i+1,j)-uz(k,i+2,j)*whx(i+1,j)) )/dx_u(i,j)

     ! v * du/dy

       vp=0.5_rsh*(vz_u(k)+ABS(vz_u(k)))
       vm=0.5_rsh*(vz_u(k)-ABS(vz_u(k)))
       vdudy=( (vp+vm)*( 0.5_rsh*(uz(k,i,j+1)-uz(k,i,j-1)) -0.5_rsh*(vp+vm)*(uz(k,i,j+1)+uz(k,i,j-1)-2.0_rsh*uz(k,i,j)) )  &
               +vp/6.0_rsh*(1.0_rsh-vp*vp)*(uz(k,i,j-2)*ms(i,j-1)*ms(i,j)-3.0_rsh*uz(k,i,j-1)+3.0_rsh*uz(k,i,j  )-uz(k,i,j+1)) &
               +vm/6.0_rsh*(1.0_rsh-vm*vm)*(uz(k,i,j-1)-3.0_rsh*uz(k,i,j  )+3.0_rsh*uz(k,i,j+1)-uz(k,i,j+2)*mn(i,j+1)*mn(i,j)) ) &
              /dy_u(i,j)

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k)*tfiusr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! u * du/dx

       uc=uz(k,i,j)
       up=0.5_rsh*(uc+ABS(uc))
       um=0.5_rsh*(uc-ABS(uc))
       ududx=up*(uz(k,i,j)-uz(k,i-1,j))/dx(i,j)+um*(uz(k,i+1,j)-uz(k,i,j))/dx(i+1,j)

     ! v * du/dy

       vp=0.5_rsh*(vz_u(k)+ABS(vz_u(k)))
       vm=0.5_rsh*(vz_u(k)-ABS(vz_u(k)))
       vdudy=vp*(uz(k,i,j)-uz(k,i,j-1))/dy_f(i,j-1)+vm*(uz(k,i,j+1)-uz(k,i,j))/dy_f(i,j)

     ! u * v * tg(phi) / R

       uv=uz(k,i,j)*vz_u(k)*tfiusr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - ududx - vdudy + uv

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * du/dz
   ! -------------------------

   k=1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   k=2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k) &
          +wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   DO k=3,kmax-2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=( (wp+wm)*( 0.5_rsh*(uz(k+1,i,j)-uz(k-1,i,j)) -0.5_rsh*(wp+wm)*(uz(k+1,i,j)+uz(k-1,i,j)-2.0_rsh*uz(k,i,j)) ) &
             +wp/6.0_rsh*(1.0_rsh-wp*wp)*(uz(k-2,i,j)-3.0_rsh*uz(k-1,i,j)+3.0_rsh*uz(k  ,i,j)-uz(k+1,i,j))              &
             +wm/6.0_rsh*(1.0_rsh-wm*wm)*(uz(k-1,i,j)-3.0_rsh*uz(k  ,i,j)+3.0_rsh*uz(k+1,i,j)-uz(k+2,i,j)) )/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz
   END DO

   k=kmax-1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i+1,j)+wz(k-1,i,j)+wz(k-1,i+1,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdudz=wm*(uz(k+1,i,j)-uz(k,i,j))/dsigw(k) &
          +wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

   k=kmax
     wmo=0.25_rsh*(wz(k-1,i+1,j)+wz(k-1,i,j))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdudz=wp*(uz(k,i,j)-uz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdudz

!************** end  including dyn3dx_adv_quickest.h    ****************************
