!**************    including dyn3dy_adv_quick.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dy_adv_quick.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the y-direction
   !&E
   !&E ** Description : explicit - QUICK scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dy_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,tfivsr,wz,yz_exp,dx_v,dy,dx_f,dy_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2009-02 (F. Dumas) Upwind scheme along open boundaries + QUICK at imin+2 and jmin+2
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quick scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

   IF (i >= imin+3 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+2) THEN 

   ! QUICK scheme
   ! ------------

     DO k=1,kmax

     ! v * dv/dy

       vc=vz(k,i,j)
       vp=0.5_rsh*(vc+ABS(vc))
       vm=0.5_rsh*(vc-ABS(vc))
       vdvdy=(vp*vz(k,i,j-2)*why(i,j-1)+(-7.0_rsh*vp-3.0_rsh*vm)*vz(k,i,j-1)+  &
             (3.0_rsh*vp-3.0_rsh*vm)*vc+(3.0_rsh*vp+7.0_rsh*vm)*vz(k,i,j+1) &
             -vm*vz(k,i,j+2)*why(i,j+1))/(8.0_rsh*dy_v(i,j))

     ! u * dv/dx

       up=0.5_rsh*(uz_v(k)+ABS(uz_v(k)))
       um=0.5_rsh*(uz_v(k)-ABS(uz_v(k)))
       udvdx=(up*vz(k,i-2,j)*mo(i-1,j)*mo(i,j)+(-7.0_rsh*up-3.0_rsh*um)*vz(k,i-1,j)+  &
             (3.0_rsh*up-3.0_rsh*um)*vc+(3.0_rsh*up+7.0_rsh*um)*vz(k,i+1,j) &
             -um*vz(k,i+2,j)*me(i+1,j)*me(i,j))/(8.0_rsh*dx_v(i,j))

     ! u**2 * tg(phi) / R

       uv=uz_v(k)*uz_v(k)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! v * dv/dy

       vc=vz(k,i,j)
       vp=0.5_rsh*(vc+ABS(vc))
       vm=0.5_rsh*(vc-ABS(vc))
       vdvdy=vp*(vz(k,i,j)-vz(k,i,j-1))/dy(i,j)+vm*(vz(k,i,j+1)-vz(k,i,j))/dy(i,j+1)

     ! u * dv/dx

       up=0.5_rsh*(uz_v(k)+ABS(uz_v(k)))
       um=0.5_rsh*(uz_v(k)-ABS(uz_v(k)))
       udvdx=up*(vz(k,i,j)-vz(k,i-1,j))/dx_f(i-1,j)+um*(vz(k,i+1,j)-vz(k,i,j))/dx_f(i,j)

     ! u**2 * tg(phi) / R

       uv=uz_v(k)*uz_v(k)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * dv/dz
   ! -------------------------

   k=1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

   k=2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)+wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

   DO k=3,kmax-2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=(wp*vz(k-2,i,j)+(-7.0_rsh*wp-3.0_rsh*wm)*vz(k-1,i,j)+         &
           (3.0_rsh*wp-3.0_rsh*wm)*vz(k,i,j)+(3.0_rsh*wp+7.0_rsh*wm)*vz(k+1,i,j) &
     	   -wm*vz(k+2,i,j))/(8.0_rsh*dsigw(k))  
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz
   END DO

   k=kmax-1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)+wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) -wdvdz

   k=kmax
     wmo=0.25_rsh*(wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdvdz=wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

!*****************     end  including dyn3dy_adv_quick.h      ******************
