!**************    including dyn3dy_adv_quickest.h      ***********************

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3dy_adv_quickest.h  ***
   !&E
   !&E ** Purpose : estimate of advection terms (3d) of the momentum
   !&E              equation along the y-direction
   !&E
   !&E ** Description : explicit - QUICKEST scheme (decentre amont)
   !&E
   !&E ** Called by : dyn3dy_tls_coef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : uz_v,vz,tfivsr,wz,yz_exp,dx_v,dy,dx_f,dy_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Kowalik, Murty "Numerical Modeling of Ocean Dynamics"
   !&E
   !&E ** History :
   !&E       !  2009-02 (F. Dumas) Original code
   !&E       !  2009-03  (V. Garnier) land mask
   !&E       !  2011-11  (F. Vandermeirsch) correct spatial extension on which we can estimate quickest scheme
   !&E
   !&E---------------------------------------------------------------------

   ! ----------------
   ! Horizontal terms
   ! ----------------

   IF (i >= imin+3 .AND. i <= imax-2 .AND. j <= jmax-2 .AND. j >= jmin+2) THEN

   ! QUICKEST scheme
   ! ---------------

     DO k=1,kmax

     ! v * dv/dy

       vc=vz(k,i,j)
       vp=0.5_rsh*(vc+ABS(vc))
       vm=0.5_rsh*(vc-ABS(vc))
       vdvdy=( (vp+vm)*( 0.5_rsh*(vz(k,i,j+1)-vz(k,i,j-1)) -0.5_rsh*(vp+vm)*(vz(k,i,j+1)+vz(k,i,j-1)-2.0_rsh*vz(k,i,j)) )       &
                       +vp/6.0_rsh*(1.0_rsh-vp*vp)*(vz(k,i,j-2)*why(i,j-1)-3.0_rsh*vz(k,i,j-1)+3.0_rsh*vz(k,i,j  )-vz(k,i,j+1)) &
                       +vm/6.0_rsh*(1.0_rsh-vm*vm)*(vz(k,i,j-1)-3.0_rsh*vz(k,i,j  )+3.0_rsh*vz(k,i,j+1)-vz(k,i,j+2)*why(i,j+1)) ) &
                      /dy(i,j)

     ! u * dv/dx

       up=0.5_rsh*(uz_v(k)+ABS(uz_v(k)))
       um=0.5_rsh*(uz_v(k)-ABS(uz_v(k)))
       udvdx=( (up+um)*( 0.5_rsh*(vz(k,i+1,j)-vz(k,i-1,j)) -0.5_rsh*(up+um)*(vz(k,i+1,j)+vz(k,i-1,j)-2.0_rsh*vz(k,i,j)) )          &
                 +up/6.0_rsh*(1.0_rsh-up*up)*(vz(k,i-2,j)*mo(i-1,j)*mo(i,j)-3.0_rsh*vz(k,i-1,j)+3.0_rsh*vz(k,i  ,j)-vz(k,i+1,j))   &
                 +um/6.0_rsh*(1.0_rsh-um*um)*(vz(k,i-1,j)-3.0_rsh*vz(k,i  ,j)+3.0_rsh*vz(k,i+1,j)-vz(k,i+2,j)*me(i+1,j)*me(i,j)) ) &
                /dx_v(i,j)

     ! u**2 * tg(phi) / R

       uv=uz_v(k)*uz_v(k)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv

     END DO

   ELSE

   ! Upwind scheme
   ! -------------

     DO k=1,kmax

     ! v * dv/dy

       vc=vz(k,i,j)
       vp=0.5_rsh*(vc+ABS(vc))
       vm=0.5_rsh*(vc-ABS(vc))
       vdvdy=vp*(vz(k,i,j)-vz(k,i,j-1))/dy(i,j)+vm*(vz(k,i,j+1)-vz(k,i,j))/dy(i,j+1)

     ! u * dv/dx

       up=0.5_rsh*(uz_v(k)+ABS(uz_v(k)))
       um=0.5_rsh*(uz_v(k)-ABS(uz_v(k)))
       udvdx=up*(vz(k,i,j)-vz(k,i-1,j))/dx_f(i-1,j)+um*(vz(k,i+1,j)-vz(k,i,j))/dx_f(i,j)

     ! u**2 * tg(phi) / R

       uv=uz_v(k)*uz_v(k)*tfivsr(i,j)

     ! add all terms

       yz_exp(k,i,j) = yz_exp(k,i,j) - vdvdy - udvdx - uv

     END DO

   END IF


   ! -------------------------
   ! Vertical term : w * dv/dz
   ! -------------------------

   k=1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

   k=2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)+wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

   DO k=3,kmax-2
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=( (wp+wm)*( 0.5_rsh*(vz(k+1,i,j)-vz(k-1,i,j)) -0.5_rsh*(wp+wm)*(vz(k+1,i,j)+vz(k-1,i,j)-2.0_rsh*vz(k,i,j)) )     &
                     +wp/6.0_rsh*(1.0_rsh-wp*wp)*(vz(k-2,i,j)-3.0_rsh*vz(k-1,i,j)+3.0_rsh*vz(k  ,i,j)-vz(k+1,i,j))          &
                     +wm/6.0_rsh*(1.0_rsh-wm*wm)*(vz(k-1,i,j)-3.0_rsh*vz(k  ,i,j)+3.0_rsh*vz(k+1,i,j)-vz(k+2,i,j)) )/dsigw(k)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz
   END DO

   k=kmax-1
     wmo=0.25_rsh*(wz(k,i,j)+wz(k,i,j+1)+wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wm=0.5_rsh*(wmo-ABS(wmo))
     wdvdz=wm*(vz(k+1,i,j)-vz(k,i,j))/dsigw(k)+wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) -wdvdz

   k=kmax
     wmo=0.25_rsh*(wz(k-1,i,j)+wz(k-1,i,j+1))
     wp=0.5_rsh*(wmo+ABS(wmo))
     wdvdz=wp*(vz(k,i,j)-vz(k-1,i,j))/dsigw(k-1)
     yz_exp(k,i,j) = yz_exp(k,i,j) - wdvdz

!*****************     end  including dyn3dy_adv_quickest.h      ******************
