!*********    including dyn3dy_ipg_dj.h      ****************************

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dy_ipg_dj.h  ***
   !&E
   !&E ** Purpose : estimate the trend term relative to the internal
   !&E              pressure gradient for the zonal momentum forcing
   !&E              (3D equation)
   !&E
   !&E ** Description : explicit
   !&E                  vertical integration of the density jacobian
   !&E
   !&E ** Called by : included into dyn3dytlscoef.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : xec,h0,boz,dy_v
   !&E
   !&E ** Modified variables : yz_exp
   !&E
   !&E ** Reference : Second order density jacobian of ROMS/SCRUM
   !&E                Shchepetkin and Mc Williams (2003)
   !&E
   !&E ** History :
   !&E       !  2010-09  (M. Garnier) Original code
   !&E
   !&E---------------------------------------------------------------------

   dyb=dy_v(i,j)

!    dxey=(xec(i,j+1)-xec(i,j))/dyb

   ! surface
   k=kmax
   z1 = f_zu(h0(i,j),xec(i,j),k,i,j)
   z3 = f_zu(h0(i,j+1),xec(i,j+1),k,i,j+1)

   ! extrapolation to estimate the buoyancy at the sea surface
   dzb=f_zu(h0(i,j  ),xec(i,j  ),k,i,j  )-f_zu(h0(i  ,j),xec(i,j  ),k-1,i,j  )
   b2surf=boz(k,i,j  )+(boz(k,i,j  )-boz(k-1,i,j  ))/dzb*(xec(i,j  )-z1)
   dzb=f_zu(h0(i,j+1),xec(i,j+1),k,i,j+1)-f_zu(h0(i,j+1),xec(i,j+1),k-1,i,j+1)
   b4surf=boz(k,i,j+1)+(boz(k,i,j+1)-boz(k-1,i,j+1))/dzb*(xec(i,j+1)-z3)

#ifdef key_song
!cval je ne comprends pas pourquoi je dois utiliser la pseudo formule en surface
!cval alors que dans les couches internes cette ecriture est moins precise ?????
   ! horizontal gradient of sigma-averaged buoyancy dybs = -(b1+b2-b3-b4)
   dybs = 0.5_rlg*(b4surf+boz(k,i,j+1)-b2surf-boz(k,i,j))
   ! vertical gradient of y-averaged depth (z4-z3+z2-z1)
   dszy = 0.5_rlg*(-z3-z1+xec(i,j+1)+xec(i,j))
   ! vertical gradient of y-averaged buoyancy (b4+b3-b2-b1)/dy
   dsby = 0.5_rlg*(b4surf+b2surf-boz(k,i,j+1)-boz(k,i,j))
   ! horizontal gradient of sigma-averaged depth dyzs = -(z1+z2-z3-z4)
   dyzs = 0.5_rlg*(z3+z4-z1-z2)

   ! vertical integration of the jacobian
   jacobian = (dybs*dszy-dsby*dyzs)/dyb

#else
   ! pseudo-flux form
   ! f21=0.5_rlg*(b1+b2)*(z2-z1)
   f21=0.5_rlg*(boz(k,i,j)+b2surf)*(xec(i,j)-z1)
   ! f42=0.5_rlg*(b2+b4)*(z4-z2)
   f42=0.5_rlg*(b2surf+b4surf)*(xec(i,j+1)-xec(i,j))
   ! f43=0.5_rlg*(b3+b4)*(z4-z3)
   f43=0.5_rlg*(boz(k,i,j+1)+b4surf)*(xec(i,j+1)-z3)
   ! f31=0.5_rlg*(b1+b3)*(z3-z1)
   f31=0.5_rlg*(boz(k,i,j)+boz(k,i,j+1))*(z3-z1)

   ! vertical integration of the jacobian
   jacobian=(f31+f43-f42-f21)/dyb
#endif

   yz_exp(k,i,j)= yz_exp(k,i,j)                   &
                  ! vertical integration
                  +jacobian
                  ! buoyancy * gradient of ssh
!                  +dxey*(boz(k,i,j)+boz(k,i,j+1))*0.5_rlg

   DO k=kmax-1,1,-1
     z2 = z1
     z4 = z3
     z1 = f_zu(h0(i,j),xec(i,j),k,i,j)
     z3 = f_zu(h0(i,j+1),xec(i,j+1),k,i,j+1)

     ! horizontal gradient of sigma-averaged buoyancy dybs = -(b1+b2-b3-b4)
     dybs = 0.5_rlg*(boz(k+1,i,j+1)+boz(k,i,j+1)-boz(k+1,i,j)-boz(k,i,j))
     ! vertical gradient of y-averaged depth (z4-z3+z2-z1)
     dszy = 0.5_rlg*(z4-z3+z2-z1)
     ! vertical gradient of y-averaged buoyancy (b4+b3-b2-b1)/dy
     dsby = 0.5_rlg*(boz(k+1,i,j+1)+boz(k+1,i,j)-boz(k,i,j+1)-boz(k,i,j))
     ! horizontal gradient of sigma-averaged depth dyzs = -(z1+z2-z3-z4)
     dyzs = 0.5_rlg*(z3+z4-z1-z2)

   ! vertical integration of the jacobian
     jacobian = jacobian + (dybs*dszy-dsby*dyzs)/dyb

     ! vertical integration
     yz_exp(k,i,j)=yz_exp(k,i,j)+jacobian

   END DO


!*********  end including dyn3dy_ipg_dj.h      ****************************
