   !!======================================================================

  SUBROUTINE init_height(xe)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE init_height  ***
   !&E
   !&E ** Purpose : Evaluation of water column height and layer depths
   !&E              at u,v locations
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main.F90,initfromfile.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hx,hy,xe
   !&E
   !&E ** Modified variables : htx,hzutx,hty,hzuty
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-07  (V. Garnier)  Original code from dyn3d_height_mask
   !&E       !  2011-03  (F. Dumas) remove case 2 and 3 on tidal flats
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : htx,hty,hx,hy,h0fond
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzutx,hzwtx,hzuty,hzwty
   USE toolgeom,  ONLY : f_hzu_x,f_hzu_y,f_hzw_x,f_hzw_y
#endif
   USE_MPI toolmpi, ONLY : ex_j_rlg,ex_i_rlg,ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT (in) :: xe

   !! * Local declarations
   INTEGER                                                     :: i,j,k

   !!----------------------------------------------------------------------
   !! * Executable part

#ifdef key_MPI_2D
!$OMP DO schedule(runtime)
#else
!$OMP DO schedule(static)
#endif
   DO j=ljmin,ljmaxm1
     DO i=limin,limaxm1
       htx(i,j)=hx(i,j)+0.5_rsh*(xe(i,j)+xe(i+1,j))
       hty(i,j)=hy(i,j)+0.5_rsh*(xe(i,j)+xe(i,j+1))
#if defined key_siggen || defined key_gencoord
       DO k=1,kmax-1
   ! dz/ds at location hx (i+1/2) and depth u or w
         hzutx(k,i,j)=f_hzu_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)
         hzwtx(k,i,j)=f_hzw_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)
         IF ( htx(i,j)<=0.0_rsh .AND. ( hx(i,j)+xe(i,j) > 0.0_rsh .OR. hx(i,j)+xe(i+1,j) > 0.0_rsh) ) THEN
           hzutx(k,i,j)=f_hzu_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)
           hzwtx(k,i,j)=f_hzw_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)
         END IF
   ! dz/ds at location hy (j+1/2) and depth of v or w
         hzuty(k,i,j)=f_hzu_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)
         hzwty(k,i,j)=f_hzw_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)
         IF ( hty(i,j)<=0.0_rsh .AND. ( hy(i,j)+xe(i,j) > 0.0_rsh .OR. hy(i,j)+xe(i,j+1) > 0.0_rsh )) THEN
           hzuty(k,i,j)=f_hzu_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)
           hzwty(k,i,j)=f_hzw_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)
         END IF
       END DO
       k=kmax
       hzutx(k,i,j)=f_hzu_x(hx(i,j),0.5_rsh*(xe(i,j)+xe(i+1,j)),k,i,j)
       IF ( htx(i,j)<=0.0_rsh .AND. ( hx(i,j)+xe(i,j) > 0.0_rsh .OR. hx(i,j)+xe(i+1,j) > 0.0_rsh) ) &
         hzutx(k,i,j)=f_hzu_x(hx(i,j),MAX(xe(i,j),xe(i+1,j)),k,i,j)
       hzuty(k,i,j)=f_hzu_y(hy(i,j),0.5_rsh*(xe(i,j)+xe(i,j+1)),k,i,j)
       IF ( hty(i,j)<=0.0_rsh .AND. ( hy(i,j)+xe(i,j) > 0.0_rsh .OR. hy(i,j)+xe(i,j+1) > 0.0_rsh )) &
         hzuty(k,i,j)=f_hzu_y(hy(i,j),MAX(xe(i,j),xe(i,j+1)),k,i,j)
#endif
       IF ( htx(i,j)<=0.0_rsh .AND. ( hx(i,j)+xe(i,j) > 0.0_rsh .OR. hx(i,j)+xe(i+1,j) > 0.0_rsh) ) &
         htx(i,j)=hx(i,j)+MAX(xe(i,j),xe(i+1,j))
       IF ( hty(i,j)<=0.0_rsh .AND. ( hy(i,j)+xe(i,j) > 0.0_rsh .OR. hy(i,j)+xe(i,j+1) > 0.0_rsh )) &
         hty(i,j)=hy(i,j)+MAX(xe(i,j),xe(i,j+1))
     END DO

     IF_MPI (limax==imax) THEN
       htx(imax,j)=hx(imax,j)+         xe(imax,j)
       hty(imax,j)=hy(imax,j)+0.5_rsh*(xe(imax,j)+xe(imax,j+1))
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hy (j+1/2) and depth of v or w
       DO k=1,kmax-1
         hzutx(k,imax,j)=f_hzu_x(hx(imax,j),         xe(imax,j)              ,k,imax,j)
         hzwtx(k,imax,j)=f_hzw_x(hx(imax,j),         xe(imax,j)              ,k,imax,j)
         hzuty(k,imax,j)=f_hzu_y(hy(imax,j),0.5_rsh*(xe(imax,j)+xe(imax,j+1)),k,imax,j)
         hzwty(k,imax,j)=f_hzw_y(hy(imax,j),0.5_rsh*(xe(imax,j)+xe(imax,j+1)),k,imax,j)
       END DO
       IF ( hty(imax,j)<=0.0_rsh .AND. ( hy(imax,j)+xe(imax,j) > 0.0_rsh .OR. hy(imax,j)+xe(imax,j+1) > 0.0_rsh )) THEN
         hzuty(k,imax,j)=f_hzu_y(hy(imax,j),MAX(xe(imax,j),xe(imax,j+1)),k,imax,j)
         hzwty(k,imax,j)=f_hzw_y(hy(imax,j),MAX(xe(imax,j),xe(imax,j+1)),k,imax,j)
       END IF
       k=kmax
       hzutx(k,imax,j)=f_hzu_x(hx(imax,j),         xe(imax,j)              ,k,imax,j)
       hzuty(k,imax,j)=f_hzu_y(hy(imax,j),0.5_rsh*(xe(imax,j)+xe(imax,j+1)),k,imax,j)
       IF ( hty(imax,j)<=0.0_rsh .AND. ( hy(imax,j)+xe(imax,j) > 0.0_rsh .OR. hy(imax,j)+xe(imax,j+1) > 0.0_rsh )) &
         hzuty(k,imax,j)=f_hzu_y(hy(imax,j),MAX(xe(imax,j),xe(imax,j+1)),k,imax,j)
#endif
       IF ( hty(imax,j)<=0.0_rsh .AND. ( hy(imax,j)+xe(imax,j) > 0.0_rsh .OR. hy(imax,j)+xe(imax,j+1) > 0.0_rsh )) &
         hty(imax,j)=hy(imax,j)+MAX(xe(imax,j),xe(imax,j+1))
     ENDIF_MPI

   END DO ! end do for j loop
!$OMP END DO

!$OMP SINGLE
   IF_MPI (ljmax==jmax) THEN
   DO i=limin,limaxm1
     htx(i,jmax)=hx(i,jmax)+0.5_rsh*(xe(i,jmax)+xe(i+1,jmax))
     hty(i,jmax)=hy(i,jmax)+         xe(i,jmax)
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hx (i+1/2) and depth u or w
     DO k=1,kmax-1
       hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),0.5_rsh*(xe(i,jmax)+xe(i+1,jmax)),k,i,jmax)
       hzwtx(k,i,jmax)=f_hzw_x(hx(i,jmax),0.5_rsh*(xe(i,jmax)+xe(i+1,jmax)),k,i,jmax)
       hzuty(k,i,jmax)=f_hzu_y(hy(i,jmax),         xe(i,jmax)              ,k,i,jmax)
       hzwty(k,i,jmax)=f_hzw_y(hy(i,jmax),         xe(i,jmax)              ,k,i,jmax)
     END DO
     IF ( htx(i,jmax)<=0.0_rsh .AND. ( hx(i,jmax)+xe(i,jmax) > 0.0_rsh .OR. hx(i,jmax)+xe(i+1,jmax) > 0.0_rsh) ) THEN
       hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),MAX(xe(i,jmax),xe(i+1,jmax)),k,i,jmax)
       hzwtx(k,i,jmax)=f_hzw_x(hx(i,jmax),MAX(xe(i,jmax),xe(i+1,jmax)),k,i,jmax)
     END IF
     k=kmax
     hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),0.5_rsh*(xe(i,jmax)+xe(i+1,jmax)),k,i,jmax)
     hzuty(k,i,jmax)=f_hzu_y(hy(i,jmax),         xe(i,jmax)              ,k,i,jmax)
     IF ( htx(i,jmax)<=0.0_rsh .AND. ( hx(i,jmax)+xe(i,jmax) > 0.0_rsh .OR. hx(i,jmax)+xe(i+1,jmax) > 0.0_rsh) ) &
       hzutx(k,i,jmax)=f_hzu_x(hx(i,jmax),MAX(xe(i,jmax),xe(i+1,jmax)),k,i,jmax)
#endif
     IF ( htx(i,jmax)<=0.0_rsh .AND. ( hx(i,jmax)+xe(i,jmax) > 0.0_rsh .OR. hx(i,jmax)+xe(i+1,jmax) > 0.0_rsh) ) &
       htx(i,jmax)=hx(i,jmax)+MAX(xe(i,jmax),xe(i+1,jmax))
   END DO
   ENDIF_MPI

   IF_MPI (ljmax==jmax .AND. limax==imax) THEN
     htx(imax,jmax)=hx(imax,jmax)+         xe(imax,jmax)
     hty(imax,jmax)=hy(imax,jmax)+         xe(imax,jmax)
#if defined key_siggen || defined key_gencoord
   ! dz/ds at location hx (i+1/2) and depth u or w
     DO k=1,kmax-1
       hzutx(k,imax,jmax)=f_hzu_x(hx(imax,jmax),         xe(imax,jmax),k,imax,jmax)
       hzwtx(k,imax,jmax)=f_hzw_x(hx(imax,jmax),         xe(imax,jmax),k,imax,jmax)
       hzuty(k,imax,jmax)=f_hzu_y(hy(imax,jmax),         xe(imax,jmax),k,imax,jmax)
       hzwty(k,imax,jmax)=f_hzw_y(hy(imax,jmax),         xe(imax,jmax),k,imax,jmax)
     END DO
     k=kmax
     hzutx(k,imax,jmax)=f_hzu_x(hx(imax,jmax),         xe(imax,jmax),k,imax,jmax)
     hzuty(k,imax,jmax)=f_hzu_y(hy(imax,jmax),         xe(imax,jmax),k,imax,jmax)
#endif
   ENDIF_MPI
!$OMP END SINGLE

   ! Maybe not usefull...
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,htx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,htx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hty(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hty(liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(htx,hty)

  END SUBROUTINE init_height
