  SUBROUTINE cas_tidalflat2dv

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE cas_tidalflat2dv  ***
   !&E
   !&E ** Purpose : definition of a test case for a westward open channel
   !&E              containing sandy and muddy sediments and a tidal flat 
   !&E              on the eastward boundary
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : cas_init
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2011-03 (F. Cayocca) new test case issued from test caschansed
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comvarp2d, ONLY : ssh,u,v,sal,temp
   USE comvarp3d, ONLY : uz,vz,ect
   USE comvars3d
   USE output, ONLY : fileresu,tdebsor,tfinsor,nb_out,tinter,tsort,pasor,   &
                      l_out_precdate,l_out_demer,l_out_ssh,l_out_ubt,l_out_u3d, &
                      l_out_sal,l_out_temp,l_out_wind,l_out_buoy,           &
                      l_out_visc,l_out_turb,l_out_off,l_out_atmsflx,nbatm,  &
                      iextmin,iextmax,l_out_nc4par,l_out_pack,              &
                      jextmin,jextmax,kextmin,kextmax,stepi,stepj,stepk,    &
                      igextmin,igextmax,jgextmin,jgextmax,l_filebydate,     &
                      l_out_zaxis_w
   USE obcogcm, ONLY : obc_coefrel
   USE cometdyn
   USE comturb
   USE comdiag, ONLY : l_diag,file_diag,tdebdiag,tfindiag
   USE trackpoints, ONLY : file_point,l_points,track_points
   USE cometemp
   USE comfluxdo
#ifdef key_substance
   USE comsubstance
#endif
   USE comaree
   USE obccombine
   USE_MPI toolmpi, ONLY: ADD_TAG,ex_i_rsh,ex_j_rsh,MASTER,min_all_mpi_rsh,max_all_mpi_rsh
#if defined key_siggen || defined key_gencoord
   USE comsiggen
#endif
#ifdef key_sedim
   USE sedimento
#endif
#ifdef key_contaminant
   USE contareaddat
#endif
#ifdef key_castestriver
   USE comfluxdo
#endif
   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   CHARACTER(LEN=19) :: date_start,date_end,date_start3d,date_startagrif
   CHARACTER(LEN=19) :: date_startout,date_endout
   CHARACTER(LEN=19) :: tool_sectodat
   CHARACTER(LEN=lchain) :: logfilename
   INTEGER           :: i,j,k,iiv,indgrp,IERR_MPI
   INTEGER           :: iv,isubs
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: hx_t,hy_t
#ifdef key_sedim
   REAL(KIND=rsh)    :: dzsf,somalp
#if defined key_castestsub || defined key_castestconta
   LOGICAL           :: l_varassoc
   INTEGER           :: ivpc
#endif
#endif
   REAL(KIND=rsh)    :: x,dxval,dyval,hdeep,hshallow,ws3maxloc
   REAL(KIND=rlg)    :: tool_datosec
#ifdef key_substance
   REAL(KIND=rlg)    :: tdb,tfi
   INTEGER           :: n
#endif
#ifdef key_contaminant
   INTEGER           :: ireac
#endif
   REAL(KIND=rsh)    :: value

   !!----------------------------------------------------------------------
   !! * Executable part

   !-----------------------------------------------------------------------
   !-----------------------------------------------------------------------
   !! test case : castidalflat2dv : 2DV sediment dynamics along a tidal flat 
   !-----------------------------------------------------------------------
   !-----------------------------------------------------------------------

   PRINT*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
   PRINT*, 'TEST CASE : castidalflat2dv '
   PRINT*, '!!!!!!!!!!!!!!!!!!!!!!!!!!!!'

   ! bathymetry
   ! ----------
   mslshift=0.0_rsh
   h0fond=0.01_rsh
   hshallow=4.0_rsh
   hdeep=16.0_rsh
   hx_g(imin,:)=hdeep
   hy_g(imin,:)=hdeep
   h0_g(imin,:)=hdeep+h0fond
   DO i=imin+1,imax
     hx_g(i,:)=hdeep-REAL(i-imin-1,rsh)/REAL(imax-imin-1,rsh)*(hdeep-hshallow)
     hy_g(i,:)=hdeep-REAL(i-imin-1,rsh)/REAL(imax-imin-1,rsh)*(hdeep-hshallow)
     h0_g(i,:)=hdeep-REAL(i-imin-1,rsh)/REAL(imax-imin-1,rsh)*(hdeep-hshallow)
   END DO
   
 
   DO j=jmin+1,jmax
   DO i=imin+1,imax
      h0_g(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))+h0fond
   END DO
   END DO

   hm=1.0_rsh
   l_tssubadv_over_smalldepth=.true.
   hy_g(:,jmin+1)=-valmanq-h0fond
   hy_g(:,jmax-1)=-valmanq-h0fond
   hx_g(imax-1,:)=-valmanq-h0fond
   hy_g(:,:)=-valmanq-h0fond

   h0(:,:) = h0_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hx(:,:) = hx_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hy(:,:) = hy_g(liminm2:limaxp2,ljminm2:ljmaxp2)

   ! grid parametrization
   ! --------------------
   dxval=500.0_rsh
   dyval=500.0_rsh
   fisac=0.0_riolg
   gwac=0.0_riolg
   dfiac=0.01_riolg
   dgac= 0.01_riolg
   DO j = ljmin,ljmax
     DO i = limin,limax
       dx(i,j)=dxval
       dx_u(i,j)=dxval
       dx_v(i,j)=dxval
       dx_f(i,j)=dxval
       dy(i,j)=dyval
       dy_u(i,j)=dyval
       dy_v(i,j)=dyval
       dy_f(i,j)=dyval
       surf(i,j)=dy(i,j)**2
       fu(i,j)=0.0_rsh
       fv(i,j)=0.0_rsh
       tfiusr(i,j)=0.0_rsh
       tfivsr(i,j)=0.0_rsh
       goulet(i,j)=1.0_rsh
       govlet(i,j)=1.0_rsh
     ENDDO
   ENDDO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,govlet(limin:limax,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dx(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dx(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limaxp1,ljminm1,ljmax,dx_v(limin:limaxp1,ljminm1:ljmax))
   CALL_MPI ex_i_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dy(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,1,liminm1,limaxp1,ljminm1,ljmaxp1,dy(liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,liminm1,limax,ljmin,ljmaxp1,dy_u(liminm1:limax,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,dx_u(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin,ljmax,dx_f(liminm1:limax,ljmin:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,dy_v(limin:limax,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin,limax,ljminm1,ljmax,dy_f(limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(govlet,dx,dx_v,dy,dy_u,dx_u,dx_f,dy_v,dy_f)

   jb(:)=jmin+1
   jh(:)=jmax
   ig(:)=imin+1
   id(:)=imax
   dhx(:,:)=0.0_rsh
   dhy(:,:)=0.0_rsh
   dhx_g(:,:)=0.0_rsh
   dhy_g(:,:)=0.0_rsh
   l_corio_cst=.TRUE.

   DO k=1,kmax
     sig(k)=(REAL(k-kmax,rsh)-0.5_rsh)/REAL(kmax,rsh)
   END DO
   sig(kmax+1)=-sig(kmax)
   sig(0)=ABS(sig(1))-2.0_rsh

   DO k=1,kmax
     dsigw(k)=sig(k+1)-sig(k)
     sigw(k)=0.5_rsh*(sig(k)+sig(k+1))
   END DO
   sigw(kmax+1)=-sigw(kmax)
   sigw(0)=-1.0_rsh

   DO k=1,kmax
     dsigu(k)=sigw(k)-sigw(k-1)
   END DO

#if defined key_siggen || defined key_gencoord

   theta_sig= 4.0_rsh ! entre 0 et 20
   theta_sig= 0.0000001_rsh ! entre 0 et 20
   theta_sig= 1.0_rsh ! entre 0 et 20
   b_sig= 0.750_rsh     ! entre 0 et 1
   hc = 5.0_rsh
   WHERE (h0_g(:,:)/=-valmanq) hc_sig_g(:,:)=MIN(h0_g(:,:)-0.00001_rsh,hc)
   WHERE (hx_g(:,:)/=-valmanq-h0fond) hcx_sig_g(:,:)=MIN(hx_g(:,:)-0.00001_rsh,hc)
   WHERE (hy_g(:,:)/=-valmanq-h0fond) hcy_sig_g(:,:)=MIN(hy_g(:,:)-0.00001_rsh,hc)
   hc_sig(:,:)  = hc_sig_g(liminm2:limaxp2,ljminm2:ljmaxp2)
   hcx_sig(:,:)= hc_sig_g(limin:limaxp1,ljmin:ljmaxp1)
   hcy_sig(:,:)= hc_sig_g(limin:limaxp1,ljmin:ljmaxp1)
   DO k=0,kmax+1
     Csu_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sig(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sig(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO
   DO k=0,kmax
     Csw_sig(k)=(1.0_rsh-b_sig)*SINH(theta_sig*sigw(k))/SINH(theta_sig) &
                + 0.5_rsh*b_sig*(TANH(theta_sig*(sigw(k)+0.5_rsh))-TANH(0.5_rsh*theta_sig))/TANH(theta_sig/2.0_rsh)
   END DO

   DO k=1,kmax
     dcusds(k)=(Csw_sig(k)-Csw_sig(k-1))/(sigw(k)-sigw(k-1))
   END DO

   DO k=1,kmax-1
     dcwsds(k)=(Csu_sig(k+1)-Csu_sig(k))/(sig(k+1)-sig(k))
   END DO
   dcwsds(kmax)=(0.0_rsh-csu_sig(kmax))/(0.0_rsh-sig(kmax))

#endif

   ! longitude latitude
   ! ------------------

   lon2d_g(:,0)=(/ (dxval *(i-1), i=0,imax) /)
   DO j=1,jmax
     lon2d_g(:,j)=lon2d_g(:,0)
   END DO
   lon2du_g(:,:)=lon2d_g(:,:)+dxval/2.0_rsh
   lon2dv_g(:,:)=lon2d_g(:,:)
   lat2d_g(0,:)=(/ (dyval *(j-1), j=0,jmax) /)
   DO i=1,imax
     lat2d_g(i,:)=lat2d_g(0,:)
   END DO
   lat2du_g(:,:)=lat2d_g(:,:)
   lat2dv_g(:,:)=lat2d_g(:,:)+dyval/2.0_rsh

   lon2du(liminm1:limax,ljmin:ljmaxp1)=lon2du_g(liminm1:limax,ljmin:ljmaxp1)
   lon2d(limin:limax,ljmin:ljmax)=lon2d_g(limin:limax,ljmin:ljmax)
   lon2dv(limin:limax,ljmin:ljmax)=lon2dv_g(limin:limax,ljmin:ljmax)

   lat2dv(limin:limax,ljminm1:ljmax)=lat2dv_g(limin:limax,ljminm1:ljmax)
   lat2d(limin:limax,ljmin:ljmax)=lat2d_g(limin:limax,ljmin:ljmax)
   lat2du(limin:limax,ljmin:ljmax)=lat2du_g(limin:limax,ljmin:ljmax)

   ! dates
   ! -----

   date_start=   '01/01/1900 00:00:00'
   date_end=   '05/01/1900 00:00:00'
   date_start3d=  '01/01/1900 00:00:00'
   date_startagrif='01/01/1900 00:00:00'
#ifdef key_substance
   date_startsub='01/01/1900 01:00:00'
   tdebsubs=tool_datosec(date_startsub)
#ifdef key_sedim
   ! consolidation only if  l_consolid=.true. (not ready yet)
   date_startconsol='30/01/2000 00:00:00'
   tstartconsol=tool_datosec(date_startconsol)
   tstartconsol=MAX(tstartconsol,tdebsubs)
   dtconsol=500.0_rlg
#endif
#endif
   date_ref = '01-Jan-1900 00:00:00'
   tdeb     = tool_datosec(date_start) 
   t=tdeb
   tfin     = tool_datosec(date_end)
   tdeb3d   = tool_datosec(date_start3d)
   tdebagrif = tool_datosec(date_startagrif)
   date_startshift= 0.0_rlg
   dtmin=30.0_rlg
   dtmax=50.0_rlg
   dt=30.0_rlg
   l_modele2d=.FALSE.
   cflcrt=0.6_rsh
   tobs=2.0_rlg*3600.0_rlg
   tdebobs=t
   cflumaxmax=0.0_rsh
   cflvmaxmax=0.0_rsh

   l_initfromfile=.FALSE.
   l_saverestart_1file=.FALSE.
   l_saverestart_bydate=.FALSE.
   file_saverestart='save.nc'
   saverestart_step=5.0_rlg*3600.0_rlg   ! in hours
   name_in_dt='dt'
   name_bathy_h0='h0'
   name_bathy_hx='hx'
   name_bathy_hy='hy'
   name_in_xev='xev'
   name_in_u='u'
   name_in_v='v'
   name_in_uz='uz'
   name_in_vz='vz'
   name_in_wz='wz'
   name_in_temp='temp'
   name_in_sal='sal'
   name_in_ect='ect'
   name_in_psi='psi'
   name_in_kz='kz'
   name_in_nz='nz'
   name_in_vis2d='name_in_vis2d'
   name_in_vis3d_xe='name_in_vis3d_xe'
   name_in_vis3d_phi='name_in_vis3d_phi'

   ! outputs
   ! -------

   nb_out=1
   ALLOCATE( fileresu(nb_out),tdebsor(nb_out),tfinsor(nb_out),tinter(nb_out) )
   ALLOCATE( tsort(nb_out),pasor(nb_out),l_filebydate(nb_out),l_out_precdate(nb_out) )
   ALLOCATE( l_out_off(nb_out),l_out_atmsflx(nb_out),l_out_visc(nb_out),l_out_turb(nb_out) )
   ALLOCATE( l_out_buoy(nb_out), l_out_zaxis_w(nb_out) )
   ALLOCATE( l_out_demer(nb_out),l_out_ubt(nb_out),l_out_u3d(nb_out) )
   ALLOCATE( l_out_sal(nb_out),l_out_temp(nb_out),l_out_ssh(nb_out),l_out_wind(nb_out) )
   ALLOCATE( iextmin(nb_out),iextmax(nb_out),jextmin(nb_out),jextmax(nb_out) )
   ALLOCATE( igextmin(nb_out),igextmax(nb_out),jgextmin(nb_out),jgextmax(nb_out) )
   ALLOCATE( kextmin(nb_out),kextmax(nb_out),stepi(nb_out),stepj(nb_out),stepk(nb_out) )

   suffix='tidalflat'
   out_dir='./'

#ifdef key_sedim
#ifdef key_castestsub
#if defined key_sedim_mixsed
   fileresu(1)='results_tflat2dv_castestsub_mixsed.nc'
#else  
   fileresu(1)='results_tflat2dv_castestsub_mudsand.nc'
#endif
#elif key_castestconta
#if defined key_sedim_mixsed                              
     fileresu(1)='results_tflat2dv_castestconta_mixsed.nc'                
#else                                                     
     fileresu(1)='results_tflat2dv_castestconta_mudsand.nc'   
#endif
#else 
#if defined key_sedim_mixsed
   fileresu(1)='results_tflat2dv_mixsed.nc'
#else 
   fileresu(1)='results_tflat2dv_mudsand.nc'
#endif
#endif
#else
   fileresu(1)='results.nc'
#endif
   fileresu(1)= TRIM(out_dir)//TRIM(fileresu(1))
   date_startout='01/01/1900 00:00:00'
   date_endout=date_end
   l_maxu=.TRUE.
   tdebsor(1)=tool_datosec(date_startout)
   tfinsor(1)=tool_datosec(date_endout)
   pasor(1)=600.0_rlg
   l_filebydate(1)=.FALSE.
   IF_MPI ( .NOT. l_filebydate(1) ) THEN
     CALL_MPI ADD_TAG(fileresu(1),LEN(fileresu(1)))
   ENDIF_MPI
   l_out_off(1)=.FALSE.
   l_out_precdate(1)=.FALSE.
   l_out_demer(1)=.FALSE.
   igextmin(1)=imin
   igextmax(1)=imax
   jgextmin(1)=jmin
   jgextmax(1)=jmax
   iextmin(1)=limin
   iextmax(1)=limax
   jextmin(1)=ljmin
   jextmax(1)=ljmax
   kextmin(1)=1
   kextmax(1)=kmax
   stepi(1)=1
   stepj(1)=1
   stepk(1)=1
   l_out_ssh(1)=.TRUE.
   l_out_ubt(1)=.TRUE.
   l_out_u3d(1)=.TRUE.
   l_out_sal(1)=.TRUE.
   l_out_temp(1)=.FALSE.
   l_out_wind(1)=.FALSE.
   l_out_atmsflx(1)=.FALSE.
   l_out_visc(1)=.FALSE.
   l_out_turb(1)=.FALSE.
   l_out_buoy(1)=.FALSE.
   l_out_zaxis_w(1)=.FALSE.
   l_obc2drank_save=.FALSE.
   l_stability=.FALSE.
   iscreen=12
   iscreenlog=18
   ierrorlog=17
   iwarnlog=16
   l_out_nc4par = .false.
   l_out_pack = .false.

   IF_MPI (MASTER) THEN
   logfilename= 'simu_'//TRIM(suffix)//'.log'
   IF (iscreenlog/=6) OPEN(unit=iscreenlog,file=logfilename,status="replace")
   logfilename='error_'//TRIM(suffix)//'.log'
   IF (ierrorlog/=6) OPEN(unit=ierrorlog,file=logfilename,status="replace")
   logfilename='warning_'//TRIM(suffix)//'.log'
   IF (iwarnlog/=6) OPEN(unit=iwarnlog,file=logfilename,status="replace")
   ENDIF_MPI

   ! diagnoses
   ! ---------

   l_diag=.FALSE.
   file_diag='diagnostic.nc'
   tdebdiag=tdebsor(1)
   tfindiag=tfinsor(1)

   l_points=.TRUE.
   file_point='points.dat'

   ! meteorology
   ! -----------

   l_meteo_stat=.TRUE.
   l_meteo_hom=.TRUE.
   wind_veloc=0.0_rsh
   wind_dir=0.0_rsh
   l_sflx_solarcst=.TRUE.
   sflx_solarcst=0.0_rsh
   nbatm=0

   ! open boundaries
   ! ---------------

   ! geometry (where are the open boundaries ?)

   l_obc_south=.FALSE.
   l_obc_north=.FALSE.
   l_obc_west=.TRUE.
   l_obc_east=.FALSE.

   ! used scheme to manage obc

   l_obc_mars = .TRUE.
   l_obc_diri = .FALSE.
   l_obc_char = .FALSE.
   l_obc_cycl = .FALSE.
   l_obc_cycl_x=.FALSE.
   l_obc_cycl_y=.FALSE.

   ! physics introduced into the simulated area (tide or ogcm solution)

   l_obc_tide=.TRUE.
   l_tide_harmcompo=.FALSE.
   l_obc_ogcm=.FALSE.

   obc_coefrel=0.0_rsh

   ! sponge layer
   !-------------



   ! Initialization of the sea surface height
   ! ----------------------------------------

   ssh(liminm2:limaxp2,ljminm2:ljmaxp2)  = MAX(0.0_rsh,-h0(liminm2:limaxp2,ljminm2:ljmaxp2))

   ! Initialization of velocity field
   !---------------------------------

   u(:,:)    = 0.0_rsh
   v(:,:)    = 0.0_rsh
   uz(:,:,:) = 0.0_rsh
   vz(:,:,:) = 0.0_rsh
   wz(:,:,:) = 0.0_rsh

   ! turbulence
   !------------

   l_incbotstress=.FALSE.
   botstressmax=100.0_rsh
   turb_nbeq=2
   turb_0eq_option=1
   turb_2eq_option=2
   z0b(:,:)   =0.0001_rsh
   z0s(:,:)   =0.0001_rsh
   hminfrot=0.001_rsh
   hmaxfrot=5.0_rsh
   nzinit=0.001_rsh
   kzinit=0.00001_rsh
   nz(:,:,:) = nzinit
   kz(:,:,:) = kzinit
   ect(:,:,:)=0.000001_rsh
   ectmin=0.000001_rsh
   bgdiff=0.000001_rsh
   fvisc=0.1_rsh
   vis2d(:,:) = fvisc*0.01_rsh*dyval**1.15_rsh
   vish_xe(:,:,:) = fvisc*0.01_rsh*dyval**1.15_rsh
   vish_phi(:,:,:)= fvisc*0.01_rsh*dyval**1.15_rsh
   l_smagor=.FALSE.
   vismin=0.001_rsh
   vismax=4.0_rsh
   cosmag=0.1_rsh
   icon=2
   kx=1.0_rsh
   ky=1.0_rsh
   qmax=8.0_rsh
   qmaxz=10.0_rsh
   rhoref=1027.34_rsh
   saliref=0.0_rsh
   tetaref=10.0_rsh
   l_stateeq_lin=.FALSE.
   saliref_lin=saliref
   tetaref_lin=tetaref
   rhoair=1.25_rsh
   chp=3986.0_rsh
   coext=0.1_rsh
   file_river='.'

   ! Initialization of tracers
   !--------------------------

   DO i=liminp1,limax
     x=(i-1)*dxval
     IF (x < 30000.0_rsh ) THEN
       sal(:,i,:)=saliref
     ELSE IF (x > 80000.0_rsh ) THEN
       sal(:,i,:)= 0.0_rsh
     ELSE
       sal(:,i,:)=saliref*(1.0_rsh-(x-30000.0_rsh)/50000.0_rsh)
     END IF
   END DO
   temp(:,:,:) = tetaref


   ! Initialization of other variables
   !----------------------------------

   phieau(:,:,:)=0.0_rlg

#if defined key_substance
   
   ! Initialization of budget parameters
   ! -----------------------------------
   l_bilan=.TRUE.
   nb_border=1
   dtout_budget=1.0_rlg  ! in hours
   name_bilfil = ''

   ! Initialization of sedimentary parameters
   ! ----------------------------------------

#ifdef key_sedim
   l_fricwave=.FALSE.      
   l_repsed=.FALSE.
   l_morphocoupl=.FALSE.
   l_consolid=.FALSE.
#if defined key_mudsand & (defined key_castestsub || defined key_castestconta)
   l_diffused=.TRUE.  
#else
   l_diffused=.FALSE.
#endif
   l_unised = .TRUE.
   z0seduni = 0.0001_rsh
   fricwav=0.06_rsh 
   hseduni = 0.1_rsh 
   cseduni = 400.0_rsh 
   ksmiuni = 1
   ksmauni = 1
   sini_sed=0.0_rsh
   tini_sed=10.0_rsh 
   dzsmin = 1.0e-5
   dzsmax = 1.0_rsh
   csupi = 400.0_rsh 
   xdifs1 = 1.e-8
   xdifs2 = 0.0_rsh 
   xdifsi1 = 1.e-6
   xdifsi2 = 0.0_rsh 
   epdifi = 0.01_rsh 

#ifdef key_sedim_mixsed
   csedmin=30.0_rsh
   xperm1=0.2_rsh
   xperm2=5.0_rsh
   xsigma1=1.5e-14
   xsigma2=6
   activlayer=0.02_rsh
   frmudcr2=0.7_rsh 
   eroa1=1000._rsh 
   eroa2=0.3_rsh 
   eroa3=0._rsh 
   erob1=1.6_rsh 
   erob2=0.0002_rsh 
   erob3=1._rsh 
   corfluer1=0.01_rsh 
   corfluer2=350._rsh 
   cmudcr=600._rsh 
   cvolmaxsort=0.58_rsh 
   cvolmaxmel=0.67_rsh 
   xsigma1sg=xsigma1/9.81_rsh 
#else             
   csmaxmel = 1400.0_rsh 
   frmudcr1=0.3_rsh
   frmudcr2=0.5_rsh
   E0sable=0.0002_rsh *0.3_rsh
   alphavase=1.0_rsh
   alphasable=1.0_rsh
   xeros = 0.0002_rsh *0.3_rsh   ! this formulation so as to have a similar erosion flux
                                 ! as for mixsed with eroa2=0.3 and erob2=0.0002
   x1toce = 0.3_rsh 
   x2toce = 0.0_rsh 
   poro1 = 0.6_rsh 
   xtas11 = 0.1e-3
   xtas12 = 50.0_rsh 
   poro2 = 0.6_rsh 
   xtas21 = 1.825e-5
   xtas22 = 57.0_rsh
   poro3 = 0.6_rsh
   xtas31 = 0.9e-08
   xtas32 = 13.0_rsh
   fexcs = 1.0_rsh
   cexcs=1._rsh-fexcs
   xcvgce=0.3_rsh
   fimpls=1.0_rsh
#endif
   fws2=fricwav*0.5_rsh
#endif

   ! Initialization of the number of variables depending on their type
   ! (gravel,sand,mud,no-constitutive,constitutive,dissolved,fixed,driving or intermediate)
   !----------------------------------------------------------------------------
   
#ifdef key_castestsub
   nv_grav=0
   nv_sand=0
   nv_mud=2
   nv_cp=0
   nv_ncp=1
   nv_dis=1
   nv_fix=0
   nv_dri=0
   nv_int=0
#elif key_castestconta
   nv_grav=0
   nv_sand=0
   nv_mud=2          ! MES_H, MES_L
   nv_cp=0
   nv_ncp=3          ! Cpcin_H, Cpeq_H, Cpeq_L
   nv_dis=3          ! Csubd,Cd,CdL
   nv_fix=0
   nv_dri=0 
   nv_int=0          
#else
   nv_grav=0
   nv_sand=0
   nv_mud=1
   nv_cp=0
   nv_ncp=0
   nv_dis=0
   nv_fix=0
   nv_dri=0
   nv_int=0
#endif

   nvpc=nv_cp+nv_mud+nv_sand+nv_grav
   nvp=nvpc+nv_ncp
   nv_adv=nvp+nv_dis
   nv_state=nv_adv+nv_fix
   nv_tot=nv_state+nv_dri+nv_int
   
   ! identification of igrav1,igrav2,isand1,isand2,imud1,imud2
   
   igrav1=1
   igrav2=nv_grav
   isand1=igrav2+1
   isand2=igrav2+nv_sand
   imud1=isand2+1
   imud2=isand2+nv_mud

   !  allocation for variables declared in comsubstance (not allocated anywhere
   !  else since subreaddat is not used for test cases
   !  ---------------------------------------------------------------

   ALLOCATE(cv_wat(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   ALLOCATE(obc_cv_name(nv_adv))
   ALLOCATE(init_cv_name(nv_tot))
   ALLOCATE(l_pelagic(nv_adv))
   ALLOCATE(l_benthic(nv_adv))
   ALLOCATE(l_out_subs(nv_state))
   ALLOCATE(t90(nv_state))
   ALLOCATE(cini_air(nv_state))
   ALLOCATE(f90(nb_var))
   ALLOCATE(sub_flx_atm(nv_adv))
   ALLOCATE(cv_rain(nv_adv))
   ALLOCATE(cini_wat(nv_state))
   ALLOCATE(typart(nv_state))
   ALLOCATE(itypv(nv_tot))
   ALLOCATE(obc_cv_wat_e(nv_adv,kmax,imax-obc_width+1:imax,jmin:jmax))
   ALLOCATE(obc_cv_wat_w(nv_adv,kmax,imin:imin+obc_width,jmin:jmax))
   ALLOCATE(obc_cv_wat_n(nv_adv,kmax,imin:imax,jmax-obc_width+1:jmax))
   ALLOCATE(obc_cv_wat_s(nv_adv,kmax,imin:imax,jmin:jmin+obc_width))
   ALLOCATE(phicon(nv_tot,kmax,limin:limax,ljmin:ljmax))
   ALLOCATE(phicon_atm(nv_tot,limin:limax,ljmin:ljmax))
   ALLOCATE(irk_mod(nv_tot))
   ALLOCATE(irk_fil(nv_tot))
   ALLOCATE(ws3(kmax,nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(xws(13,nv_adv))
   ALLOCATE(tocd(nvp))
!#if defined key_dyn_wetdry_fct & defined key_castestriver 
#ifdef key_phicondrycell  
   ALLOCATE(phicon_drycell(nv_tot,kmax,limin:limax,ljmin:ljmax))
#endif

#ifdef key_sedim
   CALL sed_alloc
#endif

   ! Mise en groupe des variables particulaires
   ! Allocation des tableaux se rapportant aux variables non dissoutes
   
   ALLOCATE(iv_ws_max(nvp,nvp))
   ALLOCATE(nv_ws_max(nvp))
   ALLOCATE(ndt_part(nvp))
   ALLOCATE(sv_part(nvp))
   ALLOCATE(ap_ws_gr(nvp))
   ALLOCATE(irkm_var_assoc(nvp))
   irkm_var_assoc(:)=0

   ! Allocate intermediate variable used in ultimate quickest macho scheme
   ALLOCATE(c2(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   c2(:,:,:,:)=0.0_rsh
   CALL alloc_fxfyfz

   ! Initialization of variables
   ! --------------------------------------------

   cv_wat(:,:,:,:) = 0.0_rsh
   obc_cv_name(:)='none'
   l_out_subs(:)=.TRUE.
#if defined key_castestsub || defined key_castestconta
   l_subflxatm=.TRUE.
#else
   l_subflxatm=.FALSE.
#endif
   sflx_sub_atm_depth=1.0_rsh
   f90(:) = 1.0_rsh
   obc_cv_wat_e(:,:,:,:)=0.0_rsh
   obc_cv_wat_w(:,:,:,:)=0.0_rsh
   obc_cv_wat_n(:,:,:,:)=0.0_rsh
   obc_cv_wat_s(:,:,:,:)=0.0_rsh
   phicon(:,:,:,:) = 0.0_rsh
#if defined key_phicondrycell 
   phicon_drycell(:,:,:,:) = 0.0_rsh
#endif
   phicon_atm(:,:,:) = 0.0_rsh
   ws3(:,:,:,:) = 0.0_rsh
   
   DO n=1,nv_tot
    irk_fil(n)=n
   END DO
   nb_ws_max=0
   iv_ws_max(:,:)=0
   nv_ws_max(:)=0
   ap_ws_gr(:)=0
   ndt_part(:)=0
   sv_part(:)=0.0_rsh
   
   ! caracteristics of each variable
   ! -----------------------------------------
! one first particulate constitutive variable (heavy vase)
! initally present in the sediment, not in water column

   isubs=1
   name_var(isubs)='vase'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=1
   itypv(isubs)=3
   xws(1,isubs)=.0001_rsh     
   xws(2,isubs)=.001_rsh 
   xws(3,isubs)=20.0_rsh 
   xws(4,isubs)=0.085_rsh 
   xws(5,isubs)=0.5_rsh 
   xws(6,isubs)=60.0_rsh 
   xws(7,isubs)=0.0_rsh 
   xws(8,isubs)=0.0_rsh  
   xws(9,isubs)=1.0_rsh  
   xws(10,isubs)=0_rsh  
   xws(11,isubs)=0.0_rsh 
   xws(12,isubs)=0.0_rsh 
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh 
#ifdef key_sedim
   ros(isubs)=2600.0_rsh
   diam_sed(isubs)=0.0002_rsh
#endif
   cini_wat(isubs)=0.0_rsh     
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
    
#if defined key_castestsub || defined key_castestconta
! a second particulate constitutive variable (lignt vase)
! initially present in the water column, not in the sediment
   isubs=2
   name_var(isubs)='vase_L'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=1.
   itypv(isubs)=3
   xws(1,isubs)=.00002_rsh    
   xws(2,isubs)=.0001_rsh
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh 
   xws(9,isubs)=1.0_rsh 
   xws(10,isubs)=0.0_rsh 
   xws(11,isubs)=0.0_rsh
   xws(12,isubs)=0.0_rsh
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh
#ifdef key_sedim 
   ros(isubs)=1600.0_rsh
   diam_sed(isubs)=0.0001_rsh
#endif
   cini_wat(isubs)=10.0_rsh     
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,80:150,:) = cini_wat(isubs)
#endif

#ifdef key_castestsub
! one non constitutive particulate substance
!  sorbed on the first constitutive particulate substance
   isubs=3
   name_var(isubs)='part_H'
   unit_var(isubs)='g.m-3'
   typart(isubs)=0.
   itypv(isubs)=4
   xws(1,isubs)=.0001_rsh     
   xws(2,isubs)=.001_rsh 
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh 
   xws(9,isubs)=1.0_rsh
   xws(10,isubs)=0.0_rsh 
   xws(11,isubs)=0.0_rsh 
   xws(12,isubs)=0.0_rsh 
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh
#ifdef key_sedim
   ros(isubs)=2600.0_rsh
   diam_sed(isubs)=0.0002_rsh
#endif
   cini_wat(isubs)=0.0_rsh     
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
 
! one dissolved substance
   isubs=4
   name_var(isubs)='diss1'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=0
   itypv(isubs)=5
   cini_wat(isubs)=100._rsh
   t90(isubs)=0._rsh
   !sub_flx_atm(isubs)=0.0000001_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0._rsh
   cv_wat(isubs,:,10:100,:) = cini_wat(isubs)
#endif

#if defined key_castestconta
! one first non constitutive particulate substance
!  sorbed on the second constitutive particulate substance
   isubs=3
   name_var(isubs)='Cp_Leq'
   unit_var(isubs)='g.m-3'
   typart(isubs)=0
   itypv(isubs)=4
   xws(1,isubs)=.00002_rsh    
   xws(2,isubs)=.0001_rsh
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh 
   xws(9,isubs)=1.0_rsh 
   xws(10,isubs)=0.0_rsh 
   xws(11,isubs)=0.0_rsh
   xws(12,isubs)=0.0_rsh
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh
#ifdef key_sedim 
   ros(isubs)=1600.0_rsh
   diam_sed(isubs)=0.0002_rsh
#endif
   cini_wat(isubs)=0.1_rsh*cini_wat(2)   
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,80:150,:) = cini_wat(isubs)
  
! a second non constitutive particulate substance
!  sorbed on the first constitutive particulate substance
! nulle dans l eau initialement, presente dans sediment sur la deuxieme variable contitutive
   isubs=4
   name_var(isubs)='Cp_Hcin'
   unit_var(isubs)='g.m-3'
   typart(isubs)=0
   itypv(isubs)=4
   xws(1,isubs)=.0001_rsh     
   xws(2,isubs)=.001_rsh 
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh 
   xws(9,isubs)=1.0_rsh
   xws(10,isubs)=0.0_rsh 
   xws(11,isubs)=0.0_rsh 
   xws(12,isubs)=0.0_rsh 
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh
#ifdef key_sedim 
   ros(isubs)=2600.0_rsh
   diam_sed(isubs)=0.0002_rsh
#endif
   cini_wat(isubs)=0.0_rsh     
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
 
! a third non constitutive particulate substance
  ! nulle dans l eau initialement, presente dans sediment sur la deuxieme variable contitutive
!  sorbed on the first constitutive particulate substance
   isubs=5
   name_var(isubs)='Cp_Heq'
   unit_var(isubs)='g.m-3'
   typart(isubs)=0
   itypv(isubs)=4
   xws(1,isubs)=.0001_rsh    
   xws(2,isubs)=.001_rsh
   xws(3,isubs)=20.0_rsh
   xws(4,isubs)=0.085_rsh
   xws(5,isubs)=0.5_rsh
   xws(6,isubs)=60.0_rsh
   xws(7,isubs)=0.0_rsh
   xws(8,isubs)=0.0_rsh 
   xws(9,isubs)=1.0_rsh
   xws(10,isubs)=0.0_rsh 
   xws(11,isubs)=0.0_rsh 
   xws(12,isubs)=0.0_rsh 
   xws(13,isubs)=0.0_rsh 
   tocd(isubs)=1.0_rsh
#ifdef key_sedim 
   ros(isubs)=2600.0_rsh
   diam_sed(isubs)=0.0002_rsh
#endif
   cini_wat(isubs)=0.0_rsh     
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
 
! one first dissolved substance
   isubs=6
   name_var(isubs)='Csubd'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=0
   itypv(isubs)=5
   cini_wat(isubs)=0.005_rsh
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   DO i=imin,imax
        cv_wat(isubs,:,i,:) = cini_wat(isubs)*(0.1+0.9*(imax-i)/imax)
   END DO
   obc_cv_wat_w(isubs,:,:,:)=cini_wat(isubs)
          
! one second dissolved substance
   isubs=7
   name_var(isubs)='Cd'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=0
   itypv(isubs)=5
   cini_wat(isubs)=0.005_rsh
   t90(isubs)=0._rsh
   sub_flx_atm(isubs)=0.0_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
   obc_cv_wat_w(isubs,:,:,:)=cini_wat(isubs)
   
! one thrid dissolved substance
   isubs=8
   name_var(isubs)='CdL'
   unit_var(isubs)='kg.m-3'
   typart(isubs)=0
   itypv(isubs)=5
   cini_wat(isubs)=0.00_rsh
   t90(isubs)=0.0_rsh
   sub_flx_atm(isubs)=0.0001_rsh
   cv_rain(isubs)=0.0_rsh
   cv_wat(isubs,:,:,:) = cini_wat(isubs)
#endif


   ! Initialization of the sediment layer
   ! ------------------------------------

#ifdef key_sedim

#ifdef key_sedim_mixsed
   DO iv=1,isand2
!     here, for simplification, water density = 1000. kg/m3 
     rosmrowsros(iv)=(ros(iv)-1000.0_rsh)/ros(iv)
     diamstar(iv)=diam_sed(iv)*10000.0_rsh*(9.81_rsh*(ros(iv)/1000.0_rsh-1.0_rsh))**0.33_rsh
!     according to Soulsby, 1997, and if viscosity = 10-6 m/s :
     ws0(iv)=.000001_rsh*((107.33_rsh+1.049_rsh*diamstar(iv)**3)**0.5_rsh-10.36_rsh)/diam_sed(iv)
     tetacri0(iv)=0.3_rsh/(1.0_rsh+1.2_rsh*diamstar(iv))+0.055_rsh*(1.0_rsh-EXP(-0.02_rsh*diamstar(iv)))
     stresscri0(iv)=tetacri0(iv)*9.81_rsh*(ros(iv)-1000.0_rsh)*diam_sed(iv)
     xnielsen(iv)=12.0_rsh*1000.0_rsh/9.81_rsh/(1.0_rsh-1000.0_rsh/ros(iv))
   END DO
#endif    
   z0sed(:,:)=z0seduni
   hsed(:,:)=0.0_rsh 
   WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) hsed(limin:limax,ljmin:ljmax)=hseduni
   ksmi(:,:)=0
   ksma(:,:)=0
   WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) ksmi(limin:limax,ljmin:ljmax)=ksmiuni
   WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) ksma(limin:limax,ljmin:ljmax)=ksmauni
!      ksma(1:150,:)=0
!      ksma(150:imax,:)=0

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! ATTTENTION !!!
!!!!!!!!!!!!!!!!!!
! ik key_mixsed , the total concentration c_sedtot is computed as the sum of the
!                 contributions of each variable (cini_sed(iv))
! if key_mudsand, the total concentration is given as an input, and the contribution
!                 of each variable (cini_sed(iv)) is given as a fraction of c_sedtot 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#ifdef key_sedim_mixsed
   c_sedtot(:,:,:)=0.0_rsh
   ! constitutives  (fraction of 1)
   cini_sed(1)=cseduni
#ifdef key_castestsub
   cini_sed(2)=0.0_rsh
   ! non constitutive in masse/kg of associated constitutive substance
   cini_sed(3)=10.0_rsh
   ! dissolved in masse/m3 of pore water 
   cini_sed(4)=0.0_rsh
#elif key_castestconta
   cini_sed(2)=0.0_rsh
   ! non constitutive in masse/kg of associated constitutive substance
   cini_sed(3)=0.0_rsh
   cini_sed(4)=1.0_rsh
   cini_sed(5)=0.01_rsh
   ! dissolved in masse/m3 of pore water 
   cini_sed(6)=0.0_rsh
   cini_sed(7)=0.0_rsh
   cini_sed(8)=0.0_rsh
#endif 
 
   DO j=jmin,jmax
     DO i=ig(j),id(j)
       DO k=ksmi(i,j),ksma(i,j)
         somalp=0.
         DO iv=1,nvpc
!           - constitutive variables
           cv_sed(iv,k,i,j)=cini_sed(iv)
           c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
           somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
         END DO
         poro(k,i,j)=1.-somalp
         DO iv=nvpc+1,nvp
!           - non constitutive variables : cini_sed in conc/kg of sediment
           cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
         END DO
            DO iv=nvp+1,nv_state
!           - dissolved variables : cini_sed in conc/m3 of pore water
              cv_sed(iv,k,i,j)=cini_sed(iv)
            END DO
            cv_sed(-1,k,i,j)=tini_sed
            cv_sed( 0,k,i,j)=sini_sed
       END DO
     END DO
   END DO
#else
   isubs=1
   cini_sed(isubs)=1.0_rsh
   c_sedtot(:,:,:)=cseduni
   ! constitutives in fraction de 1
   cini_sed(1)=1.0_rsh
#ifdef key_castestsub
   cini_sed(2)=0.0_rsh
   ! non constitutives in masse/kg of associated constitutive substance
   cini_sed(3)=10.0_rsh
   ! dissolved in masse/m3 of pore water 
   cini_sed(4)=0.0_rsh
#elif key_castestconta
   cini_sed(2)=0.0_rsh
   ! non constitutive in masse/kg of associated constitutive substance
   cini_sed(3)=0.0_rsh
   cini_sed(4)=1.0_rsh
   cini_sed(5)=0.01_rsh
   ! dissolved in masse/m3 of pore water 
   cini_sed(6)=0.0_rsh
   cini_sed(7)=0.0_rsh
   cini_sed(8)=0.0_rsh
#endif

   DO j=jmin,jmax
     DO i=ig(j),id(j)
       DO k=ksmi(i,j),ksma(i,j)
         somalp=0.
         DO iv=1,nvpc
!           - constitutive variables
           cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
           somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
         END DO
         poro(k,i,j)=1.-somalp
         DO iv=nvpc+1,nvp
!           - non constitutive variables : cini_sed in conc/kg of sediment
           cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
         END DO
            DO iv=nvp+1,nv_state
!           - dissolved variables : cini_sed in conc/m3 of pore water
              cv_sed(iv,k,i,j)=cini_sed(iv)
            END DO
            cv_sed(-1,k,i,j)=tini_sed
            cv_sed( 0,k,i,j)=sini_sed
       END DO
     END DO
   END DO
#endif
   PRINT*, 'maxval(c_sedtot) ',maxval(c_sedtot)
   PRINT*, 'maxval(cv_sed) ',maxval(cv_sed)

!   vertical discretization within the sediment
!   -------------------------------------------

   dzs=0.0_rsh
   DO j=jmin,jmax
     DO i=imin,imax
       IF (ksma(i,j).NE.0) THEN
         dzsf=hsed(i,j)/float(ksma(i,j)-ksmi(i,j)+1)
         DO k=ksmi(i,j),ksma(i,j)
           dzs(k,i,j)=dzsf
         END DO
         DO k=ksmi(i,j),ksma(i,j)-1
            dzsiin(k,i,j)=2.0_rsh/(dzs(k,i,j)+dzs(k+1,i,j))
         ENDDO
       END IF
     END DO
   END DO

   IF(l_morphocoupl) CALL sed_morphoinit
            
   CALL sed_settlveloc_ini
   
! test rejet/riviere : rajout de la riviere dans river.F90 et outflow.F90
!  avec cle key_castestriver
! besoin d initialiser nvalmax et tchrono
#if defined key_castestriver & (defined key_castestsub || defined key_castestconta)
   
   ! initialize time vector tchrono relative to
   ! river dischage and outflow events
   !-----------------------------------------

   l_chrono=.FALSE.
   perchrono=360.0_rlg
   tdb=tdeb+date_startshift*86400.0_rlg-perchrono
   tfi=tfin+perchrono
   nvalmax=NINT((tfi-tdb)/perchrono+0.5_rlg)+1
   ALLOCATE(tchrono(nvalmax))
   DO i=1,nvalmax
     tchrono(i)=tdb+(i-1.0_rlg)*perchrono
   END DO
   indchrono=1
   l_chrono=.TRUE.
#endif

#if defined key_castestriver                                          
   nbfl=1
   ALLOCATE(iriv(nbfl),jriv(nbfl),kriv(nbfl))
   ALLOCATE(l_flestat(nbfl),debmoy(nbfl))
   ALLOCATE(l_tempriv(nbfl),tmfle(nbfl),amtfle(nbfl))
   ALLOCATE(phiriver(nbfl,kmax))
   ALLOCATE(river_thickness(nbfl))
   iriv(1)=195
   jriv(1)=2
   kriv(1)=10
   river_thickness(1)=100.0_rsh
   l_flestat(1)=.TRUE.
   l_tempriv(1)=.FALSE.
   debmoy(1)=5.0_rsh
   tmfle(1)=10.0_rsh
   amtfle(1)=0.0_rsh 
 
#ifdef key_castestsub                                            
      nbrej=3                                                          
      ALLOCATE(xrej(nbrej),yrej(nbrej),krej(nbrej))                    
      ALLOCATE(nucon(nbrej),numriv(nbrej))                             
      ALLOCATE(l_rejflux(nbrej),valrej(nvalmax,nbrej))                 
      xrej(:)=195                                                      
      yrej(:)=2                                                        
      krej(:)=10                                                       
      nucon(1)=1                                                       
      nucon(2)=3                                                       
      nucon(3)=4                                                       
      numriv(:)=1                                                      
      l_rejflux(:)=.false.                                             
      valrej(:,:)=1.0_rsh                                              
#elif key_castestconta                                                
      nbrej=4                                                          
      ALLOCATE(xrej(nbrej),yrej(nbrej),krej(nbrej))                    
      ALLOCATE(nucon(nbrej),numriv(nbrej))                             
      ALLOCATE(l_rejflux(nbrej),valrej(nvalmax,nbrej))                 
      xrej(:)=195                                                      
      yrej(:)=2                                                        
      krej(:)=10                                                       
      nucon(1)=1                                                       
      nucon(2)=4                                                       
      nucon(3)=6                                                       
      nucon(4)=7                                                       
      numriv(:)=1                                                      
      l_rejflux(:)=.false.                                             
      ! vase, CP_HCIN, CSUBD, CD                                       
      valrej(:,1)=0.02_rsh                                             
      valrej(:,2)=0.003_rsh                                            
      valrej(:,3)=0.005_rsh                                            
      valrej(:,4)=0.0005_rsh                                           
#endif                                                                
#endif
      
  ! Equation of substances with settling velocity
  ! is solved with a sub time step

  ! estimate of the minimum layer thickness
  ! ---------------------------------------

  dhmin=hm
#if defined key_siggen || defined key_gencoord
  IF (hc > hm) THEN
    DO k=1,kmax
      dhmin=MIN(dhmin,hm*dsigu(k))
    ENDDO
  ELSE
    DO k=1,kmax
      dhmin=MIN(dhmin,(hc+(hm-hc)*dcusds(k))*dsigu(k))
    ENDDO
  END IF
#else
  DO k=1,kmax
    dhmin=MIN(dhmin,hm*dsigu(k))
  ENDDO
#endif

  ws3maxloc=0.0_rsh
  DO iv=1,isand2
#ifdef key_sedim_mixsed
     if (ws0(iv)>ws3maxloc) ws3maxloc=ws0(iv)
#elif key_sedim_mudsand
     if (xws(1,iv)>ws3maxloc) ws3maxloc=xws(1,iv)
#endif
  ENDDO
#ifdef key_subs_part_eqsubdt
  DO iv=1,nvp                      ! loop on particulate variables    
    IF (ap_ws_gr(iv)==0) THEN

  ! estimate the number of different settling velocities
      nb_ws_max=nb_ws_max+1
  ! number of variables with the same settling velocity
      nv_ws_max(nb_ws_max)=1
      indgrp=1
  ! index of variable associated with a settling velocity
      iv_ws_max(nb_ws_max,indgrp)=iv
      ap_ws_gr(iv)=nb_ws_max
  ! number of sub time step
      if(iv.gt.isand2 .and. xws(2,iv).gt.ws3maxloc) ws3maxloc=xws(2,iv)
      ndt_part(nb_ws_max)=NINT(dt/MIN(dhmin/ws3maxloc,dt))
      sv_part(nb_ws_max)=ws3maxloc
      IF (iv<nvp) THEN
        DO iiv=iv+1,nvp
          IF ((ap_ws_gr(iiv)==0) .AND.(xws(2,iiv)==sv_part(nb_ws_max))) THEN 
  ! number of variables with the same settling velocity
            nv_ws_max(nb_ws_max)=nv_ws_max(nb_ws_max)+1
            indgrp=indgrp+1
  ! index of variable associated with a settling velocity
            iv_ws_max(nb_ws_max,indgrp)=iiv
            ap_ws_gr(iiv)=nb_ws_max
          ENDIF
        ENDDO
       ENDIF
    ENDIF
  ENDDO

#else
  nb_ws_max=0
  DO iv=1,nvp
    if(iv.gt.isand2 .and. xws(2,iv).gt.ws3maxloc) ws3maxloc=xws(2,iv)
    nb_ws_max=iv     ! number of variables with different settling velocities
                        ! all of them for key_sedim because ws3 is a 4D
                        ! variable
    nv_ws_max(iv)=1  ! each particulate variable is relative to a unique group
                        ! because ws3 is a 4D variable
    sv_part(iv)=max(ws3maxloc,xws(2,iv))   ! important to determinate the number of iterations
                               ! index of variable associated with a settling velocity
    iv_ws_max(iv,1)=iv
    IF (xws(2,iv) == 0.0_rsh) THEN
         IF_MPI (MASTER) THEN
         WRITE(ierrorlog,*) ''
         WRITE(ierrorlog,*) ' ******* Error in the definition of settling velocity ******* '
         WRITE(ierrorlog,*) 'the variable :',name_var(irk_fil(isubs))
         WRITE(ierrorlog,*) 'is a particulate one'
         WRITE(ierrorlog,*) 'but its settling velocity is set to 0.0 in castidalflat2dv'
         WRITE(ierrorlog,*) 'Consequently, the sub time step used to solve substance equation cannot be estimated'
         WRITE(ierrorlog,*) 'The simulation is stopped'
         STOP
         ENDIF_MPI
    END IF
    ndt_part(nb_ws_max)=NINT(dt/MIN(dhmin/ws3maxloc,dt))
    PRINT*, 'ndt_part dans castidalflat2dv',nb_ws_max,ndt_part(nb_ws_max)
  ENDDO
#endif
   nvpi=nvp

   IF_MPI (MASTER) THEN
#if defined key_sedim 
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) 'SUBSTANCES WITH A VARIABLE SETTLING VELOCITY '
#else
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) 'SUBSTANCES WITH A TIME CONSTANT SETTLING VELOCITY '
#endif
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*)'NUMBER GROUP OF SUBSTANCES            : ',nb_ws_max
   ENDIF_MPI
   DO iiv=1,nb_ws_max
     IF_MPI (MASTER) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*)'group number                   : ',iiv
     WRITE(iscreenlog,*)'number of substances           : ',nv_ws_max(iiv)
     WRITE(iscreenlog,*)'group settling velocity        : ',sv_part(iiv)
     WRITE(iscreenlog,*)'group number of iterations     : ',ndt_part(iiv)
     ENDIF_MPI
     DO iv=1,nv_ws_max(iiv)
       IF_MPI (MASTER) THEN
       WRITE(iscreenlog,*) 'substance number        name'
       WRITE(iscreenlog,'(4x,i4,6x,a18,2x)')  &
       iv,TRIM(name_var(irk_fil(iv_ws_max(iiv,iv))))
       ENDIF_MPI
     ENDDO
   ENDDO
   
#if defined key_castestsub || defined key_castestconta
   ! reperage de variables particulaires constitutives associees aux variables part. SORB
   ! (si meme vitesse de chute min et max; sinon pas de part associees)
   ! ------------------------------------------------------------------------------------
   DO iv=1,nv_ncp
     isubs=nvpc+iv
     irkm_var_assoc(isubs)=0
     l_varassoc=.FALSE.
     DO ivpc=1,nvpc
       IF (xws(1,isubs) == xws(1,ivpc) .AND. xws(2,isubs) == xws(2,ivpc))THEN
         irkm_var_assoc(isubs)=ivpc
         IF_MPI (MASTER) THEN
         WRITE(iscreenlog,*) ' '
         WRITE(iscreenlog,*)'constitutive part. variable where is sorbed'
         WRITE(iscreenlog,*)'the variable :',name_var(irk_fil(isubs))
         WRITE(iscreenlog,*)'is the variable ', name_var(irk_fil(ivpc))
         ENDIF_MPI
         l_varassoc=.TRUE.
       END IF
     END DO
     IF (.NOT.l_varassoc) THEN
       IF_MPI (MASTER) THEN
       WRITE(iscreenlog,*)' '
       WRITE(iscreenlog,*)'the variable :',name_var(irk_fil(isubs))
       WRITE(iscreenlog,*)'does not have associated constitutive part. variable'
       WRITE(iscreenlog,*)'but its own settling velocity'
       ENDIF_MPI
     END IF
   END DO
!! ends key_sedim
! ends key_sedim
#endif
#endif
! ends key_substance
#endif

#if defined key_contaminant || defined key_castestconta
   ! Initialization of chemical reactions (paraconta + reaction.dat)
   ! ---------------------------------------------------
     minpt_mes_re5 = 0.00005_rsh
     minpt_conta =1.e-5
     l_sorfluxreac=.false.
     thick_volat=0.5_rsh
     massmol_h2o=18.0_rsh
     massmol_co2=44.0_rsh
     massmol_o2=32.0_rsh
     cf_difwat_co2=1.77e-9
     cf_difwat_o2=2.3e-9
     cf_difair_h2o=0.25e-4
     cst_gas_univ=8.314_rsh 
     
     nreacmax=4
     allocate(ityp_reac(nreacmax))
     allocate(nb_var_re(nreacmax))
     allocate(ityp_arh(nreacmax))
     allocate(ire_typ(nreacmax))
     allocate(name_reac(nreacmax))
     allocate(cin_max_re(nreacmax))
     allocate(cf_teta_arh(nreacmax))
     allocate(cf_xetsig_arh(nreacmax))
     allocate(tempref_arh(nreacmax))
     allocate(l_arh(nreacmax))
     allocate(l_treat_re(nreacmax))
     allocate(l_cincte_re(nreacmax))
     allocate(l_coefcte_re(nreacmax))
     allocate(nb_flim1_re(nreacmax))
     allocate(nb_flim2_re(nreacmax))
     allocate(nb_flim3_re(nreacmax))
     allocate(icon_reac(nreacmax))
     nvaractmax=4   
     allocate(irkm_var_re(nreacmax,nvaractmax))
     allocate( name_var_re(nreacmax,nvaractmax))
     allocate( cf_stochio_re(nreacmax,nvaractmax))
     allocate( cf_expovar_re(nreacmax,nvaractmax))
     allocate(ispc_reac(nreacmax,nvaractmax))
#ifdef key_species
     allocate(ifrac_var_reac(nreacmax,nvaractmax))
#endif
     nflimax=2
     allocate(irkm_flim1_re(nreacmax,nflimax))
     allocate(irkm_flim2_re(nreacmax,nflimax))
     allocate(cf_flim1_re(nreacmax,nflimax))
     allocate(cf_flim2_re(nreacmax,nflimax))
     allocate(cf_flim3_re(nreacmax,nflimax))
     l_flim3_re=.false.
     nb_reac(:)=0
 
 ! first reaction : partition between Cp_Heq and Cd and Cp_Leq and CdL
     nb_reac(5)=1
     ALLOCATE(l_pce_re5(nb_reac(5)))
     ALLOCATE(l_salt_re5(nb_reac(5)))
     ALLOCATE(cf_pce_re5(nb_reac(5),2))
     ALLOCATE(cf_salt_re5(nb_reac(5),2))
     ALLOCATE(cf_distb_re5(nb_reac(5),4))
     ALLOCATE(irkm_sub_re5(nb_reac(5),4))
     ALLOCATE(l_kds_re5(nb_reac(5)))
     ALLOCATE(ndg_kds_re5(nb_reac(5)))
     ALLOCATE(cf_kd0_re5(nb_reac(5)))
     ALLOCATE(cf_xkds_re5(nb_reac(5),1))
     ireac=1
     ityp_reac(ireac)=5
     name_reac(ireac)='partition'
     l_treat_re(ireac)=.false.
     ispc_reac(ireac,:)=0
     icon_reac(ireac)=0
     ire_typ(ireac)=nb_reac(5)
     nb_var_re(ireac)=4
     l_arh(ireac)=.false.
     l_pce_re5(1)=.false.
     l_salt_re5(1)=.false.
     l_kds_re5(1)=.false.
     irkm_var_re(ireac,1)=8
     irkm_var_re(ireac,2)=7
     irkm_sub_re5(1,2)=6
     cf_distb_re5(1,2)=13182.567_rsh
     irkm_var_re(ireac,3)=3
     irkm_sub_re5(1,3)=2
     cf_distb_re5(1,3)=13182.567_rsh
     irkm_var_re(ireac,4)=5
     irkm_sub_re5(1,4)=1
     cf_distb_re5(1,4)=13182.567_rsh
   ! iv=1 vase_H
   ! iv=2 vase_L
   ! iv=3 Cp_Leq
   ! iv=4 Cp_Hcin
   ! iv=5 Cp_Heq
   ! iv=6 Csubd
   ! iv=7 Cd
   ! iv=8 CdL
     
 ! second reaction : reversible between Cp_Heq and Cp_Hcin
     ireac=2
     name_reac(ireac)='partREV'
     ityp_reac(ireac)=3
     nb_reac(3)=1
     ALLOCATE(cf_rate_re3(nb_reac(3)))
     ALLOCATE(cv_watequ_re3(nb_reac(3)))
     ire_typ(ireac)=nb_reac(3)
     nb_var_re(ireac)=2
     cf_rate_re3(1)=0.01_rsh/86400.0_rsh
     cv_watequ_re3(1)=0.01_rsh
     l_arh(ireac)=.false.
     irkm_var_re(ireac,1)=4
     irkm_var_re(ireac,2)=5
     l_treat_re(ireac)=.true.
     
 ! third reaction : volatilisation of CdL 
     ireac=3
     name_reac(ireac)='VOL'
     ityp_reac(ireac)=4
     nb_reac(4)=1
     nb_var_re(ireac)=1
     irkm_var_re(ireac,1)=8
     massmol(8)=200.0_rsh
     nschmidt(8)=500.0_rsh
     cf_difwat(8)=0.0_rsh
     cf_difair(8)=0.0_rsh
     cf_henry(8)=0.0_rsh
     cf_henry_adim(8)=0.20_rsh
     cf_temp_henry(8)=0.0_rsh
     cf_salt_henry(8)=0.3_rsh
     l_volkl_dif(8)=.true.
     l_volkl_sal(8)=.false.
     ich_volkl_wind(8)=2
     l_volkl_cor(8)=.true.
     cf_expvol_liq(8)=-0.5_rsh
     l_volkg_cor(8)=.true.
     cf_expvol_gas(8)= -0.5_rsh 
     cini_air(8)=0.01_rsh
     l_treat_re(ireac)=.true.

 ! fourth reaction : degradation of Csubd => vase_L 
     ireac=4
     name_reac(ireac)='DEG'
     ityp_reac(ireac)=2
     nb_reac(2)=1
     nb_var_re(ireac)=2
     cin_max_re(ireac)=0.0001_rsh/86400.0_rsh
     cin_max_re(ireac)=0.000_rsh/86400.0_rsh
     l_cincte_re(ireac)=.true.
     l_coefcte_re(ireac)=.true.
     l_arh(ireac)=.false.
     irkm_var_re(ireac,1)=6
     irkm_var_re(ireac,2)=2
     cf_stochio_re(ireac,1)=-1
     cf_stochio_re(ireac,2)=1
     cf_expovar_re(ireac,1)=1
     cf_expovar_re(ireac,2)=0
     nb_flim1_re(ireac)=0
     nb_flim2_re(ireac)=0
     nb_flim3_re(ireac)=0
     l_treat_re(ireac)=.true.
     
     l_treat_re_after=.true.
     l_treat_re_dc=.true.
     
     ALLOCATE( dcdt(nv_state,kmax,limin:limax,ljmin:ljmax) )
     dcdt(:,:,:,:)=0.0_rsh
#endif

   ! Percent of wet surface on each cell
   ! -----------------------------------

   hx_t(:,:)=100000.0_rsh
   hy_t(:,:)=100000.0_rsh
   WHERE (hx_g(:,:) /= -valmanq -h0fond) hx_t(:,:)=hx_g(:,:)
   WHERE (hy_g(:,:) /= -valmanq -h0fond) hy_t(:,:)=hy_g(:,:)
   dhxy(:,:)=1.0_rsh
   DO j=ljminp1,ljmax
      DO i=liminp1,limax
         dhxy(i,j)=MAX(hx_g(i,j),hx_g(i-1,j),hy_g(i,j),hy_g(i,j-1))- &
                   MIN(hx_t(i,j),hx_t(i-1,j),hy_t(i,j),hy_t(i,j-1))+epsilon
      END DO
   END DO

   fwet(:,:) = 0.1_rsh
   fwetp(:,:) = 0.1_rsh
   fwetmin=0.1_rsh
   IF (fwetmin/=1.0_rsh) THEN
     DO j=ljmin,ljmax
       DO i=limin,limax
         IF (dhxy(i,j) == 0.0_rsh) THEN
           fwet(i,j)=1.0_rsh
           fwetp(i,j)=1.0_rsh
         ELSE
           fwet(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
           fwetp(i,j)=MIN(MAX(fwetmin,(ssh(i,j)+h0(i,j))/dhxy(i,j)),1.0_rsh)
         ENDIF
       END DO
     END DO
   END IF

   ! Initialization of names of variables in output file
   ! ---------------------------------------------------

   name_out_h0='h0'
   name_out_hx='hx'
   name_out_hy='hy'
   name_out_u='u'
   name_out_v='v'
   name_out_uz='uz'
   name_out_vz='vz'
   name_out_temp='temp'
   name_out_sal='sal'
   name_out_xe='xe'
   name_out_vis2d='VIS2D'
   name_out_vishxe='VISH_XE'
   name_out_vishphi='VISH_PHI'
   name_out_nz='NZ'
   name_out_kz='KZ'
   name_out_bz='BZ'
#ifdef key_sedim
   name_out_eptot='EPTOT'
   name_out_nbniv='NBNIV'
   name_out_dzs='DZS FOND SURF'
   name_out_tenfon='TENFON'
   name_out_tenfonc='TENFONC'
   name_out_tenfonw='TENFONW'
#endif

   ! Initialization of attributes (max and min valid values for each variable)
   ! -------------------------------------------------------------------------

   riog_valid_min_h0=-50.0_riosh
   riog_valid_max_h0=10000.0_riosh
   riog_valid_min_u=-100.0_riosh
   riog_valid_max_u=100.0_riosh
   riog_valid_min_temp=0.0_riosh
   riog_valid_max_temp=50.0_riosh
   riog_valid_min_sal=0.0_riosh
   riog_valid_max_sal=50.0_riosh
   riog_valid_min_xe=-50.0_riosh
   riog_valid_max_xe=50.0_riosh
   riog_valid_min_wstress=0.0_riosh
   riog_valid_max_wstress=2000.0_riosh
   riog_valid_min_visc=0.0_riosh
   riog_valid_max_visc=100000.0_riosh
   riog_valid_min_nz=0.0_riosh
   riog_valid_max_nz=1000.0_riosh
   riog_valid_min_kz=0.0_riosh
   riog_valid_max_kz=1000.0_riosh
   riog_valid_min_bz=-50.0_riosh
   riog_valid_max_bz=50.0_riosh
   value=MINVAL(dx(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dx=value-1.0_riosh
   value=MAXVAL(dx(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dx=value+1.0_riosh
   value=MINVAL(dy(:,:))
   CALL_MPI MIN_ALL_MPI_RSH(value)
   riog_valid_min_dy=value-1.0_riosh
   value=MAXVAL(dy(:,:))
   CALL_MPI MAX_ALL_MPI_RSH(value)
   riog_valid_max_dy=value+1.0_riosh
#ifdef key_sedim
   riog_valid_min_eptot=0.0_riosh
   riog_valid_max_eptot=100.0_riosh
   riog_valid_min_nbniv=0.0_riosh
   riog_valid_max_nbniv=200.0_riosh
   riog_valid_min_dzs=0.0_riosh
   riog_valid_max_dzs=100.0_riosh
   riog_valid_min_tenfon=0.0_riosh
   riog_valid_max_tenfon=50.0_riosh
#endif

   ! Initialization of other attributes
   ! ----------------------------------

   cg_conv_title="name of area"
   cg_conv_comment=" "
   cg_conv_source="MARS CONF"
   cg_conv_area="area"
   cg_conv_institution="IFREMER"
   cg_conv_institution_references="http://www.ifremer.fr"
   cg_conv_data_centre="DYNECO/PHYSED"
   cg_conv_data_centre_references="http://www.ifremer.physed.fr"
   cg_conv_forecast_range=" "
   cg_conv_forecast_type=" "
   cg_conv_field_type="n/a"
   cg_conv_operational_status=" "

   ! initialization of cdate,ijour,imois... for outputs and use of tides
   ! -------------------------------------------------------------------

   cdate = tool_sectodat(t)
   CALL tool_decompdate(cdate,ijour,imois,ian,iheure,iminu,isec)


   ! Initialization of vertical scale factor (part relative to dc*sds)
   !------------------------------------------------------------------

#if defined key_siggen || defined key_gencoord
#if ! defined key_siggen_g2
!$OMP DO SCHEDULE(STATIC)
   DO j=ljmin,ljmaxp1
     DO i=limin,limaxp1
       DO k=1,kmax
         hzdcwsds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcwsds(k)
         hzdcusds(k,i,j)=hc_sig(i,j)+(h0(i,j)-hc_sig(i,j))*dcusds(k)
         hzdcwsds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcwsds(k)
         hzdcusds_x(k,i,j)=hcx_sig(i,j)+(hx(i,j)-hcx_sig(i,j))*dcusds(k)
         hzdcwsds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcwsds(k)
         hzdcusds_y(k,i,j)=hcy_sig(i,j)+(hy(i,j)-hcy_sig(i,j))*dcusds(k)
       END DO
     END DO
   END DO
!$OMP END DO
#endif
#endif

  END SUBROUTINE cas_tidalflat2dv
