   !!======================================================================

  SUBROUTINE step

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE step  ***
   !&E
   !&E ** Purpose : manage the estimate of all pronostic variables
   !&E
   !&E ** Description : - save restart file
   !&E                  - update forcings
   !&E                  - update the time step
   !&E                  - call the step?dxy or step?dyx subroutines, depending on if the time-step
   !&E                    is integer or half-integer (i.e. depending on the value of nbouc)
   !&E                  - call diagnoses
   !&E                  - estimate maximum velocityies at the sea surface
   !&E
   !&E ** Called by : main
   !&E
   !&E ** External calls : step3dxy, step3dyx, tool_sectodat, tide_genpot, meteo, sflx_read, sub_sflx_read
   !&E                   : out_saverestart, outflow, river, tool_decompdate
   !&E                   : tool_julien, sflx_surf, step2dxy, step2dyx, diagnostic, track_points, output_mng
   !&E                   : tide_astroarg
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : cdate, ijour, imois, ian, iheure, iminu, isec, jjulien
   !&E                         l_boolsauv, l_boolsort
   !&E                         indchrono, nbouc, t
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E        !           (P. Lazure)  Original code
   !&E        !  2005-02-23
   !&E        !  2007-07-25  (M. Sourisseau) elimination var_read_obc
   !&E        !  2009-03  (V. Garnier) MPI2D
   !&E        !  2009-02  (P. Le Hir, F. Cayocca) introduction "new" sand/mud module
   !&E        !  2009-09  (B. Thouvenin) introduction of module MET&OR (contaminant)
   !&E        !  2009-10  (F. Cayocca) correction for sedimentology
   !&E        !  2009-10  (V. Garnier) reproductibility when using restart file
   !&E        !  2010-02  (G. Simon) introduction of module wave
   !&E        !  2010-03  (A.-C. Bennis) coupling between ww3 and mars through palm
   !&E        !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E        !  2010-07  (R. Verney) Introduction of orbital velocity variable
   !&E       !  2010-10   (B. Thouvenin) Distribution of the river discharge throughout a thickness
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comvars3d
   USE comvarp2d
   USE comvarp3d
   USE comcolig2d,   ONLY : igdv_old
   USE cometdyn
   USE cometemp,     ONLY : l_sflx_solarcst
#ifdef key_wave
   USE wave
#endif
   USE comfluxdo
   USE comdiag,      ONLY : l_diag
   USE output,       ONLY : output_mng
   USE comaree
   USE sflxread             ! read sea surface data or atmospherical fluxes
   USE trackpoints,  ONLY : l_points,track_points
   USE tssubeqtest,  ONLY : tssub_eq_test
   USE_MPI toolmpi,  ONLY : findmax_mpi_3,MASTER,MAX_ALL_MPI_RSH

#ifdef key_wave
   USE comwave,      ONLY : tosx3dm,tosy3dm,l_wave_rdir,l_wave_rhs,l_wave_rfoc,l_wave_rubot
#endif
#ifdef key_palm_ww3
   USE palmlib,      ONLY : PL_LNAME,PL_NO_TAG
#endif

#if defined key_substance
   USE comsubstance,    ONLY : cv_wat
   USE comsubstance,    ONLY : nv_dri,l_driv_cst,t_driv,nv_state,nv_int,nv_adv, &
                               l_cvrain_readfile,l_subflxatm_readfile,tdebsubs,l_bilan
   USE usersubstance,   ONLY : user_vintermediate,user_ftime_vdriv
   USE subreaddat,      ONLY : sub_sflx_read
   USE submassbalance,  ONLY : t_out_budget,sub_budget_out
#if defined key_sedim
   USE sedimento
   USE comsubstance,    ONLY : isand2,flx_s2w,flx_w2s
#ifdef key_parsub_newdt
 ! rajout benedicte pour pouvoir recalculer un nombre de sous pas de temps variable 
 !  (seulement pour key_sedim car wsmax est calcule dans key_sedim)
   USE comsubstance,    ONLY : dhmin,ndt_part,nb_ws_max,iv_ws_max                          
#endif
#endif
#if defined key_contaminant 
   USE contareaddat,    ONLY : nreacmax,l_treat_re_after
   USE contasinksource, ONLY : conta_reac_after
   USE comsubstance,    ONLY : dcdt,tdebsubs
#endif
#endif

   IMPLICIT NONE
   !! * Arguments

   !! * Local declarations
   CHARACTER(LEN=19)  :: tool_sectodat
   INTEGER            :: i,j,k,tool_julien,iumx,jumx,ivmx,jvmx
   INTEGER            :: liumx,ljumx,livmx,ljvmx
   REAL(KIND=rsh)     :: umx,vmx,lumx,lvmx
#if defined key_sedim && defined key_parsub_newdt
   INTEGER            :: num_ws,iv,ndt_part_anc
#endif

#ifdef key_palm_ww3
  INTEGER :: il_err,ifile,timea=0,timeb=0
  CHARACTER(LEN=PL_LNAME) :: cl_object, cl_space
  REAL(KIND=4), DIMENSION(0:imax,0:jmax) :: usend,usendxe
  REAL(kind=4), DIMENSION(0:imax,0:jmax,kmax) :: tosx3d
  REAL(kind=4), DIMENSION(0:imax,0:jmax,kmax) :: tosy3d
  REAL(KIND=4), DIMENSION(1:kmax) :: usendsig
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter STEP routine'

   ! Save restart file
   ! -----------------

   IF (MOD(nbouc,2) == 0) THEN
     l_boolsauv=.FALSE.
     l_boolsort=.FALSE.

     IF (l_saverestart_1file.OR.l_saverestart_bydate) THEN
       CALL out_saverestart(u,uz,v,vz,       &
                            ssh(limin:limax,ljmin:ljmax),        &
                            sal,temp,vstar,igdv_old, &
#ifdef key_substance
                            cv_wat,                              &
#endif
                            ect   )
#if defined key_sedim && defined key_substance
       CALL sed_outsaverestart
#endif
     END IF     
   END IF     ! Ends loop on odd nbouc


   ! increment the time
   ! ------------------

   t=t+0.5*dt
   cdate = tool_sectodat(t)
   CALL tool_decompdate(cdate,ijour,imois,ian,iheure,iminu,isec)
   jjulien=tool_julien(ijour,imois,ian)-tool_julien(1,1,ian)+1
!   IF (MOD(nbouc,10)==0 .AND. iscreen/=6) WRITE(*,*) cdate, ' dt=', dt
!   IF (iscreen==6)        WRITE(iscreen,*) cdate, ' dt=', dt

   ! increment nbouc
   nbouc=nbouc+1

   ! update forcings
   ! ---------------

   ! for time interpolation of river discharges...
   IF (l_chrono) THEN
     DO WHILE((t-tchrono(indchrono))*(t-tchrono(indchrono+1)) > 0.0_rlg)
       indchrono=indchrono+1
     END DO
   END IF

   ! Potential generator at t
   IF (l_genpot) CALL tide_astroarg
   IF (l_genpot) CALL tide_genpot

   ! Fluxes at t
   IF (.NOT. l_testcase) CALL meteo        ! only if l_meteo_stat or l_meteo_hom
   IF (.NOT.l_meteo_stat .AND. .NOT.l_meteo_hom) CALL sflx_read
   CALL sflx_flux(temp)

   ! Waves at t
#ifdef key_wave
   IF ( l_wave_rdir .OR. l_wave_rhs .OR. l_wave_rfoc .OR. l_wave_rubot) CALL wave_read
#endif

   ! River input at t
   IF (nbfl  > 0) CALL river(ssh)

#ifdef key_substance
   ! Not used anymore. OBC data must be in netcdf format
!   CALL var_read_obc              ! Prescribe obc for substances

   ! interpolate data from files for cvrain and/or sub_flxatm 
   ! if one wants temporal variations of cvrain or of atm.deposition 
   ! for at least one substance
   !  (l_cvrain_readfile=.true. or l_subflxatm_readfile=.true.)
   IF (l_cvrain_readfile .or. l_subflxatm_readfile) CALL sub_sflx_read
   
   ! forcing variables
   IF ( nv_dri>0 .AND. (.NOT.l_driv_cst) ) THEN
     IF (t >= t_driv) &
       CALL user_ftime_vdriv(t,cv_wat &
#ifdef key_sedim
                              ,cv_sed &
#endif
                               )
   END IF

    ! intermediate variables
   IF (nv_int > 0) THEN
     CALL user_vintermediate(t,cv_wat &
#ifdef key_sedim
                              ,cv_sed &
#endif
                                                    )
   END IF
#endif

   ! simulate the dynamics and tracers
   ! ---------------------------------

   IF (num_testcase==18 .OR. num_testcase==19 .OR. num_testcase==20 .OR. num_testcase==24) THEN

   !************************************************************************
   ! Test advection scheme without any use of the dynamics equations
   !************************************************************************

     CALL tssub_eq_test

   ELSE


!!!#ifdef key_agrif
!!!     IF (.NOT.l_modele2d .AND. t >= tdeb3d) THEN
!!!#else
     IF (MOD(nbouc,2) == 1 .AND. .NOT.l_modele2d .AND. t >= tdeb3d) THEN
!!!#endif

#if ! defined key_offline

   ! Update the time step

       CALL dyn3d_dt(uz,vz)

   ! Test the stability of the water column and
   ! apply vertical mixing on temp and sal if required

       IF (l_stability) &
         CALL dyn3d_stabts(sal,temp,boz)


   ! key_offline
#endif


#if defined key_sedim && defined key_substance
       IF (t .GE. tdebsubs) THEN
         CALL sed_skinstress
         CALL sed_erosion
         CALL sed_settlveloc
         CALL sed_depflx
         IF (isand2.GE.1) CALL sed_sandconcextrap

#if defined key_parsub_newdt
         !  modif pour recalculer le ndt_part pour chaque groupe et optimiser
         ! (que pour key_sedim car plus susceptible d avoir des vitesses de chute fortes) 
         DO num_ws=1,nb_ws_max
            iv=iv_ws_max(num_ws,1) 
            CALL_MPI MAX_ALL_MPI_RSH(ws3max(iv))
            ndt_part_anc=ndt_part(num_ws)
            ndt_part(num_ws)=NINT(dt/MIN((dhmin/ws3max(iv)),dt))
#ifdef key_MPI_2D
            IF (MASTER .AND. ndt_part_anc /=ndt_part(num_ws)) WRITE(iscreen,*) 'ndt_part(',num_ws,')= ',ndt_part(num_ws)
#else               
            IF (ndt_part_anc /=ndt_part(num_ws)) WRITE(iscreen,*) 'ndt_part(',num_ws,')= ',ndt_part(num_ws)
#endif
         ENDDO
#endif
       ENDIF
#endif

   ! Contaminant - equilibrium reactions 
   ! ----------------------------------
#if defined key_contaminant && defined key_substance
   ! after ==> before
       IF ( nreacmax > 0 .AND. l_treat_re_after .AND. t >= tdebsubs ) &
         CALL conta_reac_after(cv_wat,sal,temp,ssh)
#endif

     END IF     ! Ends loop on odd nbouc and 3d run



   !************************************************************************
   ! 1er demi-pas de temps
   ! Estimate ssh,u,uz,sal,temp
   !************************************************************************

     IF (MOD(nbouc,2) == 1) THEN

   ! Estimate of the dynamics
   ! ------------------------

       IF (l_modele2d .OR. t < tdeb3d) CALL step2dxy
       IF (.NOT.l_modele2d .AND. t < tdeb3d) THEN
         DO j=ljminm2,ljmaxp2
           DO i=liminm2,limaxp2
             uz(:,i,j)=u(i,j)
             vz(:,i,j)=v(i,j)
           END DO
         END DO
       END IF
       IF (.NOT.l_modele2d .AND. t >= tdeb3d) CALL step3dxy

   ! Estimate of diagnoses
   ! ---------------------

       IF (l_diag) CALL diagnostic(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                                   cv_wat,                 &
#endif
                                   ect,boz)

#if defined key_sedim && defined key_substance
       IF (t .GE. tdebsubs) THEN
         CALL sed_effdep(ssh)
         CALL sed_consol_diffu
       END IF
#endif
#if defined key_substance
       IF(l_bilan .AND. t .GE. tdebsubs) THEN
          IF (t >= t_out_budget) CALL sub_budget_out(t,cv_wat  &
#if defined key_sedim
                                         ,cv_sed         &
#endif
                                         ,ssh)
       END IF
#endif

   ! Save variables at each time step for debugging purpose
   ! ------------------------------------------------------

       IF (l_points) CALL track_points(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                                       cv_wat,                 &
#endif
                                       ect,boz)


     END IF

!************************************************************************
!  2eme demi-pas de temps
!  Estimate ssh,v,vz,sal,temp...
!************************************************************************

     IF (MOD(nbouc,2) == 0) THEN

   ! Estimate of the dynamics
   ! ------------------------

       IF (l_modele2d .OR. t < tdeb3d) CALL step2dyx
       IF (.NOT.l_modele2d .AND. t < tdeb3d) THEN
         DO j=ljminm2,ljmaxp2
           DO i=liminm2,limaxp2
             uz(:,i,j)=u(i,j)
             vz(:,i,j)=v(i,j)
           END DO
         END DO
       END IF
       IF (.NOT.l_modele2d .AND. t >= tdeb3d) CALL step3dyx

   ! Estimate of diagnoses
   ! ---------------------

       IF (l_diag) CALL diagnostic(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                                   cv_wat,                 &
#endif
                                   ect,boz)

   ! Sediment 
   ! ---------------------
#if defined key_sedim && defined key_substance
       IF (t .GE. tdebsubs) THEN
         CALL sed_effdep(ssh)
         CALL sed_consol_diffu
         IF(l_morphocoupl) CALL sed_morpho
       END IF
#endif

   ! Balance  
   ! ---------------------
#if defined key_substance
       IF(l_bilan .AND. t .GE. tdebsubs) THEN
          IF (t >= t_out_budget) CALL sub_budget_out(t,cv_wat  &
#if defined key_sedim
                                         ,cv_sed      &
#endif   
                                         ,ssh)
       END IF
#endif

   ! Save variables at each time step for debugging purpose
   ! ------------------------------------------------------

       IF (l_points) CALL track_points(u,v,uz,vz,ssh,sal,temp, &
#ifdef key_substance
                                       cv_wat,                 &
#endif
                                       ect,boz)


   ! Fixed variables 
   !  A VOIR si valables aussi pour BIOLO ou si deja prises en compte  ???
   !  pour l instant mis seulement pour contaminant
   ! ----------------------------------------------
#if defined key_contaminant && defined key_substance

   !   fixed variable which are not advected but which are submitted to reactions
       cv_wat(nv_adv+1:nv_state,:,limin:limax,ljmin:ljmax)=          &
                 cv_wat(nv_adv+1:nv_state,:,limin:limax,ljmin:ljmax) &
                 +dt*0.5_rlg*dcdt(nv_adv+1:nv_state,:,limin:limax,ljmin:ljmax)
!cval ne devrait on pas echanger la ?
#endif

     END IF

   END IF    ! end test on test cas casAdvCst or casAdvSpot


#ifdef key_palm_ww3
#include "palmww3.h"
#endif


   ! Outputs
   ! -------

   CALL output_mng 

   ! Statistics
   ! ----------

   IF (l_maxu .AND. l_modele2d) THEN
     umx = -100.0_rsh
     vmx = -100.0_rsh
!$OMP PARALLEL default(none) &
!$OMP& PRIVATE(i,j,liumx,ljumx,livmx,ljvmx,lumx,lvmx) &
!$OMP& shared(umx,vmx,u,v,iumx,jumx,ivmx,jvmx,limin,limax,ljmin,ljmax,hx,hy,h0fond)
     lumx = umx
     lvmx = vmx
!$OMP do SCHEDULE(static)
     DO j = ljmin,ljmax
       DO i = limin,limax
         IF (ABS(u(i,j)) > lumx .AND. hx(i,j) /= -valmanq -h0fond) THEN
           lumx = ABS(u(i,j))
           liumx = i
           ljumx = j
         END IF
         IF (ABS(v(i,j)) > lvmx .AND. hy(i,j) /= -valmanq -h0fond ) THEN
           lvmx = ABS(v(i,j))
           livmx = i
           ljvmx = j
         END IF
       END DO
     END DO
!$OMP END DO nowait
!$OMP CRITICAL
     IF (lumx.GT.umx) THEN
       umx = lumx
       iumx = liumx
       jumx = ljumx
     END IF
     IF (lvmx.GT.vmx) THEN
       vmx = lvmx
       ivmx = livmx
       jvmx = ljvmx
     END IF
!$OMP END CRITICAL
!$OMP END PARALLEL
     CALL_MPI FINDMAX_MPI_3(umx,iumx,jumx)
     CALL_MPI FINDMAX_MPI_3(vmx,ivmx,jvmx)
     IF_MPI (MASTER) THEN
     WRITE(iscreen,*) 'DATE : ',cdate
     WRITE(iscreen,*) 'dt=',dt
     WRITE(iscreen,*)'umax = ',umx,'en',iumx,jumx,'hx= ',hx_g(iumx,jumx)
     WRITE(iscreen,*)'vmax = ',vmx,'en',ivmx,jvmx,'hy= ',hy_g(ivmx,jvmx)
     ENDIF_MPI
   END IF

   IF_AGRIF ( t > tfin .AND. .NOT.Agrif_Root() .AND. Agrif_NbStepint()==INT(Agrif_Rhot()-1) ) CALL DEALLOC
   IF_AGRIF ( t > tfin .AND. .NOT.Agrif_Root() .AND. Agrif_NbStepint()==INT(Agrif_Rhot()-1) ) CALL dealloc_inter

   PRINT_DBG*, 'exit STEP routine'

  END SUBROUTINE step
