  SUBROUTINE step2dyx

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE step2dyx  ***
   !&E
   !&E ** Purpose : Estimate the dynamics and temperature, salinity
   !&E              equations at n+1
   !&E
   !&E ** Description : dyn2dyx estimates xe^n+1 and u^n+1
   !&E                  plus an intermediate variable v_i^n+1
   !&E                  tssub2d_eq solves temperature, salinity
   !&E                  and substance equations
   !&E
   !&E ** Called by : step.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : u,v,ssh,sal,temp,cv_wat
   !&E
   !&E ** Modified variables : uv,xeuv,uv2,u,ssh,v,sal,temp,cv_wat
   !&E                         phitemp,phieau,phisal
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !           (P. Lazure)  Original code
   !&E       !  2007-03  (F. Vandermeirsch) introduction of characteristics method
   !&E       !  2008-07  (L. Debreu, T. Duhaut, M. Honnorat) time scheme
   !&E       !  2008-07  (T. Odaka) MPI
   !&E       !  2009-02  (A. Mano, V. Garnier, T. Odaka) MPI2D
   !&E       !  2009-04  (V. Garnier) separation of obc origin (ogcm/tide)
   !&E                                removing of xeuv argument for tide_harmcompo and obc2d_rank
   !&E       !  2009-05  (F. Dumas)  AGRIF to new time scheme
   !&E       !  2009-05  (F. Vandermeirsch)  introduction l_obc_tide,l_obc_ogcm
   !&E       !  2009-07  (F. Vandermeirsch)  MC 2nd order scheme
   !&E       !  2010-05  (F. Vandermeirsch)  Save obc at each half time step
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,     ONLY : l_modele2d,ig,id,jb,jh,l_obc2drank_save, &
                             nbouc,l_champroche,icon,h0fond,l_tide_surge
   USE comvarp2d,     ONLY : u,v,ssh,sal,temp
   USE traobc2dapply, ONLY : ts_obc2d_apply
   USE obcapplyssh,   ONLY : obc_apply_ssh
   USE_AGRIF traobc2dapply, ONLY : ts_obc2d_apply_agrif
#ifdef key_substance
   USE comsubstance,  ONLY : nbrej,nv_state,cv_wat
   USE subobc2dapply, ONLY : sub_obc2d_apply
   USE obcogcm,       ONLY : obc_subs
   USE obccombine,    ONLY : l_obc_ogcm_rc
   USE_AGRIF subobc2dapply, ONLY : sub_obc2d_apply_agrif
#endif
   USE comfluxdo,     ONLY : xeadv,nbfl
   USE comaree,       ONLY : l_tide_harmcompo
   USE obcogcm,       ONLY : obc_ogcm
   USE obccombine,    ONLY : l_obc_ogcm_rssh,obc_combine,l_obc_tide,l_obc_ogcm,l_obc_char
#ifdef key_trajec2d
   USE traject2d,     ONLY : traj_2d
#endif
   USE obcapplyv,     ONLY : obc_apply_v
#ifdef key_MPI_2D
   USE toolmpi,       ONLY : ex_i_rsh,ex_j_rsh
#endif

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER                                        :: i,j
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:,:)    :: xeuv
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2) :: uv,uv2

   !!----------------------------------------------------------------------
   !! * Executable part

   ! ----------------------------------------
   ! Initialisation of intermediate variables
   ! ----------------------------------------

   ! xeuv = estimated sea surface height
   ! uv = estimated barotropic current

   ALLOCATE(xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
   xeuv(liminm2:limaxp2,ljminm2:ljmaxp2)=ssh(liminm2:limaxp2,ljminm2:ljmaxp2)

   uv(liminm2:limaxp2,ljminm2:ljmaxp2)=0.0_rsh
   uv2(liminm2:limaxp2,ljminm2:ljmaxp2)=0.0_rsh

   ! --------------------------------------------
   ! change the time step if possible or required
   ! --------------------------------------------

!!!   CALL_AGRIF dyn2d_dt(u,v)

   ! ----------------------------
   ! Read and apply open boundary
   ! ----------------------------

   IF_AGRIF (Agrif_Root()) THEN
     IF (l_obc_tide) THEN
       IF (l_tide_surge) THEN
         CALL tide_harmcompo
         CALL obc2d_rank
       ELSE
         IF (l_tide_harmcompo) THEN
           CALL tide_harmcompo
         ELSE
           IF (l_testcase) THEN
             CALL obc2d_cas(xeuv)
           ELSE
             CALL obc2d_rank
           END IF
         END IF
       END IF
     END IF
     IF (l_obc_ogcm) CALL obc_ogcm
#ifdef key_substance
     IF (l_obc_ogcm_rc) CALL obc_subs
#endif
     CALL obc_combine(xeuv)         ! combine ogcm and tide obc
     IF (.NOT. l_obc_char) CALL obc_apply_ssh(xeuv)       ! apply ssh obc
   ELSE_AGRIF
     CALL_AGRIF agrif_obc_ssh(xeuv)
   ENDIF_AGRIF


   ! ---------------------
   ! Estimate the dynamics
   ! ---------------------

   CALL dyn2dyx(v,u,ssh,uv,xeuv,uv2)

   ! Update u,v,ssh variables

   DO j=ljmin,ljmax
     DO i=liminp1,limax
       IF ((j.GE.jb(i)) .AND. (j.LE.jh(i))) ssh(i,j)=xeuv(i,j)
     END DO
   END DO

   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))

   DO j=ljminm2,ljmaxp2
     DO i=liminm2,limaxp2
       IF ((j.GE.jb(i)-1) .AND. (j.LE.jh(i))) v(i,j)=uv(i,j)
       IF ((j.GE.jb(i)) .AND. (j.LE.jh(i))) u(i,j)=uv2(i,j)
     END DO
   END DO
   DEALLOCATE(xeuv)

   ! --------------------------------------------------------
   ! Save open boundary conditions (xe,u,v) for the next rank
   ! --------------------------------------------------------

   IF (l_obc2drank_save) CALL tide_saveobc(ssh,u,v)

   ! Initialize phicon and phieau
   CALL init_flux

   ! -------
   ! Outflow
   ! -------

#ifdef key_substance
   IF (nbrej > 0) CALL outflow
#endif

   ! -----------------------------------------------------------
   ! Estimate the temperature, salinity and substances equations
   ! -----------------------------------------------------------

#ifdef key_substance
   IF (l_modele2d .AND. (nv_state >= 1 .OR. nbfl >= 1)) THEN
#else
   IF (l_modele2d .AND. nbfl >= 1) THEN
#endif

     CALL sflx_surf(temp,sal,ssh)

     CALL tssub2d_eq(                                             &
#ifdef key_substance
                     cv_wat,                                      &
#endif
                     sal,temp,xeadv,ssh )

     IF_AGRIF (Agrif_Root()) THEN
       CALL ts_obc2d_apply(u,v,sal,temp)

#ifdef key_substance
       CALL sub_obc2d_apply(u,v,cv_wat)
#endif
     ELSE_AGRIF
       CALL_AGRIF ts_obc2d_apply_agrif
#ifdef key_substance
       CALL_AGRIF sub_obc2d_apply_agrif
#endif
     ENDIF_AGRIF

     IF (icon == 2) CALL ts_stateeq2d(sal)

#if defined key_champroche && defined key_substance
     IF (l_champroche) THEN
!arevoir nucon  CALL particule(u,v)
       CALL eclat(cv_wat,ssh)
     END IF
#endif

     DO j=ljminp2,ljmaxm1
       xeadv(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j)=ssh(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j) !akiko5
     END DO

   ELSE IF (.NOT.l_modele2d) THEN

     DO j=ljminp2,ljmaxm1
       xeadv(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j)=ssh(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j)
     END DO

   END IF

   ! -----------------------
   ! Estimate the trajectory
   ! -----------------------

#ifdef key_trajec2d
   CALL traj_2d(u,v)
#endif

  END SUBROUTINE step2dyx
