  SUBROUTINE step3dyx

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE step3dyx  ***
   !&E
   !&E ** Purpose : estimate of the dynamics and heat/salt/substance equation
   !&E              from time n+1/2 to n+1
   !&E
   !&E ** Description : estimate of
   !&E                   - the final zonal velocities : U^n+1, UZ^n+1
   !&E                   - the intermediate meridional velocities : V^n+1, VZ^n+1
   !&E                   - the final sea surface height : XE^n+1
   !&E                   - temperature, salinity : temp^n+1, sal^n+1
   !&E                   - substances : cv_wat^n+1
   !&E
   !&E ** Called by : step
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables :
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !           (P. Lazure)  Original code
   !&E       !  2007-07  (J. Marin et F. Vandermeirsch) introduction of characteristics method
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-08  (T. Odaka) MPI
   !&E       !  2009-04  (V. Garnier) separation of obc origin (ogcm/tide)
   !&E                                 removing of xeuv argument for tide_harmcompo and obc2d_rank
   !&E       !  2009-02  (P. Le Hir F. Cayocca) removing of sedimentology calls
   !&E       !  2009-04  (A. Mano, V. Garnier) MPI2D
   !&E       !  2009-05  (F. Dumas)  AGRIF to new time scheme
   !&E       !  2009-05  (F. Vandermeirsch)  introduction l_obc_tide,l_obc_ogcm,characteristics method
   !&E       !  2009-07  (F. Vandermeirsch)  MC 2nd order scheme
   !&E       !  2009-09  (B. Thouvenin)  coupling with MET&OR (contaminant)
   !&E       !  2011-03  (M. Huret) Introduction of Individual Based Model
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comvars3d
   USE comvarp2d
   USE comvarp3d ,     ONLY : uz,vz,boz,ect,psi
   USE cometdyn
   USE comfluxdo
   USE comaree
   USE obcogcm,        ONLY : obc_ogcm
   USE obccombine,     ONLY : obc_combine,l_obc_tide,l_obc_ogcm,l_obc_char,l_obc_ogcm_ruvz
#ifdef key_trajec3d
   USE_MPI comtraj,    ONLY : down_give,up_give,right_give,left_give
   USE traject3d
#endif
#ifdef key_ibm
   USE ibm, ONLY : ibm_3d
#endif
   USE obcapplyv
   USE obcapplyvz
   USE traobc3dapply
   USE obcapplyssh
#if defined key_substance
   USE subobc3dapply,  ONLY : sub_obc3d_apply
   USE obccombine,     ONLY : l_obc_ogcm_rc
   USE obcogcm,        ONLY : obc_subs
   USE_AGRIF subobc3dapply,  ONLY : sub_obc3d_apply_agrif
   USE comsubstance,   ONLY : tdebsubs,nbrej,cv_wat, &
                              sv_part,ndt_part,nb_ws_max,nvp,l_bilan
   USE submassbalance, ONLY : sub_budget_flxcum
#if defined key_biolo
   USE comsubstance,   ONLY : dtbio
   USE biolodynzwat,   ONLY : biolo_dyn_zwat
   USE biolosinksource,ONLY : biolo_sksc_sed,biolo_sksc_wat
#endif
#if defined key_contaminant
   USE contareaddat,   ONLY : l_flim3_re,nreacmax,l_treat_re_dc
   USE contasinksource,ONLY : conta_sksc_wat
   USE contaradsigz,   ONLY : radz_reduce
#if defined key_species
   USE comsubstance,   ONLY : dc_spc_wat,l_gpspecies
#endif
#endif
#endif
#ifdef key_offline
   USE offline
#endif
#ifdef key_MPI_2D
   USE toolmpi,        ONLY :ex_j_rsh,ex_i_rsh
#endif

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   INTEGER            :: i,j,l
   ! intermediate 2d variables of velocity and sea level
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:,:)      :: xeuv
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2)   :: uv,uv2

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter STEP3DXY routine'

   !-----------------------------------------------------------------------
   ! fisrt step (half time step incrementation)
   !-----------------------------------------------------------------------

   ! Allocate and initialize intermediate variables
   ! xeuv = estimated sea surface height
   ! uv = estimated barotropic current

   ALLOCATE(xeuv(liminm2:limaxp2,ljminm2:ljmaxp2))
   xeuv(liminm2:limaxp2,ljminm2:ljmaxp2)=ssh(liminm2:limaxp2,ljminm2:ljmaxp2)

   uv(liminm2:limaxp2,ljminm2:ljmaxp2)=0.0_rsh
   uv2(liminm2:limaxp2,ljminm2:ljmaxp2)=0.0_rsh

#if ! defined key_offline

   ! ------------------------
   ! Open Boundary Conditions
   ! ------------------------

   ! offline : sea surface height is prescribed
   ! agrif : nesting of v and xe

   IF_AGRIF (Agrif_Root()) THEN
     IF (l_obc_tide) THEN
       IF (l_tide_surge) THEN
         CALL tide_harmcompo
         CALL obc2d_rank
       ELSE
         IF (l_tide_harmcompo) THEN
           CALL tide_harmcompo
         ELSE
           IF (l_testcase) THEN
             CALL obc2d_cas(xeuv)
           ELSE
             CALL obc2d_rank
           END IF
         END IF
       END IF
     END IF
     IF (l_obc_ogcm) THEN
#if defined key_siggen || defined key_gencoord
       IF (l_obc_ogcm_ruvz) CALL obc_init_height(xeuv)
#endif
       CALL obc_ogcm ! read ogcm open boundary conditions
     END IF
#ifdef key_substance
     IF (l_obc_ogcm_rc) CALL obc_subs  ! read "bio" open boundary conditions
#endif
     CALL obc_combine(xeuv)         ! combine ogcm and tide obc
     IF (.NOT. l_obc_char) CALL obc_apply_ssh(xeuv)       ! apply ssh obc
   ELSE_AGRIF
     CALL_AGRIF agrif_obc_ssh(xeuv)
   ENDIF_AGRIF

   ! ------------------------
   ! Estimate of the dynamics
   ! ------------------------

   ! final estimate of the zonal velocity : U^n+1 and UZ^n+1
   ! intermediate estimate of the meridional velocities : V^n+1 and VZ^n+1
   ! intermediate estimate of the sea surface height : XE^n+1

   CALL dyn3dyx(v,u,ssh,uv,uv2,xeuv,vz,uz,boz)

#elif key_offline
   CALL dyn3dyx_offline(ssh,xeuv,vz,uz,boz)
   ! key_offline
#endif

   ! Estimate of the vertical velocity

!toseelater We can remove uz,vz,u,v in dyn3dwz
#if defined key_siggen || defined key_gencoord
!ATTENTION modif timescheme
   CALL dyn3d_wz(ssh,xeuv,uz,vz,u,uv)
#else
   CALL dyn3d_wz(ssh,xeuv,uz,vz)
#endif

   ! Save intermediate barotropic zonal velocity and the sea surface height
   ! into corresponding global 2d variables
   ! inside the interior domain (obc excluded)

   DO j=ljmin,ljmax
     DO i=liminp1,limax
       IF ((j.GE.jb(i)) .AND. (j.LE.jh(i))) ssh(i,j)=xeuv(i,j)
     END DO
   END DO
   DEALLOCATE(xeuv)

   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))

   DO j=ljminm2,ljmaxp2
     DO i=liminm2,limaxp2
       IF ((j.GE.jb(i)-1) .AND. (j.LE.jh(i))) v(i,j)=uv(i,j)
       IF ((j.GE.jb(i)  ) .AND. (j.LE.jh(i))) u(i,j)=uv2(i,j)
     END DO
   END DO

   !----------
   ! Ecosystem
   !----------

   ! Initialize phicon and phieau
   CALL init_flux

#if defined key_substance && defined key_biolo
   IF (t .GE. tdebsubs) THEN
     dtbio=0.5_rlg*dt
     CALL biolo_dyn_zwat(ssh,cv_wat,sal,temp)
   END IF
#endif

#if (defined key_substance && defined key_contaminant )
   !  attenuation of radiation into water column
   IF (t .GE. tdebsubs .and.(l_flim3_re) ) THEN
     CALL radz_reduce(ssh)
   END IF
#endif

#ifdef key_substance

   ! Outflow

   IF (nbrej > 0) CALL outflow

#ifdef key_biolo
   IF (t .GE. tdebsubs) THEN
     CALL biolo_sksc_wat(cv_wat,sal,temp,ssh)
     CALL biolo_sksc_sed
   END IF
#elif key_contaminant
   IF (t .GE. tdebsubs) THEN
     IF(nreacmax > 0 .AND. l_treat_re_dc) THEN
        CALL conta_sksc_wat( cv_wat,sal,temp,ssh)
     END IF
   END IF
#endif

#endif

   ! -------------------------------------
   ! Equation of heat, salt and substances
   ! -------------------------------------

   ! Estimate of atmospherical surface flux
   ! Update phitemp, phisal, phieau

#if ! defined key_offline
   CALL sflx_surf(temp,sal,ssh)
#endif

   ! Solve equation of heat, salt and substances

   IF (icon >= 1) THEN

     CALL tssub_eq(xeadv,ssh,sal,temp &
#ifdef key_substance
                  ,cv_wat             &
#endif
                  )

#ifdef key_substance

   ! cumulating dissolved substances fluxes for budget estimation
       IF (l_bilan .AND. t >= tdebsubs) THEN
          CALL sub_budget_flxcum(cv_wat,ssh)
       END IF

   ! Solve equation of variables with settling velocity
   IF (nvp>0) THEN
     DO l=1,nb_ws_max
       CALL parsub_eq(xeadv,ssh,cv_wat,sv_part(l),ndt_part(l),l)
     END DO
   END IF
#endif

   ! Apply open boundary conditions on temperature, salinity and substances

     IF_AGRIF (Agrif_Root()) THEN

#if ! defined key_offline
       CALL ts_obc3d_apply(uz,vz,sal,temp)
#endif

#ifdef key_substance
       CALL sub_obc3d_apply(uz,vz,cv_wat)
#endif

     ELSE_AGRIF
       CALL_AGRIF ts_obc3d_apply_agrif
#ifdef key_substance
       CALL_AGRIF sub_obc3d_apply_agrif
#endif
     ENDIF_AGRIF

   ! Estimate the buoyancy

#if ! defined key_offline
     IF (icon >= 1 .AND.      l_stateeq_lin) CALL ts_stateeq_linear(sal,temp,boz)
     IF (icon >= 1 .AND. .NOT.l_stateeq_lin) CALL ts_stateeq_blumberg(sal,temp,boz)
#endif

     DO j=ljminp2,ljmaxm1
       xeadv(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j)=ssh(MAX0(limin,ig(j)+1):MIN0(limax,id(j)-1),j) !akiko8
     END DO

   END IF   ! ends test on icon

#if ! defined key_offline

   ! -------------------------------------------
   ! Estimate of vertical turbulent coefficients
   ! -------------------------------------------

   IF (turb_nbeq == 0) &
     CALL turb(ssh,uz,vz,boz)

   IF (turb_nbeq == 1) &
     CALL turbkl(ssh,uz,vz,boz,ect)

   IF (turb_nbeq == 2) THEN
     CALL turbk(ssh,uz,vz,boz,ect)
     CALL turbpsi(ssh,uz,vz,boz,psi,ect)
     CALL turblepskznz(ssh,boz,psi,ect)
   END IF

   ! key_offline
#endif

   ! ------------
   ! Trajectories
   ! ------------

#ifdef key_trajec3d
   CALL traj_3d(ssh,uz,vz)
!   CALL_MPI ex_traj(down_give,up_give,right_give,left_give)
#endif

   ! ------------
   ! IBM
   ! ------------

#ifdef key_ibm
   CALL ibm_3d(ssh,uz,vz,sal,temp,          &
#ifdef key_substance
                                   cv_wat,   &
#endif
                                   boz       )
#endif

     CALL_AGRIF update_agrif

   PRINT_DBG*, 'exit STEP3DXY routine'

  END SUBROUTINE step3dyx
