 MODULE obcapplyvz

   !!=======================================================================
   !!                        ***  MODULE obcapplyvz  ***
   !! Open boundary treatment : apply open boundary schemes
   !!                           for 3d velocities
   !!                           at eastern and western boundaries.
   !!=======================================================================

#include "toolcpp.h"
 
  !! * Modules used
   USE parameters
   USE comvars2d
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC obc_apply_vz,obc_apply_vz_mc  ! called by stepv, dyn3dy

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE obc_apply_vz(vz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE obc_apply_vz  ***
   !&E
   !&E ** Purpose : Apply open boundary conditions for vz
   !&E
   !&E ** Description : Available schemes :
   !&E                     - periodic along x or y
   !&E                     - Dirichlet
   !&E                     - MARS standard (CMS)
   !&E                     - Caracteristics method
   !&E
   !&E ** Called by : stepv, dyn3dy
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !           Original code
   !&E       !  2007-03  (V. Garnier) introduction of obc periodic along y
   !&E       !  2007-05  (F. Vandermeirsch, J. Marin, V. Garnier) introduction of caracteristics method
   !&E       !  2008-07  (V. Garnier T. Odaka) MPI
   !&E       !  2009-03  (V. Garnier) MPI2D
   !&E       !  2010-04 (T. Odaka) MPI2D cyclic
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE obccombine, ONLY : obc_vze,obc_vzw,obc_vzs,obc_vzn,     &
                          l_obc_mars,l_obc_diri,l_obc_cycl
   USE_MPI toolmpi, ONLY : MPI_exj_recv,MPI_exj_send

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(inout) :: vz

   !! * local declarations
   INTEGER                      :: i,j

   !!----------------------------------------------------------------------
   !! * Executable part

   !! cyclic open boundary conditions
!$OMP SINGLE
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     CALL_MPI ex_peri_x_rsh(2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
#else 
     DO j=ljmin,ljmax
       vz(:,imin  ,j)=vz(:,imax-2,j)
       vz(:,imin+1,j)=vz(:,imax-1,j)
       vz(:,imax  ,j)=vz(:,imin+2,j)
     END DO
#endif
     !IF (jmax.EQ.ljmax) vz(:,limin:limax,jmax  )=vz(:,limin:limax,jmax-1)
     !IF (jmin.EQ.ljmin) vz(:,limin:limax,jmin  )=vz(:,limin:limax,jmin+1)
     ! Mars obc are applied
     DO i=liminp2,limaxm1
   ! Southern boundary
       j=jb(i)-1
       IF_MPI (ljmin==jmin .AND. ljmin==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i,j+1  )
       ENDIF_MPI
   ! Northern boundary
       j=jh(i)
       IF_MPI (ljmax==jmax .AND. ljmax==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i,j-1)
       ENDIF_MPI
     END DO

   ELSE IF (l_obc_cycl_y) THEN
     !IF (imin.EQ.limin)   vz(:,imin  ,:)=vz(:,imin+2,:)
     !IF (imin.EQ.limin)   vz(:,imin+1,:)=vz(:,imin+2,:)
     !IF (imax.EQ.limax)   vz(:,imax  ,:)=vz(:,imax-1,:)
     DO j=ljminp1,ljmaxm1
   ! Western boundary
       i=ig(j)
       IF_MPI (limin==imin .AND. limin==i-1) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j  )=vz(:,i+2,j)
         IF (hy(i-1,j) /= -valmanq-h0fond) vz(:,i-1,j  )=vz(:,i+2,j)
       ENDIF_MPI
   ! Eastern boundary
       i=id(j)
       IF_MPI (limax==imax .AND. limax==i) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i-1,j)
       ENDIF_MPI
     END DO

#ifdef key_MPI_2D
     CALL_MPI ex_peri_y_rsh(2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
#else
     vz(:,limin:limax,jmax  )=vz(:,limin:limax,jmin+2)
     vz(:,limin:limax,jmin+1)=vz(:,limin:limax,jmax-1)
     vz(:,limin:limax,jmin  )=vz(:,limin:limax,jmax-2)
#endif
   END IF   ! ends periodic part
!$OMP END SINGLE


   !! Dirichlet
   IF (l_obc_diri) THEN
! for straight open boundary only

!$OMP DO SCHEDULE(static)
     DO j=ljmin,ljmax

       i=ig(j)
       IF_MPI (limin==imin .AND. limin==i-1) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=obc_vzw(:,i,j)
       ENDIF_MPI

       i=id(j)
       IF_MPI (limax==imax .AND. limax==i) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=obc_vze(:,i,j)
       ENDIF_MPI
     END DO
!$OMP END DO

!$OMP SINGLE
     DO i=liminp2,limaxm1

       j=jb(i)-1
       IF_MPI (ljmin==jmin .AND. ljmin==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=obc_vzs(:,i,j)
       ENDIF_MPI

       j=jh(i)
       IF_MPI (ljmax==jmax .AND. ljmax==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=obc_vzn(:,i,j)
       ENDIF_MPI

     END DO
!$OMP END SINGLE

   !! Ends Dirichlet
   END IF


   !! Default open boundary conditions (MARS standard)

   IF (l_obc_mars) THEN

!$OMP DO SCHEDULE(static)
     DO j=ljminp1,ljmaxm1

   ! Western boundary

       i=ig(j)
       IF_MPI (limin==imin .AND. limin==i-1) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j  )=vz(:,i+1,j)
       ENDIF_MPI

   ! Eastern boundary

       i=id(j)
       IF_MPI (limax==imax .AND. limax==i) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i-1,j)
       ENDIF_MPI

     END DO
!$OMP END DO


!$OMP SINGLE
     DO i=liminp2,limaxm1

   ! Southern boundary

       j=jb(i)-1
       IF_MPI (ljmin==jmin .AND. ljmin==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i,j+1  )
       ENDIF_MPI

   ! Northern boundary

       j=jh(i)
       IF_MPI (ljmax==jmax .AND. ljmax==j) THEN
         IF (hy(i,j) /= -valmanq-h0fond) vz(:,i,j)=vz(:,i,j-1)
       ENDIF_MPI

     END DO
!$OMP END SINGLE

   !! Ends MARS standard open boundary conditions (straight boundaries)
   END IF

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull CALL ex_j_rsh(-2,2,kmax,limin,limaxp1,ljminm2,ljmaxp2,vz(:,limin:limaxp1,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vz)

  END SUBROUTINE obc_apply_vz

   !!======================================================================

  SUBROUTINE obc_apply_vz_mc(uz,vz,boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE obc_apply_vz_mc  **
   !&E
   !&E ** Purpose : Apply open boundary conditions for vz
   !&E
   !&E ** Description : - Compatible with characteristics method
   !&E
   !&E ** Called by : dyn3dxy, dyn3dyx
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :  uz,vz
   !&E
   !&E ** Modified variables : vz
   !&E
   !&E ** Reference : Blayo & Debreu, 2005. Ocean modelling, 9, 231-252
   !&E
   !&E ** History :
   !&E       !  2007-05  (F. Vandermeirsch, J. Marin, V. Garnier) Original code
   !&E       !  2009-05  (F. Vandermeirsch) characteristics method with new time scheme
   !&E
   !&E---------------------------------------------------------------------
   USE comvarp2d
   USE comvars3d
   USE obccombine, ONLY : obc_vze,obc_vzw,obc_vzn,obc_vzs
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( inout )  :: vz
   REAL(KIND=rsh), DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )  :: uz
   REAL(KIND=rsh), DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2), INTENT( in )  :: boz

   !! * local declarations
   INTEGER                     :: i,j,k,i1,i2,j1,j2

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP DO SCHEDULE(RUNTIME)
   DO i=liminp2,limaxm1

   ! Southern boundary

     j1 = jb(i)
     IF_MPI (ljmin==jmin .AND. ljmin==j1-1) THEN
 !FVDM      IF (hy(i,j1-1) /= -valmanq-h0fond) vz(:,i,j1-1)=obc_vzs(:,i,j1-1)
       IF (hy(i,j1-1) /= -valmanq-h0fond) vz(:,i,j1-1)= vz(:,i,j1)
    ENDIF_MPI

   ! Northern open boundary

     j2 = jh(i)
     IF_MPI (ljmax==jmax .AND. ljmax==j2) THEN
!FVDM       IF (hy(i,j2) /= -valmanq-h0fond)   vz(:,i,j2)=obc_vzn(:,i,j2) 
        IF (hy(i,j2) /= -valmanq-h0fond)   vz(:,i,j2)= vz(:,i,j2-1) 
    ENDIF_MPI

   END DO 
!$OMP END DO


!$OMP DO SCHEDULE(RUNTIME)
!   DO j=ljminp1,ljmaxm1
   DO j=ljminp2,ljmaxm1

   ! western  boundary

     i1=ig(j)
     IF_MPI (limin==imin .AND. limin==i1-1) THEN
       IF (hx(i1,j) /= -valmanq-h0fond) THEN
!FVDM     IF (hx(i1,j) /= -valmanq-h0fond .and. hy(i1,j) /= -valmanq-h0fond) THEN
! car MC barocline n est pas completement developpee
         DO k=1,kmax
!FVDM 5/02/2014
! MC pour la partie barocline n est pas developpee
! donc on fait du Dirichlet	 
!          IF ((uz(k,i1-1,j)+uz(k,i1,j)).GE.0.0_rsh) THEN
!             vz(k,i1,j)=obc_vzw(k,i1,j)
!           ELSE       
             vz(k,i1,j)= vz(k,i1+1,j)
!FVDM Fin          END IF
         END DO
       END IF
     ENDIF_MPI

   ! eastern  boundary

     i2=id(j)
     IF_MPI (limax==imax .AND. limax==i2) THEN
       IF (hx(i2-1,j) /= -valmanq-h0fond) THEN
!FVDM     IF (hx(i2-1,j) /= -valmanq-h0fond .and. hy(i2,j) /= -valmanq-h0fond) THEN
! car MC barocline n est pas completement developpee
         DO k=1,kmax
!FVDM 5/02/2014
! MC pour la partie barocline n est pas developpee
! donc on fait du Dirichlet		 
!           IF ((uz(k,i2-1,j)+uz(k,i2,j)).LE.0.0_rsh) THEN 
!             vz(k,i2,j)=obc_vze(k,i2,j)
!           ELSE
             vz(k,i2,j)= vz(k,i2-1,j)
!FVDM Fin           END IF
         END DO
       END IF
     ENDIF_MPI

   END DO  
!$OMP END DO

!---------------------------------------------------       
!Traitement des CFO par methode des caracteristiques
!----------------------------------------------------
!
#ifdef key_obc_char_barocline

   PRINT*, 'key_obc_char_barocline not ready yet'
   STOP
#if defined key_MPI_2D
   PRINT*, 'obcapplyuz mc part not MPIed yet'
   STOP
#endif

#ifdef key_obc_char_east
   DO j=jmin+2,jmax-1
       IF (h0(id(j),j-1)/=-valmanq .AND. h0(id(j),j+1)/=-valmanq) &
         CALL obcch3d_vz_exp(uz,vz,boz,id(j),j,3)
   END DO
#endif

#ifdef key_obc_char_west
   DO j=jmin+2,jmax-1
       IF (h0(ig(j),j-1)/=-valmanq .AND. h0(ig(j),j+1)/=-valmanq) &
         !jm on applique la methode barocline que sur 1/3 de la frontiere
         !jmif (j>jmax/3 .and. j<2*jmax/3) THEN
           CALL obcch3d_vz_exp(uz,vz,boz,ig(j)-1,j,4)
         !jmendif
   END DO
#endif

#ifdef key_obc_char_north
   DO j=imin+2,imax-1
       IF (h0(j-1,jh(j))/=-valmanq .AND. h0(j+1,jh(j))/=-valmanq) &
         CALL obcch3d_vz_exp(uz,vz,boz,j,jh(j),1)
   END DO
#endif

#ifdef key_obc_char_south
   DO j=imin+2,imax-1
       IF (h0(j-1,jb(j))/=-valmanq .AND. h0(j+1,jb(j))/=-valmanq) &
         CALL obcch3d_vz_exp(uz,vz,boz,j,jb(j)-1,2)
   END DO


   WRITE (*,*) 'routine limavz ... frontiere Sud non implementee'
   STOP
#endif
#endif  
   
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
!part really usefull CALL ex_j_rsh(-2,2,kmax,limin,limaxp1,ljminm2,ljmaxp2,vz(:,limin:limaxp1,ljminm2:ljmaxp2))
!larger exchanges to make transfer in step2d* and step3d* easier
   CALL_MPI ex_j_rsh(-2,2,kmax,liminm2,limaxp2,ljminm2,ljmaxp2,vz(:,liminm2:limaxp2,ljminm2:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(vz)

END SUBROUTINE obc_apply_vz_mc

   !!======================================================================

END MODULE obcapplyvz
