
   !!======================================================================

  SUBROUTINE sflx_turb_fairall(ztemp2,zsat,zwindmod,zpa,zrh,zrad,zirlos,zrain, &
                               zsenslos,zlatlos,zevap,ztosx,ztosy)

   !&E--------------------------------------------------------------------
   !&E                 ***  ROUTINE sflx_turb_fairall  ***
   !&E
   !&E ** Purpose : Computation of the heat fluxes and wind stress
   !&E              coefficients at the ocean surface
   !&E
   !&E ** Description :
   !&E        COARE Bulk Flux Algorithm version 3.0a. See Fairall et al.,2003:
   !&E        J.Climate,16,571-591
   !&E       
   !&E        Written 05/09/2003 by Frank Bradley frank.bradley@csiro.au
   !&E       
   !&E        Some details of changes from versions 2.5 and 2.6.
   !&E        For full description see Cor30_readme.txt
   !&E       
   !&E        Wave options for Zo - models by Taylor and Yelland
   !&E        (JPO 2001,31,572) or Oost et al.(BLM 2002,103,409)
   !&E        Wind speed dependant Charnock parameter above 10 m/s
   !&E        Grachev and Fairall first guess z/L routine (JAM 36,406-414,1997)
   !&E        to cut iterations from 20 to 3
   !&E        rr-zoq,charnock relations as described in COARE 3.0 J. Climate paper
   !&E        (see above) Use separate phiu,phit functions with Grachev et al.
   !&E        (2000) constants Stable functions according to Beljaars and Holtslag
   !&E        (JAM 30, 327-341,1991) Return Webb Wbar because Webb flux correction already
   !&E        included in latent heat flux Set up to read COARE test data set, test3_0.txt
   !&E        with 4-digit year tab delimited see Bradley, Moncrieff and Weller,
   !&E        Proceedings of the COARE Flux Group
   !&E        workshop, May 14-16 1997,c NCAR/MMM, PO Box 3000, Boulder, CO 80307-3000, USA
   !&E       
   !&E        THIS version "f" uses Chris Fairall s seasnake for Ts nominal depth 0.05m
   !&E        Another version "h" uses Mike Gregg s ASP Ts data at 6m depth, prepared
   !&E        by Hemantha Wijesekera Outputs tst3_0af.out or tst3_0ah.out written from
   !&E        subroutine bulk_flux
   !&E        Wave option outputs tst30afo.out or tst30aft.out (Jwave = 1 or 2)
   !&E
   !&E ** Input data - introduced in calling program "fluxes" and passed to subroutines in COMMON
   !&E        xtime (COARE convention yyyymnddhhmmss.ss GMT)
   !&E        u (wind speed relative to the sea surface) m/s  i.e. add CURRENT VECTOR if available
   !&E        ts=temp (sea temperature)  deg. C
   !&E        t=sat (air temperature) deg. C
   !&E        q (specific humidity) g/kg or RH (as decimal) - code detects which then works internally in kg/kg
   !&E        rs (shortwave radiation) W/m2
   !&E        rl (downwelling longwave) W/m2
   !&E        zrain (average rainrate in timestep) mm/hour
   !&E        xlat (latitude) degrees  [latitude north +ve, latitude south -ve]
   !&E        xlon (longitude) degrees [longitude east +ve, longitude west -ve]
   !&E        hwt alternative sea temperature (6m depth)
   !&E        p (pressure) mb; use 1008mb if data unavailable
   !&E        zi (boundary-layer depth) m; use 600m if data unavailable
   !&E        zu (wind measurement height) m
   !&E        zt (T measurement height) m
   !&E        zq (q measurement height) m
!   !&E        zus (wind standard height) m - usually 10m
!   !&E        zts (T and q standard height) m - usually 2m
!   !&E        ts_depth (depth of ts measurement) - positive m - for warm layer correction
   !&E        jcool (=1 for cool skin calculation;  =0 if SST measured by IR radiometer)
   !&E        jwarm (=1 for warm layer calculation; =0 if SST measured by IR radiometer)
   !&E        jwave (wave state options 0=Charnock,1=Oost et al,2=Taylor and Yelland)
   !&E
   !&E ** Calculated inputs:
   !&E        qs (humidity at ocean interface)
   !&E        tsw (ts with warm layer added)
   !&E
   !&E ** Outputs available - write statements in subroutine bulk_flux
   !&E        hf W/m**2     (Sensible heat flux) - turbulent part only
   !&E        ef W/m**2     (Latent heat flux) - turbulent part only
   !&E        rf W/m**2     (Heat flux due to rainfall)
   !&E        tau N/m**2   (wind stress) - turbulent part only
   !&E        taur - momentum flux due to rain N/m**2
   !&E        usr m/s     (M-O velocity scaling parameter u* = friction velocity)
   !&E        tsr C       (M-O temperature scaling parameter t*)
   !&E        qsr kg/kg   (M-O humidity scaling parameter q*)
   !&E        Cdn - neutral drag coefficient
   !&E        Chn - neutral transfer coefficient for heat
   !&E        Cen - neutral transfer coefficient for moisture
   !&E        RR - Roughness Reynolds number
   !&E        RT - Roughness Reynolds number for temperature
   !&E        RQ - Roughness Reynolds number for moisture
   !&E        zL - stability parameter height/L where L is the Obukhov length
   !&E        zo - velocity roughness length
   !&E        zot - temperature roughness length
   !&E        zoq - humidity roughness length
   !&E        dt_wrm - total warm layer temperature difference C
   !&E        tk_pwp - thickness of warm layer
   !&E        dsea - (=dt_wrm*ts_depth/tk_pwp) warming above sea temperature measurement
   !&E        dter - cool skin temperature difference C
   !&E        tkt - cool skin thickness mm (printed out *1000)
   !&E        sst - skin temperature C (sst = ts - dter + dsea)
   !&E        Wg - gustiness factor m/s
   !&E        Wbar - Webb mean vertical velocity m/s
   !&E        ef_webb - latent flux due to the Webb correction
   !&E        u_zs - wind velocity at standard height
   !&E        t_zs - air temperature ditto
   !&E        q_zs,qa_zs,e_zs,rh_zs - humidity ditto in a variety of units
   !&E
   !&E ** Called by : sflxsurf.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays :
   !&E
   !&E ** Modified variables : zsenslos, zlatlos, zevap, zchs, zces
   !&E
   !&E ** Reference : COARE Bulk Flux Algorithm version 3.0a. See Fairall et al.,2003. J.Climate,16,571-591
   !&E                METEOCEAN report (2009)
   !&E
   !&E ** History :
   !&E       !  2003/09  (Frank Bradley) Original code
   !&E       !  2008-08  (L.-H. Ayina)
   !&E       !  2009-06  (V. Garnier) MARS implementation
   !&E
   !&E----------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : grav,rlg
   USE comvars2d, ONLY  : jb,jh,ig,id,nbouc,t,iwarnlog
   USE comfluxdo, ONLY  : rain

   !! * Arguments
   REAL(KIND=rsh), INTENT( IN )      :: ztemp2,zsat,zwindmod,zpa,zrh,zrain, &
                         zirlos, &   ! downward IR flux [W/m**2]
                         zrad        ! downward solar flux [W/m**2]
   REAL(KIND=rsh), INTENT( OUT )     :: zsenslos,zlatlos,zevap
   REAL(KIND=rsh), INTENT( INOUT )   :: ztosx,ztosy
   REAL(KIND=rsh)                    :: ztemp !MANGA

   !! * Local declarations
!   REAL(KIND=rsh), DIMENSION(imin:imax,ljmin:ljmax) ::   &
!         xlat

#ifdef key_sfls_coef
   REAL(KIND=rsh)                    ::   &
         zchs,            &  ! coefficient for sensible heat flux
         zces                ! coefficient for latent heat flux
#endif

   INTEGER, PARAMETER :: jwave=0,       &   ! implement wave dependent roughness model
                                            ! Charnock formulation
                         jcool=0            ! implement cool calculation skin (0=no, 1=yes)
                                            ! SST measured by IR radiometer


   !! * Fairall parameters
   !! --------------------

   REAL(KIND=rsh), PARAMETER ::        &
                        zu=10.0_rsh,          &    ! wind measurement height             [m]
                        zq=2.0_rsh,           &    ! humidity (q) measurement height     [m]
!                        ts_depth=1.0_rsh,     &
                        Cpa=1004.67_rsh,      &    ! specific heat of air at constant pressure [J/kg/K]
                        zs=10.0_rsh,          &    ! standard height                     [m]
                        p=1008.0_rsh,         &    ! pressure if measurement not available [mbar]
                        zi=600.0_rsh,         &    ! boundary layer depth [m]
                        Rgas=287.1_rsh,       &
                        cpw=4000.0_rsh,       &
                        rhow=1022.0_rsh,      &
                        von=0.4_rsh,          &
                        Beta=1.2_rsh,         &
                        be=0.026_rsh,         &
                        twopi=3.141593_rsh*2.0_rsh, &    ! 2*pi
                        visw=1.e-6,           &
                        tcw=0.6_rsh

   REAL(KIND=rsh) :: expo,expa,zprs,hwave,twave,ztsw,dter,zsst,ea,es,qa, &
                     Cpv, rhoa,Al,ztka,ztko,qs,                          &
                     bigc,lnut,wetc,visa,cwave,                          &
                     lwave,dqer,zo,Wg,tkt,Du,Dt,Dq,u10,                  &
                     zo10,Cd10,Ch10,Ct10,zot10,Ct,Cd,CC,Ribcu,           &
                     Ribu,zetu,L1,psu10,pst10,psq10

   REAL(KIND=rsh) :: usr,tsr,qsr,charn,rr,zoq,zot, zL,L,psu,pst,         &
                     Bf,Rnl,hsb,hlb,qout,dels,qcool,alq, xlamx,          &
                     dwat,dtmp,dqs_dt,alfac,rf,taur,Wbar,ef_webb,        &
                     CDS,CHS,CES,Cdn,Chn,Cen,                            &
                     u_zs,t_zs,q_zs,qa_zs,e_zs,esat_zs,rh_zs,            &
                     zwind,L10,tau,ef,hf

   INTEGER :: nits,iter                              


   !!----------------------------------------------------------------------
   !! * Executable part


   ! --------------
   ! Initialization
   ! --------------


      ! Relative wind velocity [m/s] (module)
      ! prevoir u_wind-u_ocean selon la direction du vent
      ! -------------------------------------
! S.P. 09/2013 seuillage ????
         ztemp=max(ztemp2,0.1_rsh) !MANGA
         zwind = zwindmod


      ! default wave parameters
      ! -----------------------

         !  ztso=hwt !un-comment to use 6m depth sea temperature; also change ts_depth above
         !  default values for wave height and period for equilibrium sea
         !  if known, add to data and to read statement above,then comment next two lines

         hwave=0.018_rsh*zwind*zwind
         hwave=hwave*(1.0_rsh+.015_rsh*zwind)
         twave=0.729_rsh*zwind


      ! Sea surface temperature [Celsius]
      ! ---------------------------------
 
         ztsw=ztemp
         dter=0.3_rsh*jcool
         zsst=ztemp-dter

         ! jump over warm layer calculation
         ! convert humidities and calculate fluxes in ASL
         IF (jwave == 0) ztsw=ztemp


      ! air and sea temperature [K]
      !----------------------------

         ztka   = zsat + kelvin                 ! air temperature [K]
         ztko   = ztemp + kelvin                ! sea temperature [K]


      ! convert humidities and calculate fluxes in ASL
      ! ----------------------------------------------

         ! Atmospherical pressure [Pa ---> mbar]
         zprs = zpa/100.0_rsh

         ! The same formulation must be used for the air and the ocean
         CALL sh_fairall(zsat,zrh,zprs,ea,qa)      ! Teten s returns sat. air qa in kg/kg
         CALL sh_fairall(ztemp,suswe,zprs,es,qs)   ! Teten s returns sat. sea qs in kg/kg


!         CALL gravity(xlat(i,j),grav)                     ! gravity


   ! ------------------
   ! Fairall parameters
   ! ------------------


      ! Constants and coefficients (Stull 1988 p640).
      !----------------------------------------------

         Cpv=Cpa*(1.0_rsh+0.00084_rsh*qa)                        ! qa (kg/kg)
         rhoa=zprs*100.0_rsh/(Rgas*ztka*(1.0_rsh+0.61_rsh*qa))   ! qa (kg/kg)  and zprs (mb)  (bug in core3.0)
         Al=2.1e-5*(ztemp+3.2_rsh)**0.79_rsh


      ! Factors :Beta=1.2            !Given as 1.25 in Fairall et al.(1996)
      !          twopi=3.14159*2.
      !
      ! Additional constants needed for cool skin
      ! visw=1.e-6                   !m2/s kinematic viscosity water
      ! tcw=0.6                      !W/m/K   Thermal conductivity water

         bigc=16.0_rsh*grav*cpw*(rhow*visw)**3
         bigc=bigc/(tcw*tcw*rhoa*rhoa)
         lnut=(2.501_rsh-0.00237_rsh*ztsw)*1e+6         ! latent heat of vaporization [J/kg]
                                                        ! at ts (3C warming=0.3%)
         wetc=0.622_rsh*lnut*qs/(Rgas*(ztsw+kelvin)**2) ! Clausius-Clapeyron

         visa= 4.84e-9*zsat*zsat*zsat
         visa= 8.301e-6*zsat*zsat-visa
         visa=1.326e-5*(1.0_rsh+6.542e-3*zsat+visa)



   ! Wave parameters of Oost et al (2002)
   ! ------------------------------------

         cwave=grav*twave/twopi
         lwave=cwave*twave



   ! -----------
   ! First guess
   ! -----------

         dter=0.3_rsh*jcool              ! cool skin Dt
         dqer=wetc*dter                  ! cool skin Dq
         zo=0.0001_rsh
         Wg=0.5_rsh                      ! Gustiness factor initial guess
         tkt= 0.001_rsh*jcool            ! Cool skin thickness first guess


      ! Air-sea differences - includes warm layer in Dt and Dq
      ! ------------------------------------------------------

         Du=(zwind**2+Wg**2)**0.5_rsh    ! include gustiness in wind spd. difference
         Dt=ztsw-zsat-0.0098_rsh*zt      ! potential temperature difference.
         Dq=qs-qa


      ! neutral coefficients
      ! --------------------

         u10=Du*LOG(10/zo)/LOG(zu/zo)
         usr=0.035_rsh*u10
         zo10=0.011_rsh*usr*usr/grav+0.11_rsh*visa/usr
         Cd10=(von/LOG(10/zo10))**2
         Ch10=0.00115_rsh
         Ct10=Ch10/sqrt(Cd10)
         zot10=10.0_rsh/EXP(von/Ct10)
         Cd=(von/LOG(zu/zo10))**2


      ! Grachev and Fairall (JAM, 1997)
      ! -------------------------------

         Ct=von/LOG(zt/zot10)                    ! Temperature transfer coefficient
         CC=von*Ct/Cd                            ! z/L vs Rib linear coefficient
         Ribcu=-zu/(zi*0.004_rsh*Beta**3)        ! Saturation or plateau Rib
         Ribu=-grav*zu*((Dt-dter)+0.61_rsh*ztka*Dq)/(ztka*Du**2)
         IF (Ribu < 0.0_rsh) THEN
           zetu=CC*Ribu/(1.0_rsh+Ribu/Ribcu)               ! Unstable G and F
         ELSE
           zetu=CC*Ribu*(1.0_rsh+27.0_rsh/9.0_rsh*Ribu/CC) ! Stable
         END IF
         IF (zetu == 0.0_rsh) THEN
           zetu = -0.1_rsh ! cval WARNING AIE WHICH VALUE IS CORRECT ?
#if ! defined key_agrif
           WRITE(iwarnlog,*) "FAIRALL at t: ", t,"zetu == 0.0_rsh and set to -0.1_rsh"
#endif
         END IF
! S.P. 09/2013 seuillage ????
         L10=zu/max(zetu,0.1_rsh)                          ! MANGA
!         L10=zu/zetu                          ! MO length
         IF (zetu > 50.0_rsh) THEN
           nits=1
         ELSE
           nits=3                       ! number of iterations
         END IF


      ! First guess M-O stability dependent scaling params.(u*,t*,q*) to estimate zo and z/L
      ! ------------------------------------------------------------------------------------

         psu10=psiu(zu/L10)
         pst10=psit(zt/L10)
         psq10=psit(zq/L10)

         usr= Du*von/(LOG(zu/zo10)-psu10)
         tsr=-(Dt-dter)*von/(LOG(zt/zot10)-pst10)
         qsr=-(Dq-dqer)*von/(LOG(zq/zot10)-psq10)


      ! Charnock for high wind speeds Chris  data
      ! -----------------------------------------

         charn=0.011_rsh     !then modify Charnock for high wind speeds Chris  data
         IF (Du > 10.0_rsh) charn=0.011_rsh+(0.018_rsh-0.011_rsh)*(Du-10.0_rsh)/(18.0_rsh-10.0_rsh)
         IF (Du > 18.0_rsh) charn=0.018_rsh



   !----------------------------------------------------------------------------
   ! **** Iterate across u*(t*,q*),zo(zot,zoq) and z/L including cool skin ****
   !----------------------------------------------------------------------------

         DO iter=1,nits


      ! zo estimate
      ! -----------

           IF (jwave == 0) THEN
             zo=charn*usr*usr/grav + 0.11_rsh*visa/usr          ! after Smith 1988
           ELSE IF (jwave == 1) THEN
             zo=(50.0_rsh/twopi)*lwave*(usr/cwave)**4.5_rsh+0.11_rsh*visa/usr ! Oost et al.
           ELSE IF (jwave == 2) THEN
             zo=1200.0_rsh*hwave*(hwave/lwave)**4.5_rsh+0.11_rsh*visa/usr     ! Taylor and Yelland
           END IF
           rr=zo*usr/visa


      ! zoq and zot fitted to results from several ETL cruises
      ! ------------------------------------------------------

           zoq=min(1.15e-4,5.5e-5/rr**0.6_rsh)
           zot=zoq

      ! 
      ! -------------------------------------------------------------------------
           zL=von*grav*zu*(tsr*(1.+0.61_rsh*qa)+0.61_rsh*ztka*qsr)           &
              /(ztka*usr*usr*(1.+0.61_rsh*qa))
           L=zu/zL
           psu=psiu(zu/L)
           pst=psit(zt/L)
           dqer=wetc*dter*jcool
           usr=Du*von/(LOG(zu/zo)-psiu(zu/L))
           tsr=-(Dt-dter)*von/(LOG(zt/zot)-psit(zt/L))
           qsr=-(Dq-dqer)*von/(LOG(zq/zoq)-psit(zq/L))
           Bf=-grav/ztka*usr*(tsr+0.61_rsh*ztka*qsr)

           IF (Bf > 0.0_rsh) THEN
             Wg=Beta*(Bf*zi)**0.333_rsh
           ELSE
             Wg=0.2_rsh
           END IF

           Du=sqrt(zwind**2+Wg**2)                     ! include gustiness in wind spd.


      ! Cool skin
      ! ---------

           IF (jcool == 1) THEN

             ! background sensible and latent heat
             hsb=-rhoa*cpa*usr*tsr
             hlb=-rhoa*lnut*usr*qsr

             ! Recompute net longwave; cool skin=-2W/m2
!             Rnl= 0.97_rsh*(5.67e-8*(ztsw-dter+kelvin)**4-zrlo)
!             qout=Rnl+hsb+hlb

             ! thermal + sensible + latent heat fluxes
             qout=zirlos+hsb+hlb         ! ATTENTION signe et unite de irlos

             ! solar flux estimate
!             dels=(0.065_rsh+11.0_rsh*tkt-6.6e-5/tkt*(1.0_rsh-EXP(-tkt/8.0e-4))) !Eq.16 Ohlmann
!             dels=zrso*0.945_rsh*dels  ! the ocean sees only 94.5% of the incident solar  (penetrative flux)
!             qcool=qout-dels

     ! ATTENTION calcul de qout et qcool n est pas clair - signe des flux ?

             ! cooling flux (the penetrative solar flux -which warms the sea surface- is removed)
             ! apres etude de sensibilite, utiliser zrad * exp(-coext*dz) (on veut utiliser le flux penetrant)
             qcool=qout-zrad            ! ATTENTION signe et unite
             alq=Al*qcool+be*hlb*cpw/lnut                         ! Eq. 7 Buoy flux water

             IF (alq > 0.0_rsh) THEN                              ! originally (qcool.gt.0)
               xlamx=6.0_rsh/(1.0_rsh+(bigc*alq/usr**4)**0.75_rsh)**0.333_rsh     ! Eq 13 Saunders coeff.
               tkt=xlamx*visw/(sqrt(rhoa/rhow)*usr)               ! Eq.11 Sublayer thickness
             ELSE
               xlamx=6.0_rsh                                      ! prevent excessive warm skins
               tkt=MIN(0.01_rsh,xlamx*visw/(sqrt(rhoa/rhow)*usr)) ! Limit tkt
             END IF

             dter=qcool*tkt/tcw                                   ! Eq.12 Cool skin
             dqer=wetc*dter

           END IF    ! ends skin cool

         END DO                                             ! end iterations

   !---------------------------
   ! **** End of iteration ****
   !---------------------------


   !--------------------------------------------
   ! compute surface fluxes and other parameters
   !--------------------------------------------

         zsst=ztsw-dter*jcool          ! final skin temperature at this timestep
                                       ! warming at the sea surface relative to the heat lost (net flux)
                                       ! between the mid ocean surface layer and the sea surface
                                       ! dT/dz = Q

         ! relations 1.3.2 in METEOCEAN 2009
         tau=rhoa*usr*usr*zwind/Du     ! stress N/m2
         hf=-cpa*rhoa*usr*tsr          ! sensible W/m2
         ef=-lnut*rhoa*usr*qsr         ! latent W/m2


   !----------------------------------
   ! compute heat flux due to rainfall
   !----------------------------------

         dwat=2.11e-5*((ztka)/kelvin)**1.94_rsh                  ! water vapour diffusivity
         dtmp=(1.0_rsh+3.309e-3*zsat-1.44e-6*zsat*zsat)*0.02411_rsh
         dtmp=dtmp/(rhoa*cpa)                                    ! heat diffusivity
         dqs_dt=qa*lnut/(Rgas*(ztka)**2)                         ! Clausius-Clapeyron
         alfac= 1.0_rsh/(1.0_rsh+(wetc*lnut*dwat)/(cpa*dtmp))    ! wet bulb factor
         !rf= zrain*alfac*cpw*((zsst-zsat)+(qs-qa-dqer)*lnut/cpa)/3600.0_rsh    ! ATTENTION signe et unite (/3600 car passe en mm/h dans sflxread originalement)
         rf= zrain*alfac*cpw*((zsst-zsat)+(qs-qa-dqer)*lnut/cpa)               ! ATTENTION signe et unite (stupide, on reste en mm/s dans sflxread)
         rf = 0.0_rsh                                            ! the rain is not taken into account at the moment


   !--------------------------------------
   ! compute momentum flux due to rainfall
   !--------------------------------------

         !taur=0.85_rsh*zrain/3600.0_rsh*zwind                    ! ATTENTION signe et unite de la pluie
         taur=0.85_rsh*zrain*zwind                    ! ATTENTION signe et unite de la pluie (on reste en mm/s dans sflxread)
         taur=0.0_rsh                                            ! the rain is not taken into account at the moment


   !-------------------------------------------------------------------------------------
   ! Webb correction to latent heat flux already in ef via zoq/rr function so return Wbar
   !-------------------------------------------------------------------------------------

!      Wbar=-1.61_rsh*usr*qsr/(1.0_rsh+1.61_rsh*qa)-usr*tsr/(ztka)
         expo=-1.61_rsh*usr*qsr/(1.0_rsh+1.61_rsh*qa)
         expa=usr*tsr
         expa=expa/ztka
         Wbar= expo-expa

         ! latent heat flux due to the Webb correction
         ef_webb=lnut*rhoa*Wbar*qa
         ef_webb=0.0_rsh



   !------------------------------
   ! compute transfer coefficients
   !------------------------------
! S.P. 2013
     !    CDS=(usr/Du)**2   ! MANGA
     !    CHS=usr*tsr/(Du*(zsat-zsst+0.0098_rsh*zt)) !MANGA
     !    CES=usr*qsr/(Du*(qa-qs+dqer)) !MANGA


   !---------------------------------------------------------------------------
   ! compute neutral transfer coefficients and met variables at standard height
   !---------------------------------------------------------------------------

         Cdn=(von/LOG(zs/zo))**2
         Chn=von*von/(LOG(zs/zo)*LOG(zs/zot))
         Cen=von*von/(LOG(zs/zo)*LOG(zs/zoq))

!MANGA
	 CDS=(usr/Du)**2
         if ( Du*(zsat-zsst+0.0098_rsh*zt)/=0.0_rsh ) then
            CHS=usr*tsr/(Du*(zsat-zsst+0.0098_rsh*zt))
         else 
           CHS =CHN
         endif
         if ( Du*(qa-qs+dqer)/=0.0_rsh ) then
           CES=usr*qsr/(Du*(qa-qs+dqer))
         else 
           CES =Cen
         endif
!END MANGA

   !---------------------------------------------------
   ! adjust meteorological variables to standard height
   ! (aboard, measurements are done at different atitudes)
   !---------------------------------------------------

#ifdef key_metvar_fairall
         u_zs=usr/von*(LOG(zs/zo)-psiu(zL))
         t_zs=zsst+tsr/von*(LOG(zs/zot)-psit(zL))
         q_zs=(qs-dqer)+qsr/von*(LOG(zs/zoq)-psit(zL))  ! kg/kg
         qa_zs=1000.0_rsh*q_zs                          ! g/kg
         e_zs=q_zs*zprs/(0.62197_rsh+0.378_rsh*q_zs)    ! mb
         call humidity(t_zs,p,esat_zs)                  ! mb
         rh_zs=e_zs/esat_zs                             ! RH as decimal
#endif


   !--------------
   ! output fluxes
   !--------------

         ! Wind stress (N/m2)

         ztosx=rhoa*CDS*ztosx
         ztosy=rhoa*CDS*ztosy

         ! Du is the modified module of the wind 
         ! from which all the exchange coefficients have been estimated.
         ! ztosx = ||V||*Vx ; ztosy = ||V||*Vy, so Du/zwindmod removes
         ! the module of the wind in the formula,
         ! the second factor Du/zwindmod corrects the component of the wind

         IF (zwindmod > 0.0_rsh) THEN
           ztosx=ztosx*(Du/zwindmod)**2
           ztosy=ztosy*(Du/zwindmod)**2
         ENDIF

#ifdef key_windstress_module
         ztosx=rhoa*CDS*zwindmod*zwindmod
         ztosy=rhoa*CDS*Du*Du
#endif

         ! Turbulent fluxes

         zsenslos = -(hf+rf)
         zlatlos = -(ef+ef_webb)
         zevap = -zlatlos/lnut
#ifdef key_sfls_coef
         zchs = CHS
         zces = CES

       ELSE

         zchs = 0.0_rsh
         zces = 0.0_rsh
#endif


  END SUBROUTINE sflx_turb_fairall


   !!======================================================================

  SUBROUTINE sh_fairall(ssti,rhim,pris,pvis,shim)

   !&E---------------------------------------------------------------------
   !&E                       ***  ROUTINE sh_fairall ***
   !&E      
   !&E       ** Purpose : calculate specific humidity and saturated vapour pressure
   !&E      
   !&E       ** Description : - determined as function of relative humidity
   !&E                        - alternative using dew point temperature instead
   !&E                          of relative humidity is possible but not included
   !&E      
   !&E       ** Called by :
   !&E      
   !&E       ** External calls :
   !&E      
   !&E       ** Reference : - Teten s formula for saturation vo Buck (1981)
   !&E                                                   JAM 20, 1527-1532
   !&E       ** History :
   !&E             !
   !&E              press = 1013.25 ! standard atmospheric pressure [mb]
   !&E---------------------------------------------------------------------
   REAL(KIND=rsh), INTENT( IN )  :: ssti,rhim,pris
   REAL(KIND=rsh), INTENT( OUT ) :: shim
   REAL(KIND=rsh), INTENT( OUT)  :: pvis
   REAL(KIND=rsh), PARAMETER     ::         &
                                    cons1=1.0007_rsh,    &
                                    cons2=3.46e-6,       &
                                    cons3=6.112_rsh,     &
                                    cons4=17.502_rsh,    &
                                    cons5=240.97_rsh,    &
                                    cons6=0.622_rsh,     &
                                    cons7=1.0_rsh-cons6

   REAL(KIND=rsh)                :: expo,expo1,expa

   expa = cons4*ssti/(cons5+ssti)
   expo = exp(expa)
   expo1 = (cons1+cons2*pris)*cons3*expo
!   IF (rhim < 2.0_rsh) THEN
     pvis = expo1*rhim
     shim = (cons6*pvis)/(pris-cons7*pvis)
!   ELSE
!     shim = rhim/1000.0_rsh                 !g/kg to kg/kg
!   END IF

  END SUBROUTINE sh_fairall


   !!======================================================================

  SUBROUTINE humidity(T,P,esat)

   !&E------------------------------------------------------------------
   !&E      Tetens  formula for saturation vp Buck(1981) JAM 20, 1527-1532
   !&E------------------------------------------------------------------
   REAL(KIND=rsh), INTENT( IN )  ::  T,P
   REAL(KIND=rsh), INTENT( OUT ) ::  esat

   esat = (1.0007_rsh+3.46e-6*P)*6.1121_rsh*EXP(17.502_rsh*T/(240.97_rsh+T)) ! mb

  END SUBROUTINE humidity


   !!======================================================================

  FUNCTION psiu(zL)
   !&E---------------------------------------------------------------------
   !&Epsiu and psit evaluate stability function for wind speed and scalars
   !&E  matching Kansas and free convection forms with weighting f
   !&E     convective form follows Fairall et al (1996) with profile constants
   !&E       from Grachev et al (2000) BLM
   !&E       stable form from Beljaars and Holtslag (1991)
   !&E--------------------------------------------------------------------
   REAL(KIND=rsh), INTENT( IN ) :: zL
   REAL(KIND=rsh)               :: x,y,psik,psic,f,psiu,c

   IF (zL <= 0.0_rsh) THEN
     x=(1.0_rsh-15.0_rsh*zL)**0.25_rsh              ! Kansas unstable
     psik=2.0_rsh*LOG((1.0_rsh+x)/2.0_rsh)+LOG((1.0_rsh+x*x)/2.0_rsh)  &
          - 2.0_rsh*ATAN(x)+2.0_rsh*ATAN(1.0_rsh)
     y=(1.0_rsh-10.15_rsh*zL)**0.3333_rsh                    ! Convective
     psic=1.5_rsh*LOG((1.0_rsh+y+y*y)/3.0_rsh)                       &
            - SQRT(3.0_rsh)*ATAN((1.0_rsh+2.0_rsh*y)/SQRT(3.0_rsh))  &
            + 4.0_rsh*ATAN(1.0_rsh)/SQRT(3.0_rsh)
     f=zL*zL/(1.+zL*zL)
     psiu=(1.0_rsh-f)*psik+f*psic
   ELSE
     c=min(50.0_rsh,0.35_rsh*zL)                       ! Stable
!cval       psiu=-((1.0_rsh+1.0_rsh*zL)**1.+.6667*(zL-14.28)/EXP(c)+8.525)
     psiu=-((1.0_rsh+1.0_rsh*zL)+0.6667_rsh*(zL-14.28_rsh)/EXP(c)+8.525_rsh)
   END IF

  END FUNCTION psiu


   !!======================================================================

  FUNCTION psit(zL)
   !&E---------------------------------------------------------------------
   !&E       psiu and psit evaluate stability function for wind speed and scalars
   !&E       matching Kansas and free convection forms with weighting f
   !&E       convective form follows Fairall et al (1996) with profile constants
   !&E       from Grachev et al (2000) BLM
   !&E       stable form from Beljaars and Holtslag (1991)
   !&E--------------------------------------------------------------------

   REAL(KIND=rsh), INTENT( IN )  :: zL
   REAL(KIND=rsh)                :: x,y,psik,psic,f,psit,c

   IF (zL <= 0.0_rsh) THEN
     x=(1.0_rsh-15.0_rsh*zL)**0.5_rsh                        ! Kansas unstable
     psik=2.0_rsh*LOG((1.0_rsh+x)/2.0_rsh)
     y=(1.0_rsh-34.15_rsh*zL)**0.3333_rsh                    ! Convective
     psic=1.5_rsh*LOG((1.0_rsh+y+y*y)/3.0_rsh)                       &
            - SQRT(3.0_rsh)*ATAN((1.0_rsh+2.0_rsh*y)/SQRT(3.0_rsh))  &
            + 4.0_rsh*ATAN(1.0_rsh)/SQRT(3.0_rsh)
     f=zL*zL/(1.0_rsh+zL*zL)
     psit=(1.0_rsh-f)*psik+f*psic
   ELSE
     c=min(50.0_rsh,0.35_rsh*zL)                        !Stable
     psit=-((1.0_rsh+2.0_rsh*zL/3.0_rsh)**1.5_rsh+0.6667_rsh*(zL-14.28_rsh)/EXP(c)+8.525_rsh)
   END IF

  END FUNCTION psit

   !!======================================================================

  SUBROUTINE gravity(zzlat,zgra)
   !&E--------------------------------------------------------------------------
   !&E      calculates g as a function of latitude using the 1980 IUGG formula
   !&E
   !&E      Bulletin Geodesique, Vol 62, No 3, 1988 (Geodesist s Handbook)
   !&E      p 356, 1980 Gravity Formula (IUGG, H. Moritz)
   !&E      units are in m/sec^2 and have a relative precision of 1 part
   !&E      in 10^10 (0.1 microGal)
   !&E      code by M. Zumberge.
   !&E
   !&E      check values are:
   !&E      
   !&E      g = 9.780326772 at latitude  0.0
   !&E      g = 9.806199203 at latitude 45.0
   !&E      g = 9.832186368 at latitude 90.0
   !&E--------------------------------------------------------------------------
   USE parameters, ONLY : rlg
   REAL(KIND=rsh), INTENT( IN )  :: zzlat
   REAL(KIND=rsh), INTENT( OUT ) :: zgra
   REAL(KIND=rlg), PARAMETER     :: ppi=3.14159265358979_rlg
   REAL(KIND=rlg)                :: gamma, c1, c2, c3, c4, phi

   phi = REAL(zzlat,rlg) * ppi/180.0_rlg
   gamma = 9.7803267715_rlg
   c1 = 0.0052790414_rlg
   c2 = 0.0000232718_rlg
   c3 = 0.0000001262_rlg
   c4 = 0.0000000007_rlg
   zgra = REAL ( gamma * (1.0_rlg    &
          + c1 * ((sin(phi))**2)     &
          + c2 * ((sin(phi))**4)     &
          + c3 * ((sin(phi))**6)     &
          + c4 * ((sin(phi))**8)), rsh)

  END SUBROUTINE gravity

