#ifdef key_substance

  SUBROUTINE parsub_eq(xe2,xe3,cv_wat,setvel,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_eq_wcen  ***
   !&E
   !&E ** Purpose : first possibility of subroutine body for the computation 
   !&E              of tracers with settling velocity. 
   !&E
   !&E ** Description : A centred scheme is used for vertical convection
   !&E                  (resulting in a series of problems as sigma levels are not equidistant).
   !&E                  For low water heights, a special computation is eventually performed
   !&E                  in parsub_eq_hm, depending on the value of hm.
   !&E                  This subroutine body is the selected one by default.
   !&E
   !&E ** Important remark about the settling velocity with the use of key_subs_part_eqsubdt cpp key
   !&E              To save computation time, we assume the substances have the same settling velocity
   !&E              in a group (since the group and the corresponding sub time step depend on
   !&E              the diameter and the density of the particule).
   !&E              This assumption is not necessary valid for the biology. If so, modify slightly
   !&E              the maximum settling velocity in variable.dat file, to create a new group.
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls : parsub_advdifx, parsub_advdify
   !&E
   !&E ** Used ij-arrays : cv_wat,h0,xe2,xe3,wz,kz,flx_s2w,flx_w2s,flx_w2s_sum
   !&E                     fx,fy,phicon,dy_u,dx_v,surf
   !&E  
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (P. Douillet) substances with settling velocities
   !&E       !                         issued from tssub_eq_wcen.h
   !&E       !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E       !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E       !  2011-02 (B. Thouvenin) correction for key_subs_part_eqsubdt key
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,     ONLY : ig,id,h0,hm,dy_u,dx_v,surf,l_tssubadv_over_smalldepth,halfdt,t,fwet,fwetp
   USE comvars3d,     ONLY : wz,kz,dsigw,dsigu
   USE comvarp3d,     ONLY : uz,vz
   USE comsubstance,  ONLY : fx,fy,phicon,f90,nv_ws_max,iv_ws_max,l_bilan,tdebsubs
#if defined key_biolo || defined key_contaminant || defined key_eco3m
   USE comsubstance,  ONLY : dcdt
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m || defined key_sedim
   USE comsubstance,  ONLY : ws3
#endif
#if defined key_sedim
   USE comsubstance,  ONLY : flx_s2w,flx_w2s,flx_w2s_sum
#endif
   USE parsubobc3dapply, ONLY : parsub_obc3d_apply
   USE submassbalance,   ONLY : parsub_budget_flxcum
   USE_AGRIF parsubobc3dapply, ONLY : parsub_obc3d_apply_agrif

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in)    :: niter,num_ws
   REAL(KIND=rsh),                                                       INTENT(in)    :: setvel
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                                :: i,j,k,l,ll,l1,iv
   REAL(KIND=rsh)                         :: dtm,d3,d2,unsd3
   REAL(KIND=rsh),DIMENSION(1:kmax)       :: ws
   REAL(KIND=rlg),DIMENSION(kmax,nb_var)  :: y
   REAL(KIND=rlg)                         :: xk,yk
   REAL(KIND=rlg),DIMENSION(kmax)         :: a,b,c,u0,v0,p0
   REAL(KIND=rlg),DIMENSION(0:kmax)       :: q0

   !!----------------------------------------------------------------------
   !! * Executable part


!$OMP PARALLEL DEFAULT(none) PRIVATE(d2,d3,unsd3,i,j,k,l,         &
!$OMP& iv,ll,l1,xk,yk,dtm,p0,q0,u0,v0,a,b,c,y,ws)                 &
!$OMP& SHARED(halfdt,l_tssubadv_over_smalldepth,xe2,xe3,hm,ig,id,h0 &
!$OMP& ,limin,limax,ljmin,ljmax,ljminp2,ljmaxm1,fwet,fwetp        &
!$OMP& ,liminm1,limaxp2,ljminm1,ljmaxp2,l_bilan,t,tdebsubs        &
!$OMP& ,uz,vz,cv_wat,fx,fy,phicon,f90,nv_ws_max,iv_ws_max         &
#if defined key_biolo || defined key_contaminant || defined key_eco3m
!$OMP& ,dcdt                                                      &
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m || defined key_sedim
!$OMP& ,ws3                                                       &
#endif
#if defined key_sedim
!$OMP& ,flx_s2w,flx_w2s,flx_w2s_sum                               &
#endif
!$OMP& ,niter,num_ws,setvel,wz,kz,dsigw,dsigu,dy_u,dx_v,surf)

  DO ll=1,niter

   dtm=halfdt/REAL(niter,rlg)

   ! calcul des flux horizontaux (advectifs et diffusifs)
   !---------------------------------------------------

   CALL parsub_advdifx(cv_wat,niter,num_ws)
   CALL parsub_advdify(cv_wat,niter,num_ws)

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

   ! get the settling velocity
   ! -------------------------

   ! we assume all the substances of a group have the same settling velocity (iv=1 for each group)

#if defined key_sedim || defined key_biolo
       l=iv_ws_max(num_ws,1)
       ws(:)=-ws3(:,l,i,j)
#else
       l=1
       ws(:)=-setvel
#endif


       d3=h0(i,j)+REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)
       d2=h0(i,j)+REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)
       IF(d3 > hm) THEN
         unsd3=1.0_rsh/d3

   ! matrice tridiagonale du modele vertical
   ! (resolution a faire pour chaque traceur du fait de la prise en compte
   ! des depots se faisant a des vitesses de chute differentes)
   !----------------------------------------

         DO l1=1,nv_ws_max(num_ws)         !  vitesse de chute,
           iv=iv_ws_max(num_ws,l1)
   !-------------------------
           k=1
   !-------------------------
           c(k)=dtm*(d3*0.5_rlg*(wz(k,i,j)+0.5_rsh*(ws(k)+ws(k+1))/d3)-kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
           b(k)=d3*fwet(i,j)+dtm*(d3*0.5_rlg*(wz(k,i,j)+0.5_rsh*(ws(k)+ws(k+1))/d3)+kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
#ifdef key_sedim
           b(k)=b(k)+dtm*flx_w2s(iv,i,j)/dsigu(k)
#endif


           q0(k)=-c(k)/b(k)        ! factorisation LU

   !-------------------------
           DO k=2,kmax-1
   !-------------------------
             c(k)=dtm*(d3*0.5_rlg*(wz(k,i,j)+0.5_rsh*(ws(k)+ws(k+1))/d3)-kz(k,i,j)*unsd3/dsigw(k))/dsigu(k)
             a(k)=-dtm*(d3*0.5_rlg*(wz(k-1,i,j)+0.5_rsh*(ws(k-1)+ws(k))/d3)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)
             b(k)=d3*fwet(i,j)+dtm*(d3*0.5_rlg*(wz(k,i,j)+0.5_rsh*(ws(k)+ws(k+1))/d3)+kz(k,i,j)*unsd3/dsigw(k)         &
                 -d3*0.5_rlg*(wz(k-1,i,j)+0.5_rsh*(ws(k-1)+ws(k))/d3)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)


             p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU
             q0(k)=-c(k)/p0(k)

   !-------------------------
           ENDDO
   !-------------------------

   !-------------------------
           k=kmax
   !-------------------------
           a(k)=-dtm*(d3*0.5_rlg*(wz(k-1,i,j)+0.5_rsh*(ws(k-1)+ws(k))/d3)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)
           b(k)=d3*fwet(i,j)+dtm*(-d3*0.5_rlg*(wz(k-1,i,j)+0.5_rsh*(ws(k-1)+ws(k))/d3)+kz(k-1,i,j)*unsd3/dsigw(k-1))/dsigu(k)

           p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU


   ! estimation des traceurs
   !------------------------

           DO k=1,kmax
             y(k,iv)=d2*fwetp(i,j)*cv_wat(iv,k,i,j)-(dy_u(i,j)*fx(iv,k,i,j)-dy_u(i-1,j)*fx(iv,k,i-1,j) &
                                       +dx_v(i,j)*fy(iv,k,i,j)-dx_v(i,j-1)*fy(iv,k,i,j-1) &
                                       -(phicon(iv,k,i,j)/REAL(niter,rsh))/dsigu(k))/surf(i,j)
#if defined key_biolo || defined key_contaminant || defined key_eco3m
             y(k,iv)=y(k,iv)+dcdt(iv,k,i,j)*dtm*d2*fwetp(i,j)
#endif
           ENDDO
#ifdef key_sedim
           y(1,iv)=y(1,iv)+flx_s2w(iv,i,j)*dtm/dsigu(1)  
#endif
           u0(1)=y(1,iv)/b(1)
           DO k=2,kmax
             u0(k)=(y(k,iv)-a(k)*u0(k-1))/p0(k)
           ENDDO
           xk=u0(kmax)
           cv_wat(iv,kmax,i,j)=f90(iv)*(xk)
           DO k=kmax-1,1,-1
             xk=q0(k)*xk+u0(k)
             cv_wat(iv,k,i,j)=f90(iv)*(xk)
           ENDDO
#ifdef key_sedim
           ! integration of effectif settling => flx_w2s_sum=kg/m2
           flx_w2s_sum(iv,i,j)=flx_w2s_sum(iv,i,j)+flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)*dtm
#endif
        ENDDO

       ENDIF

     ENDDO
   ENDDO
!$OMP END DO

   IF (l_tssubadv_over_smalldepth) CALL parsub_eq_hm(xe2,xe3,cv_wat,ll,niter,num_ws)

   IF (l_bilan .AND. t >= tdebsubs) CALL parsub_budget_flxcum(xe2,xe3,cv_wat,dtm,niter,num_ws,ll)

   ! Apply open boundary conditions on temperature, salinity and substances

   IF_AGRIF (Agrif_Root()) THEN
#ifdef key_substance
     CALL parsub_obc3d_apply(uz,vz,cv_wat,dtm,num_ws)
#endif
   ELSE_AGRIF
#ifdef key_substance
     CALL_AGRIF parsub_obc3d_apply_agrif(num_ws)
#endif
   ENDIF_AGRIF
 
  END DO  ! ends loop on subdt


!$OMP END PARALLEL

  END SUBROUTINE parsub_eq

#endif
