#ifdef key_substance

  SUBROUTINE parsub_advdifx_fourthorder(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx_fourthorder  ***
   !&E
   !&E ** Purpose : Estimate of zonal fluxes of particulate substances (advection and diffusion)
   !&E              Advection uses a fourth order centered scheme equivalent 
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y : correct for eastward propagation,
   !&E                    uncorrect for westward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,fxeau,c(cv_wat),hzex,dx_u 
   !&E                     fx 
   !&E  
   !&E ** Modified variables : fx
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E           !  2010-11 (L. Debreu, T. Duhaut, V. Garnier) original code
   !&E           !                         issued from tssub_advdifx_fourthorder
   !&E           !  2011-02 (B. Thouvenin) correction diffusion terms of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : ig,id,hex,hx,kx,h0fond,hminkxky,dx_u,halfdt,l_obc_cycl_x,dy_u
#if defined key_siggen || defined key_gencoord
   USE comsiggen,   ONLY : hzex
#endif
   USE comfluxdo,   ONLY : fxeau
   USE comsubstance,ONLY : iv_ws_max,nv_ws_max,nv_adv,fx
   USE_MPI toolmpi, ONLY : ex_i_rsh

   IMPLICIT NONE

   !! * Arguments

   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
!   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: uu,uuu,cfl,d1,d2,d3,d4,kxb
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,yd,delta

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP DO schedule(runtime),private(i,j,k,kxb   &
!$OMP& ,l,iv,ya,yb,yc,yd,delta                 &
!$OMP& ,d1,d2,d3,d4,uu,uuu,cfl)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       DO k=1,kmax
         IF (hex(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           fx(iv,k,i,j)=0.0_rsh
          END DO
         ELSE
           kxb=kx
           IF (hex(i,j) < hminkxky) kxb=0.0_rsh ! no diffusion for small water depth
           uu=fxeau(k,i,j)
#if defined key_siggen || defined key_gencoord
           uuu = uu/(halfdt*hzex(k,i,j))
#else
           uuu = uu/(halfdt*hex(i,j))
#endif
           cfl = uuu*halfdt/dx_u(i,j)/REAL(niter,rsh)
           d1 = (1.0_rsh/24.0_rsh)*(-2.0_rlg+cfl+2.0_rlg*cfl**2-cfl**3)
           d2 = (1.0_rsh/24.0_rsh)*(14.0_rlg-15.0_rlg*cfl-2.0_rlg*cfl**2+3.0_rlg*cfl**3)
           d3 = (1.0_rsh/24.0_rsh)*(14.0_rlg+15.0_rlg*cfl-2.0_rlg*cfl**2-3.0_rlg*cfl**3)
           d4 = (1.0_rsh/24.0_rsh)*(-2.0_rlg-cfl+2.0_rlg*cfl**2+cfl**3)

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c(iv,k,i+1,j)
             yc(iv)=c(iv,k,i,j)
             delta(iv)=yb(iv)-yc(iv)

             IF (hex(i+1,j) > 0.0_rsh .AND. i+1 < id(j)) THEN
               ya(iv)=c(iv,k,i+2,j)
             ELSE
               ya(iv)=yb(iv)
             END IF

             IF (hex(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
!              IF (hex(i-1,j) > 0.0_rsh) THEN
               yd(iv)=c(iv,k,i-1,j)
             ELSE
               yd(iv)=yc(iv)
             END IF
           END DO



   ! advection flux of other substances
   ! ----------------------------------

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
!             IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
!               ! no horizontal gradient
!             ELSE
!             END IF
             fx(iv,k,i,j) = uu*(d1*ya(iv)+d2*yb(iv)+d3*yc(iv)+d4*yd(iv))
           END DO


           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
             fx(iv,k,i,j) = fx(iv,k,i,j)*dy_u(i,j)/REAL(niter,rsh)
#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hzex(k,i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#else
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hex(i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#endif
#endif
           END DO
         END IF ! ends test on water depth
       END DO ! ends loop on k
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,nv_adv*kmax,liminm1,limax,ljmin,ljmax,fx(:,:,liminm1:limax,ljmin:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fx)

  END SUBROUTINE parsub_advdifx_fourthorder

#endif
