#ifdef key_substance

  SUBROUTINE parsub_advdifx_uq(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdifx_ultimatequickest  ***
   !&E
   !&E ** Purpose : Estimate of zonal fluxes of particulate substances (advection and diffusion)
   !&E              Advection uses QUICKEST scheme with ULTIMATE limiter.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y : correct for eastward propagation,
   !&E                    uncorrect for westward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,fxeau,c(cv_wat),hzex,dx_u 
   !&E                     fx 
   !&E  
   !&E ** Modified variables : fx
   !&E
   !&E ** Reference : Leonard B. P. (1991) : The ULTIMATE conservative
   !&E                difference scheme applied to unsteady one-dimensional
   !&E                advection. Computer Methods in Applied Mechanics and
   !&E                Engineering, 88, 17-74
   !&E
   !&E ** History :
   !&E           !  2010-03  (P. Douillet) substances with settling velocities
   !&E           !                         issued from tssub_advdifx_uq
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : ig,id,hex,hx,kx,h0fond,hminkxky,dx_u,halfdt,l_obc_cycl_x
#if defined key_siggen || defined key_gencoord
   USE comsiggen, ONLY : hzex
#endif
   USE comfluxdo, ONLY : fxeau
   USE comsubstance, ONLY: fx,iv_ws_max,nv_ws_max,nv_adv
   USE_MPI toolmpi, ONLY : ex_i_rsh

   IMPLICIT NONE

   !! * Arguments

   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: uu,dyds,dygs,dydt,d0,d1,uuu,courant,     &
                                dygt,kxb,thetap,psip
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: dyd,dyg,ya,yb,yc,delta

   !!----------------------------------------------------------------------
   !! * Executable part

!$OMP DO schedule(runtime),private(i,j,k,kxb               &
!$OMP& ,dyds,dygs,dydt,dygt                                &
!$OMP& ,l,iv,dyd,dyg,ya,yb,yc,delta                        &
!$OMP& ,d0,d1,uu,uuu,courant,thetap,psip)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)
       DO k=1,kmax
         IF (hex(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           fx(iv,k,i,j)=0.0_rsh
          END DO
         ELSE
           kxb=kx
           IF (hex(i,j) < hminkxky) kxb=0.0_rsh ! no diffusion for small water depth
           uu=fxeau(k,i,j)
#if defined key_siggen || defined key_gencoord
           uuu = uu/(halfdt*hzex(k,i,j))
#else
           uuu = uu/(halfdt*hex(i,j))
#endif
           courant = ABS(uuu)*halfdt/dx_u(i,j)/REAL(niter,rsh)
           d0=(1.0_rsh/6.0_rsh)*(2.0_rsh-courant)*(1.0_rsh-courant)
           d1=(1.0_rsh/6.0_rsh)*(1.0_rsh-courant)*(1.0_rsh+courant)
             
           IF (uu <= 0.0_rsh) THEN  ! westward advection
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c(iv,k,i+1,j)
             yc(iv)=c(iv,k,i,j)
             IF (hex(i+1,j) > 0.0_rsh .AND. i+1 < id(j)) THEN
               ya(iv)=c(iv,k,i+2,j)
             ELSE
               ya(iv)=yb(iv)
             END IF
             dyd(iv)=yc(iv)-yb(iv)
             dyg(iv)=yb(iv)-ya(iv)
             delta(iv)=yb(iv)-yc(iv)
            END DO

           ELSE  ! eastward advection

            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             yb(iv)=c(iv,k,i,j)
             yc(iv)=c(iv,k,i+1,j)
             IF (hex(i-1,j) > 0.0_rsh .AND. (i > ig(j) .OR. l_obc_cycl_x)) THEN
!              IF (hex(i-1,j) > 0.0_rsh) THEN
               ya(iv)=c(iv,k,i-1,j)
             ELSE
               ya(iv)=yb(iv)
             END IF
             dyg(iv)=yb(iv)-ya(iv)
             dyd(iv)=yc(iv)-yb(iv)
             delta(iv)=yc(iv)-yb(iv)
            END DO

           END IF    ! ends test on velocity sign


   ! advection flux of other substances
   ! ----------------------------------

           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
               ! no horizontal gradient
               thetap = 0.0_rsh
               psip = 0.0_rsh
             ELSE
               ! ultimate limiter
               thetap=(dyg(iv)+dsmall)/(dyd(iv)+dsmall)
               psip = d0+d1*thetap
               psip=MAX(0.0_rsh,MIN(MIN(1.0_rsh,psip),((1.0_rsh-courant)/(courant+dsmall))*thetap))
             END IF
             fx(iv,k,i,j) = uu*(yb(iv)+psip*dyd(iv))
           END DO


           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
             fx(iv,k,i,j) = fx(iv,k,i,j)/REAL(niter,rsh)*dy_u(i,j)

#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hzex(k,i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#else
             fx(iv,k,i,j) = (fx(iv,k,i,j) - hex(i,j)*kxb*delta(iv)/dx_u(i,j)*halfdt)/REAL(niter,rsh)
#endif

#endif
           END DO
         END IF ! ends test on water depth
       END DO ! ends loop on k
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,0,nv_adv*kmax,liminm1,limax,ljmin,ljmax,fx(:,:,liminm1:limax,ljmin:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fx)

  END SUBROUTINE parsub_advdifx_uq

#endif
