#ifdef key_substance

  SUBROUTINE parsub_advdify_quickupwind(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify_quickupwind  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of temperature and salinity
   !&E              and other substances (advection and diffusion).
   !&E              Advection uses TVD scheme (QUICK and upwind) owing to limiter.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeau,c(cv_wat),hzey,dy_v,fy
   !&E
   !&E ** Modified variables : fy
   !&E
   !&E ** Reference : Kowalik Z. and T. S. Murty : Numerical modeling of ocean
   !&E                dynamics in Advanced series on ocean engineering, vol 5.
   !&E
   !&E ** History :
   !&E           !  2010-03  (P. Douillet) substances with settling velocities
   !&E                                     issued from tssub_advdify_quickupwind
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : jb,jh,hey,hy,h0fond,qmax,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y
#if defined key_siggen || defined key_gencoord
   USE comsiggen,    ONLY : hzey
#endif
   USE comfluxdo,    ONLY : fyeau
   USE comsubstance, ONLY : fy,iv_ws_max,nv_ws_max,nv_adv
#ifdef key_sedim
   USE sedimento,    ONLY : corfluy
#endif
   USE_MPI toolmpi,      ONLY : ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   LOGICAL                 :: decentt,decents
   INTEGER                 :: i,j,k
   REAL(KIND=rsh)          :: unsqmax,vv,yr,qot,dyds,dygs,dydt
   REAL(KIND=rsh)          :: dygt,kyb,epsi
   LOGICAL, DIMENSION(nv_adv)           :: decent
   INTEGER                              :: l,iv
   REAL(KIND=rsh)                       :: dyg
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,dyd

   !!----------------------------------------------------------------------
   !! * Executable part

   unsqmax=1.0_rsh/qmax
   epsi=1.0e-10

!$OMP DO schedule(runtime),private(i,j,k,kyb               &
!$OMP& ,vv,yr,qot,decents,decentt,dyds,dygs,dydt,dygt      &
!$OMP& ,l,iv,ya,yb,yc,decent,dyd,dyg)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hey(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fy(iv,k,i,j)=0.0_rsh
            END DO
           ELSE
             kyb=ky
             IF (hey(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
             IF (vv <= 0.0_rsh) THEN  ! southward advection
              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c(iv,k,i,j+1)
               yc(iv)=c(iv,k,i,j)
               IF (hey(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
                 ya(iv)=c(iv,k,i,j+2)
               ELSE
                 ya(iv)=yb(iv)
               END IF

                 dyd(iv)=yb(iv)-yc(iv)
                 dyg=yb(iv)-ya(iv)
                 decent(iv)=(dyd(iv)*dyg) >= 0.0_rsh
                 IF (.NOT.decent(iv)) THEN
                   qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                   IF (qot <= qmax .AND. qot >= unsqmax) THEN
                     decent(iv)=.FALSE.
                   ELSE
                     decent(iv)=.TRUE.
                   END IF
                 END IF
              END DO

             ELSE      ! northward advection

              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c(iv,k,i,j)
               yc(iv)=c(iv,k,i,j+1)
               IF (hey(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                 ya(iv)=c(iv,k,i,j-1)
               ELSE
                 ya(iv)=yb(iv)
               END IF

                 dyd(iv)=yc(iv)-yb(iv)
                 dyg=yb(iv)-ya(iv)
                 decent(iv)=(dyd(iv)*dyg) <= 0.0_rsh
                 IF (.NOT.decent(iv)) THEN
                   qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                   IF (qot <= qmax .AND. qot >= unsqmax) THEN
                     decent(iv)=.FALSE.
                   ELSE
                     decent(iv)=.TRUE.
                   END IF
                 END IF
               END DO

             END IF       ! ends test on velocity sign


   ! advection flux of other substances
   ! ----------------------------------

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho

  ! advection flux alone
  ! --------------------
               IF (decent(iv)) THEN
                 fy(iv,k,i,j)=fyeau(k,i,j)/REAL(niter,rsh)*yb(iv)*dx_v(i,j)
               ELSE
                 yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
                 fy(iv,k,i,j)=fyeau(k,i,j)/REAL(niter,rsh)*yr*dx_v(i,j)
               END IF

#else

  ! diffusion/advection flux
  ! ------------------------
               IF (decent(iv)) THEN
#if defined key_siggen || defined key_gencoord
                 fy(iv,k,i,j)=(fyeau(k,i,j)*yb(iv)-hzey(k,i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#else
                 fy(iv,k,i,j)=(fyeau(k,i,j)*yb(iv)-hey(i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#endif
               ELSE
                 yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
#if defined key_siggen || defined key_gencoord
                 fy(iv,k,i,j)=(fyeau(k,i,j)*yr-hzey(k,i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#else
                 fy(iv,k,i,j)=(fyeau(k,i,j)*yr-hey(i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#endif
               END IF

#endif

#ifdef key_sedim
               IF (k.eq.1) fy(iv,k,i,j)=fy(iv,k,i,j)*corfluy(iv,i,j)
#endif
             END DO

           END IF    ! ends test on water depth
         END DO  ! ends loop on k
       END IF  ! for selecting right j loop
     END DO  ! ends loop on i
   END DO  ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fy)

  END SUBROUTINE parsub_advdify_quickupwind

#endif
