#ifdef key_substance

  SUBROUTINE parsub_advdify_uqm(c,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE parsub_advdify_ultimatequickestmacho  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of substances with settling velocity
   !&E              (advection and diffusion)
   !&E              Advection uses QUICKEST scheme with ULTIMATE limiter and
   !&E              its MACHO multidimensional extension.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y: correct for eastward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hex,hx,hey,hy,fxeau,fyeau,c(cv_wat),hzey,dx,dy_v
   !&E                     fy
   !&E
   !&E ** Modified variables : fy
   !&E
   !&E ** Reference : Leonard B. P., Lock A. P. and M. K. MacVean (1996) :
   !&E                Conservative Explicit Unrestricted-Time-Step
   !&E                Multidimensional COnstancy-Preserving Advections Schemes.
   !&E                Monthly Weather Review, 124, 2588:2605
   !&E
   !&E ** History :
   !&E           !  2010-03 (P. Douillet) substances with settling velocities
   !&E           !                         issued from tssub_advdify_uqm
   !&E           !  2011-02  (B. Thouvenin) correction diffusion term of substances
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : jb,jh,hey,hy,h0fond,ky,hminkxky,dy_v,dx,hex,    &
                            ig,id,hx,halfdt,l_obc_cycl_y
#if defined key_siggen || defined key_gencoord
   USE comsiggen,    ONLY : hzex,hzey
#endif
   USE comfluxdo,    ONLY : fyeau,fxeau
   USE comsubstance, ONLY : nv_adv,fy,nvp,iv_ws_max,nv_ws_max,c2
   USE_MPI toolmpi,  ONLY : ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   INTEGER,                                                              INTENT(in) :: niter,num_ws
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh), PARAMETER :: dsmall = 1.0d-20
   REAL(KIND=rsh)            :: vv,dyds,dygs,dydt,d0,d1,vvv,courant,uu,  &
                                dygt,kyb,thetap,psip
   REAL(KIND=rlg), DIMENSION(kmax) :: ful,fur
   REAL(KIND=rsh), DIMENSION(kmax) :: sall,salr,templ,tempr
   INTEGER                              :: l,iv
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: dyd,dyg,ya,yb,yc,delta
   REAL(KIND=rsh), DIMENSION(1:nvp,kmax) :: cl,cr

   !!----------------------------------------------------------------------
   !! * Executable part

   ! estimate of intermediate tracers (from upwind scheme)
   ! -----------------------------------------------------

   ! at open boundaries (north and south)

!$OMP SINGLE

   DO l=1,nv_ws_max(num_ws)
   iv=iv_ws_max(num_ws,l)
   IF_MPI (ljmin==jmin) THEN
     c2(iv,:,limin:limax,jmin:jmin+1) = c(iv,:,limin:limax,jmin:jmin+1)
   ENDIF_MPI
   IF_MPI (ljmax==jmax) THEN
     c2(iv,:,limin:limax,jmax) = c(iv,:,limin:limax,jmax)
   ENDIF_MPI
   END DO

!$OMP END SINGLE

   ! inside the domain

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,l,iv, &
!$OMP& cr,fur,cl,ful,uu)

   DO j=ljmin,ljmax
#ifdef key_MPI_2D
     DO i=liminp1m1,limax
     IF (i==ig(j) .OR. i==liminp1m1) THEN
#else
     i=ig(j)
#endif

       DO k=1,kmax
         IF (hex(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
          DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           cr(iv,k)=0.0_rsh
          END DO
           fur(k)=0.0_rlg
         ELSE

           fur(k)=fxeau(k,i,j)   ! h*uz*dt/2

           DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           ! upwind scheme
           IF (fur(k)<= 0.0_rlg) THEN
             cr(iv,k)=c(iv,k,i+1,j)
           ELSE
             cr(iv,k)=c(iv,k,i,j)
           END IF   ! ends test on the direction of propagation
           END DO

         END IF   ! ends test on the water depth
       END DO   ! ends loop on k

#ifdef key_MPI_2D
     ELSE IF(i >= ig(j)+1 .AND. i <=id(j)-1 .AND. i>liminp1m1) THEN
#else
     DO i=ig(j)+1,id(j)-1
#endif

       DO k=1,kmax

         ! left of the stencil
         DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           cl(iv,k)=cr(iv,k)
         END DO
         ful(k)=fur(k)

         ! right of the stencil
         IF (hex(i,j) <= 0.0_rsh .OR. hx(i,j) == -valmanq-h0fond) THEN
           DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             cr(iv,k)=0.0_rsh
           END DO
           fur(k)=0.0_rlg
         ELSE

           fur(k)=fxeau(k,i,j)   ! h*uz*dt/2

           DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)

           ! upwind scheme
           IF (fur(k)<= 0.0_rlg) THEN
             cr(iv,k)=c(iv,k,i+1,j)
           ELSE
             cr(iv,k)=c(iv,k,i,j)
           END IF   ! ends test of the direction of propagation
           END DO

         END IF   ! ends test on the water depth

     ! estimate of intermediate tracers (from upwind scheme)

         IF ( (ful(k)>0.0_rlg) .AND. (fur(k)>0.0_rlg) ) THEN
#if defined key_siggen || defined key_gencoord
           uu = ful(k)/(halfdt*hzex(k,i-1,j))
#else
           uu = ful(k)/(halfdt*hex(i-1,j))
#endif
         ELSE IF ( (ful(k)<0.0_rlg) .AND. (fur(k)<0.0_rlg) ) THEN
#if defined key_siggen || defined key_gencoord
           uu = ful(k)/(halfdt*hzex(k,i,j))
#else
           uu = fur(k)/(halfdt*hex(i,j))
#endif
         ELSE
           ! no time evolution of the intermediate temperature
           ! along the boundaries
           uu = 0.0_rsh
         END IF

         ! apply euler (time) - upwind (x) scheme
         DO l=1,nv_ws_max(num_ws)
           iv=iv_ws_max(num_ws,l)
           c2(iv,k,i,j)  = c(iv,k,i,j)  - (halfdt/REAL(niter,rsh)/dx(i,j))*uu*(cr(iv,k)-cl(iv,k))  
         END DO

       END DO   ! ends loop on k
#ifdef key_MPI_2D
     END IF
#endif
     END DO   ! ends loop on i
   END DO   ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,2,nv_adv*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,c2(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(c2)

   ! estimate of fluxes (from ultimate quickest scheme)
   ! --------------------------------------------------

!$OMP DO SCHEDULE(RUNTIME),PRIVATE(i,j,k,kyb,l, &
!$OMP& iv,dyd,dyg,ya,yb,yc,delta,               &
!$OMP& d0,d1,vv,vvv,courant,thetap,psip)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hey(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
            DO l=1,nv_ws_max(num_ws)
             iv=iv_ws_max(num_ws,l)
             fy(iv,k,i,j)=0.0_rsh
            END DO
           ELSE
             kyb=ky
             IF (hey(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
#if defined key_siggen || defined key_gencoord
             vvv = vv/(halfdt*hzey(k,i,j))
#else
             vvv = vv/(halfdt*hey(i,j))
#endif
             courant = ABS(vvv)*halfdt/dy_v(i,j)/REAL(niter,rsh)
             d0=(1.0_rsh/6.0_rsh)*(2.0_rsh-courant)*(1.0_rsh-courant)
             d1=(1.0_rsh/6.0_rsh)*(1.0_rsh-courant)*(1.0_rsh+courant)
                           
             IF (vv <= 0.0_rsh) THEN  ! southward advection
              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c2(iv,k,i,j+1)
               yc(iv)=c2(iv,k,i,j)

               IF (hey(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
                 ya(iv)=c2(iv,k,i,j+2)
               ELSE
                 ya(iv)=yb(iv)
               END IF

               dyg(iv)=yb(iv)-ya(iv)
               dyd(iv)=yc(iv)-yb(iv)
               delta(iv)=yb(iv)-yc(iv)
              END DO

             ELSE      ! northward advection

              DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               yb(iv)=c2(iv,k,i,j)
               yc(iv)=c2(iv,k,i,j+1)
               IF (hey(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
                 ya(iv)=c2(iv,k,i,j-1)
               ELSE
                 ya(iv)=yb(iv)
               END IF

               dyg(iv)=yb(iv)-ya(iv)
               dyd(iv)=yc(iv)-yb(iv)
               delta(iv)=yc(iv)-yb(iv)
               END DO

             END IF       ! ends test on velocity sign


   ! advection flux of other substances
   ! ----------------------------------

             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)
               IF ((ABS(dyd(iv)).LE.dsmall).OR.(ABS(dyg(iv)).LE.dsmall)) THEN
                 ! no horizontal gradient
                 thetap = 0.0_rsh
                 psip = 0.0_rsh
               ELSE
                 ! ultimate limiter
                 thetap = (dyg(iv)+dsmall)/(dyd(iv)+dsmall)
                 psip = d0+d1*thetap
                 psip=MAX(0.0_rsh,MIN(MIN(1.0_rsh,psip),((1.0_rsh-courant)/(courant+dsmall))*thetap))
               END IF
               fy(iv,k,i,j) = vv*(yb(iv)+psip*dyd(iv))
             END DO


             DO l=1,nv_ws_max(num_ws)
               iv=iv_ws_max(num_ws,l)

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   ! Advection term (iso-sigma) alone
   ! --------------------------------
               fy(iv,k,i,j) = fy(iv,k,i,j)/REAL(niter,rsh)*dx_v(i,j)

#else
   ! Advection/diffusion term (iso-sigma)
   ! ------------------------------------
#if defined key_siggen || defined key_gencoord
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hzey(k,i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#else
               fy(iv,k,i,j) = (fy(iv,k,i,j) - hey(i,j)*kyb*delta(iv)/dy_v(i,j)*halfdt)/REAL(niter,rsh)
#endif

#endif
             END DO

           END IF ! ends test on water depth
         END DO ! ends loop on k
       END IF ! for selecting right j loop
     END DO ! ends loop on i
   END DO ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(fy)

  END SUBROUTINE parsub_advdify_uqm

#endif
