#ifdef key_substance

  SUBROUTINE parsub_eq_hm(xe2,xe3,cv_wat,ll,niter,num_ws)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tssub_eq_hm  ***
   !&E
   !&E ** Purpose : compute the transport of tracers with fall velocity, 
   !&E              for meshes with low water heights (< hm).
   !&E
   !&E ** Description : WARNING : The resulting vertical profiles are homogenous.
   !&E
   !&E ** Called by : adv
   !&E
   !&E ** External calls :                  :
   !&E
   !&E ** Used ij-arrays : h0,xe3,xe2,dy_u,dx_v,
   !&E                     surf,flx_s2w,flx_w2s,flx_w2s_sum
   !&E                     cv_wat,fx,fy,phicon,dcdt
   !&E
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-03  (P. Douillet) substances with settling velocities
   !&E       !                         issued from tssub_eq_hm
   !&E       !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : ig,id,h0,h0fond,hm,dy_u,dx_v,surf,dt,halfdt,fwet,fwetp
   USE comvars3d,    ONLY : dsigu
#if defined key_siggen || defined key_gencoord
   USE toolgeom,     ONLY : f_dzu
#endif
   USE comsubstance, ONLY : fx,fy,nvp,nv_adv,phicon,f90,nv_ws_max,iv_ws_max
#if defined key_sedim
   USE comsubstance, ONLY : flx_s2w,flx_w2s,flx_w2s_sum
#endif
#if defined key_contaminant || defined key_biolo || defined key_eco3m
   USE comsubstance, ONLY : dcdt
#endif

   IMPLICIT NONE

   !! * Arguments
   INTEGER, INTENT(in) :: ll,niter,num_ws
   REAL(KIND=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                                         :: i,iv,j,l1,k
   REAL(KIND=rsh)                                  :: cnouv,d3c3,d3,d2,xei
   REAL(KIND=rsh)                                  :: fxmg,fxmd,fymb,fymh,cm,phi,dcdthm
   REAL(KIND=rlg)                                  :: dtm
   REAL(KIND=rsh),DIMENSION(kmax)                  :: dz
   

   !!----------------------------------------------------------------------
   !! * Executable part

   dtm=0.5_rlg*dt/REAL(niter,rlg)

#ifdef key_MPI_2D
!$OMP DO schedule(runtime) private(i,l1,j,k,cnouv,d3c3,d3,d2 &
!$OMP& ,fxmg,fxmd,fymb,fymh,cm,phi,dcdthm,dz,xei)
#else
!$OMP DO schedule(dynamic,3)
#endif
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       d3=h0(i,j)+REAL(ll  ,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh) +xe2(i,j)
#if ! defined key_siggen && ! defined key_gencoord
       d2=h0(i,j)+REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh) +xe2(i,j)
#endif
       IF(d3 > h0fond.AND.d3 <= hm) THEN

#if defined key_siggen || defined key_gencoord
         xei=REAL(ll-1,rsh)*(xe3(i,j)-xe2(i,j))/REAL(niter,rsh)+xe2(i,j)
         DO k=1,kmax
           dz(k)=f_dzu(h0(i,j),xei,k,i,j)   ! thickness of the layer [m]
         END DO
#else
         dz(:)=dsigu(:)   !  no unit
#endif

   !-----------------------------------------------
   ! integration des flux de traceurs particulaires
   !-----------------------------------------------

         DO l1=1,nv_ws_max(num_ws) ! number of variables in the group of settling velocity
           iv=iv_ws_max(num_ws,l1)  ! initialize index of the sustance
                                   ! according to the number of settling velocity group
                                   ! and the number of variables in this group

           cm=SUM(cv_wat(iv,:,i,j)*dz(:))
           fxmg=SUM(fx(iv,:,i-1,j)*dsigu(:))
           fxmd=SUM(fx(iv,:,i,j)*dsigu(:))
           fymb=SUM(fy(iv,:,i,j-1)*dsigu(:))
           fymh=SUM(fy(iv,:,i,j)*dsigu(:))
           phi =SUM(phicon(iv,:,i,j))/REAL(niter,rsh)
#if defined key_biolo || defined key_contaminant || defined key_eco3m
#if defined key_siggen || defined key_gencoord
           dcdthm=SUM(dcdt(iv,:,i,j)*dz(:))
#else
           dcdthm=d2*SUM(dcdt(iv,:,i,j)*dsigu(:))
#endif
#endif
#if defined key_siggen || defined key_gencoord
           d3c3=(cm*fwetp(i,j)-(fxmd*dy_u(i,j)-fxmg*dy_u(i-1,j)+fymh*dx_v(i,j)-fymb*dx_v(i,j-1)-phi)/surf(i,j))/fwet(i,j)
#else
           d3c3=(d2*cm*fwetp(i,j)-(fxmd*dy_u(i,j)-fxmg*dy_u(i-1,j)+fymh*dx_v(i,j)-fymb*dx_v(i,j-1)-phi)/surf(i,j))/fwet(i,j)
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m
           d3c3=d3c3+dtm*dcdthm
#endif
#if defined key_sedim
           d3c3=d3c3+dtm*flx_s2w(iv,i,j)
           cnouv=d3c3/(d3+dtm*flx_w2s(iv,i,j))
#else
           cnouv=d3c3/d3
#endif
           IF(cnouv < 1.e-10) cnouv=0.0_rsh
#if defined key_sedim
           flx_w2s_sum(iv,i,j)=flx_w2s_sum(iv,i,j)+flx_w2s(iv,i,j)*cnouv*dtm
#endif
           cv_wat(iv,:,i,j)=f90(iv)*(cnouv)
         ENDDO

       ENDIF ! test sur la hauteur d eau
     ENDDO ! fin boucle en i
   ENDDO ! fin boucle en j

!$OMP end DO

  END SUBROUTINE parsub_eq_hm

#endif
