 MODULE subobc3dapply

#ifdef key_substance

   !!======================================================================
   !!                   ***  MODULE  subobc3dapply  ***
   !! Ocean dynamics:  apply open boundary conditions of variables
   !!                  (tracers other than temperature or salinity)
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC sub_obc3d_apply,           &  ! routine called by stepu.F90 stepv.F90
          sub_obc3d_apply_agrif               ! routine called by stepu.F90 stepv.F90

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE sub_obc3d_apply(u,v,cv_wat)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sub_obc3d_apply  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : cv_wat,u,obc_cv_wat_w,hx,dx_u,obc_cv_wat_e,dy_v,
   !&E                     hy,obc_cv_wat_s,v,obc_cv_wat_n
   !&E 
   !&E ** Modified variables : cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-06
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04 (A. Mano) MPI2D
   !&E       !  2010-04 (T. Odaka) MPI2D cyclic
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : dt,ig,hx,h0fond,dx_u,id,jb,hy,dy_v,jh,          &
                            l_obc_cycl_x,l_obc_cycl_y
   USE comsubstance, ONLY : obc_cv_wat_w,nvp,nv_adv,obc_cv_wat_e,           &
                            obc_cv_wat_s,obc_cv_wat_n
   USE obcogcm,      ONLY : obc_coefrel
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u,v
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat

   !! * Local declarations
   INTEGER                   :: i,j,k
   REAL(KIND=rsh)            :: uu,vv,cou,dtrelax
   REAL(KIND=rlg)            :: dtm

   !!----------------------------------------------------------------------
   !! * Executable part

   dtm=0.5_rlg*dt
   dtrelax=dtm*obc_coefrel  ! dt / time scale of relaxation
   IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh

!$OMP SINGLE
   ! periodic conditions along x-axis
   IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
     CALL_MPI ex_peri_x_rsh(1,(nv_adv-nvp)*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(nvp+1:nv_adv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
     DO j=ljmin,ljmax
   ! western boundary
       cv_wat(nvp+1:nv_adv,:,imin+1,j)=cv_wat(nvp+1:nv_adv,:,imax-1,j)
   ! eastern boundary
       cv_wat(nvp+1:nv_adv,:,imax  ,j)=cv_wat(nvp+1:nv_adv,:,imin+2,j)
     END DO
#endif
   END IF

   ! periodic conditions along y-axis
   IF (l_obc_cycl_y) THEN
#ifdef key_MPI_2D
     CALL_MPI ex_peri_y_rsh(1,(nv_adv-nvp)*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(nvp+1:nv_adv,:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
   ! southern boundary
     cv_wat(nvp+1:nv_adv,:,limin:limax,jmin+1)=cv_wat(nvp+1:nv_adv,:,limin:limax,jmax-1)
   ! northern boundary
     cv_wat(nvp+1:nv_adv,:,limin:limax,jmax  )=cv_wat(nvp+1:nv_adv,:,limin:limax,jmin+2)
#endif
   END IF
!$OMP END SINGLE

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites gauche et droite
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_x) THEN
     DO j=ljminp2,ljmaxm1

   ! western boundary
       i=ig(j)
       IF_MPI (limin==i-1) THEN
       IF (hx(i,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i,j)
           IF (uu > 0.0_rsh) THEN
             cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-dtrelax)+obc_cv_wat_w(nvp+1:nv_adv,k,1,j)*dtrelax
           ELSE
             cou=uu*dtm/dx_u(i,j)
             cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(nvp+1:nv_adv,k,i+1,j)
           END IF
         END DO
       END IF
       ENDIF_MPI

   ! eastern boundary
       i=id(j)
       IF_MPI (limax==i) THEN
       IF (hx(i-1,j) /= -valmanq-h0fond) THEN
         DO k=1,kmax
           uu=u(k,i-1,j)
           IF (uu < 0.0_rsh) THEN
             cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-dtrelax)+obc_cv_wat_e(nvp+1:nv_adv,k,imax,j)*dtrelax
           ELSE
             cou=uu*dtm/dx_u(i-1,j)
             cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(nvp+1:nv_adv,k,i-1,j)
           END IF
         END DO
       END IF
       ENDIF_MPI

     END DO
   END IF

   !----------------------------------------------
   ! on traite l advection quand le courant sort
   ! sur les limites basse et haute
   !----------------------------------------------

   IF (.NOT.l_obc_cycl_y) THEN
     DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

   ! southern boundary
       IF (j == jb(i)) THEN
         IF (hy(i,j) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j)
             IF (vv > 0.0_rsh) THEN
               cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-dtrelax)+obc_cv_wat_s(nvp+1:nv_adv,k,i,1)*dtrelax
             ELSE
               cou=vv*dtm/dy_v(i,j)
               cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh+cou)-cou*cv_wat(nvp+1:nv_adv,k,i,j+1)
             END IF
           END DO
         END IF
       END IF

   ! northern boundary
       IF (j == jh(i)) THEN
         IF (hy(i,j-1) /= -valmanq-h0fond) THEN
           DO k=1,kmax
             vv=v(k,i,j-1)
             IF (vv < 0.0_rsh) THEN
               cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-dtrelax)+obc_cv_wat_n(nvp+1:nv_adv,k,i,jmax)*dtrelax
             ELSE
               cou=vv*dtm/dy_v(i,j-1)
               cv_wat(nvp+1:nv_adv,k,i,j)=cv_wat(nvp+1:nv_adv,k,i,j)*(1.0_rsh-cou)+cou*cv_wat(nvp+1:nv_adv,k,i,j-1)
             END IF
           END DO
         END IF
       END IF

     END DO
     END DO
   END IF

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(cv_wat)

  END SUBROUTINE sub_obc3d_apply

   !!======================================================================

  SUBROUTINE sub_obc3d_apply_agrif

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sub_obc3d_apply_agrif  ***
   !&E
   !&E ** Purpose : open boundary conditions for tracers other than temperature and salinity
   !&E              For incoming conditions, used values are stored in the 
   !&E              obc_cv_wat_s, obc_cv_wat_n, obc_cv_wat_w and obc_cv_wat_e arrays,
   !&E              filled in the "subreaddat" routine.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :
   !&E    
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2005-10
   !&E       !  2006-11 (V. Garnier) module - mise en forme
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#ifdef key_agrif
   USE parameters
   USE comvarp2d
   USE toolmpi
   USE agrifcommonid
   USE comsubstance, ONLY : nvp,nv_adv

   !! * Arguments

   !! * Local declarations
   INTEGER  :: l
   REAL(kind=rsh), DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: sub_temp
   EXTERNAL :: interp3dsub

   !!----------------------------------------------------------------------
   !! * Executable part

   CALL Agrif_Bc_Variable(sub_temp,cv_wat_id,procname=interp3dsub)
   IF_MPI(limin==imin) THEN
    do l=nvp+1,nv_adv
     cv_wat(l,:,imin+1,ljminm1:ljmaxp2)=sub_temp(l,:,imin+1,ljminm1:ljmaxp2)
    end do
   ENDIF_MPI
   IF_MPI(limax==imax) THEN
    do l=nvp+1,nv_adv
     cv_wat(l,:,imax,ljminm1:ljmaxp2)=sub_temp(l,:,imax,ljminm1:ljmaxp2)
    end do
   ENDIF_MPI
   IF_MPI(ljmax==jmax) THEN
    do l=nvp+1,nv_adv
     cv_wat(l,:,liminm1:limaxp2,jmax  )=sub_temp(l,:,liminm1:limaxp2,jmax  )
    end do
   ENDIF_MPI
   IF_MPI(ljmin==jmin) THEN
    do l=nvp+1,nv_adv
     cv_wat(l,:,liminm1:limaxp2,jmin+1)=sub_temp(l,:,liminm1:limaxp2,jmin+1)
    end do
   ENDIF_MPI
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,nb_var*kmax,liminm1,limaxp2,ljminm1,ljmaxp2,cv_wat(:,:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(cv_wat)


#endif
  END SUBROUTINE sub_obc3d_apply_agrif
#endif

 END MODULE subobc3dapply

   !!======================================================================
#ifdef key_substance
#ifdef key_agrif
      subroutine interp3dsub(tabres,l1,l2,k1,k2,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
USE comsubstance
implicit none

       integer i1,i2,j1,j2,l1,l2,k1,k2
       real tabres(l1:l2,k1:k2,i1:i2,j1:j2)
       integer i,j,l,k

       do l=l1,l2
        do j=j1,j2
         do i=i1,i2
          IF (h0(i,j)+ssh(i,j) < 2.0_rsh*h0fond) THEN
           do k=k1,k2
             tabres(l,k,i,j) = Agrif_SpecialValue
           enddo
          ELSE
           do k=k1,k2
             tabres(l,k,i,j) =cv_wat(l,k,i,j)
           enddo
          ENDIF
         enddo
        enddo
       enddo
      return
      end

#endif
#endif


