  SUBROUTINE ts_stateeq_blumberg(sal,temp,boz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_stateeq_blumberg  ***
   !&E
   !&E ** Purpose : buoyancy estimation from a unlinear state equation
   !&E
   !&E ** Description :
   !&E                  TP potential temperature in DEG C
   !&E                  = in situ temperature for small depths
   !&E                  S salinity in o/oo
   !&E                  p pressure in decibars = depth in meters
   !&E
   !&E ** Called by : main, stepu, stepv
   !&E
   !&E ** Used ij-arrays : h0,temp,sal
   !&E 
   !&E ** Modified variables : boz
   !&E
   !&E ** Reference : Mellor, 609-611 JAOT 1991
   !&E
   !&E ** History :
   !&E       !  2000-01  (P. Garreau) Original code
   !&E       !  2007-10  (V. Garnier) Cleaning
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2010-12  (P. Garreau) Minimum of S and T to avoid underflow
   !&E                   Later, we should take care of the validity range of this state equation
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : h0
   USE comvars3d, ONLY : sig,rhoref
   !USE toolgeom, ONLY : f_zu
   USE_MPI toolmpi,   ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(out) :: boz

   !! * Local declarations
   INTEGER                  :: k,i,j
   REAL(KIND=rsh)           :: TP,S,p,roref
   REAL(KIND=rlg)           :: row,rost,c,pcc,rostp

   !!----------------------------------------------------------------------
   !! * Executable part

   roref=1040.0_rsh

!$OMP PARALLEL DO PRIVATE(TP,S,P,row,rost,rostp,c,pcc,k,i,j) &
!$OMP& SCHEDULE(RUNTIME)
   DO j=ljminp1,ljmax
    DO i=liminp1,limax

     IF (h0(i,j) > -valmanq) THEN

       DO k=1,kmax

         TP = MAX(temp(k,i,j),0.001_rsh)
         S = MAX(sal(k,i,j),0.0001_rsh)
!         P = -f_zu(h0(i,j),0.0_rsh,k,i,j)
         P = 10.0_rsh

   ! density of pure water

         row = 999.842594_rlg + (  6.793952d-02 + ( - 9.095290d-03 +                &
               (1.001685d-04 - 1.120083d-06*TP + 6.536332d-09*TP*TP)*TP )*TP  )*TP

   ! density of sea water at the surface

         rost = row + S * (   0.824493_rlg + (  -4.0899d-03 + ( 7.6438d-05 +        &
                          (-8.2467d-07+5.3875d-09*TP)*TP )*TP  )*TP   ) +           &
                S**1.5_rlg * ( -5.72466d-03 + (1.0227d-04-1.6546d-06*TP)*TP ) +     &
                S*S*4.8314d-04

   ! density of sea water at pressure p (decibars)

         c = 1449.2_rlg + 1.34_rlg*(S-35.0_rlg) + 4.55_rlg*TP - 0.045_rlg*TP*TP +   &
             0.00821_rlg*p + 15.0d-09*p*p
         pcc = p/(c*c)
         rostp = rost + 1.0d4*(1.0_rlg-0.2_rlg*pcc)*pcc

   ! buoyancy

         boz(k,i,j) = grav*( roref - REAL(rostp,rsh) )/roref

       END DO      ! fin de la boucle en k

     END IF       ! fin du test sur le masque terre 

    END DO        ! fin de la boucle en i
   END DO         ! fin de la boucle en j 

!$OMP END PARALLEL DO

   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))

   PRINT_DBG*, 'exit TS_STATEEQ_BLUMBERG routine'

  END SUBROUTINE ts_stateeq_blumberg
