  SUBROUTINE ts_stateeq_blumberg(sal,temp,boz,xec)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_stateeq_blumberg  ***
   !&E
   !&E ** Purpose : buoyancy estimation from a unlinear state equation
   !&E
   !&E ** Description :
   !&E                  TP potential temperature in DEG C
   !&E                  = in situ temperature for small depths
   !&E                  S salinity in o/oo
   !&E                  p pressure in decibars = depth in meters
   !&E
   !&E ** Called by : main, stepu, stepv
   !&E
   !&E ** Used ij-arrays : h0,temp,sal
   !&E 
   !&E ** Modified variables : boz
   !&E
   !&E ** Reference : Mellor, 609-611 JAOT 1991
   !&E
   !&E ** History :
   !&E       !  2000-01  (P. Garreau) Original code
   !&E       !  2007-10  (V. Garnier) Cleaning
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2010-12  (P. Garreau) Minimum of S and T to avoid underflow
   !&E                   Later, we should take care of the validity range of this state equation
   !&E       !  2014-10  (F. Vandermeirsch) formulation equation  etat du code ROMS  
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d, ONLY : h0
! S.P 09/2013 utilisation du profil de densite moyenne
   USE comvars3d, ONLY : rhoref,qp1,boz2,rho,rho2
   USE comcolig3d, ONLY : zr
   USE toolgeom, ONLY : f_zu
   USE_MPI toolmpi,   ONLY : ex_i_rsh,ex_j_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in)  :: sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(out) :: boz
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: xec

   !! * Local declarations
   INTEGER                  :: k,i,j
   REAL(KIND=rsh)           :: TP,S,p,roref
   REAL(KIND=rlg)           :: row,rost,c,pcc,rostp,rho1,rho0
!!----------------------------------------------------------------------   
      real(KIND=rlg)  K0, K1,K2,				      &
     	  r00,r01,r02,r03,r04,r05,    K00,K01,K02,K03,K04,  dr00,     &
     	  r10,r11,r12,r13,r14,        K10,K11,K12,K13,      Ts, Tt,   &
     	  rS0,rS1,rS2,  	      KS0,KS1,KS2,	    sqrtTs,   &
     	  r20,  					    dpth,     &
     		  B00,B01,B02,B03,    E00,E01,E02,		      &
     		  B10,B11,B12,        E10,E11,E12,	    cff,      &
     		      BS1,				    cff1      

      parameter(r00=999.842594,  r01=6.793952E-2,  r02=-9.095290E-3,  &
     				 r03=1.001685E-4,  r04=-1.120083E-6,  &
     						   r05=6.536332E-9,   &
     	       r10=0.824493,	 r11=-4.08990E-3,  r12=7.64380E-5,    &
     				 r13=-8.24670E-7,  r14=5.38750E-9,    &
     	       rS0=-5.72466E-3,  rS1=1.02270E-4,   rS2=-1.65460E-6,   &
     	       r20=4.8314E-4,					      &	       
     	       K00=19092.56,	 K01=209.8925,     K02=-3.041638,     &
     				 K03=-1.852732e-3, K04=-1.361629e-5,  &
     	       K10=104.4077,	 K11=-6.500517,    K12=0.1553190,     &
     						   K13=2.326469e-4,   &
     	       KS0=-5.587545,	 KS1=+0.7390729,   KS2=-1.909078e-2,  &
     	       B00=0.4721788,	 B01=0.01028859,   B02=-2.512549e-4,  &
     						   B03=-5.939910e-7,  &
     	       B10=-0.01571896,  B11=-2.598241e-4, B12=7.267926e-6,   &
     				 BS1=2.042967e-3,		      &
     	       E00=+1.045941e-5, E01=-5.782165e-10,E02=+1.296821e-7,  &
     	       E10=-2.595994e-7, E11=-1.248266e-9, E12=-3.508914e-9)
   !!----------------------------------------------------------------------
   !! * Executable part

!   roref=1040.0_rsh
   rho0 = rhoref
   
#  ifdef key_TOTO1

!$OMP PARALLEL DO PRIVATE(TP,S,P,row,rost,rostp,c,pcc,k,i,j) &
!$OMP& SCHEDULE(RUNTIME)

   DO j=ljminp1,ljmax
    DO i=liminp1,limax
     IF (h0(i,j) > -valmanq) THEN
       DO k=1,kmax

         TP = MAX(temp(k,i,j),0.001_rsh)
         S = MAX(sal(k,i,j),0.0001_rsh)
!         P = -f_zu(h0(i,j),0.0_rsh,k,i,j)
         P = 10.0_rsh

   ! density of pure water

         row = 999.842594_rlg + (  6.793952d-02 + ( - 9.095290d-03 +                &
               (1.001685d-04 - 1.120083d-06*TP + 6.536332d-09*TP*TP)*TP )*TP  )*TP

   ! density of sea water at the surface

         rost = row + S * (   0.824493_rlg + (  -4.0899d-03 + ( 7.6438d-05 +        &
                          (-8.2467d-07+5.3875d-09*TP)*TP )*TP  )*TP   ) +           &
                S**1.5_rlg * ( -5.72466d-03 + (1.0227d-04-1.6546d-06*TP)*TP ) +     &
                S*S*4.8314d-04

   ! density of sea water at pressure p (decibars)

         c = 1449.2_rlg + 1.34_rlg*(S-35.0_rlg) + 4.55_rlg*TP - 0.045_rlg*TP*TP +   &
             0.00821_rlg*p + 15.0d-09*p*p
         pcc = p/(c*c)
         rostp = rost + 1.0d4*(1.0_rlg-0.2_rlg*pcc)*pcc

   ! buoyancy

         boz(k,i,j)  = grav*( rho0 - REAL(rostp,rsh) )/rho0
         rho(k,i,j)  = REAL(rostp,rsh) - rho0
!         boz2(k,i,j) = grav*( rho0 - REAL(rostp,rsh) )/rho0 
	 boz2(k,i,j) = grav*( rho2(k,i,j) - REAL(rostp,rsh) )/rho0 ! key_dyn_pg_pint
       END DO      ! fin de la boucle en k

     END IF       ! fin du test sur le masque terre 

    END DO        ! fin de la boucle en i
   END DO         ! fin de la boucle en j 
!----------------------------------------------------------------------------------------------------     
#else  
      dr00=r00-rho0
!
!$OMP PARALLEL DO PRIVATE(Tt,Ts,sqrtTs,rho1,K0,K1,K2,dpth,cff,cff1,rostp,k,i,j,TP,S,P,row,rost,c,pcc) &
!$OMP& SCHEDULE(RUNTIME)
!    
!     DO i=liminp1,limax    
    DO j=ljmin,ljmax
     DO i=limin,limax
     IF (h0(i,j) .NE. -valmanq) THEN    
         DO k =1,kmax
           zr(k,i,j) = f_zu(h0(i,j),xec(i,j),k,i,j)	 
!           zr(k,i,j) = f_zu(h0(i,j),0.0_rsh,k,i,j)	 
           Tt =temp(k,i,j)
           Ts = MAX(sal(k,i,j),0.000_rsh)	 
           sqrtTs=sqrt(Ts)
           rho1=( dr00 +Tt*( r01+Tt*( r02+Tt*( r03+Tt*(          &   
     						 r04+Tt*r05 )))) &
     			      +Ts*( r10+Tt*( r11+Tt*( r12+Tt*(   &
     						 r13+Tt*r14 )))  &
     				   +sqrtTs*(rS0+Tt*(		 &
     					 rS1+Tt*rS2 ))+Ts*r20 ))    

            K0= Tt*( K01+Tt*( K02+Tt*( K03+Tt*K04 )))  &
     	       +Ts*( K10+Tt*( K11+Tt*( K12+Tt*K13 ))   &
     		    +sqrtTs*( KS0+Tt*( KS1+Tt*KS2 )))

            K1=B00+Tt*(B01+Tt*(B02+Tt*B03)) +Ts*( B10+Tt*( B11    &
                                          +Tt*B12 )+sqrtTs*BS1 )

            K2=E00+Tt*(E01+Tt*E02) +Ts*(E10+Tt*(E11+Tt*E12))

#ifdef SPLIT_EOS
            boz2(k,i,j) =rho1
            qp1(k,i,j)=0.1*(K00*rho1-rho0*K0)/(K00*(K00+K0))
            rostp=rho1+qp1(k,i,j)*(xec(i,j)-zr(k,i,j))
#  else      
            dpth=xec(i,j)-zr(k,i,j)
!             dpth=-zr(k,i,j)
            cff=K00-0.1*dpth
            cff1=K0+dpth*(K1+K2*dpth)
            rostp=( rho1*cff*(K00+cff1)              &
                                 -0.1*dpth*rho0*cff1 &
                                  )/(cff*(cff+cff1))      
#  endif /* SPLIT_EOS */ 
            boz2(k,i,j) =rho1

#ifdef key_dyn_pg_djcs
! pour le gradient de pression type key_dyn_pg_djcs
        
 	 rho(k,i,j)  = REAL(rostp,rsh)                 ! SPLIT_EOS
#  else      
! rostp est deja une anomalie : ne pas lui retirer rho0
! pour le gradient de pression type key_dyn_pg_pint 

!         incompatible : il faut crer au-dessus d'autre variable  rho1(k,i,j)
!         boz2(k,i,j) = grav*(- REAL(rostp,rsh) )/rho0 ! key_dyn_pg_pint
#   endif  
!          boz(k,i,j)  = grav*(- REAL(rostp,rsh) )/rho0  ! NO SPLIT_EOS + turbulence verticale
!---------------------------------------------------------------------------------------------------
         TP = MAX(temp(k,i,j),0.001_rsh)
         S = MAX(sal(k,i,j),0.0001_rsh)
         P = 10.0_rsh
   ! density of pure water
         row = 999.842594_rlg + (  6.793952d-02 + ( - 9.095290d-03 +                &
               (1.001685d-04 - 1.120083d-06*TP + 6.536332d-09*TP*TP)*TP )*TP  )*TP
   ! density of sea water at the surface
         rost = row + S * (   0.824493_rlg + (  -4.0899d-03 + ( 7.6438d-05 +        &
                          (-8.2467d-07+5.3875d-09*TP)*TP )*TP  )*TP   ) +           &
                S**1.5_rlg * ( -5.72466d-03 + (1.0227d-04-1.6546d-06*TP)*TP ) +     &
                S*S*4.8314d-04
   ! density of sea water at pressure p (decibars)
         c = 1449.2_rlg + 1.34_rlg*(S-35.0_rlg) + 4.55_rlg*TP - 0.045_rlg*TP*TP +   &
             0.00821_rlg*p + 15.0d-09*p*p
         pcc = p/(c*c)
         rostp = rost + 1.0d4*(1.0_rlg-0.2_rlg*pcc)*pcc
   ! buoyancy
         boz(k,i,j)  = grav*( rho0 - REAL(rostp,rsh) )/rho0
	  
!---------------------------------------------------------------------------------------------------
!           zw(k) = f_zw(h0(i,j),xec(i,j),k,i,j)	 
!           dpth=xec(i,j)-zw(k)
!           K_up(k)=K0+dpth*(K1+K2*dpth)
!	    
!           zw(k-1) = f_zw(h0(i,j),xec(i,j),k-1,i,j)	 
!           dpth=xec(i,j)-zw(k-1)
!           K_dw(k)=K0+dpth*(K1+K2*dpth)
       END DO      ! fin de la boucle en k

!        cff=grav/rho0
!        do k=1,kmax-1
!            cff1=0.1*(xec(i,j)-zw(k))
!            bvf(k,i,j)=-cff*(   (boz2(k+1,i,j)-boz2(k,i,j))           & ! turbulence verticale
!     			     *(K00+K_dw(k+1))*(K00+K_up(k))	      &
!     			  -cff1*( rho0*(K_dw(k+1)-K_up(k))	      &
!     				 +K00*(boz2(k+1,i,j)-boz2(k,i,j))     &
!     				       +boz2(k+1,i,j)*K_dw(k+1)       &
!     					   -boz2(k,i,j)*K_up(k)       &
!     	       ) )/(  (K00+K_dw(k+1)-cff1)*(K00+K_up(k)-cff1)         &
!     				       *(zr(k+1,i,j)-zr(k,i,j))  )
!				       print*,i,j,k
!				       print*,boz(k,i,j),K_dw(k),K_dw(k+1)				       
!        end do      ! fin de la boucle en k
!---------------------------------------------------------------------------------------------------
     END IF       ! fin du test sur le masque terre 
       
    END DO        ! fin de la boucle en i
   END DO         ! fin de la boucle en j 	 	   
#   endif  

!$OMP END PARALLEL DO

   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,zr(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
! utile ?
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,zr(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
   
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz2(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz2(:,liminm1:limaxp2,ljminm1:ljmaxp2))	
 
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,qp1(:,liminm1:limaxp2,ljminm1:ljmaxp2)) 
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,qp1(:,liminm1:limaxp2,ljminm1:ljmaxp2)) 

   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,rho(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,rho(:,liminm1:limaxp2,ljminm1:ljmaxp2))      
   
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,boz(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   
!   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,bvf(:,liminm1:limaxp2,ljminm1:ljmaxp2))
!   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,bvf(:,liminm1:limaxp2,ljminm1:ljmaxp2))     
   PRINT_DBG*, 'exit TS_STATEEQ_BLUMBERG routine'

  END SUBROUTINE ts_stateeq_blumberg
