  SUBROUTINE tssub_advdify_quickupwind(sal,temp &
#ifdef key_substance
                                       ,c      &
#endif
                                       )

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tssub_advdify_quickupwind  ***
   !&E
   !&E ** Purpose : Estimate of meridional fluxes of temperature and salinity
   !&E              and other substances (advection and diffusion).
   !&E              Advection uses TVD scheme (QUICK and upwind) owing to limiter.
   !&E
   !&E ** Description : No horizontal diffusion at lower depths ( < hminkxky)
   !&E                  - case of cyclicity in y : correct for northward propagation,
   !&E                    uncorrect for southward propagation (stencil not long enough)
   !&E
   !&E ** Called by :
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : hey,hy,fyeau,sal,temp,c(cv_wat),hzey,dy_v
   !&E                     fsaly,ftempy,fy
   !&E
   !&E ** Modified variables : fsaly,ftempy,fy
   !&E
   !&E ** Reference : Kowalik Z. and T. S. Murty : Numerical modeling of ocean
   !&E                dynamics in Advanced series on ocean engineering, vol 5.
   !&E
   !&E ** History :
   !&E           !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E           !  2008-09 (V. Garnier) correction for southern boundary
   !&E           !  2009-04 (A. Mano) MPI2D
   !&E           !  2009-10  (F. Cayocca) correction for sedimentology
   !&E           !  2010-10 (T. Duhaut) introduction of diffusion along isopycnes
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,    ONLY : jb,jh,hey,hy,h0fond,qmax,ky,hminkxky,dy_v,halfdt,l_obc_cycl_y,dx_v, &
! FVDM 2014 : ajout declaration
                            vismul,forwrka

#if defined key_siggen || defined key_gencoord
   USE comsiggen,    ONLY : hzey
#endif
   USE comfluxdo,    ONLY : fsaly,fyeau
   USE cometemp,     ONLY  : ftempy
#ifdef key_substance
   USE comsubstance, ONLY : nv_adv,fy,nvpi
#ifdef key_sedim
   USE sedimento,    ONLY : corfluy
#endif
#endif
   USE_MPI toolmpi,      ONLY : ex_j_rsh,ex_j_rlg

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: sal,temp
#ifdef key_substance
   REAL(KIND=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: c
#endif

   !! * Local declarations
   LOGICAL                 :: decentt,decents
   INTEGER                 :: i,j,k
   REAL(KIND=rsh)          :: unsqmax,vv,yr,qot,dyds,dygs,dydt
   REAL(KIND=rsh)          :: dygt,kyb,yas,yat,ybs,ybt,ycs,yct,epsi
#ifdef key_substance
   LOGICAL, DIMENSION(nv_adv)           :: decent
   INTEGER                              :: iv
   REAL(KIND=rsh)                       :: dyg
   REAL(KIND=rsh), DIMENSION(nv_adv)    :: ya,yb,yc,dyd
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   unsqmax=1.0_rsh/qmax
   epsi=1.0e-10

!$OMP DO schedule(runtime),private(i,j,k,kyb               &
!$OMP& ,vv,yr,qot,decents,decentt,dyds,dygs,dydt,dygt      &
#ifdef key_substance
!$OMP& ,iv,ya,yb,yc,decent,dyd,dyg                         &
#endif
!$OMP& ,yas,yat,ybs,ybt,ycs,yct)

   DO j=ljmin,ljmax
     DO i=liminp2,limaxm1

       IF (j >= jb(i) .AND. j <= jh(i)-1) THEN
         DO k=1,kmax
           IF (hey(i,j) <= 0.0_rsh .OR. hy(i,j)==-valmanq-h0fond) THEN
#if ! defined key_offline
             fsaly(k,i,j)=0.0_rlg
             ftempy(k,i,j)=0.0_rlg
#endif
#ifdef key_substance
             fy(nvpi+1:nv_adv,k,i,j)=0.0_rsh
#endif
           ELSE
!FVDM 5/02/2014 
! kyb=ky
! La diffusion est fonction d un cosinus
! dans la couche eponge (tableau forwrka)
! et continue avec l interieur du domaine 
	   kyb=ky + vismul*forwrka(i,j)! FVDM
             IF (hey(i,j) < hminkxky) kyb=0.0_rsh ! no diffusion for small water depth
             vv=fyeau(k,i,j)
             IF (vv <= 0.0_rsh) THEN  ! southward advection
#if ! defined key_offline
               ybs=sal(k,i,j+1)
               ycs=sal(k,i,j)
               ybt=temp(k,i,j+1)
               yct=temp(k,i,j)
#endif
#ifdef key_substance
               yb(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j+1)
               yc(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j)
#endif

               IF (hey(i,j+1) > 0.0_rsh .AND. j+1 < jh(i)) THEN
#if ! defined key_offline
                 yas=sal(k,i,j+2)
                 yat=temp(k,i,j+2)
#endif
#ifdef key_substance
                 ya(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j+2)
#endif
               ELSE
#if ! defined key_offline
                 yas=ybs
                 yat=ybt
#endif
#ifdef key_substance
                 ya(nvpi+1:nv_adv)=yb(nvpi+1:nv_adv)
#endif
               END IF

#if ! defined key_offline
               dyds=ybs-ycs
               dygs=ybs-yas
               dydt=ybt-yct
               dygt=ybt-yat
               decents=(dyds*dygs) >= 0.0_rsh
               IF (.NOT.decents) THEN
                 qot=ABS(dyds/MAX(ABS(dygs),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decents=.FALSE.
                 ELSE
                   decents=.TRUE.
                 END IF
               END IF

               decentt=(dydt*dygt) >= 0.0_rsh
               IF (.NOT.decentt) THEN
                 qot=ABS(dydt/MAX(ABS(dygt),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decentt=.FALSE.
                 ELSE
                   decentt=.TRUE.
                 END IF
               END IF
#endif

#ifdef key_substance
               DO iv=nvpi+1,nv_adv
                 dyd(iv)=yb(iv)-yc(iv)
                 dyg=yb(iv)-ya(iv)
                 decent(iv)=(dyd(iv)*dyg) >= 0.0_rsh
                 IF (.NOT.decent(iv)) THEN
                   qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                   IF (qot <= qmax .AND. qot >= unsqmax) THEN
                     decent(iv)=.FALSE.
                   ELSE
                     decent(iv)=.TRUE.
                   END IF
                 END IF
               END DO
#endif

             ELSE      ! northward advection

#if ! defined key_offline
               ybs=sal(k,i,j)
               ycs=sal(k,i,j+1)
               ybt=temp(k,i,j)
               yct=temp(k,i,j+1)
#endif
#ifdef key_substance
               yb(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j)
               yc(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j+1)
#endif
               IF (hey(i,j-1) > 0.0_rsh .AND. (j > jb(i) .OR. l_obc_cycl_y)) THEN
#if ! defined key_offline
                 yas=sal(k,i,j-1)
                 yat=temp(k,i,j-1)
#endif
#ifdef key_substance
                 ya(nvpi+1:nv_adv)=c(nvpi+1:nv_adv,k,i,j-1)
#endif
               ELSE
#if ! defined key_offline
                 yas=ybs
                 yat=ybt
#endif
#ifdef key_substance
                 ya(nvpi+1:nv_adv)=yb(nvpi+1:nv_adv)
#endif
               END IF

#if ! defined key_offline
               dyds=ycs-ybs
               dygs=ybs-yas
               dydt=yct-ybt
               dygt=ybt-yat

               decents=(dyds*dygs) <= 0.0_rsh
               IF (.NOT.decents) THEN
                 qot=ABS(dyds/MAX(ABS(dygs),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decents=.FALSE.
                 ELSE
                   decents=.TRUE.
                 END IF
               END IF

               decentt=(dydt*dygt) <= 0.0_rsh
               IF (.NOT.decentt) THEN
                 qot=ABS(dydt/MAX(ABS(dygt),epsi))
                 IF (qot <= qmax .AND. qot >= unsqmax) THEN
                   decentt=.FALSE.
                 ELSE
                   decentt=.TRUE.
                 END IF
               END IF
#endif

#ifdef key_substance
               DO iv=nvpi+1,nv_adv
                 dyd(iv)=yc(iv)-yb(iv)
                 dyg=yb(iv)-ya(iv)
                 decent(iv)=(dyd(iv)*dyg) <= 0.0_rsh
                 IF (.NOT.decent(iv)) THEN
                   qot=ABS(dyd(iv)/MAX(ABS(dyg),epsi))
                   IF (qot <= qmax .AND. qot >= unsqmax) THEN
                     decent(iv)=.FALSE.
                   ELSE
                     decent(iv)=.TRUE.
                   END IF
                 END IF
               END DO
#endif

             END IF       ! ends test on velocity sign

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho

   ! advection flux of salinity alone
   ! --------------------------------

             IF (decents) THEN
               fsaly(k,i,j)=vv*ybs*dx_v(i,j)
             ELSE
               yr=0.375_rsh*ycs+0.75_rsh*ybs-0.125_rsh*yas
               fsaly(k,i,j)=vv*yr*dx_v(i,j)
             END IF

   ! advection flux of temperature alone
   ! -----------------------------------

             IF (decentt) THEN
               ftempy(k,i,j)=vv*ybt*dx_v(i,j)
             ELSE
               yr=0.375_rsh*yct+0.75_rsh*ybt-0.125_rsh*yat
               ftempy(k,i,j)=vv*yr*dx_v(i,j)
             END IF

#else

   ! advection/diffusion flux of salinity
   ! ------------------------------------

             IF (decents) THEN
#if defined key_siggen || defined key_gencoord
               fsaly(k,i,j)=vv*ybs-hzey(k,i,j)*kyb*dyds/dy_v(i,j)*halfdt
#else
               fsaly(k,i,j)=vv*ybs-hey(i,j)*kyb*dyds/dy_v(i,j)*halfdt
#endif
             ELSE
               yr=0.375_rsh*ycs+0.75_rsh*ybs-0.125_rsh*yas
#if defined key_siggen || defined key_gencoord
               fsaly(k,i,j)=vv*yr-hzey(k,i,j)*kyb*dyds/dy_v(i,j)*halfdt
#else
               fsaly(k,i,j)=vv*yr-hey(i,j)*kyb*dyds/dy_v(i,j)*halfdt
#endif
             END IF

   ! advection/diffusion flux of temperature
   ! ---------------------------------------

             IF (decentt) THEN
#if defined key_siggen || defined key_gencoord
               ftempy(k,i,j)=vv*ybt-hzey(k,i,j)*kyb*dydt/dy_v(i,j)*halfdt
#else
               ftempy(k,i,j)=vv*ybt-hey(i,j)*kyb*dydt/dy_v(i,j)*halfdt
#endif
             ELSE
               yr=0.375_rsh*yct+0.75_rsh*ybt-0.125_rsh*yat
#if defined key_siggen || defined key_gencoord
               ftempy(k,i,j)=vv*yr -hzey(k,i,j)*kyb*dydt/dy_v(i,j)*halfdt
#else
               ftempy(k,i,j)=vv*yr -hey(i,j)*kyb*dydt/dy_v(i,j)*halfdt
#endif
             END IF

#endif

   ! key_offline
#endif

#ifdef key_substance

   ! advection flux of other substances
   ! ----------------------------------

             DO iv=nvpi+1,nv_adv

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
               IF (decent(iv)) THEN
                 fy(iv,k,i,j)=fyeau(k,i,j)*yb(iv)*dx_v(i,j)
               ELSE
                 yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
                 fy(iv,k,i,j)=fyeau(k,i,j)*yr*dx_v(i,j)
               END IF
#else
               IF (decent(iv)) THEN
#if defined key_siggen || defined key_gencoord
                 fy(iv,k,i,j)=fyeau(k,i,j)*yb(iv)-hzey(k,i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt
#else
                 fy(iv,k,i,j)=fyeau(k,i,j)*yb(iv)-hey(i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt
#endif
               ELSE
                 yr=0.375_rsh*yc(iv)+0.75_rsh*yb(iv)-0.125_rsh*ya(iv)
#if defined key_siggen || defined key_gencoord
                 fy(iv,k,i,j)=fyeau(k,i,j)*yr-hzey(k,i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt
#else
                 fy(iv,k,i,j)=fyeau(k,i,j)*yr-hey(i,j)*kyb*dyd(iv)/dy_v(i,j)*halfdt
#endif
               END IF
#endif

#ifdef key_sedim
               IF (k.eq.1) fy(iv,k,i,j)=fy(iv,k,i,j)*corfluy(iv,i,j)
#endif
             END DO
#endif

           END IF    ! ends test on water depth
         END DO  ! ends loop on k
       END IF  ! for selecting right j loop
     END DO  ! ends loop on i
   END DO  ! ends loop on j
!$OMP END DO

OMPMPI barrier
OMPMPI master
# if ! defined key_offline
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,fsaly(:,limin:limax,ljminm1:ljmax))
   CALL_MPI ex_j_rlg(-1,0,kmax,limin,limax,ljminm1,ljmax,ftempy(:,limin:limax,ljminm1:ljmax))
#endif
#ifdef key_substance
   CALL_MPI ex_j_rsh(-1,0,nv_adv*kmax,limin,limax,ljminm1,ljmax,fy(:,:,limin:limax,ljminm1:ljmax))
#endif
OMPMPI end master
OMPMPI barrier
# if ! defined key_offline
OMPMPI flush(fsaly,ftempy)
#endif
#ifdef key_substance
OMPMPI flush(fy)
#endif

  END SUBROUTINE tssub_advdify_quickupwind
