  SUBROUTINE turblepskznz(xeuv,boz,psi,ect)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE turblepskznz  ***
   !&E
   !&E ** Purpose : compute the vertical viscosity and diffusion coefficients,
   !&E              using the k parameterisation
   !&E
   !&E ** Description : compute the turbulent kinetic energy (ect),
   !&E                  estimate nz and kz from ect and from an anlytical 
   !&E                  formulation of the mixing length (l)
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,xeuv,ect,psi,eps,boz,kz,nz
   !&E
   !&E ** Modified variables : eps,nz,kz
   !&E
   !&E ** History :
   !&E       !  2004-09
   !&E       !  2009-03  (T. Duhaut) corrections
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2009-12  (T. Duhaut) stability function
   !&E       !  2011-10  (V. Garnier) OMP
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2012-02  (F. Dumas) Initialization of nz, kz at kmax
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d
   USE comvars3d
   USE cometdyn
   USE comturb
   USE comvarp3d,   ONLY : uz,vz
   USE toolgeom,    ONLY : f_zw
   USE_MPI toolmpi, ONLY : ex_j_rsh,ex_i_rsh

   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: xeuv
   REAL(kind=rsh),DIMENSION(kmax,limin  :limax  ,ljmin  :ljmax  ),INTENT(in) :: psi,ect
   REAL(kind=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: boz

   !! * Local declarations
   INTEGER        :: i,j,k
   REAL(kind=rsh) :: A1,A2,B1,B2,C2,C3,Gh0,Ghcrit,Ghmin,Gh,Ghu,gls
   REAL(kind=rsh) :: C1 
   REAL(kind=rsh) :: hp,N2,Sh,Sm,invB113
   REAL(kind=rsh) :: zlim
   REAL(kind=rsh) :: M2, Denom, t1, t2, t3, t4, s0, s1, s4, s5, s6,Gm 
   REAL(kind=rsh) :: bb1, bb2, bb3, bb4, bb5, aa1, aa2, aa3, aa4, aa5, aa6 
   REAL(kind=rsh) :: immwk
   !! * Parametre en DUR : choix de la fonction de stabilite 1,2 ou 3
   INTEGER,PARAMETER        :: OptionSF=3
   !!----------------------------------------------------------------------
   !! * Executable part
   
   !------------------------------------------------------------------------------
   ! calcul  de la GLS et du taux de dissipation de l ect
   ! des fonctions de stabilite et des kz et nz 
   !------------------------------------------------------------------------------

!$OMP PARALLEL DEFAULT(none) SHARED(h0,hm,xeuv,ect,eps,psi,boz,uz,vz,kz,nz,   &
!$OMP&  ljminp2,ljmaxm1,limin,limax,ljmin,ljmax,id,ig,dsigu,dsigw,sigw,nbouc, &
!$OMP&  turb_2eq_option,cmu,bgdiff)                                           &
!$OMP& PRIVATE(Ghcrit,Gh0,Ghmin,A1,A2,B1,invB113,B2,C1,C2,C3,                 &
!$OMP& i,j,k,hp,t1,t2,t3,t4,s0,s1,s4,s5,s6,M2,N2,Gh,Gm,gls,Denom,Sm,Sh,Ghu,   &
!$OMP& aa1,aa2,aa3,aa4,aa5,aa6,bb1,bb2,bb3,bb4,bb5,immwk,zlim)

   Ghcrit= 0.02_rsh
   Gh0   = 0.0233_rsh
   Ghmin =-0.28_rsh
   A1=0.92_rsh
   A2=0.74_rsh
   B1=16.6_rsh
   invB113=1.0_rsh/(B1**(1.0_rsh/3.0_rsh))
   B2=10.1_rsh
   C1=0.08_rsh
   C2= 0.7_rsh
   C3= 0.2_rsh

!$OMP DO SCHEDULE(RUNTIME)
   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       hp=h0(i,j)+xeuv(i,j)
       IF (hp > hm) THEN

         IF (turb_2eq_option==1) eps(:,i,j)=(cmu**3)*(ect(:,i,j)**2.5_rsh)/psi(:,i,j)
         IF (turb_2eq_option==2) eps(:,i,j)=psi(:,i,j)
         IF (turb_2eq_option==3) eps(:,i,j)=cmu**4*ect(:,i,j)*psi(:,i,j)
         IF (turb_2eq_option==4) eps(:,i,j)=psi(:,i,j)**1.5_rsh

         eps(:,i,j) = MAX(eps(:,i,j),1.e-12)

         IF ( OptionSF == 1) THEN  ! Fonction de stabilite de type : Kantha et Clayson KC1

           t1=-32.23_rsh
           t2=5.08_rsh
           t3=159.96_rsh
           t4=-83.59_rsh
           s0=0.6979_rsh
           s1=-9.345_rsh
           s4=0.74_rsh
           s5=0.9070_rsh
           s6=-4.534_rsh

           DO k=1,kmax-1
             IF (turb_2eq_option==1) gls=psi(k,i,j)/ect(k,i,j)
             IF (turb_2eq_option==2 .or. turb_2eq_option==3 .or. turb_2eq_option==4) gls=cmu**3*ect(k,i,j)**1.5_rsh/eps(k,i,j)
             M2= 0.5_rsh*((vz(k+1,i,j  )-vz(k,i,j  ))**2 &
                         +(vz(k+1,i,j-1)-vz(k,i,j-1))**2 &
                         +(uz(k+1,i,j  )-uz(k,i,j  ))**2 &
                         +(uz(k+1,i-1,j)-uz(k,i-1,j))**2)/(hp*dsigw(k))**2
             N2=(boz(k+1,i,j)-boz(k,i,j))/(hp*dsigw(k))
             IF (N2>0.0_rsh) gls=MIN(SQRT(0.56_rsh*ect(k,i,j)/N2),gls) 

             Gh=MIN(-N2*gls**2/(2.0_rsh*ect(k,i,j)),0.033_rsh)
             Gm=MIN( M2*gls**2/(2.0_rsh*ect(k,i,j)),0.825_rsh-25.0_rsh*Gh)

             Denom=1.0_rsh+t1*Gh+t2*Gm+t3*Gh**2+t4*Gm*Gh
             Sm=(s0+s1*Gh)/Denom
             Sh=(s4+s5*Gm+s6*Gh)/Denom

             nz(k,i,j)=SQRT(2.0_rsh*ect(k,i,j))*gls*Sm
             nz(k,i,j)=MIN(MAX(nz(k,i,j),bgdiff),0.9_rsh)
             kz(k,i,j)=SQRT(2.0_rsh*ect(k,i,j))*gls*Sh
             kz(k,i,j)=MIN(MAX(kz(k,i,j),bgdiff),0.9_rsh)

           END DO 

         ELSE IF ( OptionSF == 2 ) THEN ! Fonction de stabilite de type : Galperin G2

           DO k=1,kmax-1
             IF (turb_2eq_option==1) gls=psi(k,i,j)/ect(k,i,j)
             IF (turb_2eq_option==2 .or. turb_2eq_option==3 .or. turb_2eq_option==4) gls=cmu**3*ect(k,i,j)**1.5_rsh/eps(k,i,j)
             N2=(boz(k+1,i,j)-boz(k,i,j))/(hp*dsigw(k))
             IF (N2>0.0_rsh) gls=MIN(SQRT(0.56_rsh*ect(k,i,j)/N2),gls) 

             Gh=-N2*gls**2/ect(k,i,j)
             Gh=MAX(Gh,-0.0466_rsh)
             Gh=MIN(Gh,0.0233_rsh)
           
             Sm=(0.5562_rsh+2.182_rsh*Gh)/(1.0_rsh+20.40_rsh*Gh+53.12_rsh*Gh**2)
             Sh=0.6985_rsh/(1.0_rsh+17.34_rsh*Gh)

             nz(k,i,j)=SQRT(ect(k,i,j))*gls*Sm
             nz(k,i,j)=MIN(MAX(nz(k,i,j),bgdiff),0.9_rsh)
             kz(k,i,j)=SQRT(ect(k,i,j))*gls*Sh
             kz(k,i,j)=MIN(MAX(kz(k,i,j),bgdiff),0.9_rsh)

           END DO

         ELSE IF ( OptionSF == 3) THEN ! Fonction de stabilite de type : Kantha et Ckayson KC2

           Ghcrit= 0.02_rsh
           Gh0   = 0.0233_rsh
           Ghmin =-0.28_rsh
           A1=0.92_rsh
           A2=0.74_rsh
           B1=16.6_rsh
           invB113=1.0_rsh/(B1**(1.0_rsh/3.0_rsh))
           B2=10.1_rsh
           C1= 0.08_rsh
           C2= 0.7_rsh
           C3= 0.2_rsh
           ! C0=1.0 ! ou SQRT(2.)*cmu**3 pour canuto A et B
           ! C0= SQRT(2.)*cmu**3 !  pour canuto A et B

           DO k=1,kmax-1
             IF (turb_2eq_option==1) gls=psi(k,i,j)/ect(k,i,j)
             IF (turb_2eq_option==2 .or. turb_2eq_option==3 .or. turb_2eq_option==4) gls=cmu**3*ect(k,i,j)**1.5_rsh/eps(k,i,j)
             N2=(boz(k+1,i,j)-boz(k,i,j))/(hp*dsigw(k))
             IF (N2>0.0_rsh) gls=MIN(SQRT(0.56_rsh*ect(k,i,j)/N2),gls) 

             Ghu=-N2*gls**2/(2.0_rsh*ect(k,i,j))
             !print*,Ghu-(Ghu-Ghcrit)**2,Ghu+Gh0-2.0_rsh*Ghcrit,Ghu+Gh0
             !Gh=MIN(MAX((Ghu-(Ghu-Ghcrit)**2)/(Ghu+Gh0-2.0_rsh*Ghcrit),Ghmin),Gh0)
             Gh=MIN(MAX(Ghu,Ghmin),Gh0)

             Sh=(A2*(1.0_rsh-6.0_rsh*A1/B1)) &
                /(1.0_rsh-3.0_rsh*A2*Gh*(6.0_rsh*A1+B2*(1.0_rsh-C3)))
             Sm=(invB113+(18.0_rsh*A1*A1+9.0_rsh*A1*A2*(1.0_rsh-C2))*Sh*Gh) &
                /(1.0_rsh-9.0_rsh*A1*A2*Gh)

             !nz(k,i,j)=SQRT(ect(k,i,j))*gls*Sm
             nz(k,i,j)=SQRT(2.*ect(k,i,j))*gls*Sm
             nz(k,i,j)=MIN(MAX(nz(k,i,j),bgdiff),.9)
             !kz(k,i,j)=SQRT(ect(k,i,j))*gls*Sh
             kz(k,i,j)=SQRT(2.*ect(k,i,j))*gls*Sh
             kz(k,i,j)=MIN(MAX(kz(k,i,j),bgdiff),0.9_rsh)

           END DO

         ELSE IF ( OptionSF == 4) THEN ! Fonction de stabilite de type : Canuto CNB 

           aa1=0.1270_rsh
           aa2=0.01526_rsh
           aa3=-0.00016_rsh
           aa4=0.1190_rsh
           aa5=0.004290_rsh
           aa6=0.00066_rsh
           bb1=0.1977_rsh
           bb2=0.03154_rsh
           bb3=0.005832_rsh
           bb4=0.004127_rsh
           bb5=-0.000042_rsh

           DO k=1,kmax-1
            ! IF (turb_2eq_option==1) gls=psi(k,i,j)/ect(k,i,j)
            ! IF (turb_2eq_option==2 .or. turb_2eq_option==3 .or. turb_2eq_option==4) gls=cmu**3*ect(k,i,j)**1.5_rsh/eps(k,i,j)
             M2= 0.5_rsh*((vz(k+1,i,j  )-vz(k,i,j  ))**2 &
                         +(vz(k+1,i,j-1)-vz(k,i,j-1))**2 &
                         +(uz(k+1,i,j  )-uz(k,i,j  ))**2 &
                         +(uz(k+1,i-1,j)-uz(k,i-1,j))**2)/(hp*dsigw(k))**2
             N2=(boz(k+1,i,j)-boz(k,i,j))/(hp*dsigw(k))
             ! IF (N2>0.0_rsh) gls=MIN(SQRT(0.56_rsh*ect(k,i,j)/N2),gls)

             Gh=N2*ect(k,i,j)**2/(eps(k,i,j)**2)
             Gm=M2*ect(k,i,j)**2/(eps(k,i,j)**2)

             Denom=1+bb1*Gh+bb2*Gm+bb3*Gh**2+bb4*Gh*Gm+bb5*Gm**2
             Sm=(aa1+aa2*Gh+aa3*Gm)/Denom
             Sh=(aa4+aa5*Gh+aa6*Gm)/Denom

             nz(k,i,j)=ect(k,i,j)**2*Sm/eps(k,i,j)
             nz(k,i,j)=MIN(MAX(nz(k,i,j),bgdiff),0.9_rsh)
             kz(k,i,j)=ect(k,i,j)**2*Sh/eps(k,i,j)
             kz(k,i,j)=MIN(MAX(kz(k,i,j),bgdiff),0.9_rsh)

           END DO

         ELSE ! fonction de stabilite standard

           DO k=1,kmax-1
             IF (turb_2eq_option==1) gls=psi(k,i,j)/ect(k,i,j)
             IF (turb_2eq_option==2 .or. turb_2eq_option==3) gls=cmu**3*ect(k,i,j)**1.5_rsh/eps(k,i,j) 
             N2=(boz(k+1,i,j)-boz(k,i,j))/(hp*dsigw(k))
             IF (N2>0.0_rsh) gls=MIN(SQRT(0.56_rsh*ect(k,i,j)/N2),gls) 
             Ghu=-N2*gls**2/(2.0_rsh*ect(k,i,j))
            ! Gh=MIN(MAX((Ghu-(Ghu-Ghcrit)**2)/(Ghu+Gh0-2.0_rsh*Ghcrit),Ghmin),Gh0)

             Gh=MAX(Ghu,-0.28_rsh)
             Gh=MIN(Gh,0.0233_rsh)

             ! Sm=A1*(1.-3.*C1-6.*A1/B1-3.*A2*Gh*((B2-3.*A1)*(1-6.*A1/B1)-3.*C1*(B2+6.*A1)))   &
             !          /(1-3.*A2*Gh*(6.*A1+B2))/(1.-9.*A1*A2*Gh)
             ! Sh=A2*(1-6.*A1/B1)/(1-3.*A2*Gh*(6.*A1+B2))
             Sh=(A2*(1.0_rsh-6.0_rsh*A1/B1))/(1.0_rsh-3.0_rsh*A2*Gh*(6.0_rsh*A1+B2*(1.0_rsh-C3)))
             Sm=(invB113+(18.0_rsh*A1*A1+9.0_rsh*A1*A2*(1.0_rsh-C2))*Sh*Gh)/(1.0_rsh-9.0_rsh*A1*A2*Gh)

             nz(k,i,j)=SQRT(2.0_rsh*ect(k,i,j))*gls*Sm
             nz(k,i,j)=MIN(MAX(nz(k,i,j),bgdiff),0.9_rsh)
             kz(k,i,j)=SQRT(2.0_rsh*ect(k,i,j))*gls*Sh
             kz(k,i,j)=MIN(MAX(kz(k,i,j),bgdiff),0.9_rsh)

           END DO 

         END IF

         ! Estimate of the viscosity and diffusivity at the surface  ! What can we do otherwise ????
         ! We can also use the intermediate variable with no vertical gradient of uz and vz
         ! or use a logarithmic gradient of velocity ?
         nz(kmax,i,j)=nz(kmax-1,i,j)
         kz(kmax,i,j)=kz(kmax-1,i,j)

       END IF 

#ifdef key_ifrequired
       DO k=1,kmax-1
         immwk=f_zw(h0(i,j),xeuv(i,j),k,i,j)
         ! test bride kz et nz en dessous de 200 m comme gaspard
         IF (h0(i,j) > 200.0.and. -immwk > 200.0_rsh) kz(k,i,j)=1.e-5
         IF (h0(i,j) > 200.0.and. -immwk > 200.0_rsh) nz(k,i,j)=1.e-5
       END DO
#endif

     END DO 
   END DO
!$OMP END DO

!$OMP SINGLE
   IF (limin==imin) THEN
     nz(:,imin+1,ljmin:ljmax)=nz(:,imin+2,ljmin:ljmax)
     kz(:,imin+1,ljmin:ljmax)=kz(:,imin+2,ljmin:ljmax)
   END IF
   IF (limax==imax) THEN
     nz(:,imax,ljmin:ljmax)=nz(:,imax-1,ljmin:ljmax)
     kz(:,imax,ljmin:ljmax)=kz(:,imax-1,ljmin:ljmax)
   END IF
   IF (ljmin==jmin) THEN
     nz(:,limin:limax,jmin+1)=nz(:,limin:limax,jmin+2)
     kz(:,limin:limax,jmin+1)=kz(:,limin:limax,jmin+2)
   END IF
   IF (ljmax==jmax) THEN
     nz(:,limin:limax,jmax)=nz(:,limin:limax,jmax-1)
     kz(:,limin:limax,jmax)=kz(:,limin:limax,jmax-1)
   END IF
!$OMP END SINGLE

!$OMP SINGLE
   IF ( l_testcase .AND. num_testcase==22 ) THEN

     zlim=50.0_rsh

     DO k=kmax-1,1,-1
       IF ( ect(k,imax/2,2) > 0.00001_rsh ) THEN
         zlim=50*(dsigu(1)*0.5_rsh +(k-1)*dsigu(1))
       ELSE
         IF ( ect(k,imax/2,2)-ect(k+1,imax/2,2) < 0.0_rsh ) THEN
           WRITE(153,*) nbouc*15_rsh/3600.0_rsh,(50.0_rsh-zlim)
           GOTO 45
         ELSE
           zlim=dsigu(1)*0.5_rsh +(k-1)*dsigu(1)
         END IF
       END IF
     END DO

45   IF (MOD(nbouc,54) ==0) THEN
       DO k=kmax,1,-1
         WRITE(13,*) dsigu(1)*0.5_rsh +(k-1)*dsigu(1),ect(k,imax/2,2)
       END DO
       WRITE(13,'(/)') ! ecriture d une ligne blanche
       WRITE(13,'(/)') ! ecriture d une ligne blanche
     END IF

   END IF
!$OMP END SINGLE

!$OMP END PARALLEL

   CALL_MPI ex_i_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,nz(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,kmax,limin,limaxp1,ljmin,ljmaxp1,nz(:,limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,kz(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,kz(:,liminm1:limaxp1,ljminm1:ljmaxp1))

  END SUBROUTINE turblepskznz
