
 MODULE wave

#ifdef key_wave

   !&E=====================================================================
   !&E
   !&E                      ***  MODULE  waveread  ***
   !&E ** Purpose : Read, spatial and time interpolation of waves datas
   !&E
   !&E
   !&E ** Called by:
   !&E       step (in file step.F90) 
   !&E
   !&E ** HOWTO: add New Waves Quantity (-nwq-)
   !&E       !    Search -nwq- in present file and add your quantity as others
   !&E       !    Open comwave.F90, search -nwq- and add your quantity as others
   !&E       !    Open outputsave.F90 search -nwq- and add your quantity as others
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Built on the basis of sflxread.F90
   !&E       Rq val OMP+MPI not done yet
   !&E       !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E
   !&E=====================================================================

#include "toolcpp.h"

   !! * Modules used
  USE parameters, ONLY : rsh,riosh,rlg,    &
        kmax,valmanq,     &
        imin,imax,        &
        jmin,jmax,        &
        limin,limax,      &
        ljmin,ljmax,      &
        limaxp1,ljmaxp1,  &
        ljminp1,liminp1,  &
        limaxp2,ljmaxp2,  &
        pi,epsilon
   USE ionc4, ONLY : ionc4_openr,ionc4_corres,ionc4_flimax,             &
                     ionc4_fljmax,ionc4_read_lat,ionc4_read_lon,        &
                     ionc4_read_dimt,ionc4_read_time,ionc4_read_subxyt

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC wave_read ! function called by step

   !! * Shared module variables

   !! * Private variables
#ifdef key_sflx_interp_scrip
   LOGICAL,DIMENSION(:,:),ALLOCATABLE        :: maskdst,masksrc
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: matrix,matrix_u,matrix_v
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: tabvar2d,tabinterp2d
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: lonwave2d,latwave2d
   INTEGER,DIMENSION(:),ALLOCATABLE          :: src_add,dst_add,src_add_u,dst_add_u,src_add_v,dst_add_v
   INTEGER :: nxfin,nyfin
#endif

 CONTAINS

   !!======================================================================

  SUBROUTINE wave_read

   !&E---------------------------------------------------------------------
   !&E                         ***  ROUTINE wave_read  ***
   !&E
   !&E ** Purpose : Read waves file data generated from differents waves models.
   !&E              These data are provided in a NetCDF file with an AO structure :
   !&E              the grid is oriented from South to North and from West to East
   !&E              and spatial steps are constant.
   !&E
   !&E ** Description : 
   !&E              Read the following variables :
   !&E                Mean direction of the total sea (dir=ATAN2(b1tot/a1tot)) (in [degrees])
   !&E                Significant wave height (in [m])
   !&E                Energy flux from waves to current in in ([W.m^{-2}])
   !&E
   !&E ** Map : 
   !&E             -onr- Open Netcdf, Read lat and lonfile (init: only do for nbouc=1)
   !&E             -faa- Find size of 2D Array to Allocate (init)
   !&E             -lmw- Locate Mars cell from Wave cell (init)
   !&E             -ctd- Checkout Time Dependences of inputed waves data (init)
   !&E             -ri2- Read and spatial Interpolate of 2 time step of input file (init)
   !&E             -cou- CheckOut Units (init)
   !&E             -fse- Find time Step in the file that Exceed computational time step
   !&E             -2i1- We transfert array 2 In array 1
   !&E             -rin- Read and spatial Interpolate Next time step of input file while computational time step is not exceed
   !&E             -tli- Do time Linear Interpolation
   !&E             (-mmg- Mpi Managing for Gradient computation)
   !&E
   !&E ** Called by : 
   !&E                 step
   !&E
   !&E ** External calls : tool_interp2d_sflx,ionc4_openr,ionc4_corres,ionc4_flimax,
   !&E                     ionc4_fljmax,ionc4_read_lat,ionc4_read_lon, ionc4_read_dimt,
   !&E                     ionc4_read_time,ionc4_read_subxyt
   !&E
   !&E ** Reference :
   !&E      SWAN: TODO
   !&E      WAVEWATCH III:
   !&E        TOLMAN Hendrik L., 2002, User manual and system documentation of WAVEWATCH-III version 2.22
   !&E        TOLMAN Hendrik L., 2007, The 2007 release of WAVEWATCH III
   !&E
   !&E ** History :
   !&E       !  2010-02 (G. Simon) Build on the basis of module sflx_read
   !&E       !  2010-07 (R. Verney) Introduction of orbital velocity and wave period variables
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used: (definitions of variable in module)
   USE parameters, ONLY : tref

   USE comvars2d, ONLY  : t,h0,                              &
                          nbouc,                             &
                          iscreenlog,ierrorlog,iwarnlog,     &
                          lon2d,lat2d,lon2d_g,lat2d_g,       &
                          lon2du,lon2dv,lat2du,lat2dv
#if defined key_grid_rotated || defined key_sflx_interp_scrip
   USE comvars2d, ONLY  : lonmin,latmin,lonmax,latmax,grid_angle
#endif
   !! -nwq-
   USE comwave,  ONLY   : idimt,                                                  &
                          file_wave,valfill_wave,cdf_id_wave,tnc1_wave,tnc2_wave, & 
                          ilecmem_wave,ilec_wave,                                 &
                          iwavemin,iwavemax,jwavemin,jwavemax,                    &
                          l_wave_rdir,wave_dir,name_wave_dir,wave_dir1,wave_dir2, &
                          l_wave_rfoc,wave_foc,name_wave_foc,wave_foc1,wave_foc2, &
                          l_wave_rubot,wave_ubot,name_wave_ubot,wave_ubot1,wave_ubot2, &
                          l_wave_ruubot,wave_uubot,name_wave_uubot,wave_uubot1,wave_uubot2, &
                          l_wave_rvubot,wave_vubot,name_wave_vubot,wave_vubot1,wave_vubot2, &
                          l_wave_rhs,wave_hs,name_wave_hs,wave_hs1,wave_hs2,      &
                          l_wave_rtp,wave_tp,name_wave_tp,wave_tp1,wave_tp2,      &
                          i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,              &
                          i1u_wave,j1u_wave,xdpu_wave,ydpu_wave,                  &
                          i1v_wave,j1v_wave,xdpv_wave,ydpv_wave,                  &
                          l_wave_marsgrid,iwmin0

   USE_MPI toolmpi,   ONLY : ex_i_rsh,ex_j_rsh,BCAST_MPI,MASTER
#ifdef key_sflx_interp_scrip
   USE bicubic_interp,ONLY : get_remap_bicub,make_bicubic_remap
#endif

   IMPLICIT NONE
   !! * Local declarations
   LOGICAL           :: l_pb,IERR_MPI
   CHARACTER(LEN=19) :: tool_sectodat,tdate
   REAL(KIND=rlg)    :: tool_datosec ! Function for convert time date->second
   INTEGER           :: ncerror ! Handle IONC-NetCDF error
   INTEGER           :: ixwave,jxwave ! size of the lon and lat array
   INTEGER           :: i,j ! Loop indice
   REAL(KIND=rsh)    :: dgwave,dfiwave ! differentail element of longitude and latitude [degret]
   REAL(KIND=rlg),  ALLOCATABLE,DIMENSION(:)   :: lonwave,latwave ! longitude and latitude in wave file [degret]
   REAL(KIND=riosh),ALLOCATABLE,DIMENSION(:,:) :: tab1,tab2 ! array for process wave quantity
   REAL(KIND=rsh)    :: tabmin,tabmax ! size maximum of process array
   REAL(KIND=rlg)    :: torigin ! Used for comute offset between Mars and wave time [s]
   REAL(KIND=rlg)    :: twave ! The same as t [s]
   REAL(KIND=rlg)    :: dt1,dt2 ! Used for linear interpolation in time [s]
   REAL(KIND=rsh)    :: radeg ! Identity for convert radien angle in  degret
   REAL(KIND=rsh)    :: valfillm ! Left limit to valfill_wave
   REAL(KIND=rsh)    :: valfillp ! Right limit to valfill_wave

   !! For locate MARS cell from Wave cell
   REAL(KIND=rsh)    :: xm,ym

   LOGICAL           :: dbug ! dbug flag

   !!----------------------------------------------------------------------
   !! * Executable part
   !!----------------------------------------------------------------------

!$OMP PARALLEL DEFAULT(none)                                          &
!$OMP& PRIVATE (                                                      &
#ifdef key_sflx_interp_scrip
!$OMP& src_add,dst_add,src_add_u,dst_add_u,src_add_v,dst_add_v,       &
!$OMP& nxfin,nyfin,                                                   &
#endif
!$OMP& IERR_MPI,l_pb,tdate,                                           &
!$OMP&ncerror,                                                        &
!$OMP&ixwave,jxwave,                                                  &
!$OMP&i,j,                                                            &
!$OMP&dgwave,dfiwave,                                                 &
!$OMP&lonwave,latwave,                                                &
!$OMP&tab1,tab2,                                                      &
!$OMP&tabmin,tabmax,                                                  &
!$OMP&torigin,                                                        &
!$OMP&twave,                                                          &
!$OMP&dt1,dt2,                                                        &
!$OMP&radeg,                                                          &
!$OMP& xm,ym,                                                         &
!$OMP& dbug,                                                          &
!$OMP& valfillm,                                                      &
!$OMP& valfillp                                                       &
!$OMP& )                                                              &
!$OMP& SHARED(                                                        &
#ifdef key_MPI_2D
!$OMP& MASTER,                                                        &
#endif
!$OMP& lon2d,lat2d,lon2du,lat2du,lon2dv,lat2dv,lon2d_g,lat2d_g,       &
#if defined key_grid_rotated || defined key_sflx_interp_scrip
!$OMP& lonmin,latmin,lonmax,latmax,grid_angle,                        &
#endif
#ifdef key_sflx_interp_scrip
!$OMP& maskdst,masksrc,matrix,matrix_u,matrix_v,tabvar2d,tabinterp2d, &
!$OMP& lonwave2d,latwave2d,                                             &
#endif
!$OMP& file_wave,valfill_wave,cdf_id_wave,tnc1_wave,tnc2_wave,        & 
!$OMP& idimt,h0,                                                      &
!$OMP& ilecmem_wave,ilec_wave,iwmin0,                                 &
!$OMP& iwavemin,iwavemax,jwavemin,jwavemax,l_wave_marsgrid,           &
!$OMP& l_wave_rdir,wave_dir,name_wave_dir,wave_dir1,wave_dir2,        &
!$OMP& l_wave_rfoc,wave_foc,name_wave_foc,wave_foc1,wave_foc2,        &
!$OMP& l_wave_rhs,wave_hs,name_wave_hs,wave_hs1,wave_hs2,             &
!$OMP& l_wave_rubot,wave_ubot,name_wave_ubot,wave_ubot1,wave_ubot2,   &
!$OMP& l_wave_ruubot,wave_uubot,name_wave_uubot,wave_uubot1,wave_uubot2,   &
!$OMP& l_wave_rvubot,wave_vubot,name_wave_vubot,wave_vubot1,wave_vubot2,   &
!$OMP& l_wave_rtp,wave_tp,name_wave_tp,wave_tp1,wave_tp2,             &
!$OMP& i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,                     &
!$OMP& i1u_wave,j1u_wave,xdpu_wave,ydpu_wave,                         &
!$OMP& i1v_wave,j1v_wave,xdpv_wave,ydpv_wave,                         &
!$OMP& t,nbouc,limin,limax,liminp1,limaxp1,ljmin,ljmax,               &
!$OMP& ljminp1,limaxp2,ljmaxp2,iscreenlog,ierrorlog,iwarnlog          &
!$OMP )

   dbug=.false.
   l_pb = .false. ! No problem

   radeg = pi/180.0_rsh

   twave=t
   torigin=tref
   torigin=0.0_rlg
   ! include time lag if time origine is not 01/01/1900 in wave file
   !torigin=tool_datosec('01/01/1900 00:00:00')-tool_datosec('01/01/0000 00:00:00')
#ifdef key_menor
   torigin=tool_datosec('01/01/1900 00:00:00')-tool_datosec('01/01/0000 00:00:00')
#endif

   IF (dbug) print *,'ENTER WAVEREAD'

   IF (nbouc==1) THEN ! START if (nbouc==1) then -init-
   !!----------------------------------------------------------------------
   !! * Open NetCDF file, read lat and lon -onr-
   !!----------------------------------------------------------------------

      iwmin0=1

!$OMP single
     CALL ionc4_openr(file_wave,l_out_nc4par=.true.)
     CALL ionc4_corres(file_wave,cdf_id_wave)
     IF (dbug) print *,'cfd_id',cdf_id_wave
     ixwave=ionc4_flimax(cdf_id_wave)
     IF (dbug) print *,'ixwave',ixwave
     jxwave=ionc4_fljmax(cdf_id_wave)  
     IF (dbug) print *,'jxwave',jxwave

     ALLOCATE(latwave(1:jxwave),lonwave(1:ixwave))
     CALL ionc4_read_lat(file_wave,latwave,1,jxwave)
     CALL ionc4_read_lon(file_wave,lonwave,1,ixwave)
     IF (dbug) print *,'latwave',latwave
     IF (dbug) print *,'lonwave',lonwave

   !!----------------------------------------------------------------------
   !! * Find size of 2D array to allocate -faa-
   !!----------------------------------------------------------------------
#ifdef key_grid_rotated
     !toseelater
     ! to estimate min and max indexes relative to the rotated grid
     ! to be used in atmospherical file
     ! Work indifferently if wave. model longitudes decrease or increase.
     ! dimensions of the wave file usefull for each cpu
     iwavemin=1               ! global dimensions of the wave file by default
     iwavemax=ixwave
     jwavemin=1
     jwavemax=jxwave
     DO i=1,ixwave-1
       IF ((lonmin-lonwave(i))*(lonmin-lonwave(i+1)) <= 0.0_rlg) iwavemin=i
       IF ((lonmax-lonwave(i)+epsilon)*(lonmax-lonwave(i+1)-epsilon) <= 0.0_rlg) iwavemax=i+1
     END DO
     ! Work indifferently if wave. model latitudes decrease or increase.
     DO j=1,jxwave-1
       IF ((latmin-latwave(j))*(latmin-latwave(j+1)) <= 0.0_rlg) jwavemin=j
       IF ((latmax-latwave(j)+epsilon)*(latmax-latwave(j+1)-epsilon) <= 0.0_rlg) jwavemax=j+1
     END DO

#else

     ! dimensions of the wave file usefull for each cpu
     iwavemin=1               ! global dimensions of the wave file by default
     iwavemax=ixwave
     jwavemin=1
     jwavemax=jxwave
     ! check the useful indexes of the wave file
     ! Works indifferently if wave. model longitudes decrease or increase.
     DO i=1,ixwave-1
       IF ((lon2d_g(liminp1,ljminp1)-lonwave(i)) * &
           (lon2d_g(liminp1,ljminp1)-lonwave(i+1)) <= 0.0_rlg) iwavemin=i
       IF ((lon2d_g(limaxp2,ljmaxp2)-lonwave(i)  +epsilon) * &
           (lon2d_g(limaxp2,ljmaxp2)-lonwave(i+1)-epsilon) <= 0.0_rlg) iwavemax=i+1
     END DO
     ! Work indifferently if wave. model latitudes decrease or increase.
     DO j=1,jxwave-1
       IF ((lat2d_g(liminp1,ljminp1)-latwave(j)) * &
           (lat2d_g(liminp1,ljminp1)-latwave(j+1)) <= 0.0_rlg) jwavemin=j
       IF ((lat2d_g(limaxp2,ljmaxp2)-latwave(j)  +epsilon) * & 
           (lat2d_g(limaxp2,ljmaxp2)-latwave(j+1)-epsilon) <= 0.0_rlg) jwavemax=j+1
     END DO

#endif

     IF (l_wave_marsgrid) THEN  ! the grid of the wave file is equal to the MARS grid
                            ! but on 1:imax+1,1:jmax+1
       iwmin0=0
       iwavemin=limin
       iwavemax=limax
       jwavemin=ljmin
       jwavemax=ljmax
     END IF

     DO i=1,ixwave
       IF(lonwave(i) >= 180.0_rlg) lonwave(i)=lonwave(i)-360.0_rlg
     END DO

     IF (iwavemin==iwavemax .OR. jwavemin==jwavemax) THEN
       WRITE(ierrorlog,*) 'The size of the domain read from the wave field is not large enough'
       WRITE(ierrorlog,*) 'iwavemin =',iwavemin,' iwavemax =',iwavemax
       WRITE(ierrorlog,*) 'jwavemin =',jwavemin,' jwavemax =',jwavemax
       WRITE(ierrorlog,*) 'You use too many cpus for a so coarse wave grid'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF

     IF_MPI (MASTER) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) 'Waves file : spatial limits'
     WRITE(iscreenlog,*) 'iwavemin = ',iwavemin
     WRITE(iscreenlog,*) 'iwavemax = ',iwavemax
     WRITE(iscreenlog,*) 'jwavemin = ',jwavemin
     WRITE(iscreenlog,*) 'jwavemax = ',jwavemax
     WRITE(iscreenlog,*) ' '
     ENDIF_MPI

#ifdef key_sflx_interp_scrip
     IF (iwavemin > 2) THEN
       iwavemin = iwavemin - 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Eastern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (jwavemin > 2) THEN
       jwavemin = jwavemin - 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Southern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (iwavemax < ixwave - 1) THEN
       iwavemax = iwavemax + 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Western boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (jwavemax < jxwave - 1) THEN
       jwavemax = jwavemax + 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Northern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
#endif

     ALLOCATE(tab1(iwavemin:iwavemax,jwavemin:jwavemax))
     ALLOCATE(tab2(iwavemin:iwavemax,jwavemin:jwavemax))
     tab1(:,:)=0.0_riosh
     tab2(:,:)=0.0_riosh

#ifdef key_sflx_interp_scrip
     ALLOCATE(lonwave2d(iwavemin:iwavemax,jwavemin:jwavemax))
     ALLOCATE(latwave2d(iwavemin:iwavemax,jwavemin:jwavemax))
     ALLOCATE(masksrc(iwavemin:iwavemax,jwavemin:jwavemax))
     ALLOCATE(tabvar2d(iwavemin:iwavemax,jwavemin:jwavemax))

     ALLOCATE(tabinterp2d(limin:limax,ljmin:ljmax))
     ALLOCATE(maskdst(limin:limax,ljmin:ljmax))

     masksrc(:,:)=.true.
     maskdst(:,:)=.true.
     tabvar2d(:,:)=0.0_rlg
#endif

!$OMP end single

   ! -2- Ponderation weights for  the spatial interpolation
   ! ------------------------------------------------------

     dgwave =ABS(lonwave(2)-lonwave(1))
     dfiwave=ABS(latwave(2)-latwave(1))

#ifdef key_sflx_interp_scrip

!$OMP single
     DO j=jwavemin,jwavemax
       lonwave2d(iwavemin:iwavemax,j)=lonwave(iwavemin:iwavemax)
     ENDDO
     DO i=iwavemin,iwavemax
       latwave2d(i,jwavemin:jwavemax)=latwave(jwavemin:jwavemax)
     ENDDO

     nxfin=limax-limin+1
     nyfin=ljmax-ljmin+1
!$OMP end single

     CALL get_remap_bicub(latwave2d,REAL(lat2d(limin:limax,ljmin:ljmax),rlg),lonwave2d,REAL(lon2d(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix,src_add,dst_add)
     CALL get_remap_bicub(latwave2d,REAL(lat2du(limin:limax,ljmin:ljmax),rlg),lonwave2d,REAL(lon2du(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix_u,src_add_u,dst_add_u)
     CALL get_remap_bicub(latwave2d,REAL(lat2dv(limin:limax,ljmin:ljmax),rlg),lonwave2d,REAL(lon2dv(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix_v,src_add_v,dst_add_v)

#else

   !!----------------------------------------------------------------------
   !! * Locate MARS cell from WAVE cell -lmw-
   !!----------------------------------------------------------------------
!$OMP do schedule(static)
     DO j=ljminp1,ljmax
       DO i=liminp1,limax
         xm = (lon2d(i,j)-lonwave(1))/dgwave+1        !pour les points xe
         ym = (lat2d(i,j)-latwave(1))/dfiwave+1
         xdpxe_wave(i,j) = xm - REAL(INT(xm),rsh)
         ydpxe_wave(i,j) = ym - REAL(INT(ym),rsh)
         i1xe_wave(i,j) = INT(xm)
         j1xe_wave(i,j) = INT(ym)

         xm = (lon2du(i,j)-lonwave(1))/dgwave+1        !pour les points u 
         ym = (lat2du(i,j)-latwave(1))/dfiwave+1
         xdpu_wave(i,j) = xm - REAL(INT(xm),rsh)
         ydpu_wave(i,j) = ym - REAL(INT(ym),rsh)
         i1u_wave(i,j) = INT(xm)
         j1u_wave(i,j) = INT(ym)

         xm = (lon2dv(i,j)-lonwave(1))/dgwave+1        !pour les points v 
         ym = (lat2dv(i,j)-latwave(1))/dfiwave+1
         xdpv_wave(i,j) = xm - REAL(INT(xm),rsh)
         ydpv_wave(i,j) = ym - REAL(INT(ym),rsh)
         i1v_wave(i,j) = INT(xm)
         j1v_wave(i,j) = INT(ym)
       END DO
     END DO
!$OMP end do

#endif

   ! Time index in the wave file
   ! ---------------------------

     IF_MPI (MASTER) THEN

!$OMP single
     idimt=ionc4_read_dimt(file_wave)
     ilec_wave=0 
     tnc1_wave=0.0_rlg
     tnc2_wave=0.0_rlg
     DO WHILE((twave-(tnc1_wave-torigin))*(twave-(tnc2_wave-torigin)) > 0.0_rlg .AND. ilec_wave < idimt)
       ilec_wave=ilec_wave+1
       IF (ilec_wave == idimt) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'All records are read, dates are not compatible with simulation time.'
         WRITE(ierrorlog,*) 'Make sure your wave file is correct: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'Make sure the origin time is 01/01/1900 00:00:00 in the wave file.'
         WRITE(ierrorlog,*) 'If not, modify variable "torigin" in the routine SFLXREAD.F90.'
         WRITE(ierrorlog,*) ''
         tdate = tool_sectodat(twave)
         WRITE(ierrorlog,*) 't in MARS model =',tdate
         tdate = tool_sectodat(tnc1_wave-torigin)
         WRITE(ierrorlog,*) 't1 in atmospherical file =',tdate
         tdate = tool_sectodat(tnc2_wave-torigin)
         WRITE(ierrorlog,*) 't2 in atmospherical file =',tdate
         WRITE(ierrorlog,*) 'Simulation stopped.'
         WRITE(ierrorlog,*) ''
         l_pb = .true.
       END IF
       IF (.NOT. l_pb) THEN
         CALL ionc4_read_time(file_wave,ilec_wave,tnc1_wave)
!         tnc1_wave=tnc1_wave*86400.0_rlg
         CALL ionc4_read_time(file_wave,ilec_wave+1,tnc2_wave)
!        tnc2_wave=tnc2_wave*86400.0_rlg
       END IF
     END DO
     ilecmem_wave=ilec_wave
     WRITE(iscreenlog,*) 'ilec_wave',ilec_wave
!$OMP end single

     ENDIF_MPI

     ! the MASTER sends informations to every cpus
OMPMPI barrier
OMPMPI master
     CALL_MPI BCAST_MPI(l_pb)
     CALL_MPI BCAST_MPI(idimt)
     CALL_MPI BCAST_MPI(ilec_wave)
     CALL_MPI BCAST_MPI(ilecmem_wave)
     CALL_MPI BCAST_MPI(tnc1_wave)
     CALL_MPI BCAST_MPI(tnc2_wave)
OMPMPI end master
OMPMPI barrier
OMPMPI flush(l_pb,idimt,ilec_wave,ilecmem_wave,tnc1_wave,tnc2_wave)

     IF (l_pb) THEN
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF

     ! -3- lecture des tableaux deux indices qui encadrent le pas de temps courant

   !!----------------------------------------------------------------------
   !! * Read and spatial interpolate of 2 time step of input file -ri2-
   !!----------------------------------------------------------------------
   !! -nwq-
     IF (l_wave_rdir) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_dir),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_dir),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_dir1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_dir2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_dir1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_dir2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_dir1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_dir2)
#endif
       END IF

     END IF

     IF (l_wave_rhs) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_hs),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_hs),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_hs1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_hs2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_hs1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_hs2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_hs1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_hs2)
#endif
       END IF

     END IF


     IF (l_wave_rubot) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_ubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_ubot),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_ubot1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_ubot2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_ubot1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_ubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_ubot1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_ubot2)
#endif
       END IF

     END IF

     IF (l_wave_ruubot .AND. l_wave_rvubot) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_uubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_uubot),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_uubot1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_uubot2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_uubot1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_uubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_uubot1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_uubot2)
#endif
       END IF

!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_vubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_vubot),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_vubot1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_vubot2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_vubot1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_vubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_vubot1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_vubot2)
#endif
       END IF

     END IF

     IF (l_wave_rfoc) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_foc),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_foc),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_foc1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_foc2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_foc1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_foc2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_foc1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_foc2)
#endif
       END IF

     END IF


     IF (l_wave_rtp) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_tp),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_tp),tab2,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_tp1(limin:limax,ljmin:ljmax)=tab1(:,:)
         wave_tp2(limin:limax,ljmin:ljmax)=tab2(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_tp1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
         tabvar2d(:,:)=tab2(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_tp2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_tp1)
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab2,wave_tp2)
#endif
       END IF

     END IF

!$OMP single
       DEALLOCATE(tab1,tab2,latwave,lonwave)
#ifdef key_sflx_interp_scrip
       DEALLOCATE(lonwave2d,latwave2d)
#endif
!$OMP end single

   !!----------------------------------------------------------------------
   !! * Checkout units -cou-
   !!----------------------------------------------------------------------
!$OMP single
     ncerror=0

     IF (l_wave_rdir) THEN
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_dir1(i,j),wave_dir2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_dir1(i,j),wave_dir2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<0_rsh).OR.(tabmax>360_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the direction of the wave is not expressed in [degree].'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_dir)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF

     IF (l_wave_rhs) THEN
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_hs1(i,j),wave_hs2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_hs1(i,j),wave_hs2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<0_rsh).OR.(tabmax>20_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'wave hight (Hs) is not expressed in [m].'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_hs)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF

     IF (l_wave_rfoc) THEN
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_foc1(i,j),wave_foc2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_foc1(i,j),wave_foc2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<0.0_rsh).OR.(tabmax>20.0_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the coefficient of energy flux of wave to ocean expressed in [] is not correct.'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_foc)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF

     IF (l_wave_rubot) THEN
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_ubot1(i,j),wave_ubot2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_ubot1(i,j),wave_ubot2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<0.0_rsh).OR.(tabmax>1.0_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the orbital velocity expressed in [m/s] is not correct.'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_ubot)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF

     IF (l_wave_ruubot .AND. l_wave_rvubot) THEN
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_uubot1(i,j),wave_uubot2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_uubot1(i,j),wave_uubot2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<-1.0_rsh).OR.(tabmax>1.0_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the Ubot (U) expressed in [m/s] is not correct.'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_uubot)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_vubot1(i,j),wave_vubot2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_vubot1(i,j),wave_vubot2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       IF ((tabmin<-1.0_rsh).OR.(tabmax>1.0_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the Ubot (V) expressed in [m/s] is not correct.'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_vubot)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF



     IF (l_wave_rtp) THEN

       tabmin=1000000.0_rsh
       tabmax=-10000000.0_rsh
       DO j=ljminp1,ljmax
         DO i=liminp1,limax
           IF (h0(i,j)/=-valmanq) THEN
             tabmin=MIN(tabmin,(MIN(wave_tp1(i,j),wave_tp2(i,j))))
             tabmax=MAX(tabmax,(MAX(wave_tp1(i,j),wave_tp2(i,j))))
           ENDIF
         ENDDO
       ENDDO
       ! print*,'tabmin tp=SQRT',tabmin,tabmax
       IF ((tabmin<0_rsh).OR.(tabmax>50_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
         WRITE(ierrorlog,*) 'In the waves file: ',TRIM(file_wave)
         WRITE(ierrorlog,*) 'the peak wave period expressed in [s] is not correct.'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_wave_tp)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       END IF
     END IF

     IF (ncerror>0) THEN
       WRITE(ierrorlog,*) ' '
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF
!$OMP end single

   END IF ! ENDOF if (nbouc==1) then -init-

   ! deroulement normal

   !!----------------------------------------------------------------------
   !! * Find time step in the file that exceed computational time step -fse-
   !!----------------------------------------------------------------------
!$OMP single
   DO WHILE((twave-(tnc1_wave-torigin))*(twave-(tnc2_wave-torigin)) > 0.0_rlg .AND. ilec_wave < idimt)
     ilec_wave=ilec_wave+1 
     IF (ilec_wave == idimt) THEN
       WRITE(ierrorlog,*) ' '
       WRITE(ierrorlog,*) 'ERROR: routine WAVESREAD.F90'
       WRITE(ierrorlog,*) 'All records are already read.'
       WRITE(ierrorlog,*) 'The waves file is not long enough: ',TRIM(file_wave)
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF
     CALL ionc4_read_time(file_wave,ilec_wave,tnc1_wave)
!     tnc1_wave=tnc1_wave*86400.0_rlg
     CALL ionc4_read_time(file_wave,ilec_wave+1,tnc2_wave)
!     tnc2_wave=tnc2_wave*86400.0_rlg
   END DO
!$OMP end single

   IF (ilecmem_wave/=ilec_wave) THEN

     IF (nbouc/=1) THEN
!$OMP single
       ALLOCATE(tab1(iwavemin:iwavemax,jwavemin:jwavemax))
!$OMP end single
     END IF

     ilecmem_wave=ilec_wave
   !!----------------------------------------------------------------------
   !! * We transfert array 2 in array 1 -2i1-
   !!----------------------------------------------------------------------
!$OMP do schedule(static)
     DO j=ljmin,ljmax
   !! -nwq-
       IF (l_wave_rdir) wave_dir1(:,j)=wave_dir2(:,j)
       IF (l_wave_rhs)  wave_hs1(:,j) =wave_hs2(:,j)
       IF (l_wave_rubot) wave_ubot1(:,j)=wave_ubot2(:,j)+1.0e-6
       IF (l_wave_ruubot) wave_uubot1(:,j)=wave_uubot2(:,j)+1.0e-6
       IF (l_wave_rvubot) wave_vubot1(:,j)=wave_vubot2(:,j)+1.0e-6
       IF (l_wave_rtp) wave_tp1(:,j)=wave_tp2(:,j)+1.0e-6
       IF (l_wave_rfoc) wave_foc1(:,j)=wave_foc2(:,j)
     END DO
!$OMP end do

   !!----------------------------------------------------------------------
   !! * Read and spatial interpolate next time step of input file while computational time step is not exceed -rin-
   !!----------------------------------------------------------------------
   !! -nwq-
     IF (l_wave_rdir) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_dir),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_dir2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_dir2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_dir2)
#endif
       END IF

     END IF

     IF (l_wave_rhs) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_hs),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_hs2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_hs2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_hs2)
#endif
       END IF

     END IF

     IF (l_wave_rubot) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_ubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_ubot2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_ubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_ubot2)
#endif
       END IF

     END IF

     IF (l_wave_ruubot .AND. l_wave_rvubot) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_uubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_uubot2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_uubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_uubot2)
#endif
       END IF

!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_vubot),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_vubot2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_vubot2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_vubot2)
#endif
       END IF

     END IF

     IF (l_wave_rfoc) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_foc),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_foc2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_foc2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_foc2)
#endif
       END IF

     END IF
     
     IF (l_wave_rtp) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_wave,TRIM(name_wave_tp),tab1,iwavemin,iwavemax,jwavemin,jwavemax,ilec_wave+1, &
                              iwmin0,iwmin0,fillvar=0.0_rsh)
!$OMP end single

       IF (l_wave_marsgrid) THEN  ! no interpolation : the grid of the wave file is equal to the MARS grid
!$OMP single
         wave_tp2(limin:limax,ljmin:ljmax)=tab1(:,:)
!$OMP end single
       ELSE
#ifdef key_sflx_interp_scrip
!$OMP single
         tabvar2d(:,:)=tab1(:,:)
         CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
         wave_tp2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
!$OMP end single
#else
         CALL tool_interp2d_wave(i1xe_wave,j1xe_wave,xdpxe_wave,ydpxe_wave,tab1,wave_tp2)
#endif
       END IF

     END IF

!$OMP single
     DEALLOCATE(tab1)
!$OMP end single

   END IF  ! test ilecmem_wave/=ilec_wave

   !!----------------------------------------------------------------------
   !! * Time linear interpolation -tli-
   !!----------------------------------------------------------------------
   dt1=(tnc2_wave-torigin-twave)/(tnc2_wave-tnc1_wave)
   dt2=1.0_rlg-dt1

!$OMP do schedule(static)
   !! -nwq-
   DO j=ljmin,ljmax
     IF (l_wave_rdir) wave_dir(limin:limax,j) =  wave_dir1(limin:limax,j)*dt1   &
           + wave_dir2(limin:limax,j)*dt2
     IF (l_wave_rhs) wave_hs(limin:limax,j)   =  wave_hs1(limin:limax,j)*dt1   &
           + wave_hs2(limin:limax,j)*dt2
     IF (l_wave_rubot) wave_ubot(limin:limax,j) =  wave_ubot1(limin:limax,j)*dt1   &
           + wave_ubot2(limin:limax,j)*dt2
     IF (l_wave_ruubot) wave_uubot(limin:limax,j) =  wave_uubot1(limin:limax,j)*dt1   &
           + wave_uubot2(limin:limax,j)*dt2
     IF (l_wave_rvubot) wave_vubot(limin:limax,j) =  wave_vubot1(limin:limax,j)*dt1   &
           + wave_vubot2(limin:limax,j)*dt2
     IF (l_wave_rfoc) wave_foc(limin:limax,j) =  wave_foc1(limin:limax,j)*dt1   &
           + wave_foc2(limin:limax,j)*dt2
     IF (l_wave_rtp) wave_tp(limin:limax,j) =  wave_tp1(limin:limax,j)*dt1   &
           + wave_tp2(limin:limax,j)*dt2
   END DO
!$OMP end do

   IF ((l_wave_ruubot .AND. l_wave_rvubot) .AND. (.NOT. l_wave_rubot)) THEN 
     DO j=ljmin,ljmax
        DO i=limin,limax
           wave_ubot(i,j)=sqrt(wave_uubot(i,j)**2+wave_vubot(i,j)**2)
        END DO
     END DO
   END IF

!$OMP end parallel

   !     !!----------------------------------------------------------------------
   !     !! * MPI managing for gradient computation -mmg-
   !     !!----------------------------------------------------------------------
   !     !! -nwq-
   !#ifdef key_MPI_2D
   !toseelater on ne fait les echanges que s il y a besoin. Donc voir quels indices
   !vont etre utiles: a garder en commentaire car ca peux etre utils.
   !   IF (l_wave_rdir) THEN
   !     CALL ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_dir(limin:limaxp1,ljmin:ljmax))
   !     CALL ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_dir(limin:limax,ljmin:ljmaxp1))
   !   END IF
   !   IF (l_wave_rhs) THEN
   !     CALL ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_hs(limin:limaxp1,ljmin:ljmax))
   !     CALL ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_hs(limin:limax,ljmin:ljmaxp1))
   !  END IF
   !   IF (l_wave_rfoc) THEN
   !     CALL ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_foc(limin:limaxp1,ljmin:ljmax))
   !     CALL ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_foc(limin:limax,ljmin:ljmaxp1))
   !   END IF
   !   IF (l_wave_rubot) THEN
   !     CALL ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmax,wave_bot(limin:limaxp1,ljmin:ljmax))
   !     CALL ex_j_rsh(0,1,1,limin,limax,ljmin,ljmaxp1,wave_bot(limin:limax,ljmin:ljmaxp1))
   !   END IF

   !#endif

   IF_MPI (MASTER) THEN
   IF (dbug.and.(mod(nbouc,2)==0)) THEN
   !! -nwq-
     print *,'dir=',sum(sum(wave_dir,1),1)/(size(wave_dir,1)*size(wave_dir,2))
     print *,'hs=',sum(sum(wave_hs,1),1)/(size(wave_hs,1)*size(wave_hs,2))
     print *,'ubot=',sum(sum(wave_ubot,1),1)/(size(wave_ubot,1)*size(wave_ubot,2))
     print *,'tp=',sum(sum(wave_tp,1),1)/(size(wave_tp,1)*size(wave_tp,2))
     print *,'foc=',sum(sum(wave_foc,1),1)/(size(wave_foc,1)*size(wave_foc,2))
   END IF
   IF (dbug) THEN
     print *,'OUT OF WAVEREAD'
   END IF
   ENDIF_MPI

  END SUBROUTINE wave_read

   !!======================================================================

#endif

END MODULE wave

