 MODULE trajinitsave
   !!======================================================================
   !!                   ***  MODULE trajinitsave  ***
   !! Initialisation and saving of particles position
   !! Read circle or rectangle boundaries to calculate initial position, 
   !! or directly real position in netcdf file
   !! Save in a netcdf file
   !&E       !  01-2012 (M. Huret) separation from traject3d
   !&E
   !&E Description :
   !&E    traj_init3d
   !&E    traj_save3d
   !&E
   !&E ** History :
   !&E       !  2011-10 (T. Odaka, M. Huret, V. Garnier) Introduction of MPI
   !&E       !  2012-01 (M. Huret) Read netcdf
   !&
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters,  ONLY : imin,imax,jmin,jmax,kmax,ljmin,ljmax,ljmaxp1,  &
                           limaxp1,liminm1,limaxp2,ljminm1,ljmaxp2,limax, &
                           rsh,rlg,riosh,lchain,limin,liminm2,ljminm2,    &
                           rg_valmanq_io,dg_valmanq_io
   USE ionc4,       ONLY : ionc4_createfile_traj,ionc4_torigine,ionc4_tunite, &
                           ionc4_createvar_traj,ionc4_write_time,ionc4_sync, &
                           ionc4_write_trajt,ionc4_openr,ionc4_read_dimtraj,  &
                           ionc4_read_trajt,ionc4_close
   USE comtraj
   USE_MPI toolmpi, ONLY : iproc,nprocs

   IMPLICIT NONE
   PRIVATE

#if defined key_ibm || defined key_trajec3d

   !! * Accessibility
   PUBLIC traj_save3d          ! routine called by traject3d
   PUBLIC traj_init3d          ! routine called by step.F90, ibm_init.F90

   !! * Shared module variables

   !! * Private variables


 CONTAINS

   !!======================================================================

   SUBROUTINE traj_init3d(xe)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE traj_init3d  ***
   !&E
   !&E ** Purpose : Read file traject.dat to initialize trajectories variables.
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : step.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  12-2006 (M. Chiffle, V. Garnier)  Original code
   !&E       !  05-2010 (M. Sourisseau)  Introduction of derivate type
   !&E       !  01-2011 (M. Huret)  Adaptation to siggen
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,  ONLY : pi
   USE comvars2d,   ONLY : iscreenlog,tdeb3d,tfin,h0_g,radearth,lat2d_g,lon2du_g,lat2dv_g,ierrorlog
   USE trajectools, ONLY : h0int,xeint,loc_h0
#if defined key_siggen
   USE trajectools, ONLY : ztosiggen,hc_sigint
#endif
   USE_MPI toolmpi, ONLY : ADD_ALL_MPI_INT,exchange_vectcpu_int

   !! * Arguments
   REAL(KIND=rsh), DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2), INTENT( in )       :: xe

   !! * Local declarations
   LOGICAL                                     :: ex,l_posit
   CHARACTER(LEN=5)                            :: comment
   CHARACTER(LEN=19)                           :: dateread,tool_sectodat,comdate
   CHARACTER(LEN=lchain)                       :: rec
   CHARACTER(LEN=lchain), ALLOCATABLE, DIMENSION(:)  :: name_patch, filetrajin
   INTEGER                                     :: eof,i,j,k,nn,n,m,kk,l,ii
   INTEGER                                     :: IERR_MPI
   REAL(KIND=rsh)                              :: dxc,dyc  ! dx and dy at the center of the circle
   REAL(KIND=rsh)                              :: fi,g,radeg
   REAL(KIND=rsh)                              :: tool_latlon2i,tool_latlon2j,rx,ry,dl, &
                                                  d3,xtemp,ytemp,xe_lag,h0_lag,hc_sig_lag
   INTEGER, ALLOCATABLE, DIMENSION(:)          :: imin_patch,imax_patch,   &
                                                  jmin_patch,jmax_patch,   &
                                                  kmin_patch,kmax_patch,   &
                                                  nbpart_patch,nb_part_intro
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: ray_patch,depthnc
   REAL(KIND=rlg), ALLOCATABLE, DIMENSION(:)   :: lonnc,latnc
   INTEGER, ALLOCATABLE, DIMENSION(:)          :: istep_patch,jstep_patch, &
                                                  kstep_patch,ntrajnc
   REAL(KIND=rlg)                              :: tool_datosec,gmin,gmax,phimin,phimax
   REAL(KIND=rsh)                              :: kint,spos
   REAL(KIND=rsh), DIMENSION(0:kmax)           :: zpos
   INTEGER                                     :: kw,kwm,km
!  parameters for interpolation at particle location
   REAL(KIND=rsh)                       :: px,py
   INTEGER                              :: igg,idd,jbb,jhh,hlb,hlt,hrb,hrt,ipos,jpos

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter TRAJ_INIT3D routine'

   ! save into simu.log
   !-------------------
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) '**************************************************'
   WRITE(iscreenlog,*) '***************** TRAJ_INIT3D.F90 ****************'
   WRITE(iscreenlog,*) '**************************************************'
   WRITE(iscreenlog,*) ' '
   WRITE(iscreenlog,*) 'fichier definissant les caracteristiques des trajectoires : ',trim(file_trajec)

   ! Make sure the type of trajectory is correct
   eof = 0
   INQUIRE(file=file_trajec,exist=ex)
   IF (ex /= .true.) THEN
       print*,'Trajectory file does not exist'
       print*,'Check in paraspec.txt'
       print*,'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       stop
   END IF
   IF (itypetraj/=1 .AND. itypetraj/=2 .AND. itypetraj/=3) THEN
      print*,'Type of trajectory is not defined correctly'
      print*,'Must be 1 (circle patch) or 2 (rectangle patch)'
      print*,'Simulation stopped.'
      CALL_MPI MPI_FINALIZE(IERR_MPI)
      stop
   END IF

   ! Read trajec.dat file

   eof = 0
   nb_patch = 0

   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Initialize circle patches
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   IF (itypetraj==1 .AND. ex) THEN
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO
     DO WHILE(eof==0)
       READ(49,'(a)',iostat=eof) !lect  namepatch
       READ(49,'(a)',iostat=eof) !lect tbegtraj
       READ(49,'(a)',iostat=eof) !lect tendtraj
       READ(49,*,iostat=eof)  ! dtsave_traj
       READ(49,*,iostat=eof)  !location
       READ(49,*,iostat=eof)  !center of the circle
       READ(49,*,iostat=eof)  !ray size
       READ(49,*,iostat=eof)  !number of particles
       READ(49,*,iostat=eof)  !ktachemin,ktachemax
       READ(49,*,iostat=eof)  !kstep_patch
       READ(49,*,iostat=eof)  !nb_part_intro
       READ(49,*,iostat=eof)  !itypevert
       READ(49,'(a)',iostat=eof) !****
       IF (eof==0) nb_patch = nb_patch+1
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)
     WRITE(iscreenlog,*) 'Number of circular patches for estimation of trajectories = ',nb_patch

   ! allocate arrays for trajectories variables
     ALLOCATE( name_patch(nb_patch),filetraj3dout(nb_patch))
     ALLOCATE( itypevert(nb_patch) )
     ALLOCATE( nb_part(nb_patch), dtsave_traj(nb_patch) )
     ALLOCATE( tbegtraj(nb_patch),tendtraj(nb_patch),tsavtraj(nb_patch) )
     ALLOCATE( imin_patch(nb_patch),jmin_patch(nb_patch) )
     ALLOCATE( kmin_patch(nb_patch),kmax_patch(nb_patch) )
     ALLOCATE( kstep_patch(nb_patch))
     ALLOCATE( nb_part_intro(nb_patch))
     ALLOCATE( nbpart_patch(nb_patch),ray_patch(nb_patch) )
#ifdef key_MPI_2D
     ALLOCATE( nb_part_percpu(nb_patch,0:nprocs-1) )
     nb_part_percpu(:,:)=0
#endif

   ! read trajectories variables
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO

     DO n = 1,nb_patch

     ! Read name of patch
       READ(49,'(a)',iostat=eof) name_patch(n)

     ! Read starting date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tbegtraj(n) = tool_datosec(dateread)
       IF(tbegtraj(n)<tdeb3d .OR. tbegtraj(n)>tfin) THEN
         comdate=tool_sectodat(tdeb3d)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Starting date ,',trim(dateread),', is uncorrect,'
         write(iscreenlog,*) 'WARNING : consequently trajectory starts with departure of 3D run on', comdate
         tbegtraj(n)=tdeb3d
       ENDIF
       tsavtraj(n)=tbegtraj(n)

     ! Read ending date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tendtraj(n) = tool_datosec(dateread)
       IF(tendtraj(n)<tdeb3d .OR. tendtraj(n)>tfin) THEN
         comdate=tool_sectodat(tfin)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Ending date ,',trim(dateread),', is uncorrect,' 
         write(iscreenlog,*) 'WARNING : consequently trajectory ends with run on', comdate
         tendtraj(n)=tfin
       ENDIF

     ! Read time step for outputs
       READ(49,*,iostat=eof) dtsave_traj(n)
       comdate=tool_sectodat(tbegtraj(n))
       write(iscreenlog,*) 'PATCH NUMBER : ',n,', trajectory from ',trim(comdate)
       comdate=tool_sectodat(tendtraj(n))
       write(iscreenlog,*) '                                 to ',trim(comdate),' with a ',dtsave_traj(n), ' hours time step'

     ! Location of initial patch in indexes or long. lati. ?
       READ(49,*,iostat=eof) l_posit

     ! Read location of initial patch
       IF (l_posit) THEN
         READ(49,*,iostat=eof) imin_patch(n),jmin_patch(n)
       ELSE
         READ(49,*,iostat=eof) gmin,phimin
         imin_patch(n)=NINT(tool_latlon2i(gmin,phimin))
         jmin_patch(n)=NINT(tool_latlon2j(gmin,phimin))
         IF ( imin_patch(n)<imin .OR. imin_patch(n)>imax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           imin_patch(n)=MAX(imin_patch(n),imin)
           imin_patch(n)=MIN(imin_patch(n),imax)
           print*,'The longitude of patch number ',n,'is :',imin_patch(n)
         END IF
         IF ( jmin_patch(n)<jmin .OR. jmin_patch(n)>jmax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           jmin_patch(n)=MAX(jmin_patch(n),jmin)
           jmin_patch(n)=MIN(jmin_patch(n),jmax)
           print*,'The latitude of patch number ',n,'is :',jmin_patch(n)
         END IF
       END IF

     ! Read ray of initial patch
       READ(49,*,iostat=eof) ray_patch(n)

     ! Read nbpart_patch
       READ(49,*,iostat=eof) nbpart_patch(n)

     ! Read depth of initial patch (read as immersion !)
       READ(49,*,iostat=eof) kmin_patch(n),kmax_patch(n)

     ! Read resolution depth of initial patch
       READ(49,*,iostat=eof) kstep_patch(n)

     ! Correct if depth not set as imersion
       kstep_patch(n)=ABS(kstep_patch(n))
       kmin_patch(n)=ABS(kmin_patch(n))
       kmax_patch(n)=ABS(kmax_patch(n))

       ! Number of particles set at each initial position
       READ(49,*,iostat=eof) nb_part_intro(n)

     ! Type of vertical behavior (integer):
         ! itypevert(n) = 0 if constant depth
         ! itypevert(n) = -1 if no random walk (vertical advection only)
         ! itypevert(n) = 1 if random walk (advection + diffusion)
       READ(49,*,iostat=eof) itypevert(n)


     ! Read output file
       READ(49,'(a)',iostat=eof) rec
       kk=index(rec,',|')
       IF (kk /= 0 ) THEN
         filetraj3dout(n)=rec(1:kk-1)
       ELSE
         filetraj3dout(n)=rec
       END IF
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)

     ! Estimate number of particle inside the circle patch

     nbmax_part=0
     DO n = 1,nb_patch

       ! mean spatial size of the cell 
       radeg = pi/180.0_rsh
       fi = REAL(lat2d_g(imin_patch(n),jmin_patch(n)),rsh)
       IF (imin_patch(n)==imin) THEN
         dxc=radearth*COS(fi*radeg)*REAL(lon2du_g(imin_patch(n)+1,jmin_patch(n))-lon2du_g(imin_patch(n),jmin_patch(n)),rsh)*radeg
       ELSE
         dxc=radearth*COS(fi*radeg)*REAL(lon2du_g(imin_patch(n),jmin_patch(n))- lon2du_g(imin_patch(n)-1,jmin_patch(n)),rsh)*radeg
       END IF
       IF (jmin_patch(n)==jmin) THEN
         dyc=radearth*REAL(lat2dv_g(imin_patch(n),jmin_patch(n)+1)-lat2dv_g(imin_patch(n),jmin_patch(n)),rsh)*radeg
       ELSE
         dyc=radearth*REAL(lat2dv_g(imin_patch(n),jmin_patch(n))-lat2dv_g(imin_patch(n),jmin_patch(n)-1),rsh)*radeg
       END IF

       nn=INT(sqrt(4/pi*nbpart_patch(n)))+1
       dl=2*ray_patch(n)/nn
       m=0                               ! initialize nb_part for each patch
       IF (h0_g(imin_patch(n),jmin_patch(n)) < kmin_patch(n)) THEN
         nb_part(n)=0
         print*,'Patch on land or too deep'
         print*,'Check patch number',n
         print*,'in file "',trim(file_trajec),'"'
         print*,'Simulation stopped.'
         CALL_MPI MPI_FINALIZE(IERR_MPI)
         stop
       ELSE
         DO j=1,nn
         DO i=1,nn
           rx=-ray_patch(n)+real(i-1)*dl
           ry=-ray_patch(n)+real(j-1)*dl
           IF (sqrt(rx*rx+ry*ry) <= ray_patch(n)) THEN
              xtemp=imin_patch(n)+rx/dxc
              ytemp=jmin_patch(n)+ry/dyc
              IF_MPI (limin<=NINT(xtemp) .AND. NINT(xtemp)<=limax   .AND. ljmin<=NINT(ytemp) .AND. NINT(ytemp)<=ljmax ) THEN
                DO k=kmin_patch(n),kmax_patch(n),kstep_patch(n)
                  IF (h0_g(NINT(xtemp),NINT(ytemp)) > k) THEN
                    CALL loc_h0(xtemp,ytemp,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    d3=h0_lag+xe_lag
                    IF (d3 > k) THEN
                       m=m+nb_part_intro(n)
                    END IF
                  END IF
                END DO
              ENDIF_MPI
           END IF
         END DO
         END DO
       ENDIF
#ifdef key_MPI_2D
       nb_part_percpu(n,iproc)=m
       ! get the number of particles per cpus
       CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
       nb_part(n)=m
       CALL_MPI ADD_ALL_MPI_INT(nb_part(n))
       nbmax_part=MAX(nbmax_part,nb_part(n))    ! maximum number of particles in the largest patch
       WRITE(iscreenlog,*) 'Number of particles inside circle number ',n
       WRITE(iscreenlog,*) 'between depth ',kmin_patch(n),' and ',kmax_patch(n)
       WRITE(iscreenlog,*) ' = ',nb_part(n)
#if ! defined key_MPI_2D
       IF ( m==0 ) THEN
         print*,'Problem with patch number',n
         print*,'in file "',trim(file_trajec),'"'
         print*,'Simulation stopped.'
         CALL_MPI MPI_FINALIZE(IERR_MPI)
         stop
       END IF
#endif
     END DO
     WRITE(iscreenlog,*) 'The maximum number of particles is ',nbmax_part

     ! Allocate particles
     ALLOCATE(part(nb_patch,nbmax_part))

     DO n = 1,nb_patch

       part(n,:)%date_orig=tbegtraj(n)
       part(n,:)%itypevert=itypevert(n)
       part(n,:)%flag=0.0_rsh
       m=0

#ifdef key_MPI_2D
       part(n,:)%inoutflag=.FALSE.
       part(n,:)%limitbye=0

       IF (iproc /= 0) THEN
         m = SUM(nb_part_percpu(n,0:iproc-1))
       ENDIF

#endif

       ! mean spatial size of the cell 
       radeg = pi/180.0_rsh
       fi = REAL(lat2d_g(imin_patch(n),jmin_patch(n)),rsh)
       IF (imin_patch(n)==imin) THEN
         dxc=radearth*COS(fi*radeg)*REAL(lon2du_g(imin_patch(n)+1,jmin_patch(n))-lon2du_g(imin_patch(n),jmin_patch(n)),rsh)*radeg
       ELSE
         dxc=radearth*COS(fi*radeg)*REAL(lon2du_g(imin_patch(n),jmin_patch(n))- lon2du_g(imin_patch(n)-1,jmin_patch(n)),rsh)*radeg
       END IF
       IF (jmin_patch(n)==jmin) THEN
         dyc=radearth*REAL(lat2dv_g(imin_patch(n),jmin_patch(n)+1)-lat2dv_g(imin_patch(n),jmin_patch(n)),rsh)*radeg
       ELSE
         dyc=radearth*REAL(lat2dv_g(imin_patch(n),jmin_patch(n))-lat2dv_g(imin_patch(n),jmin_patch(n)-1),rsh)*radeg
       END IF

       nn=INT(sqrt(4/pi*nbpart_patch(n)))+1
       dl=2*ray_patch(n)/nn
       DO j=1,nn
       DO i=1,nn
         rx=-ray_patch(n)+real(i-1)*dl
         ry=-ray_patch(n)+real(j-1)*dl
         IF (sqrt(rx*rx+ry*ry) <= ray_patch(n)) THEN
            xtemp=imin_patch(n)+rx/dxc
            ytemp=jmin_patch(n)+ry/dyc
            IF_MPI (limin<=NINT(xtemp) .AND. NINT(xtemp)<=limax .AND. ljmin<=NINT(ytemp) .AND. NINT(ytemp)<=ljmax ) THEN
            DO k=kmin_patch(n),kmax_patch(n),kstep_patch(n)
               IF (h0_g(NINT(xtemp),NINT(ytemp)) > k) THEN
                  m=m+nb_part_intro(n)
                  ! i index of initial location
                  part(n,m-nb_part_intro(n)+1:m)%xpos=imin_patch(n)+rx/dxc
                  ! j index of initial location
                  part(n,m-nb_part_intro(n)+1:m)%ypos=jmin_patch(n)+ry/dyc
#ifdef key_MPI_2D
                  part(n,m-nb_part_intro(n)+1:m)%inoutflag=.TRUE.
!#else
!                  DO ii=m-nb_part_intro(n)+1,m
!                    part(n,ii)%num = ii
!                  END DO
#endif
                  ! total depth at particle s location
                  CALL loc_h0(part(n,m)%xpos,part(n,m)%ypos,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                  xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                  h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                  d3=h0_lag+xe_lag
                  IF (d3 > k) THEN
                  
                     part(n,m-nb_part_intro(n)+1:m)%zpos=k ! immersion depth
                     ! sigma position
#ifdef key_siggen 
                     kint=-float(k)+xe_lag ! switch from immersion to real z
                     hc_sig_lag=hc_sigint(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                     CALL ztosiggen(kint,spos,xe_lag,h0_lag,hc_sig_lag)
                     part(n,m-nb_part_intro(n)+1:m)%spos=spos
                     part(n,m-nb_part_intro(n)+1:m)%hc=hc_sig_lag
#else
                     part(n,m-nb_part_intro(n)+1:m)%spos=-(k/d3)
#endif
                     part(n,m-nb_part_intro(n)+1:m)%h0=h0_lag
                     part(n,m-nb_part_intro(n)+1:m)%xe=xe_lag
                     part(n,m-nb_part_intro(n)+1:m)%d3=d3
                  ELSE
                     m=m-nb_part_intro(n)
                  END IF
               END IF
            END DO
            ENDIF_MPI
         END IF
       END DO
       END DO

!#ifdef key_MPI_2D
       ! Assign the number of the particle
!       IF (iproc==0) THEN
!         m=0
!       ELSE
!         m = SUM(nb_part_percpu(n,0:iproc-1))
!       ENDIF
!       DO i=1,nb_part_percpu(n,iproc)
!         m = m+1
!         part(n,i)%num = m
!       END DO
!#endif

     END DO ! loop on patches

     DEALLOCATE(nbpart_patch,ray_patch)
     DEALLOCATE(imin_patch,jmin_patch,kmin_patch,kmax_patch,kstep_patch)


   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Initialize rectangular patches
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ELSEIF (itypetraj==2 .AND. ex) THEN
     WRITE(iscreenlog,*) 'debut init trajectories = '   
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO
     DO WHILE(eof==0)
       READ(49,'(a)',iostat=eof) !lect  namepatch
       READ(49,'(a)',iostat=eof) !lect tbegtraj
       READ(49,'(a)',iostat=eof) !lect tendtraj
       READ(49,*,iostat=eof)  ! dtsave_traj
       READ(49,*,iostat=eof)  !location
       READ(49,*,iostat=eof)  !itachemin,..
       READ(49,*,iostat=eof)  !istep_patch,..
       READ(49,*,iostat=eof)  !ktachemin,ktachemax
       READ(49,*,iostat=eof)  !kstep_patch
       READ(49,*,iostat=eof)  !nb_part_intro
       READ(49,*,iostat=eof)  !itypevert
       READ(49,'(a)',iostat=eof) !****
       IF (eof==0) nb_patch = nb_patch+1
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)
     WRITE(iscreenlog,*) 'Number of rectangular patches for estimation of trajectories = ',nb_patch

   ! allocate arrays for trajectories variables
     ALLOCATE( name_patch(nb_patch),filetraj3dout(nb_patch) )
     ALLOCATE( itypevert(nb_patch))
     ALLOCATE( nb_part(nb_patch), dtsave_traj(nb_patch) )
     ALLOCATE( tbegtraj(nb_patch),tendtraj(nb_patch),tsavtraj(nb_patch) )
     ALLOCATE( imin_patch(nb_patch),imax_patch(nb_patch) )
     ALLOCATE( jmin_patch(nb_patch),jmax_patch(nb_patch) )
     ALLOCATE( kmin_patch(nb_patch),kmax_patch(nb_patch) )
     ALLOCATE( istep_patch(nb_patch),jstep_patch(nb_patch),kstep_patch(nb_patch) )
     ALLOCATE( nb_part_intro(nb_patch))
#ifdef key_MPI_2D
     ALLOCATE( nb_part_percpu(nb_patch,0:nprocs-1) )
     nb_part_percpu(:,:)=0
#endif

   ! read trajectories variables
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO

     DO n = 1,nb_patch

     ! Read name of patch
       READ(49,'(a)',iostat=eof) name_patch(n)

     ! Read starting date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tbegtraj(n) = tool_datosec(dateread)
       IF(tbegtraj(n)<tdeb3d .OR. tbegtraj(n)>tfin) THEN
         comdate=tool_sectodat(tdeb3d)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Starting date ,',trim(dateread),', is uncorrect (before 3D simulations starting) . '
         write(iscreenlog,*) 'WARNING : consequently trajectory starts with departure of 3D run on', comdate
         tbegtraj(n)=tdeb3d
       ENDIF
       tsavtraj(n)=tbegtraj(n)

     ! Read ending date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tendtraj(n) = tool_datosec(dateread)
       IF(tendtraj(n)<tdeb3d .OR. tendtraj(n)>tfin) THEN
         comdate=tool_sectodat(tfin)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Ending date ,',trim(dateread),', is uncorrect, '    
         write(iscreenlog,*) 'WARNING : consequently trajectory ends with run on', comdate
         tendtraj(n)=tfin
       ENDIF

     ! Read time step for outputs
       READ(49,*,iostat=eof) dtsave_traj(n)
       comdate=tool_sectodat(tbegtraj(n))
       write(iscreenlog,*) 'PATCH NUMBER : ',n,', trajectory from ',trim(comdate)
       comdate=tool_sectodat(tendtraj(n))
       write(iscreenlog,*) '                                 to ',trim(comdate),' with a ',dtsave_traj(n), ' hours time step'

     ! Spatial extension of initial patch in indexes or long. lati. ?
       READ(49,*,iostat=eof) l_posit

     ! Read spatial extension of initial patch
       IF (l_posit) THEN
         READ(49,*,iostat=eof) imin_patch(n),imax_patch(n),jmin_patch(n),jmax_patch(n)
       ELSE
         READ(49,*,iostat=eof) gmin,gmax,phimin,phimax
         imin_patch(n)=NINT(tool_latlon2i(gmin,phimin))
         imax_patch(n)=NINT(tool_latlon2i(gmax,phimax))
         jmin_patch(n)=NINT(tool_latlon2j(gmin,phimin))
         jmax_patch(n)=NINT(tool_latlon2j(gmax,phimax))
         IF ( imin_patch(n)<imin .OR. imin_patch(n)>imax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           imin_patch(n)=MAX(imin_patch(n),imin)
           imin_patch(n)=MIN(imin_patch(n),imax)
           print*,'The western longitude of patch number ',n,'is :',imin_patch(n)
         END IF
         IF ( jmin_patch(n)<jmin .OR. jmin_patch(n)>jmax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           jmin_patch(n)=MAX(jmin_patch(n),jmin)
           jmin_patch(n)=MIN(jmin_patch(n),jmax)
           print*,'The southern latitude of patch number ',n,'is :',jmin_patch(n)
         END IF
         IF ( imax_patch(n)<imin .OR. imax_patch(n)>imax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           imax_patch(n)=MAX(imax_patch(n),imin)
           imax_patch(n)=MIN(imax_patch(n),imax)
           print*,'The eastern longitude of patch number ',n,'is :',imax_patch(n)
         END IF
         IF ( jmax_patch(n)<jmin .OR. jmax_patch(n)>jmax ) THEN
           print*,'LOCATION OUT OF THE DOMAIN'
           print*,'Have a look at file ',trim(file_trajec),' trajectory number ',n
           jmax_patch(n)=MAX(jmax_patch(n),jmin)
           jmax_patch(n)=MIN(jmax_patch(n),jmax)
           print*,'The northern latitude of patch number ',n,'is :',jmax_patch(n)
         END IF
       END IF

     ! Read spatial dispersion of particles inside initial patch
       READ(49,*,iostat=eof) istep_patch(n),jstep_patch(n)

     ! Read depth of initial patch (read as immersion in meters)
       READ(49,*,iostat=eof) kmin_patch(n),kmax_patch(n)

     ! Read resolution depth of initial patch
       READ(49,*,iostat=eof) kstep_patch(n)
       
       ! Number of particles set at each initial position
       READ(49,*,iostat=eof) nb_part_intro(n)

     ! Type of vertical behavior (integer):
         ! itypevert(n) = 0 if constant depth
         ! itypevert(n) < 0 if no random walk (vertical advection only)
         ! itypevert(n) > 0 if random walk (advection + diffusion)
         ! abs(itypevert(n)) = 1 if no vertical swimming 
         ! abs(itypevert(n)) > 1 if vertical swimming (larval behavior): 2 for nycthemeral migration, 3 for ontogenic migration (sakina), ...
       READ(49,*,iostat=eof) itypevert(n)       


     ! Read output file
       READ(49,'(a)',iostat=eof)  rec
       kk=index(rec,',|')
       IF (kk /= 0 ) THEN
         filetraj3dout(n)=rec(1:kk-1)
       ELSE
         filetraj3dout(n)=rec
       END IF
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)


     ! Estimate number of particle inside the rectangular patch
     nbmax_part=0
     DO n = 1,nb_patch
        m=0                               ! initialize nb_part for each patch
        DO j=MAX0(ljmin,jmin_patch(n)),MIN0(ljmax,jmax_patch(n)),jstep_patch(n)
           DO i=MAX0(limin,imin_patch(n)),MIN0(limax,imax_patch(n)),istep_patch(n)
              DO k=kmin_patch(n),kmax_patch(n),kstep_patch(n)
                 IF (h0_g(i,j) > k) THEN
                    xtemp=FLOAT(i)
                    ytemp=FLOAT(j)
                    CALL loc_h0(xtemp,ytemp,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    d3=h0_lag+xe_lag
                    IF (d3 > k) THEN
                       m=m+nb_part_intro(n)
                    END IF
                 END IF
              END DO
           END DO
        END DO
#ifdef key_MPI_2D
        nb_part_percpu(n,iproc)=m
       ! get the number of particles per cpus
        CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
        nb_part(n)=m
        CALL_MPI ADD_ALL_MPI_INT(nb_part(n))
        nbmax_part=MAX(nbmax_part,nb_part(n))    ! maximum number of particles in the largest patch
        WRITE(iscreenlog,*) 'Number of particles inside rectangle number ',n
        WRITE(iscreenlog,*) 'between depth ',kmin_patch(n),' and ',kmax_patch(n)
        WRITE(iscreenlog,*) ' = ',nb_part(n)
#if ! defined key_MPI_2D
        IF (m==0 ) THEN
           print*,'Patch on land or too deep'
           print*,'Check patch number',n
           print*,'in file "',trim(file_trajec),'"'
           print*,'Simulation stopped.'
           CALL_MPI MPI_FINALIZE(IERR_MPI)
           stop
        END IF
#endif
     END DO
     WRITE(iscreenlog,*) 'The maximum number of particles is ',nbmax_part

     ALLOCATE (part(nb_patch,nbmax_part))

     ! Location of particles of each patch
     DO n = 1,nb_patch
        part(n,:)%date_orig=tbegtraj(n)
        part(n,:)%itypevert=itypevert(n)
        part(n,:)%flag=0.0_rsh
        m=0

#ifdef key_MPI_2D
        part(n,:)%inoutflag=.FALSE.
        part(n,:)%limitbye=0

        IF (iproc /= 0) THEN
          m = SUM(nb_part_percpu(n,0:iproc-1))
       ENDIF
#endif

        DO j=MAX0(ljmin,jmin_patch(n)),MIN0(ljmax,jmax_patch(n)),jstep_patch(n)
           DO i=MAX0(limin,imin_patch(n)),MIN0(limax,imax_patch(n)),istep_patch(n)
              DO k=kmin_patch(n),kmax_patch(n),kstep_patch(n)
                 IF (h0_g(i,j) > k) THEN
                    m=m+nb_part_intro(n)
                    part(n,m-nb_part_intro(n)+1:m)%xpos=FLOAT(i)   ! i index of initial location    
                    part(n,m-nb_part_intro(n)+1:m)%ypos=FLOAT(j)   ! j index                        
#ifdef key_MPI_2D
                    part(n,m-nb_part_intro(n)+1:m)%inoutflag=.TRUE.
#endif                 
                   ! total depth at particle s location
                    CALL loc_h0(part(n,m)%xpos,part(n,m)%ypos,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                    d3=h0_lag+xe_lag
                    IF (d3 > k) THEN
                       part(n,m-nb_part_intro(n)+1:m)%zpos=k                                ! immersion depth
#ifdef key_siggen        
                       kint=-float(k)+xe_lag ! switch from immersion to real z
                       hc_sig_lag=hc_sigint(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                       CALL ztosiggen(kint,spos,xe_lag,h0_lag,hc_sig_lag)
                       part(n,m-nb_part_intro(n)+1:m)%spos=spos
                       part(n,m-nb_part_intro(n)+1:m)%hc=hc_sig_lag
#else
                       part(n,m-nb_part_intro(n)+1:m)%spos=-(k/d3)
#endif		
                       part(n,m-nb_part_intro(n)+1:m)%d3=d3
                       part(n,m-nb_part_intro(n)+1:m)%h0=h0_lag
                       part(n,m-nb_part_intro(n)+1:m)%xe=xe_lag
                    ELSE
                       m=m-nb_part_intro(n)
                    END IF
                 END IF
              END DO
           END DO
        END DO

     END DO  ! loop on patch (n)

     DEALLOCATE(imax_patch,jmax_patch,istep_patch,jstep_patch)
     DEALLOCATE(imin_patch,jmin_patch,kmin_patch,kmax_patch,kstep_patch)

   ! end test on itypetraj==2
  
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ! Initialize netcdf patches
   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   ELSEIF (itypetraj==3 .AND. ex) THEN
     WRITE(iscreenlog,*) 'debut init trajectories = '   
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO
     DO WHILE(eof==0)
       READ(49,'(a)',iostat=eof) !lect  namepatch
       READ(49,'(a)',iostat=eof) !lect tbegtraj
       READ(49,'(a)',iostat=eof) !lect tendtraj
       READ(49,*,iostat=eof)  ! dtsave_traj
       READ(49,'(a)',iostat=eof) !lect filetrajin
       READ(49,*,iostat=eof)  !nb_part_intro
       READ(49,*,iostat=eof)  !itypevert
       READ(49,'(a)',iostat=eof) !****
       IF (eof==0) nb_patch = nb_patch+1
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)
     WRITE(iscreenlog,*) 'Number of netcdf patches for estimation of trajectories = ',nb_patch

   ! allocate arrays for trajectories variables
     ALLOCATE( name_patch(nb_patch),filetraj3dout(nb_patch) )
     ALLOCATE( itypevert(nb_patch),filetrajin(nb_patch))
     ALLOCATE( nb_part(nb_patch), dtsave_traj(nb_patch) )
     ALLOCATE( tbegtraj(nb_patch),tendtraj(nb_patch),tsavtraj(nb_patch) )
     ALLOCATE( nb_part_intro(nb_patch),ntrajnc(nb_patch))
#ifdef key_MPI_2D
     ALLOCATE( nb_part_percpu(nb_patch,0:nprocs-1) )
     nb_part_percpu(:,:)=0
#endif

   ! read trajectories variables
     OPEN(49,file = file_trajec,form='formatted')
     comment='debut'
     DO WHILE (comment /= '*****')
       READ(49,'(a)',iostat=eof) comment
     END DO

     DO n = 1,nb_patch

     ! Read name of patch
       READ(49,'(a)',iostat=eof) name_patch(n)

     ! Read starting date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tbegtraj(n) = tool_datosec(dateread)
       IF(tbegtraj(n)<tdeb3d .OR. tbegtraj(n)>tfin) THEN
         comdate=tool_sectodat(tdeb3d)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Starting date ,',trim(dateread),', is uncorrect (before 3D simulations starting) . '
         write(iscreenlog,*) 'WARNING : consequently trajectory starts with departure of 3D run on', comdate
         tbegtraj(n)=tdeb3d
       ENDIF
       tsavtraj(n)=tbegtraj(n)

     ! Read ending date of trajectory
       READ(49,'(a)',iostat=eof) dateread
       tendtraj(n) = tool_datosec(dateread)
       IF(tendtraj(n)<tdeb3d .OR. tendtraj(n)>tfin) THEN
         comdate=tool_sectodat(tfin)
         write(iscreenlog,*) ' '
         write(iscreenlog,*) 'WARNING : PATCH NUMBER : ',n
         write(iscreenlog,*) 'WARNING : Ending date ,',trim(dateread),', is uncorrect, '    
         write(iscreenlog,*) 'WARNING : consequently trajectory ends with run on', comdate
         tendtraj(n)=tfin
       ENDIF

     ! Read time step for outputs
       READ(49,*,iostat=eof) dtsave_traj(n)
       comdate=tool_sectodat(tbegtraj(n))
       write(iscreenlog,*) 'PATCH NUMBER : ',n,', trajectory from ',trim(comdate)
       comdate=tool_sectodat(tendtraj(n))
       write(iscreenlog,*) '                                 to ',trim(comdate),' with a ',dtsave_traj(n), ' hours time step'

     ! Read name of input netcdf file
       READ(49,'(a)',iostat=eof) rec
       kk=index(rec,',|')
       IF (kk /= 0 ) THEN
          filetrajin(n)=rec(1:kk-1)
       ELSE
          filetrajin(n)=rec
       END IF
       
       ! Number of particles set at each initial position
       READ(49,*,iostat=eof) nb_part_intro(n)

     ! Type of vertical behavior (integer):
         ! itypevert(n) = 0 if constant depth
         ! itypevert(n) < 0 if no random walk (vertical advection only)
         ! itypevert(n) > 0 if random walk (advection + diffusion)
         ! abs(itypevert(n)) = 1 if no vertical swimming 
         ! abs(itypevert(n)) > 1 if vertical swimming (larval behavior): 2 for nycthemeral migration, 3 for ontogenic migration (sakina), ...
       READ(49,*,iostat=eof) itypevert(n)       


     ! Read output file
       READ(49,'(a)',iostat=eof)  rec
       kk=index(rec,',|')
       IF (kk /= 0 ) THEN
         filetraj3dout(n)=rec(1:kk-1)
       ELSE
         filetraj3dout(n)=rec
       END IF
       READ(49,*,iostat=eof)
     END DO
     CLOSE(49)

     ! Estimate/correct number of particle inside the patch
     nbmax_part=0
     DO n = 1,nb_patch

        CALL ionc4_openr(trim(filetrajin(n)),.false.)

        ! read number of part in file
        call ionc4_read_dimtraj(trim(filetrajin(n)),ntrajnc(n))

        ALLOCATE (lonnc(ntrajnc(n)),latnc(ntrajnc(n)))
        ALLOCATE (depthnc(ntrajnc(n)))

        ! read lat,lon,depth of particles in file
        call ionc4_read_trajt(trim(filetrajin(n)),"longitude",lonnc,1,ntrajnc(n),1)
        call ionc4_read_trajt(trim(filetrajin(n)),"latitude",latnc,1,ntrajnc(n),1)
        call ionc4_read_trajt(trim(filetrajin(n)),"DEPTH",depthnc,1,ntrajnc(n),1)

        

        m=0                              ! initialize nb_part for each patch
        DO nn=1,ntrajnc(n)
           i=NINT(tool_latlon2i(lonnc(nn),latnc(nn)))
           j=NINT(tool_latlon2j(lonnc(nn),latnc(nn)))
           IF (i >= limin .and. i<=limax .and. j>=ljmin .and. j<=ljmax) THEN

              if (depthnc(nn) < 0.0_rsh) THEN
                 WRITE(ierrorlog,*) 'Function INIT_TRAJ'
                 WRITE(ierrorlog,*) 'depth of particle has to be > 0'
                 WRITE(ierrorlog,*) 'Check the netcdf traj file :', filetrajin(n)
                 WRITE(ierrorlog,*) 'The simulation is stopped'
                 CALL_MPI MPI_FINALIZE(IERR_MPI)
                 STOP
              END IF

              IF (h0_g(i,j) > depthnc(nn)) THEN
                 xtemp=tool_latlon2i(lonnc(nn),latnc(nn))
                 ytemp=tool_latlon2j(lonnc(nn),latnc(nn))
                 CALL loc_h0(xtemp,ytemp,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                 xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                 h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                 d3=h0_lag+xe_lag
                 IF (d3 > depthnc(nn)) THEN
                    m=m+nb_part_intro(n)
                 END IF
              END IF
           END IF
        END DO
#ifdef key_MPI_2D
        nb_part_percpu(n,iproc)=m
       ! get the number of particles per cpus
        CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
        nb_part(n)=m
        CALL_MPI ADD_ALL_MPI_INT(nb_part(n))
        nbmax_part=MAX(nbmax_part,nb_part(n))    ! maximum number of particles in the largest patch
        WRITE(iscreenlog,*) 'Number of particles inside netcdf patch ',n
        WRITE(iscreenlog,*) ' = ',nb_part(n)
#if ! defined key_MPI_2D
        IF (m==0 ) THEN
           print*,'Patch on land or too deep'
           print*,'Check patch number',n
           print*,'in file "',trim(file_trajec),'"'
           print*,'Simulation stopped.'
           CALL_MPI MPI_FINALIZE(IERR_MPI)
           stop
        END IF
#endif
        DEALLOCATE(lonnc,latnc,depthnc)


     END DO
     WRITE(iscreenlog,*) 'The maximum number of particles is ',nbmax_part

     ALLOCATE (part(nb_patch,nbmax_part))

     ! Location of particles of each patch
     DO n = 1,nb_patch

        ALLOCATE (lonnc(ntrajnc(n)),latnc(ntrajnc(n)))
        ALLOCATE (depthnc(ntrajnc(n)))

        ! read lat,lon,depth of particles in file
        call ionc4_read_trajt(trim(filetrajin(n)),"longitude",lonnc,1,ntrajnc(n),1)
        call ionc4_read_trajt(trim(filetrajin(n)),"latitude",latnc,1,ntrajnc(n),1)
        call ionc4_read_trajt(trim(filetrajin(n)),"DEPTH",depthnc,1,ntrajnc(n),1)
        
        part(n,:)%date_orig=tbegtraj(n)
        part(n,:)%itypevert=itypevert(n)
        part(n,:)%flag=0.0_rsh
        m=0

#ifdef key_MPI_2D
        part(n,:)%inoutflag=.FALSE.
        part(n,:)%limitbye=0
        IF (iproc /= 0) THEN
          m = SUM(nb_part_percpu(n,0:iproc-1))
       ENDIF
#endif

       DO nn=1,ntrajnc(n)
          i=NINT(tool_latlon2i(lonnc(nn),latnc(nn)))
          j=NINT(tool_latlon2j(lonnc(nn),latnc(nn)))
          IF (i >= limin .and. i<=limax .and. j>=ljmin .and. j<=ljmax) THEN
             IF (h0_g(i,j) > depthnc(nn)) THEN
                m=m+nb_part_intro(n)
                part(n,m-nb_part_intro(n)+1:m)%xpos=tool_latlon2i(lonnc(nn),latnc(nn))
                part(n,m-nb_part_intro(n)+1:m)%ypos=tool_latlon2j(lonnc(nn),latnc(nn))                   
#ifdef key_MPI_2D
                part(n,m-nb_part_intro(n)+1:m)%inoutflag=.TRUE.
#endif                 
                ! total depth at particle s location
                CALL loc_h0(part(n,m)%xpos,part(n,m)%ypos,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                xe_lag=xeint(xe,px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                h0_lag=h0int(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                d3=h0_lag+xe_lag
                IF (d3 > depthnc(nn)) THEN
                   part(n,m-nb_part_intro(n)+1:m)%zpos=depthnc(nn)   ! immersion depth
#ifdef key_siggen        
                   kint=-depthnc(nn)+xe_lag ! switch from immersion to real z
                   hc_sig_lag=hc_sigint(px,py,igg,idd,jbb,jhh,hlb,hrb,hlt,hrt)
                   CALL ztosiggen(kint,spos,xe_lag,h0_lag,hc_sig_lag)
                   part(n,m-nb_part_intro(n)+1:m)%spos=spos
                   part(n,m-nb_part_intro(n)+1:m)%hc=hc_sig_lag
#else
                   part(n,m-nb_part_intro(n)+1:m)%spos=-(depthnc(nn)/d3)
#endif		
                   part(n,m-nb_part_intro(n)+1:m)%d3=d3
                   part(n,m-nb_part_intro(n)+1:m)%h0=h0_lag
                   part(n,m-nb_part_intro(n)+1:m)%xe=xe_lag
                ELSE
                   m=m-nb_part_intro(n)
                END IF
             END IF
          END IF
       END DO
       DEALLOCATE(lonnc,latnc,depthnc)


       ! close netcdf file
       call ionc4_close(filetrajin(n))

     END DO  ! loop on patch (n)
     
     DEALLOCATE(ntrajnc,filetrajin)

   END IF  ! end test on itypetraj==3
  

   INQUIRE(file=file_trajec,exist=ex)
   IF (ex) THEN
     write(iscreenlog,*) 'VERTICAL COMPONENT OF THE DISPLACEMENT:  '
     DO n = 1,nb_patch
        write(iscreenlog,*) ' ndtz : ',ndtz
        IF (itypevert(n) == 0) THEN
           write(iscreenlog,*) ' Patch number : ',n,': Trajectories at constant depth ' 
        ELSEIF (itypevert(n) < 0) THEN
           write(iscreenlog,*) ' Patch number : ',n,': Advection only (no random walk) '
        ELSE
           write(iscreenlog,*) ' Patch number : ',n,': With random walk (Advection + Diffusion) '
        ENDIF
     END DO
     DEALLOCATE(name_patch,tbegtraj,nb_part_intro)
   ENDIF


#ifdef key_trajec3d 
  ! save initialization
   CALL traj_save3d
#endif

   PRINT_DBG*, 'exit TRAJ_INIT3D routine'

END SUBROUTINE traj_init3d

   !!======================================================================

   SUBROUTINE traj_save3d

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE traj_save3d  ***
   !&E
   !&E ** Purpose : Save trajectories
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : step.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  12-2006 (M. Chiffle, V. Garnier)  Original code
   !&E       !  05-2010 (M. Sourisseau, M. Huret) Coupling with environment variables
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,   ONLY : nbouc,date_ref,t,iscreenlog

   USE_MPI toolmpi, ONLY : exchange_vectcpu_int

   !! * Arguments

   !! * Local declarations
   LOGICAL                                     :: l_out_nc4par
   INTEGER                                     :: n,m,k,kw,kwm,i,j,num1,num2,nbpart_out,p,nbproc,ind1
   REAL(kind=rsh)                              :: fillval
   REAL(KIND=rlg)                              :: tool_ind2lat,tool_ind2lon
   INTEGER,        ALLOCATABLE, DIMENSION(:)   :: num_out
   REAL(KIND=rlg), ALLOCATABLE, DIMENSION(:)   :: lat_out,lon_out
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:)   :: xpos_out,ypos_out,w_part_out,flag_out,spos_out,zpos_out,h0pos_out
   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter TRAJ_SAVE3D routine'

   IF(rsh==8) then 
      fillval = dg_valmanq_io
   ELSE
      fillval = rg_valmanq_io
   ENDIF

   ALLOCATE( xpos_out(0:nbmax_part),ypos_out(0:nbmax_part) )
   ALLOCATE( spos_out(0:nbmax_part),zpos_out(0:nbmax_part) )
   ALLOCATE( lat_out(0:nbmax_part),lon_out(0:nbmax_part) )
   ALLOCATE( h0pos_out(0:nbmax_part),num_out(0:nbmax_part) )
   ALLOCATE( flag_out(0:nbmax_part) )
   xpos_out(:)=fillval ; ypos_out(:)=fillval ; spos_out(:)=fillval ; zpos_out(:)=-fillval ; h0pos_out(:)=-fillval
   flag_out(:)=fillval ; lat_out(:)=dg_valmanq_io ; lon_out(:)=dg_valmanq_io ; num_out(:)=0
  
   ! Create output file
   IF (nbouc == 0) THEN

#ifdef key_MPI_2D
     l_out_nc4par=.true.
#else
     l_out_nc4par=.false.
#endif

     DO n=1,nb_patch
#ifdef key_MPI_2D
        nbpart_out=nb_part(n)+1  ! part 0 fictive just for output of CPU without particles
#else
        nbpart_out=nb_part(n)
#endif
       CALL ionc4_createfile_traj(filetraj3dout(n),nbpart_out,0,0,l_out_nc4par=l_out_nc4par)
       CALL ionc4_torigine(filetraj3dout(n),date_ref)
       CALL ionc4_tunite(filetraj3dout(n),'seconds')
       CALL ionc4_createvar_traj(filetraj3dout(n),"DEPTH","m","depth",fill_value=-fillval)
       CALL ionc4_createvar_traj(filetraj3dout(n),"H0","m","h0",fill_value=-fillval)
       CALL ionc4_createvar_traj(filetraj3dout(n),"NUM","","number of the particle",fill_value=0)
       CALL ionc4_createvar_traj(filetraj3dout(n),"flag","nbr","flag",fill_value=fillval)
     END DO

   END IF

   DO n=1,nb_patch
     IF ((t >= tsavtraj(n)).and.(t <= tendtraj(n))) THEN

        ! get the number of particles per cpus
#ifdef key_MPI_2D
        nb_part_percpu(n,iproc)=count(part(n,:)%inoutflag)
        CALL_MPI exchange_vectcpu_int(nb_part_percpu(n,:))
#endif
        
        IF_MPI ( nb_part_percpu(n,iproc) /= 0 ) THEN
        p=0
        DO m = 1,nb_part(n)
           IF_MPI ( part(n,m)%inoutflag ) THEN
           p=p+1
           xpos_out(p)=part(n,m)%xpos
           ypos_out(p)=part(n,m)%ypos
           lat_out(p)=tool_ind2lat(part(n,m)%xpos,part(n,m)%ypos)
           lon_out(p)=tool_ind2lon(part(n,m)%xpos,part(n,m)%ypos)
           spos_out(p)=part(n,m)%spos
           zpos_out(p)=part(n,m)%zpos ! for output as immersion
           h0pos_out(p)=part(n,m)%d3
   !        num_out(p)=part(n,m)%num
           num_out(p)=m
           flag_out(p)=part(n,m)%flag
           ENDIF_MPI


        END DO
        ENDIF_MPI

       CALL ionc4_write_time(filetraj3dout(n),0,t)

#ifdef key_MPI_2D
       nbproc=nb_part_percpu(n,iproc)
       IF_MPI ( nbproc /= 0 ) THEN
         ind1=1
         IF (iproc==0) THEN
            num1=2 ! 1 reserve a quand absence de part dans une CPU
         ELSE
            num1=SUM(nb_part_percpu(n,0:iproc-1))+2
         END IF
         num2=num1+nbproc-1
       ELSE_MPI  ! on met tout a 0 pour l indice 0.Indice d ecriture par defaut
          ind1=0
          num1=1
          num2=1
          lat_out(ind1)=REAL(ind1,rsh)
          lon_out(ind1)=lat_out(ind1)
          spos_out(ind1)=lat_out(ind1)
          zpos_out(ind1)=lat_out(ind1)
          xpos_out(ind1)=lat_out(ind1)
          ypos_out(ind1)=lat_out(ind1)
          h0pos_out(ind1)=lat_out(ind1)
          flag_out(ind1)=lat_out(ind1)
          num_out(ind1)=ind1
       ENDIF_MPI
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(ind1:nbproc),num1,num2,0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(ind1:nbproc),num1,num2,0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(ind1:nbproc),num1,num2,0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(ind1:nbproc),num1,num2,0,fillval)
#else
       CALL ionc4_write_trajt(filetraj3dout(n),'latitude',lat_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'longitude',lon_out(1:nb_part(n)),1,nb_part(n),0,dg_valmanq_io)
       CALL ionc4_write_trajt(filetraj3dout(n),'DEPTH',zpos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'H0',h0pos_out(1:nb_part(n)),1,nb_part(n),0,-fillval)
       CALL ionc4_write_trajt(filetraj3dout(n),'NUM',num_out(1:nb_part(n)),1,nb_part(n),0)
       CALL ionc4_write_trajt(filetraj3dout(n),'flag',flag_out(1:nb_part(n)),1,nb_part(n),0,fillval)
#endif

       ! To write the data on the disk and not loose data in case of run crash
       CALL ionc4_sync(filetraj3dout(n))

       tsavtraj(n)=tsavtraj(n)+dtsave_traj(n)*3600.d0
     END IF
   END DO

   DEALLOCATE( xpos_out,ypos_out,zpos_out,spos_out,lat_out,lon_out,h0pos_out,flag_out,num_out)

   PRINT_DBG*, 'exit TRAJ_SAVE3D routine'

   END SUBROUTINE traj_save3d


  !!----------------------------------------------------------------------

#endif
 END MODULE
