  SUBROUTINE dyn3d_dt(uz,vz)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE dyn3d_dt  ***
   !&E
   !&E ** Purpose : adaptative computation of the time-step by examining
   !&E              by examining the surface current number.
   !&E
   !&E ** Description : The time-step increase is applied after a lap of tobs,
   !&E                  if during this observation period the stability criterion
   !&E                  has been exceeded.
   !&E
   !&E ** Called by : stepu
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Used ij-arrays : uz, vz, dy_v, htx, hty
   !&E
   !&E ** Modified variables : dt, cflumaxmax, cflvmaxmax, tdebobs, tobs
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-12-29
   !&E       !  2007-09    (V. Garnier) homogeneneity of dx dy...
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters,ONLY : limin,limax,ljmin,ljmax,kmax,ljminp1,ljmaxm1, &
                         liminm2,limaxp2,ljminm2,ljmaxp2,epsilon,rsh,rlg
   USE comvars2d, ONLY : ig,id,dx_u,dy_v,htx,hty
   USE comvars2d, ONLY : nbouc
   USE comvars2d, ONLY : cflumaxmax,cflvmaxmax,cflcrt ,cdate
   USE comvars2d, ONLY : dt,halfdt,tobs,dtmin,t,tdebobs,dtmax,iscreen
   USE_MPI toolmpi, ONLY : FINDMAX_MPI,MASTER

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uz,vz

   !! * Local declarations
   INTEGER        :: i,j,iumax,jumax,ivmax,jvmax
   INTEGER        :: liumx,ljumx,livmx,ljvmx
   REAL(KIND=rsh) :: flumx,flvmx
   REAL(KIND=rsh) :: uzmax,htxmax,dxumax,vzmax,htymax,dymax
   REAL(KIND=rsh) :: cflumax,cflvmax,cflmax,cflu,cflv
   REAL_AGRIF(KIND=rlg) :: dtmaxp

   character(8)  :: date
   character(10) :: time
   character(5)  :: zone
   integer,dimension(8) :: values

   !!----------------------------------------------------------------------
   !! * Executable part

   ! on examine le nb de courant max en surface
   !-------------------------------------------

   cflumax=-100.0_rsh
   cflvmax=-100.0_rsh
   iumax=limin
   jumax=ljmin
   ivmax=limin
   jvmax=ljmin

!$OMP PARALLEL DEFAULT(NONE) &
!$OMP& PRIVATE(i,j,liumx,ljumx,livmx,ljvmx,flumx,flvmx,cflu,cflv) &
!$OMP& SHARED(cflumax,cflvmax,uz,vz,dt,dx_u,iumax,jumax,ivmax,jvmax,ig,id,dy_v, &
!$OMP&        ljminp1,ljmaxm1,ljmin,ljmax,limin,limax)

   flumx = cflumax
   flvmx = cflvmax

#ifdef key_MPI_2D
!$OMP do SCHEDULE(runtime)
#else
!$OMP do SCHEDULE(static)
#endif

   DO j=ljminp1,ljmaxm1
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j)-1)

       cflu=ABS(uz(kmax,i,j))*dt/dx_u(i,j)
       IF(cflu > flumx) THEN
         flumx=cflu
         liumx=i
         ljumx=j
       ENDIF

       cflv=ABS(vz(kmax,i,j))*dt/dy_v(i,j)
       IF(cflv > flvmx) THEN
         flvmx=cflv
         livmx=i
         ljvmx=j
       ENDIF

     ENDDO
   ENDDO
!$OMP end do nowait

!$OMP CRITICAL
   IF(flumx.GT.cflumax) THEN
     cflumax = flumx
     iumax = liumx
     jumax = ljumx
   ENDIF
   IF(flvmx.GT.cflvmax) THEN
     cflvmax = flvmx
     ivmax = livmx
     jvmax = ljvmx
   ENDIF
!$OMP END CRITICAL
!$OMP END PARALLEL
   uzmax=uz(kmax,iumax,jumax)
   htxmax=htx(iumax,jumax)
   dxumax=dx_u(iumax,jumax)
   vzmax=vz(kmax,ivmax,jvmax)
   htymax=hty(ivmax,jvmax)
   dymax=dy_v(ivmax,jvmax)
   CALL_MPI FINDMAX_MPI(cflumax,iumax,jumax,uzmax,htxmax,dxumax)
   CALL_MPI FINDMAX_MPI(cflvmax,ivmax,jvmax,vzmax,htymax,dymax)

   IF_MPI (MASTER) THEN 
!     call date_and_time(date,time,zone,values)
!     call date_and_time(DATE=date,ZONE=zone)
!     call date_and_time(TIME=time)
!     call date_and_time(VALUES=values)
   WRITE(iscreen,*) 'DATE : ',cdate!,' (nbouc : ',int((nbouc-1)*.5)+1,' ',time
!   WRITE(iscreen,*) 'dt=',dt
!   WRITE(iscreen,*) 'cflumax=',cflumax
!   WRITE(iscreen,*) 'cflvmax=',cflvmax
   WRITE(iscreen,*) 'max en u = ',uzmax,iumax,jumax,htxmax
   WRITE(iscreen,*) 'max en v = ',vzmax,ivmax,jvmax,htymax
!   WRITE(iscreen,*) 'max en u = ',uzmax,iumax,jumax
!   WRITE(iscreen,*) 'max en v = ',vzmax,ivmax,jvmax
flush(iscreen)
   ENDIF_MPI

   IF(MAX(cflumax,cflvmax) > cflcrt) THEN
     IF_MPI (MASTER) THEN 
     WRITE(iscreen,*) 'ALERTE ADS!!!!!!!!!!!!!'
     ENDIF_MPI
     IF_AGRIF (agrif_root()) THEN
       dt=MIN(0.83_rsh*cflcrt*dymax/MAX(ABS(vzmax),epsilon), &
              0.83_rsh*cflcrt*dxumax/MAX(ABS(uzmax),epsilon))
       dt=MAX(dt,dtmin)
     ELSE_AGRIF
       INST_AGRIF dtmaxp=MAX(MIN(0.5_rsh*dymax/MAX(ABS(vzmax),epsilon), &
       INST_AGRIF 0.5_rsh*dxumax/MAX(ABS(uzmax),epsilon)),dtmin)
       CALL_AGRIF Agrif_Set_Parent(dt, MIN(Agrif_parent(dtmax),dtmaxp*Agrif_Rhot()))
     ENDIF_AGRIF
     IF_MPI (MASTER) THEN 
     WRITE(iscreen,*) 'HIIIIIIIII!!!! nouveau dt=',dt
     ENDIF_MPI
   ENDIF

   IF_AGRIF (.NOT.Agrif_Root()) RETURN

   cflumaxmax=MAX(cflumaxmax,cflumax)
   cflvmaxmax=MAX(cflvmaxmax,cflvmax)

   !***************************************
   ! fin de la periode d observation
   !***************************************

   IF (t > tdebobs+tobs) THEN
     cflmax=MAX(cflumaxmax,cflvmaxmax)
     IF_MPI (MASTER) THEN 
     WRITE(iscreen,*) 'ATTENTION  le ',cdate
     WRITE(iscreen,*) 'cflumaxmax=',cflumaxmax
     WRITE(iscreen,*) 'cflvmaxmax=',cflvmaxmax
     ENDIF_MPI
     IF(cflmax < cflcrt*0.84_rsh) THEN
       IF_MPI (MASTER) THEN 
       WRITE(iscreen,*) 'TURBOOOOOOOOOOOOOOOOOO.......!!!!!! old dt=',dt
       ENDIF_MPI
       dt=MIN(dt*1.2_rlg,dtmax)
       IF_MPI (MASTER) THEN 
       WRITE(iscreen,*) 'Nouveau dt : ',dt
       ENDIF_MPI
     ENDIF
     cflumaxmax=0.0_rsh
     cflvmaxmax=0.0_rsh
     tdebobs=t
   ENDIF

   halfdt=dt*0.5_rlg

  END SUBROUTINE dyn3d_dt
