  SUBROUTINE dyn3dx_tls_coef(vz,boz,uz,u,v,xec,vstar)

   !&E---------------------------------------------------------------------
   !&E                 ***  dyn3dx_tls_coef.F90  ***
   !&E
   !&E ** Purpose : estimate all the coefficient of the tridiagonal
   !&E              linear system representative of the zonal momentum
   !&E              and continuity equations (2D and 3D parts)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : dyn3dxy.F90, dyn3dyx.F90
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : h0,hc_sig,xec(xeu,xev),zr,boz,igdu,fu,vstar
   !&E                     fu,vz    (dyn3dx_corio.h)
   !&E                     xec,h0,boz   (dyn3dx_ipg_*.h)
   !&E                     uz,wz,whx,ms,mn,tfiusr,dx,dy_f,dx_u,dy_u (dyn3dx_adv_*.h)
   !&E                     uz,vish_xe,vish_phi,mn,ms,dx_u,dy_f  (dyn3dx_hdiff.h)
   !&E                     uz,htx,z0b (dyn3dx_botstress.h)
   !&E                     pa,tosx,dx_u,htx (dyn3dx_atmforc.h)
   !&E                     potgene,dx_u (dyn3dx_genpot.h)
   !&E                     u,v,xec(xeu,xev),igdu,hx,tfiusr,htx,hty,vis2d,fu,
   !&E                     pa,potgene,intz_exp,dx,dx_u,dy_u,dy_f (dyn3dx_mtm.h)
   !&E                     hex,hey,surf,dx_v,dy_u,vstar,phieau (dyn3dx_cont.h)
   !&E
   !&E
   !&E ** Modified variables : zr,pr,yz_exp,intz_exp
   !&E                         av,bv,cv,y2dv     (dyn3dx_mtm.h)
   !&E                         axe,bxe,cxe,y2dxe (dyn3dx_cont.h)
   !&E
   !&E
   !&E ** Reference :
   !&E                T. Duhaut, M. Honnorat, L. Debreu, 2008 : Rapport
   !&E                contrat PREVIMER - Ref : 06/2 210 290 (pages 100-125)
   !&E
   !&E ** History :
   !&E       !  1987-02  (P. Lazure)  Original code
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2008-07  (L. Debreu, M. Honnorat, T. Duhaut) time scheme
   !&E       !  2008-12  (V. Garnier) shift of sigma values : -1 < sig < 0
   !&E       !  2009-04  (A. Mano) MPI2D
   !&E       !  2010-03  (A.-C. Bennis)  Introduction of waves radiative stress
   !&E       !  2011-03  (F. Dumas) bottom stress parametrisation + remove case 2 and 3 for tidal flat 
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E       !  2014-10  (F. Vandermeirsch) formulation equation etat du code ROMS  
   !&E
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters                              ! imin imax rsh etc
                                            ! cte(dyn3dx_adv_centered.h)
   USE comaree,    ONLY : l_genpot, potgene        ! dyn3dx_mtm.h dyn3dx_genpot.h

   USE comvars2d,  ONLY : ig,id                    ! dyn3dx_ipg_cs.h
   USE comvars2d,  ONLY : fu                       !dyn3dx_corio.h,dyn3dx_mtm.h
   USE comvars2d,  ONLY : icon
   USE comvars2d,  ONLY : h0,fwet,fwetp,h0fond              !dyn3dx_ipg_pj.h,dyn3dx_ipg_cs.h
   USE comvars2d,  ONLY : vish_phi,vish_xe         !diffh_u.h(vish_xe),dyn3dx_hdiff.h(vish_xe and vish_phi)
   USE comvars2d,  ONLY : goulet,govlet,surf       !dyn3dx_cont
   USE comvars2d,  ONLY : hex,hey,hty              !dyn3dx_cont
   USE comvars2d,  ONLY : dx_v,dy_u                !dyn3dx_cont.h,dyn3dx_adv_quickest.h
   USE comvars2d,  ONLY : htx                      !dyn3dx_cont.h, dyn3dx_atmforc.h, dyn3dx_mtm.h, tofd_x1.h, dyn3dx_botstress.h
   USE comvars2d,  ONLY : hx                       !dyn3dx_cont.h, dyn3dx_mtm.h
   USE comvars2d,  ONLY : dy_f                     !dyn3dx_adv_centered.h, dyn3dx_adv_quickest.h, dyn3dx_cont.h, dyn3dx_hdiff.h, dyn3dx_mtm.h
   USE comvars2d,  ONLY : dt,halfdt                !dyn3dx_cont.h, dyn3dx_mtm.h
   USE comvars2d,  ONLY : dx_u                     !grdpsl_xexp.h, dyn3dx_mtm.h
   USE comvars2d,  ONLY : dx                       !adv_u.h, dyn3dx_adv_centered.h, dyn3dx_adv_quickest.h, adv_uq.h, 
                                            !dyn3dx_ipg_pj.h, dyn3dx_ipg_cs.h
                                            !diffh_u.h, dyn3dx_hdiff.h, dyn3dx_atmforc.h,
                                            !grdpsl_xexp.h, dyn3dx_mtm.h, dyn3dx_genpot.h, 
   USE comvars2d,  ONLY : frot_lin,z0b             !dyn3dx_botstress.h, tofd_x1.h
!S.P. Modification du frottement. A revoir suite dernier boggue de fevrier 2014
   USE comvars2d,  ONLY : hminfrot,hmaxfrot        !dyn3dx_atmforc.h, dyn3dx_mtm.h
   USE comvars2d,  ONLY : vis2d,botstressmax     !dyn3dx_mtm.h
   USE comvars2d,  ONLY : tfiusr                   !dyn3dx_adv_centered.h, dyn3dx_adv_quickest.h
!FVDM 2014 : utilisation de forwrka
   USE comvars2d,  ONLY : forwrka

   USE comcolig2d, ONLY : igdu                    ! dyn3dx_mtm.h
   USE comcolig2d, ONLY : mn,ms,whx               ! dyn3dx_adv_scheme.h,dyn3dx_hdiff.h 
   USE comcolig2d, ONLY : av,bv,cv,y2dv           ! dyn3dx_mtm.h
   USE comcolig2d, ONLY : axe,bxe,cxe,y2dxe       ! dyn3dx_cont.h
#if key_dyn_pg_djcs
   USE comcolig3d, ONLY : zr,pr                   !
#endif
   USE comcolig3d, ONLY : yz_exp                  !  dyn3dx_adv_centered.h,dyn3dx_adv_quickest.h,dyn3dx_corio.h,dyn3dx_ipg_pj.h,dyn3dx_ipg_cs.h,
                                            !dyn3dx_hdiff.h,dyn3dx_atmforc.h,grdpsl_xexp.h,
                                            !dyn3dx_intz.h,dyn3dx_genpot.h,dyn3dx_botstress.h
   USE comcolig3d, ONLY : intz_exp                !dyn3dx_atmforc.h,grdpsl_xexp.h,dyn3dx_intz.h,
                                            !dyn3dx_mtm.h,dyn3dx_genpot.h,dyn3dx_botstress.h
   USE comcolig3d, ONLY : cdvit                   !tofd_x1.h, dyn3dx_botstress.h
   USE comvars3d,  ONLY :  sig                     !dyn3dx_ipg_pj.h, tofd_x1.h, dyn3dx_botstress.h 
   USE comvars3d,  ONLY :  dsigw                     !adv_u.h,dyn3dx_adv_centered.h,dyn3dx_adv_quickest.h,adv_uq.h,dyn3dx_ipg_pj.h
   USE comvars3d,  ONLY :  dsigu                     !dyn3dx_atmforc.h, integration_x.h,
                                            !dyn3dx_intz.h, dyn3dx_botstress.h
! S.Petton 09/2013 : Utilisation d un profil moyen  
   USE comvars3d,  ONLY :  rhoref,boz2,qp1,rho         !dyn3dx_atmforc.h,dyn3dx_mtm.h,dyn3dx_genpot.h
   USE comvars3d,  ONLY :  wz                      !adv_u.h,dyn3dx_adv_centered.h,dyn3dx_adv_quickest.h,adv_uq.h

#ifdef key_palm_ww3
   use comwave,   ONLY : tosx3dm
#endif

#if defined key_siggen || defined key_gencoord
   USE comsiggen,  ONLY : hzutx,hzwtx,hzex,hzey,hc_sig,csu_sig
#endif
   USE toolgeom,   ONLY : f_hzw,f_zu,f_hzu
   USE cometdyn ,  ONLY : pa,tosx                  !dyn3dx_atmforc.h and dyn3dx_mtm.h 
   USE comfluxdo,  ONLY : phieau                   !dyn3dx_cont.h
#ifdef key_MPI_2D
   USE toolmpi,    ONLY :  ex_i_rsh
#endif
#ifdef key_diagdyn
   USE diagdyn,   ONLY : mask_diagx,diagx_cor,diagx_corcor,diagx_ipg,diagx_nlt,diagx_hdif,diagx_vdif,diagx_epg,diag_boz
#endif
!FVDM utilisation de obc_uzall, obc_xeall, tau_out
   USE obccombine,    ONLY : obc_uzall,obc_xeall
   USE obcogcm,       ONLY : tau_out

   IMPLICIT NONE

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: u,v
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: xec
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),     INTENT(in) :: vstar
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: uz,vz
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: boz

   !! * Local declarations
   INTEGER                                  :: i,j,k
   REAL(KIND=rsh),DIMENSION(kmax)           :: vz_u
#ifdef key_dyn_pg_djcs
   REAL(KIND=rsh), PARAMETER :: demi   = 0.5_rsh                    !dyn3dx_ipg_cs.h
   REAL(KIND=rsh), PARAMETER :: uncinq = 0.2_rsh
   REAL(KIND=rsh), PARAMETER :: undouz = 1.0_rsh/12.0_rsh
   REAL(KIND=rsh)            :: cff,tbc,dxzr,dxboz, &
                                dxzrp1,dxbozp1,     &
                                dxzrp2,dxbozp2
   REAL(KIND=rsh),DIMENSION(0:kmax)       :: dzr,dboz
#elif key_dyn_pg_pint
   REAL(KIND=rsh),DIMENSION(kmax)   :: p_primu_i,p_primu_ip1,sec_gp
   REAL(KIND=rsh),DIMENSION(0:kmax) :: p_prim_interface_i,  &
                                       p_prim_interface_ip1
   REAL(KIND=rsh)                   :: hz_i_kp1,hz_i,hz_ip1_kp1,hz_ip1, &
                                       zk_i,zk_ip1,htxee,htxeo,dxb
#elif key_dyn_pg_dj
   REAL(KIND=rsh)    :: dxb,jacobian,dxbs,dszx,dsbx,dxzs,z1,z2,z3,z4,dzb,b2surf,b4surf
   REAL(KIND=rsh)    :: f31,f43,f42,f21
#else
#if defined key_siggen || defined key_gencoord
   REAL(KIND=rlg)    :: bms,bm,pri,tbc,dxb,dhxx,dhcx,dxex,bhze,bhzo       !dyn3dx_ipg_pj_sig.h
#else
   REAL(KIND=rsh)    :: bms,bm,pri,tbc,dxb,dhxx,dhtx,htxee,htxeo   !dyn3dx_ipg_pj.h
#endif
#endif
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#if defined key_dyn_adv_quick || defined key_dyn_adv_quickest
   REAL(KIND=rsh)    :: wmo,wm,wp,wdudz,uc,up,um,ududx,vp,vm,vdudy,uv !dyn3dx_adv_quickest.h, adv_uq.h
#else
   REAL(KIND=rsh)    :: wdus,wdui                                  !dyn3dx_adv_centered.h, adv_u.h
#endif
#endif
#endif
   REAL(KIND=rsh)    :: vitf,cdf,tofxexp                           !dyn3dx_botstress.h
   REAL(KIND=rsh)    :: gdt,v_utmp,vit,factft,ft,dusdy             !dyn3dx_mtm.h
#ifdef key_diagdyn  
   REAL(KIND=rsh)    :: zp,effet_pression ! zp :save previous value of yz_exp(kmax) before adding wind stress
   REAL(kind=rsh),DIMENSION(1:kmax)  :: z_prev
#endif

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter DYN3DX_TLS_COEF routine'

#if key_dyn_pg_djcs

   ! pressure : vertical interpolation (preliminary step)
   ! ----------------------------------------------------

#ifdef key_MPI_2D
!$OMP DO schedule(runtime) private(i,j,k,cff,dboz,dzr)
#else
!$OMP DO schedule(dynamic,5) private(i,j,k,cff,dboz,dzr)
#endif
   DO j = ljminp2, ljmaxm1
     DO i = MAX0(limin,ig(j)), MIN0(limax,id(j))

!       IF (h0(i,j) > -valmanq) THEN
        IF (h0(i,j)+xec(i,j) > h0fond) THEN
!       IF (h0(i,j) > h0fond) THEN
#  ifdef key_TOTO1
           zr(kmax,i,j) = f_zu(h0(i,j),xec(i,j),kmax,i,j)
         DO k = kmax-1, 1, -1
           zr(k,i,j) = f_zu(h0(i,j),xec(i,j),k,i,j)
#else	   
! calcule de  f_zu  fait dans ts_stateeq_blumberg
         DO k = kmax-1, 1, -1
#endif
           dzr(k) = zr(k+1,i,j)-zr(k,i,j)
# ifdef SPLIT_EOS 
           dboz(k)=boz2(k+1,i,j)-boz2(k,i,j)     &   ! Adiabatic
     		      +(qp1(k+1,i,j)-qp1(k,i,j))  &  ! elementary 
     	      *( xec(i,j)-0.5*( zr(k+1,i,j)	  &  ! difference
     			       +zr(k,i,j) ))
# else
!            dboz(k) = boz(k+1,i,j)-boz(k,i,j)
           dboz(k) = rho(k+1,i,j)-rho(k,i,j)
# endif	   
         END DO
           dzr(kmax) = dzr(kmax-1)
           dboz(kmax) = dboz(kmax-1)
           dzr(0) = dzr(1)
           dboz(0) = dboz(1)

         DO k = kmax,1,-1
           cff = 2.0_rsh*dboz(k)*dboz(k-1)
           IF (cff > 0) THEN
             dboz(k) = cff / (dboz(k)+dboz(k-1))
           ELSE
             dboz(k) = 0.0_rsh
           END  IF
             dzr(k) = 2.0_rsh*dzr(k)*dzr(k-1) / (dzr(k)+dzr(k-1))
#ifdef SPLIT_EOS
             dboz(k)  = dboz(k) - qp1(k,i,j)*dzr(k)
#endif
         END DO
	 
         pr(kmax,i,j) = (  boz(kmax,i,j) + &
                  0.5_rsh * (xec(i,j)-zr(kmax,i,j)) * (boz(kmax,i,j)-boz(kmax-1,i,j)) / (zr(kmax,i,j)-zr(kmax-1,i,j))  ) &
                  * (xec(i,j)-zr(kmax,i,j))
         pr(kmax,i,j) = (  rho(kmax,i,j) + &
                  0.5_rsh * (xec(i,j)-zr(kmax,i,j)) * (rho(kmax,i,j)-rho(kmax-1,i,j)) / (zr(kmax,i,j)-zr(kmax-1,i,j))  ) &
                  * (xec(i,j)-zr(kmax,i,j)) *(grav/rhoref)
         DO k = kmax-1, 1, -1
!	   pr(k,i,j) = pr(k+1,i,j) +                                               &
!                       demi*(  (boz(k+1,i,j)+boz(k,i,j))*(zr(k+1,i,j)-zr(k,i,j))   &
!                       - uncinq*( (dboz(k+1,i)-dboz(k))*(zr(k+1,i,j)-zr(k,i,j)   &
!                                           -undouz*(dzr(k+1,i)+dzr(k)))          &
!                                 -(dzr(k+1,i)-dzr(k))*(boz(k+1,i,j)-boz(k,i,j)   &
!                                           -undouz*(dboz(k+1,i)+dboz(k))) )  )
           pr(k,i,j) = pr(k+1,i,j) +                                               &
                       demi*(  (rho(k+1,i,j)+rho(k,i,j))*(zr(k+1,i,j)-zr(k,i,j))   &
                       - uncinq*( (dboz(k+1)-dboz(k))*(zr(k+1,i,j)-zr(k,i,j)   &
                                           -undouz*(dzr(k+1)+dzr(k)))          &
                                 -(dzr(k+1)-dzr(k))*(rho(k+1,i,j)-rho(k,i,j)   &
                                           -undouz*(dboz(k+1)+dboz(k))) )  )*(grav/rhoref)
         END DO

       END IF
     END DO
   END DO
!$OMP end do

OMPMPI barrier
OMPMPI master
#  ifdef key_TOTO1
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,zr(1:kmax,liminm1:limaxp2,ljminm1:ljmaxp2))
#endif   
   CALL_MPI ex_i_rsh( 0,1,kmax,limin  ,limaxp1,ljmin  ,ljmaxp1,pr(1:kmax,limin  :limaxp1,ljmin  :ljmaxp1))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(zr,pr)

#endif

#ifdef key_MPI_2D
!$OMP DO schedule(runtime)                                     &
#else
!$OMP DO schedule(dynamic,5)                                   &
#endif
!$OMP& private(i,j,k,vz_u                                      &
#ifdef key_dyn_pg_djcs
!$OMP& ,cff,tbc,dxzr,dxboz,dxzrp1,dxbozp1,dxzrp2,dxbozp2       &
#elif key_dyn_pg_pint
!$OMP& ,p_primu_i,p_primu_ip1,p_prim_interface_i               &
!$OMP& ,p_prim_interface_ip1,sec_gp,zk_i,zk_ip1,htxee,htxeo    &
!$OMP& ,hz_i_kp1,hz_i,hz_ip1_kp1,hz_ip1,dxb                    &
#elif key_dyn_pg_dj
!$OMP& ,dxb,jacobian,dxbs,dszx,dsbx,dxzs,z1,z2,z3,z4           &
!$OMP& ,dzb,b2surf,b4surf,f31,f43,f42,f21                      &
#else
#if defined key_siggen || defined key_gencoord
!$OMP& ,bms,bm,pri,tbc,dxb,dhxx,dhcx,dxex,bhze,bhzo            &
#else
!$OMP& ,bms,bm,pri,tbc,dxb,dhxx,dhtx,htxee,htxeo               &
#endif
#endif
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#if defined key_dyn_adv_quick || defined key_dyn_adv_quickest
!$OMP& ,wmo,wm,wp,wdudz,uc,up,um,ududx,vp,vm,vdudy,uv          &
#else
!$OMP& ,wdus,wdui                                              &
#endif
#endif
#endif
#if defined key_diagdyn
!$OMP& ,z_prev                                                 &
#endif
!$OMP& ,vitf,cdf,tofxexp                                       &
!$OMP& ,gdt,v_utmp,vit,factft,ft,dusdy                         &
!$OMP& )

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(ig(j),limin),MIN0(limax,id(j)-1)

       IF(igdu(i,j) == 1) THEN
!
!  debut des calculs des termes de tendances
!-------------------------------------------
                                      ! 1- coriolis
#        include "dyn3dx_corio.h"

#ifdef key_diagdyn
         IF (mask_diagx(i,j)) THEN
           diagx_cor(:,i,j)=-fu(i,j)*vz_u(:)
           diag_boz(:,i,j)=boz(:,i,j)
         END IF
#endif

#        include "dyn3dx_intz.h"

   ! removes barotropic part of coriolis term at time n (or n+1/2 depending on the order of calculation of u and v)
         yz_exp(:,i,j) = yz_exp(:,i,j)-intz_exp(i,j)

#ifdef key_diagdyn
         ! premiere correction pour Coriolis
         IF (mask_diagx(i,j)) diagx_corcor(:,i,j)=yz_exp(:,i,j)
         IF (icon<=1 .AND. mask_diagx(i,j)) z_prev(:)=yz_exp(:,i,j)
#endif	 

                                     ! 1.1- coupling with WW3
#ifdef key_palm_ww3
#     include "dyn3dx_wave.h"
#endif

         IF(icon==2) THEN                 ! 2- gradient de pression interne a n-1/2

#ifdef key_diagdyn
           IF (mask_diagx(i,j)) z_prev(:)=yz_exp(:,i,j)
#endif	 
 
#ifdef key_dyn_pg_djcs
#          include "dyn3dx_ipg_cs.h"
#elif key_dyn_pg_pint
#          include "dyn3dx_ipg_pint.h"
#elif key_dyn_pg_dj
#          include "dyn3dx_ipg_dj.h"
#else
#if defined key_siggen || defined key_gencoord
#          include "dyn3dx_ipg_pj_sig.h"
#else
#          include "dyn3dx_ipg_pj.h"
#endif
#endif

#ifdef key_diagdyn
           IF (mask_diagx(i,j)) THEN
             diagx_ipg(:,i,j)=yz_exp(:,i,j)-z_prev(:)
             z_prev(:)=yz_exp(:,i,j)
           END IF
#endif

         ENDIF

                                      ! 3- termes d advection a l instant n-1
#if ! defined key_dyn_adv_none
#if ! defined key_dyn_adv_linear
#ifdef key_dyn_adv_quick
#        include "dyn3dx_adv_quick.h"
#elif defined key_dyn_adv_quickest
#        include "dyn3dx_adv_quickest.h"
#else
#        include "dyn3dx_adv_centered.h"
#endif
#endif
#endif

#ifdef key_diagdyn
         IF (mask_diagx(i,j)) THEN
           diagx_nlt(:,i,j)=-(yz_exp(:,i,j)-z_prev(:))
           z_prev(:)=yz_exp(:,i,j)
         END IF
#endif

                                      ! 4- terme de diffusion horizontale a n-1
#        include "dyn3dx_hdiff.h"

#ifdef key_diagdyn
         IF (mask_diagx(i,j)) diagx_hdif(:,i,j)=yz_exp(:,i,j)-z_prev(:)
#endif	 

 				      ! 4bis- terme de relaxation 
!FVDM 5/02/2014
!Relaxation sur uz
!FVDM : pas utilisable si maree avec l_tide_harmcompo=.true.
!         yz_exp(:,i,j)=yz_exp(:,i,j)-halfdt*tau_out*forwrka(i,j)*(uz(:,i,j)-obc_uzall(:,i,j))

                                      ! 5- integration des termes precedents 1 a 4
                                      !    sur la verticale a destination du mode
                                      !    barotrope.
#        include "dyn3dx_intz.h" 

                                      ! 6- tension sur le fond a l instant n-1
#if ! defined key_no_bottom_stress
#        include "dyn3dx_botstress.h"
#endif
                                      ! 8- forcing meteo tension de surface et
                                      !    pression a l instant n-1/2
#ifdef key_diagdyn
         IF (mask_diagx(i,j)) THEN
           zp=yz_exp(kmax,i,j)  ! on sauve en surface pour recuperer ensuite  le vent
           z_prev(:)=yz_exp(:,i,j) ! surla verticale on va ensuite ajouter la pression
         END IF
#endif	

#        include "dyn3dx_atmforc.h"

#ifdef key_diagdyn
         IF (mask_diagx(i,j)) THEN
           effet_pression=yz_exp(1,i,j)-z_prev(1)
           diagx_vdif(kmax,i,j)=yz_exp(kmax,i,j)-zp-effet_pression
           diagx_epg(:,i,j)=diagx_epg(:,i,j)+effet_pression ! on aura pu creer un tableau en plus...
         ENDIF
#endif	

         IF(l_genpot) THEN              ! 9- calcul du potentiel generateur de maree
                                      !    a l instant n-1/2
#          include "dyn3dx_genpot.h"
         ENDIF

   ! adds barotropic part of coriolis term estimated with velocity extrapolated in time (vstar)
         yz_exp(:,i,j)=yz_exp(:,i,j)+fu(i,j)*0.25_rlg*(vstar(i,j)+vstar(i+1,j)+vstar(i,j-1)+vstar(i+1,j-1))

#ifdef key_diagdyn
         ! seconde correction pour Coriolis
         IF (mask_diagx(i,j)) diagx_corcor(:,i,j)= &
                         -(diagx_corcor(:,i,j)+fu(i,j)*0.25_rlg*(vstar(i,j)+vstar(i+1,j)+vstar(i,j-1)+vstar(i+1,j-1)))
#endif	  

         intz_exp(i,j)=intz_exp(i,j)+fu(i,j)*0.25_rlg*(vstar(i,j)+vstar(i+1,j)+vstar(i,j-1)+vstar(i+1,j-1))

       ENDIF                              ! fin test igdu

! 15- calcul des coefficients des matrices tridiagonales pour chaque colonne de la
!     grille C, issus de l implicitation des termes de gradient de pression externe
!     (equation quantite de mouvement dyn3dx_mtm) et des termes de divergence du flux 
!     Nord-Sud (equation de conservation de la masse conty) et des vecteurs 2nd-membre

#      include "dyn3dx_mtm.h"
#      include "dyn3dx_cont.h"

     END DO
   END DO
!$OMP end do 

   PRINT_DBG*, 'exit DYN3DX_TLS_COEF routine'

  END SUBROUTINE dyn3dx_tls_coef
