 MODULE sflxread_1

   !!======================================================================
   !!                   ***  MODULE  sflxread  ***
   !! Atmospherical fluxes :  read of data and fluxes
   !!
   !!   History :
   !!        !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !!
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used
   USE parameters, ONLY : rsh,rlg,imin,imax,jmin,jmax,limin,limax, &
                          ljmin,ljmax,limaxp1,ljmaxp1,pi,valmanq
   USE ionc4, ONLY : ionc4_openr,ionc4_corres,ionc4_flimax,ionc4_fljmax, &
                     ionc4_read_lat,ionc4_read_lon,ionc4_read_dimt,ionc4_read_time,&
                     ionc4_read_subxyt
   USE comvars2d,  ONLY : h0
   USE sflxatm,    ONLY : rad,senslos,dsenslos,irlos,dirlos,latlos,dlatlos
#ifdef key_sflx_analyse
   USE sflxatm,    ONLY : windx,windy
#endif

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC sflx_read_1                                 ! function called by step

   !! * Shared module variables
#ifdef key_sflx_analyse
   REAL(KIND=rsh), ALLOCATABLE, DIMENSION(:,:), PUBLIC ::                  &
                                    windx1,     &   ! wind at u-location and time 1
                                    windx2,     &   ! wind at u-location and time 2
                                    windy1,     &   ! wind at v-location and time 1
                                    windy2          ! wind at v-location and time 2
#endif

   !! * Private variables
#ifdef key_sflx_interp_scrip
   LOGICAL,DIMENSION(:,:),ALLOCATABLE        :: maskdst,masksrc
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: matrix,matrix_u,matrix_v
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: tabvar2d,tabinterp2d
   REAL(kind=rlg),DIMENSION(:,:),ALLOCATABLE :: lonmet2d,latmet2d
   INTEGER,DIMENSION(:),ALLOCATABLE          :: src_add,dst_add,src_add_u,dst_add_u,src_add_v,dst_add_v
   INTEGER :: nxfin,nyfin
#endif
   INTEGER           :: ixmet,jxmet

 CONTAINS

   !!======================================================================

  SUBROUTINE sflx_read_1

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE sflx_read_1  ***
   !&E
   !&E ** Purpose : Read met. data generated from met. models analyses.
   !&E              These data are provided in a NetCDF file with an AO structure :
   !&E              the grid is oriented from South to North and from West to East
   !&E              and spatial steps are constant.
   !&E
   !&E ** Description : Read the following variables :
   !&E              air temperature at 2m, 
   !&E              relative humidity at 2m, 
   !&E              2 wind components at 10m (in m/s),
   !&E              air pressure at 2m.
   !&E              These data are subsequently interpolated onto the computation grid.
   !&E
   !&E ** Called by : step
   !&E
   !&E ** External calls : tool_interp2d_sflx_1, ionc4_openr, ionc4_corres, ionc4_flimax
   !&E                     ionc4_fljmax, ionc4_read_lat, ionc4_read_lon, ionc4_read_dimt
   !&E                     ionc4_read_time, ionc4_read_subxyt
   !&E
   !&E ** Used ij-arrays : 
   !&E
   !&E ** Modified variables : pa, sat, rh, tosx, tosy, ventvent
   !&E                         pa1, sat1, rh1, tosx1, tosy1, vv1
   !&E                in/out : pa2, sat2, rh2, tosx2, tosy2, vv2
   !&E                         xdpxe_1, ydpxe_1, i1xe_1, j1xe_1, xdpu, ydpu
   !&E                         i1u, j1u, xdpv, ydpv, i1v, j1v, ilec_1, ilecmem_1
   !&E                         tnc1_1, tnc2_1
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  1987-02  (J.-C. Salomon, P. Lazure)  Original code
   !&E       !  1990-10  (F. Dumas ) new formulation
   !&E       !  2004-08-19
   !&E       !  2008-07  (T. Odaka) MPI
   !&E       !  2009-04  (V. Garnier) MPI2D
   !&E       !  2009-11  (S. Michel, HOCER) Check meteorological fields units,
   !&E                                      correct the precipitation unit,
   !&E                                      deal with increasing or decreasing lon/lat in met.model
   !&E       !  2011-01  (M. Caillaud, V. Garnier) Integration of SCRIP interpolation
   !&E       !  01-2011 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E       !  2012-02  (V. Garnier) Reading of the correct record done by the MASTER only
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters,ONLY : rsh,imin,imax,jmin,jmax,ljmin,ljmax,ljmaxp2,limaxp2,  &
                         riosh,rlg,kmax,tref,epsilon,ljminp1,liminp1,liminm2,ljminm2
   USE comvars2d, ONLY : t,nbouc,iscreenlog,ierrorlog,iwarnlog,   &
                         lon2d,lat2d,lon2d_g,lat2d_g,             &
                         lon2du,lon2dv,lat2du,lat2dv
#if defined key_grid_rotated || defined key_sflx_interp_scrip
   USE comvars2d, ONLY : lonmin,latmin,lonmax,latmax,grid_angle
#endif
   USE cometdyn,  ONLY : xdpxe_1,ydpxe_1,i1xe_1,j1xe_1,xdpu,ydpu,i1u,j1u,xdpv,ydpv
   USE cometdyn,  ONLY : i1v,j1v,idimt_1,ilec_1,tnc1_1,tnc2_1,ilecmem_1,cdf_id
   USE cometdyn,  ONLY : name_sflx_rain
   USE cometdyn,  ONLY : file_meteo_1
   USE cometdyn,  ONLY : imetmin_1,imetmax_1,jmetmin_1,jmetmax_1
   USE cometdyn,  ONLY : rain2,rain1
   USE cometdyn,  ONLY : l_sflx_rrain
   USE comfluxdo, ONLY : rain
   USE comvarp2d, ONLY : temp
   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh,MASTER,BCAST_MPI
#ifdef key_sflx_interp_scrip
   USE bicubic_interp,ONLY : get_remap_bicub,make_bicubic_remap
#endif

   IMPLICIT NONE

   !! * Arguments

   !! * Local declarations
   LOGICAL           :: l_pb
   CHARACTER(LEN=19) :: tool_sectodat,tdate
   INTEGER           :: i,j,ncerror,IERR_MPI
   REAL(kind=rsh)    :: dgmet,dfimet,xm,ym,tdif
   REAL(kind=rsh)    :: tabmin,tabmax
   REAL(kind=rlg)    :: tnc0
   REAL(kind=rlg),  ALLOCATABLE,DIMENSION(:)   :: lonmet,latmet
   REAL(kind=riosh),ALLOCATABLE,DIMENSION(:,:) :: uvent1,vvent1,tab1_1,tosi,tosi2,tab2_1,uvent2,vvent2

   REAL(kind=riosh),ALLOCATABLE,DIMENSION(:,:) :: tosi_2,tosi2_2
   REAL(kind=rlg)    :: torigin,tmet,dt1,dt2
#ifdef key_grid_rotated
   REAL(kind=riosh),ALLOCATABLE,DIMENSION(:,:) :: uvent1new,vvent1new,uvent2new,vvent2new
#endif
   REAL(kind=rsh) :: radeg
   REAL(kind=rlg)                              :: tool_datosec

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter sflx_read_1 routine, nbouc=',nbouc

!$OMP PARALLEL DEFAULT(none) PRIVATE (i,j,dt1,dt2,tdif,dgmet,dfimet,   &
#ifdef key_sflx_interp_scrip
!$OMP& src_add,dst_add,src_add_u,dst_add_u,src_add_v,dst_add_v,        &
!$OMP& nxfin,nyfin,                                                    &
#endif
!$OMP& IERR_MPI,l_pb,tdate,                                            &
!$OMP& xm,ym,torigin,radeg,ncerror,tabmin,tabmax,tnc0)                 &
!$OMP& SHARED(t,nbouc,tmet,h0,iscreenlog,ierrorlog,iwarnlog,           &
#ifdef key_MPI_2D
!$OMP& MASTER,                                                         &
#endif
!$OMP& lonmet,latmet,uvent1,vvent1,tab1_1,tosi,tosi2,tab2_1,uvent2,vvent2, &
#ifdef key_sflx_analyse
!$OMP& windx1,windy1,windx2,windy2,windx,windy,                        &
#endif
!$OMP& lon2d,lat2d,lon2du,lat2du,lon2dv,lat2dv,lon2d_g,lat2d_g,        &
#if defined key_grid_rotated || defined key_sflx_interp_scrip
!$OMP& lonmin,latmin,lonmax,latmax,grid_angle,                         &
#endif
#ifdef key_grid_rotated
!$OMP& uvent1new,vvent1new,uvent2new,vvent2new,                        &
#endif
#ifdef key_sflx_interp_scrip
!$OMP& maskdst,masksrc,matrix,matrix_u,matrix_v,tabvar2d,tabinterp2d,  &
!$OMP& lonmet2d,latmet2d,                                              &
#endif
!$OMP& tosi_2,tosi2_2,                                                 &
!$OMP& ixmet,jxmet,cdf_id,i1v,j1v,idimt_1,ilec_1,tnc1_1,tnc2_1,ilecmem_1,        &
!$OMP& xdpxe_1,ydpxe_1,i1xe_1,j1xe_1,xdpu,ydpu,i1u,j1u,xdpv,ydpv,              &
!$OMP& name_sflx_rain, &
!$OMP& file_meteo_1,     &
!$OMP& imetmin_1,imetmax_1,jmetmin_1,jmetmax_1,      &
!$OMP& rain2,rain1,                      &
!$OMP& l_sflx_rrain,                                                   &
!$OMP& limin,limax,ljminp1,ljmax,ljmin,liminp1,ljmaxp2,limaxp2,        &
!$OMP& liminm2,ljminm2, &
!$OMP& rain,temp,rad,irlos)

   l_pb = .false. ! No problem

   radeg = pi/180.0_rsh

   tmet=t
   torigin=tref
   torigin=0.0_rlg
   ! include time lag if time origine is not 01/01/1900 in meteorological file
   !torigin=tool_datosec('01/01/1900 00:00:00')-tool_datosec('01/01/0000 00:00:00')
#ifdef key_menor
   torigin=tool_datosec('01/01/1900 00:00:00')-tool_datosec('01/01/0000 00:00:00')
#endif

   IF(nbouc==1) THEN

!$OMP single
     CALL ionc4_openr(file_meteo_1,l_out_nc4par= .true.)
     CALL ionc4_corres(file_meteo_1,cdf_id)
     ! global dimension of the atmospherical file
     ixmet=ionc4_flimax(cdf_id)
     jxmet=ionc4_fljmax(cdf_id)

     ALLOCATE(latmet(1:jxmet),lonmet(1:ixmet))
     CALL ionc4_read_lat(file_meteo_1,latmet(1:jxmet),1,jxmet)
     CALL ionc4_read_lon(file_meteo_1,lonmet(1:ixmet),1,ixmet)


#ifdef key_grid_rotated
     ! to estimate min and max indexes relative to the rotated grid
     ! to be used in atmospherical file
     ! Work indifferently if met. model longitudes decrease or increase.
     imetmin_1=1               ! global dimensions of the meteorological file by default
     imetmax_1=ixmet
     jmetmin_1=1
     jmetmax_1=jxmet
     DO i=1,ixmet-1
       IF((lonmin-lonmet(i))*(lonmin-lonmet(i+1)) <= 0.0_rlg) imetmin_1=i
       IF((lonmax-lonmet(i)+epsilon)*(lonmax-lonmet(i+1)-epsilon) <= 0.0_rlg) imetmax_1=i+1
     ENDDO
     ! Work indifferently if met. model latitudes decrease or increase.
     DO j=1,jxmet-1
       IF((latmin-latmet(j))*(latmin-latmet(j+1)) <= 0.0_rlg) jmetmin_1=j
       IF((latmax-latmet(j)+epsilon)*(latmax-latmet(j+1)-epsilon) <= 0.0_rlg) jmetmax_1=j+1
     ENDDO

#else

!toseelater (if pb) cval warning before it was xmin=REAL(imin+1,rsh) and ymin=REAL(jmin+1,rsh)
     ! lon2d_g(liminp1,ljminp1) ! p1 because the first rowx and column (imin,jmin)
     ! llatd_g(liminp1,ljminp1) ! are outside the domain of calculation
                                ! p1 because we interpolate on u and v-locations (i+0.5 and j+0.5)
     ! lon2d_g(limaxp2,ljmaxp2) ! + p1 because of the definition of ii2 in toolinterp2dsflx.F90
     ! lat2d_g(limaxp2,ljmaxp2) ! total = p2 but maybe too safe

     ! dimensions of the meteorological file usefull for each cpu
     imetmin_1=1               ! global dimensions of the meteorological file by default
     imetmax_1=ixmet
     jmetmin_1=1
     jmetmax_1=jxmet
     ! check the useful indexes of the meteorological file
     ! Work indifferently if met. model longitudes decrease or increase.
     DO i=1,ixmet-1
       IF((lon2d_g(liminp1,ljminp1)-lonmet(i)) * &
          (lon2d_g(liminp1,ljminp1)-lonmet(i+1)) <= 0.0_rlg) imetmin_1=i
       IF((lon2d_g(limaxp2,ljmaxp2)-lonmet(i)  +epsilon) * &
          (lon2d_g(limaxp2,ljmaxp2)-lonmet(i+1)-epsilon) <= 0.0_rlg) imetmax_1=i+1
     ENDDO
     ! Work indifferently if met. model latitudes decrease or increase.
     DO j=1,jxmet-1
       IF((lat2d_g(liminp1,ljminp1)-latmet(j)) * &
          (lat2d_g(liminp1,ljminp1)-latmet(j+1)) <= 0.0_rlg) jmetmin_1=j
       IF((lat2d_g(limaxp2,ljmaxp2)-latmet(j)  +epsilon) * & 
          (lat2d_g(limaxp2,ljmaxp2)-latmet(j+1)-epsilon) <= 0.0_rlg) jmetmax_1=j+1
     ENDDO

#endif

     DO i=1,ixmet
       IF(lonmet(i) >= 180.0_rlg) lonmet(i)=lonmet(i)-360.0_rlg
     END DO

     IF (imetmin_1==imetmax_1 .OR. jmetmin_1==jmetmax_1) THEN
       WRITE(ierrorlog,*) 'The size of the domain read from the meteorological field is not large enough'
       WRITE(ierrorlog,*) 'imetmin_1 =',imetmin_1,' imetmax_1 =',imetmax_1
       WRITE(ierrorlog,*) 'jmetmin_1 =',jmetmin_1,' jmetmax_1 =',jmetmax_1
       WRITE(ierrorlog,*) 'You use too many cpus for a so coarse meteorological grid'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF

     IF_MPI (MASTER) THEN
     WRITE(iscreenlog,*) ' '
     WRITE(iscreenlog,*) 'Meteorological file : spatial limits'
     WRITE(iscreenlog,*) 'imetmin_1 = ',imetmin_1
     WRITE(iscreenlog,*) 'imetmax_1 = ',imetmax_1
     WRITE(iscreenlog,*) 'jmetmin_1 = ',jmetmin_1
     WRITE(iscreenlog,*) 'jmetmax_1 = ',jmetmax_1
     WRITE(iscreenlog,*) ' '
     ENDIF_MPI

#ifdef key_sflx_interp_scrip
     IF (imetmin_1 > 2) THEN
       imetmin_1 = imetmin_1 - 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Eastern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (jmetmin_1 > 2) THEN
       jmetmin_1 = jmetmin_1 - 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Southern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (imetmax_1 < ixmet - 1) THEN
       imetmax_1 = imetmax_1 + 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Western boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
     IF (jmetmax_1 < jxmet - 1) THEN
       jmetmax_1 = jmetmax_1 + 2
     ELSE
       WRITE(ierrorlog,*) 'Meteorological file too small'
       WRITE(ierrorlog,*) 'At the Northern boundary'
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
#endif

#ifdef key_grid_rotated
     ALLOCATE(uvent1new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(vvent1new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(uvent2new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(vvent2new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
#endif

     ALLOCATE( uvent1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE( vvent1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE( uvent2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE( vvent2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(   tosi(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(  tosi2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE( tosi_2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(tosi2_2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(   tab1_1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(   tab2_1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))

#ifdef key_sflx_interp_scrip
     ALLOCATE(lonmet2d(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(latmet2d(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(masksrc(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
     ALLOCATE(tabvar2d(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))

     ALLOCATE(tabinterp2d(limin:limax,ljmin:ljmax))
     ALLOCATE(maskdst(limin:limax,ljmin:ljmax))

     masksrc(:,:)=.true.
     maskdst(:,:)=.true.
     tabvar2d(:,:)=0.0_rlg
#endif

!$OMP end single

   ! -2- Ponderation weights for  the spatial interpolation
   ! ------------------------------------------------------

     dgmet =ABS(lonmet(2)-lonmet(1))
     dfimet=ABS(latmet(2)-latmet(1))

#ifdef key_sflx_interp_scrip

!$OMP single
     DO j=jmetmin_1,jmetmax_1
       lonmet2d(imetmin_1:imetmax_1,j)=lonmet(imetmin_1:imetmax_1)
     ENDDO
     DO i=imetmin_1,imetmax_1
       latmet2d(i,jmetmin_1:jmetmax_1)=latmet(jmetmin_1:jmetmax_1)
     ENDDO

     nxfin=limax-limin+1
     nyfin=ljmax-ljmin+1
!$OMP end single

     CALL get_remap_bicub(latmet2d,REAL(lat2d(limin:limax,ljmin:ljmax),rlg),lonmet2d,REAL(lon2d(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix,src_add,dst_add)
     CALL get_remap_bicub(latmet2d,REAL(lat2du(limin:limax,ljmin:ljmax),rlg),lonmet2d,REAL(lon2du(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix_u,src_add_u,dst_add_u)
     CALL get_remap_bicub(latmet2d,REAL(lat2dv(limin:limax,ljmin:ljmax),rlg),lonmet2d,REAL(lon2dv(limin:limax,ljmin:ljmax),rlg),masksrc,maskdst,matrix_v,src_add_v,dst_add_v)

#else

!$OMP do schedule(static)
     DO j=ljminp1,ljmax
       DO i=liminp1,limax
         xm = (lon2d(i,j)-lonmet(1))/dgmet+1        !pour les points xe 
         ym = (lat2d(i,j)-latmet(1))/dfimet+1
         xdpxe_1(i,j) = xm - REAL(INT(xm))
         ydpxe_1(i,j) = ym - REAL(INT(ym))
         i1xe_1(i,j) = INT(xm)
         j1xe_1(i,j) = INT(ym)

         xm = (lon2du(i,j)-lonmet(1))/dgmet+1        !pour les points u 
         ym = (lat2du(i,j)-latmet(1))/dfimet+1
         xdpu(i,j) = xm - REAL(INT(xm))
         ydpu(i,j) = ym - REAL(INT(ym))
         i1u(i,j) = INT(xm)
         j1u(i,j) = INT(ym)

         xm = (lon2dv(i,j)-lonmet(1))/dgmet+1        !pour les points v 
         ym = (lat2dv(i,j)-latmet(1))/dfimet+1
         xdpv(i,j) = xm - REAL(INT(xm))
         ydpv(i,j) = ym - REAL(INT(ym))
         i1v(i,j) = INT(xm)
         j1v(i,j) = INT(ym)
       ENDDO
     ENDDO
!$OMP end do

#endif

   ! Time index in the meteorological file
   ! -------------------------------------

     IF_MPI (MASTER) THEN

!$OMP single
     idimt_1=ionc4_read_dimt(file_meteo_1)
     ilec_1=0 
     tnc1_1=0.0_rlg
     tnc2_1=0.0_rlg
     DO WHILE((tmet-(tnc1_1-torigin))*(tmet-(tnc2_1-torigin)) > 0.0_rlg .AND. ilec_1 < idimt_1)
       ilec_1=ilec_1+1 
       IF (ilec_1 == idimt_1) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine SFLXREAD.F90'
         WRITE(ierrorlog,*) 'All records are read, dates are not compatible with simulation time.'
         WRITE(ierrorlog,*) 'Make sure your meteorological file is correct: ',TRIM(file_meteo_1)
         WRITE(ierrorlog,*) 'Make sure the origin time is 01/01/1900 00:00:00 in the met. file.'
         WRITE(ierrorlog,*) 'If not, modify variable "torigin" in the routine SFLXREAD.F90.'
         WRITE(ierrorlog,*) ''
         tdate = tool_sectodat(tmet)
         WRITE(ierrorlog,*) 't in MARS model =',tdate
         tdate = tool_sectodat(tnc1_1-torigin)
         WRITE(ierrorlog,*) 't1 in atmospherical file =',tdate
         tdate = tool_sectodat(tnc2_1-torigin)
         WRITE(ierrorlog,*) 't2 in atmospherical file =',tdate
         WRITE(ierrorlog,*) 'Simulation stopped.'
         WRITE(ierrorlog,*) ''
         l_pb = .true.
       END IF
       IF (.NOT. l_pb) THEN
         CALL ionc4_read_time(file_meteo_1,ilec_1,tnc1_1)
         CALL ionc4_read_time(file_meteo_1,ilec_1+1,tnc2_1)
       END IF
     ENDDO
     ilecmem_1=ilec_1
!$OMP end single

     ENDIF_MPI

     ! the MASTER sends informations to every cpus
OMPMPI barrier
OMPMPI master
     CALL_MPI BCAST_MPI(l_pb)
     CALL_MPI BCAST_MPI(idimt_1)
     CALL_MPI BCAST_MPI(ilec_1)
     CALL_MPI BCAST_MPI(ilecmem_1)
     CALL_MPI BCAST_MPI(tnc1_1)
     CALL_MPI BCAST_MPI(tnc2_1)
OMPMPI end master
OMPMPI barrier
OMPMPI flush(l_pb,idimt_1,ilec_1,ilecmem_1,tnc1_1,tnc2_1)

     IF (l_pb) THEN
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF

     ! -3- lecture des tableaux deux indices qui encadrent le pas de temps courant

     IF(l_sflx_rrain) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_meteo_1,TRIM(name_sflx_rain),tab1_1,imetmin_1,imetmax_1,jmetmin_1,jmetmax_1,ilec_1,1,1)
       CALL ionc4_read_subxyt(file_meteo_1,TRIM(name_sflx_rain),tab2_1,imetmin_1,imetmax_1,jmetmin_1,jmetmax_1,ilec_1+1,1,1)
!$OMP end single
#ifdef key_sflx_interp_scrip
!$OMP single
       tabvar2d(:,:)=tab1_1(:,:)
       CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
       rain1(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
       WHERE( rain1(:,:) < 0.0_rsh ) rain1(:,:) = 0.0_rsh
       tabvar2d(:,:)=tab2_1(:,:)
       CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
       rain2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
       WHERE( rain2(:,:) < 0.0_rsh ) rain2(:,:) = 0.0_rsh
!$OMP end single
#else
       CALL tool_interp2d_sflx_1(i1xe_1,j1xe_1,xdpxe_1,ydpxe_1,tab1_1,rain1)
       CALL tool_interp2d_sflx_1(i1xe_1,j1xe_1,xdpxe_1,ydpxe_1,tab2_1,rain2)
#endif
       ! Convert the precipitation amount (cumulated between 2 meteorological steps, in kg.m-2=mm)
       ! into a precipitation flux (per unit of time, i.e. mm.s-1).
!$OMP single
       IF (ilec_1>1) THEN
         CALL ionc4_read_time(file_meteo_1,ilec_1-1,tnc0)
         rain1(:,:)=rain1(:,:)/(tnc1_1-tnc0)
       ELSE
         rain1(:,:)=rain1(:,:)/(tnc2_1-tnc1_1)
       ENDIF
       rain2(:,:)=rain2(:,:)/(tnc2_1-tnc1_1)
!$OMP end single
     END IF

    

 

     ! Check meteorological fields units.
!$OMP single
     ncerror=0

   

     IF(l_sflx_rrain) THEN
       tabmin=MIN(MINVAL(rain1),MINVAL(rain2))*(tnc2_1-tnc1_1)
       tabmax=MAX(MAXVAL(rain1),MAXVAL(rain2))*(tnc2_1-tnc1_1)
       IF ((tabmin<0.0_rsh).OR.(tabmax>1000.0_rsh)) THEN
         WRITE(ierrorlog,*) ' '
         WRITE(ierrorlog,*) 'ERROR: routine SFLXREAD.F90'
         WRITE(ierrorlog,*) 'In the meteorological file: ',TRIM(file_meteo_1)
         WRITE(ierrorlog,*) 'the precipitation amount is not expressed in kg.m-2 (=1 mm).'
         WRITE(ierrorlog,*) 'variable name: ',TRIM(name_sflx_rain)
         WRITE(ierrorlog,*) 'min. value = ',tabmin,', max. value = ',tabmax
         ncerror=ncerror+1
       ENDIF
     ENDIF

  
     IF (ncerror>0) THEN
       WRITE(ierrorlog,*) ' '
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     ENDIF
!$OMP end single

   END IF !endif for nbouc=1?

   ! deroulement normal

!$OMP single
   DO WHILE((tmet-(tnc1_1-torigin))*(tmet-(tnc2_1-torigin)) > 0.0_rlg .AND. ilec_1 < idimt_1)
     ilec_1=ilec_1+1 
     IF (ilec_1 == idimt_1) THEN
       WRITE(ierrorlog,*) ' '
       WRITE(ierrorlog,*) 'ERROR: routine SFLXREAD.F90'
       WRITE(ierrorlog,*) 'All records are already read.'
       WRITE(ierrorlog,*) 'The meteorological file is not long enough: ',TRIM(file_meteo_1)
       WRITE(ierrorlog,*) 'Simulation stopped.'
       CALL_MPI MPI_FINALIZE(IERR_MPI)
       STOP
     END IF
     CALL ionc4_read_time(file_meteo_1,ilec_1,tnc1_1)
     CALL ionc4_read_time(file_meteo_1,ilec_1+1,tnc2_1)
   ENDDO
!$OMP end single

   IF(ilecmem_1/=ilec_1) THEN
     IF(nbouc/=1) THEN
!$OMP single
#ifdef key_grid_rotated 
       ALLOCATE(uvent1new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
       ALLOCATE(vvent1new(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
#endif
       ALLOCATE(uvent1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
       ALLOCATE(vvent1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
       ALLOCATE(  tosi(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
       ALLOCATE( tosi2(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
       ALLOCATE(  tab1_1(imetmin_1:imetmax_1,jmetmin_1:jmetmax_1))
!$OMP end single
     END IF

     ilecmem_1=ilec_1

     ! -1- on transfert les tableaux 2 en 1

!$OMP do schedule(static)
     DO j=ljmin,ljmax
       IF(l_sflx_rrain)  rain1(:,j) =rain2(:,j)
  
     ENDDO
!$OMP end do

     IF(l_sflx_rrain) THEN
!$OMP single
       CALL ionc4_read_subxyt(file_meteo_1,TRIM(name_sflx_rain),tab1_1,imetmin_1,imetmax_1,jmetmin_1,jmetmax_1,ilec_1+1,1,1)
!$OMP end single
#ifdef key_sflx_interp_scrip
!$OMP single
       tabvar2d(:,:)=tab1_1(:,:)
       CALL make_bicubic_remap(tabvar2d,masksrc,tabinterp2d(limin:limax,ljmin:ljmax),nxfin,nyfin,matrix,src_add,dst_add)
       rain2(:,:)=tabinterp2d(limin:limax,ljmin:ljmax)
       WHERE( rain2(:,:) < 0.0_rsh ) rain2(:,:) = 0.0_rsh
!$OMP end single
#else
       CALL tool_interp2d_sflx_1(i1xe_1,j1xe_1,xdpxe_1,ydpxe_1,tab1_1,rain2)
#endif
!$OMP single
       rain2(:,:)=rain2(:,:)/(tnc2_1-tnc1_1)
!$OMP end single
     END IF

!$OMP single
#ifdef key_grid_rotated 
     DEALLOCATE(uvent1new,vvent1new)
#endif
     DEALLOCATE(uvent1,vvent1,tosi,tosi2,tab1_1)
!$OMP end single

   END IF

   dt1=(tnc2_1-torigin-tmet)/(tnc2_1-tnc1_1)
   dt2=1.0_rlg-dt1

!$OMP do schedule(static)
   DO j=ljmin,ljmax
     IF(l_sflx_rrain) rain(limin:limax,j)     = rain1(limin:limax,j)*dt1 + rain2(limin:limax,j)*dt2
   ENDDO
!$OMP end do
!$OMP end parallel



 18 FORMAT(i2.2,'/',i2.2,'/',i4.4,' ',2(i2.2,':'),i2.2)
 19 FORMAT(i2,1x,a3,1x,i4,1x,2(i2,'x'),i2)

   PRINT_DBG*, 'exit sflx_read_1 routine'

  END SUBROUTINE sflx_read_1

 END MODULE
