 MODULE traobc3dapply

   !!======================================================================
   !!                   ***  MODULE  traobc3dapply  ***
   !! Ocean dynamics : apply open boundary conditions of variables
   !!                  temperature and salinity
   !!======================================================================

#include "toolcpp.h"

   !! * Modules used

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC ts_obc3d_apply                ! routine called by stepu.F90 stepv.F90
#ifdef key_agrif
   PUBLIC ts_obc3d_apply_agrif          ! routine called by stepu.F90 stepv.F90
#endif

   !! * Shared module variables

   !! * Private variables

 CONTAINS
   !!======================================================================

  SUBROUTINE ts_obc3d_apply(u,v,salav,sal,tempav,temp)

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_obc3d_apply  ***
   !&E
   !&E ** Purpose : apply open boundary conditions for temperature
   !&E              and salinity field
   !&E
   !&E ** Description : apply external value when water mass enters the domain
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls :
   !&E
   !&E ** Used ij-arrays : sal,temp,hx,u,obc_sw,obc_tw,dx_u,obc_se,obc_te,
   !&E                     v,obc_ss,obc_ts,dy_v,obc_sn,obc_tn
   !&E 
   !&E ** Modified variables : sal,temp
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !                       Original code
   !&E       !  2007-05 (V. Garnier) Mise en forme - introduction de obc_* variables
   !&E       !  2007-09 (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04 (A. Mano) MPI2D
   !&E       !  2009-07 (F. Vandermeirsch) MC 2nd order scheme
   !&E       !  2010-04  (T. Odaka) MPI2D cyclic
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
   USE parameters, ONLY : imin,imax,jmin,jmax,kmax,liminp1,ljminp1,              &
                          ljminp2,ljmaxm1,ljmin,ljmax,ljminm1,ljmaxp1,           &
                          liminm1,limaxp1,limin,limax,limaxp2,liminp2,limaxm1,   &
                          ljmaxp2,liminm2,ljminm2,limaxp1,rsh,rlg,num_testcase
   USE comvars2d,  ONLY : dt,ig,h0,hx,h0fond,dx_u,id,jb,hy,dy_v,jh,l_obc_cycl_x,l_obc_cycl_y
   USE comvars3d,  ONLY : l_testcase,valmanq
   USE obcogcm,    ONLY : obc_tw,obc_se,obc_te,obc_ss,obc_ts,obc_sn,obc_tn,obc_sw,tau_in,obc_coefrel,obc_coefnud
   USE obccombine, ONLY : l_obc_char,l_obc_south,l_obc_north,l_obc_east,l_obc_west
   USE_MPI toolmpi,ONLY : ex_i_rsh,ex_j_rsh,ex_peri_x_rsh,ex_peri_y_rsh
   USE comcolig2d, ONLY : igdu,igdv

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: sal,temp
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(in) :: salav,tempav
   REAL(KIND=rsh),DIMENSION(kmax,liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(in) :: u,v
   REAL(KIND=rsh),DIMENSION(kmax,liminm1:limaxp1,ljminm1:ljmaxp1) ::  grad1T ,grad1S,grad2T,grad2S

   !! * Local declarations
   INTEGER                   :: i,j,k,ii,jj
   REAL(KIND=rlg)            :: uu,vv,cou,dtrelax
   REAL(KIND=rlg)            :: dtm
   REAL(KIND=rlg)    tau,eps,cff, cx,cy,dft,dfx,dfy 
   parameter (eps=1.D-20)

   !!----------------------------------------------------------------------
   !! * Executable part

   IF (l_testcase .AND. (num_testcase==1 &
       .OR. num_testcase==6 .OR. num_testcase==33 &
       .OR. num_testcase==30 &
! cas kelvin a virer
       .OR. num_testcase==3) &
      ) THEN

     ! None

   ELSE ! general

     dtm=0.5_rlg*dt
!     dtrelax=dtm*tau_in  ! dt / time scale of relaxation 
     dtrelax=obc_coefrel  
     IF (obc_coefrel==0.0_rsh) dtrelax=1.0_rsh

!$OMP SINGLE
   ! periodic conditions along x-axis
     IF (l_obc_cycl_x) THEN
#ifdef key_MPI_2D
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_x_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
       DO j=ljmin,ljmax
   ! western boundary
         sal(:,imin+1,j)= sal(:,imax-1,j)
         temp(:,imin+1,j)=temp(:,imax-1,j)
   ! eastern boundary
         sal(:,imax,j)  = sal(:,imin+2,j)
         temp(:,imax,j)  =temp(:,imin+2,j)
       END DO
#endif
     END IF  ! ends periodic conditions along x-axis

   ! periodic conditions along y-axis
     IF (l_obc_cycl_y) THEN

#ifdef key_MPI_2D
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
       CALL_MPI ex_peri_y_rsh(1,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
#else
   ! southern boundary
       sal(:,limin:limax,jmin+1) = sal(:,limin:limax,jmax-1)
       temp(:,limin:limax,jmin+1) = temp(:,limin:limax,jmax-1)
   ! northern boundary
       sal(:,limin:limax,jmax  ) = sal(:,limin:limax,jmin+2)
       temp(:,limin:limax,jmax  ) = temp(:,limin:limax,jmin+2)
#endif
     END IF  ! ends periodic conditions along y-axis
!$OMP END SINGLE

     IF (.NOT.l_obc_cycl_x .AND. (l_obc_west .OR. l_obc_east)) THEN
#ifdef key_OBC_TEXP
      DO j=ljminp2,ljmax 
       DO k=1,kmax     
      	 i=ig(j)
      IF_MPI (limin==i-1) THEN	  
	 IF (tempav(k,i,j) /= valmanq .AND. tempav(k,i,j-1) /= valmanq ) THEN
          grad1T(k,i,j)   =tempav(k,i,j)-tempav(k,i,j-1)
          grad1S(k,i,j)   = salav(k,i,j)- salav(k,i,j-1)
	 ELSE
	  grad1T(k,i,j)   =0   
	  grad1S(k,i,j)   =0   
	 ENDIF
	 i=ig(j)+1
	 IF (tempav(k,i,j-1) /= valmanq .AND. tempav(k,i,j) /= valmanq ) THEN
          grad1T(k,i,j)   =tempav(k,i,j)-tempav(k,i,j-1)
          grad1S(k,i,j)   = salav(k,i,j)- salav(k,i,j-1)
	 ELSE
	  grad1T(k,i,j)   =0   
	  grad1S(k,i,j)   =0   
	 ENDIF
      ENDIF_MPI   	  
       ENDDO 
      ENDDO 
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
 	   
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
#endif    	   
           
       DO j=ljminp2,ljmaxm1
   ! Western boundary
         IF_MPI (limin==imin) THEN
	 i=imin+1
           IF (hx(i,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
!               IF (l_obc_char) THEN
! FVDM 02/2014 simplification des CFO sur T et S
!FVDM                 uu=u(k,i-1,j)+u(k,i,j) 
!                      uu=u(k,i,j)
!               ELSE
!                 uu=u(k,i,j)
!               END IF
!               IF (uu > 0.0_rsh) THEN
!                sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_sw(k,1,j)*dtrelax
!                temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_tw(k,1,j)*dtrelax
!                 sal(k,i,j)=sal(k,i+1,j)
!                 temp(k,i,j)=temp(k,i+1,j)		 
!               ELSE
!                 IF (l_obc_char) THEN
               ! Extrapolation d ordre 2
!                   sal(k,i,j)=2.0_rsh*sal(k,i+1,j)-sal(k,i+2,j)
!                   temp(k,i,j)=2.0_rsh*temp(k,i+1,j)-temp(k,i+2,j)
!                 ELSE
!                   cou=uu*dtm/dx_u(i,j)
!FVDM                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh+cou)-cou*sal(k,i+1,j)
!FVDM                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh+cou)-cou*temp(k,i+1,j)
!                   sal(k,i,j)=sal(k,i+1,j)
!                   temp(k,i,j)=temp(k,i+1,j)		   
!                 END IF
!               END IF

#ifdef key_OBC_TEXP ! FVDM TEXPO
!TEMP  
          dft=tempav(k,i+1,j)-temp(k,i+1,j)
          dfx=temp(k,i+1,j)-temp(k,i+2,j)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad1T(k,i+1,j)+grad1T(k,i+1,j+1)) .gt. 0.) then
            dfy=grad1T(k,i+1,j)
          else
            dfy=grad1T(k,i+1,j+1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 temp(k,i,j)  = (dft*dfx*temp(k,i+1,j)+ cff*tempav(k,i,j)  &
                      -max(cy,0.)*grad1T(k,i,j )       &
                      -min(cy,0.)*grad1T(k,i,j+1))     &
		                              /(cff+cx)			      					      					      
         temp(k,i,j)  =(1.-tau)*temp(k,i,j)+tau*obc_tw(k,i,j)
!	 
!SAL
          dft=salav(k,i+1,j)-sal(k,i+1,j)
          dfx=sal(k,i+1,j)-sal(k,i+2,j)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad1S(k,i+1,j)+grad1S(k,i+1,j+1)) .gt. 0.) then
            dfy=grad1S(k,i+1,j)
          else
            dfy=grad1S(k,i+1,j+1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 sal(k,i,j)  = (dft*dfx*sal(k,i+1,j)+ cff*salav(k,i,j)  &
                      -max(cy,0.)*grad1S(k,i,j )       &
                      -min(cy,0.)*grad1S(k,i,j+1))     &
		                              /(cff+cx)			      					      					      
         sal(k,i,j)  =(1.-tau)*sal(k,i,j)+tau*obc_sw(k,i,j)
#endif    	   	 
             END DO
           END IF
         ENDIF_MPI

   ! Eastern boundary
         i=id(j)
         IF_MPI (limax==i) THEN
           IF (hx(i-1,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
!               IF (l_obc_char) THEN
!FVDM                  uu=u(k,i-1,j)+u(k,i,j)
!                 uu=u(k,i-1,j)		 
!               ELSE
!                 uu=u(k,i-1,j)
!               END IF
!               IF (uu < 0.0_rsh) THEN
!        	  sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_se(k,imax,j)*dtrelax
!  	          temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_te(k,imax,j)*dtrelax
!                 sal(k,i,j)=sal(k,i-1,j)
!                 temp(k,i,j)=temp(k,i-1,j)	 
!               ELSE
!                 IF (l_obc_char) THEN
!                   sal(k,i,j)=2.0_rsh*sal(k,i-1,j)-sal(k,i-2,j)
!                   temp(k,i,j)=2.0_rsh*temp(k,i-1,j)-temp(k,i-2,j)
!                 ELSE
!                   cou=uu*dtm/dx_u(i-1,j)
 !FVDM  	      sal(k,i,j)=sal(k,i,j)*(1.0_rsh-cou)+cou*sal(k,i-1,j)
 !FVDM  	      temp(k,i,j)=temp(k,i,j)*(1.0_rsh-cou)+cou*temp(k,i-1,j)
!		   sal(k,i,j)=sal(k,i-1,j)
!                   temp(k,i,j)=temp(k,i-1,j)		   
!                 END IF
!               END IF       DO j=ljmin,ljmax
             END DO
           END IF
         ENDIF_MPI

       END DO

     END IF  ! ends test on .not.l_obc_cycl_x

     IF (.NOT.l_obc_cycl_y .AND. (l_obc_north .OR. l_obc_south)) THEN
#ifdef key_OBC_TEXP
   DO k=1,kmax
     DO i=liminp1,limax 
	IF_MPI ( jmin==ljmin ) THEN
         j =jmin+1     
         IF (hy(i-1,j) /= -valmanq-h0fond .AND. hy(i,j) /= -valmanq-h0fond ) THEN
!	  IF (tempav(k,i-1,j) /= valmanq .AND. tempav(k,i,j) /= valmanq ) THEN
           grad1T(k,i,j)   =tempav(k,i,j)-  tempav(k,i-1,j)
           grad1S(k,i,j)   = salav(k,i,j)-   salav(k,i-1,j)
	  ELSE
	   grad1T(k,i,j)   =0   
	   grad1S(k,i,j)   =0   
	  ENDIF
         IF (hy(i-1,j+1) /= -valmanq-h0fond .AND. hy(i,j+1) /= -valmanq-h0fond ) THEN
!	  IF (tempav(k,i-1,j+1) /= valmanq .AND. tempav(k,i,j+1) /= valmanq ) THEN
          grad1T(k,i,j+1)   =tempav(k,i,j+1)-  tempav(k,i-1,j+1)
          grad1S(k,i,j+1)   = salav(k,i,j+1)-   salav(k,i-1,j+1)
	  ELSE
	  grad1T(k,i,j+1)   =0   
	  grad1S(k,i,j+1)   =0   
	 ENDIF
        ENDIF_MPI
	 
        IF_MPI ( jmax==ljmax) THEN
         j = jmax  	 	 	 	 	 	        
         IF (hy(i-1,j) /= -valmanq-h0fond .AND. hy(i,j) /= -valmanq-h0fond ) THEN
!	 IF (tempav(k,i-1,j) /= valmanq .AND. tempav(k,i,j) /= valmanq ) THEN	 
          grad2T(k,i,j)   = tempav(k,i,j)  -tempav(k,i-1,j)
          grad2S(k,i,j)   =  salav(k,i,j) -  salav(k,i-1,j)
	 ELSE
	  grad2T(k,i,j)   =0   
	  grad2S(k,i,j)   =0 	    
	 ENDIF
         IF (hy(i-1,j-1) /= -valmanq-h0fond .AND. hy(i,j-1) /= -valmanq-h0fond ) THEN
!	 IF (tempav(k,i-1,j-1) /= valmanq .AND. tempav(k,i,j-1) /= valmanq ) THEN	 
          grad2T(k,i,j-1)   = tempav(k,i,j-1)  -tempav(k,i-1,j-1)
          grad2S(k,i,j-1)   =  salav(k,i,j-1) -  salav(k,i-1,j-1)
	 ELSE 
	  grad2T(k,i,j-1)   =0   
	  grad2S(k,i,j-1)   =0  	   
	 ENDIF
        ENDIF_MPI 		 	 
      ENDDO 
    ENDDO 
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad2T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_i_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad2S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
 	   
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad2T(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad1S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
   CALL_MPI ex_j_rsh(-1,1,kmax,liminm1,limaxp1,ljminm1,ljmaxp1,grad2S(:,liminm1:limaxp1,ljminm1:ljmaxp1))
#endif   
   	      
       DO i=liminp2,limaxm1
   ! Southern boundary
	IF_MPI ( jmin==ljmin ) THEN
         j =jmin+1  
	 IF (hy(i,j) /= -valmanq-h0fond) THEN
             DO k=1,kmax
!               IF (l_obc_char) THEN
!FVDM                 vv=v(k,i,j-1)+v(k,i,j) 
!		 vv=v(k,i,j)
!               ELSE
!                 vv=v(k,i,j)
!               END IF
!               IF (vv > 0.0_rsh) THEN
!		  sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_ss(k,i,1)*dtrelax
!		  temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_ts(k,i,1)*dtrelax
!                 sal(k,i,j)=sal(k,i,j+1)
!                 temp(k,i,j)=temp(k,i,j+1)
!               ELSE
!                IF (l_obc_char) THEN
!                   sal(k,i,j)=2.0_rsh*sal(k,i,j+1)-sal(k,i,j+2)
!                   temp(k,i,j)=2.0_rsh*temp(k,i,j+1)-temp(k,i,j+2)
!                 ELSE
!                   cou=vv*dtm/dy_v(i,j)
!FVDM		      sal(k,i,j)=sal(k,i,j)*(1.0_rsh+cou)-cou*sal(k,i,j+1)
!FVDM		      temp(k,i,j)=temp(k,i,j)*(1.0_rsh+cou)-cou*temp(k,i,j+1)
!                   sal(k,i,j)=sal(k,i,j+1)
!                   temp(k,i,j)=temp(k,i,j+1)		   
!                 END IF
!               END IF

#ifdef key_OBC_TEXP
!TEMP  
          dft=tempav(k,i,j+1)-temp(k,i,j+1)
          dfx=temp(k,i,j+1)-temp(k,i,j+2)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad1T(k,i,j+1)+grad1T(k,i+1,j+1)) .gt. 0.) then
            dfy=grad1T(k,i,j+1)
          else
            dfy=grad1T(k,i+1,j+1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 temp(k,i,j)  = (dft*dfx*temp(k,i,j+1)+ cff*tempav(k,i,j)  &
                      -max(cy,0.)*grad1T(k,i,j )       &
                      -min(cy,0.)*grad1T(k,i+1,j))     &
		                              /(cff+cx)			      					      					      
         temp(k,i,j)  =(1.-tau)*temp(k,i,j)+tau*obc_ts(k,i,j)
!	 
!SAL
          dft=salav(k,i,j+1)-sal(k,i,j+1)
          dfx=sal(k,i,j+1)-sal(k,i,j+2)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad1S(k,i,j+1)+grad1S(k,i+1,j+1)) .gt. 0.) then
            dfy=grad1S(k,i,j+1)
          else
            dfy=grad1S(k,i+1,j+1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 sal(k,i,j)  = (dft*dfx*sal(k,i,j+1)+ cff*salav(k,i,j)  &
                      -max(cy,0.)*grad1S(k,i,j )       &
                      -min(cy,0.)*grad1S(k,i+1,j))     &
		                              /(cff+cx)			      					      					      
         sal(k,i,j)  =(1.-tau)*sal(k,i,j)+tau*obc_ss(k,i,j)	 	 
#endif
             ENDDO   
           ENDIF 	   
         ENDIF_MPI
      
   ! Northern boundary
     IF_MPI ( jmax==ljmax) THEN
         j =jmax  
           IF (hy(i,j-1) /= -valmanq-h0fond) THEN
             DO k=1,kmax
!               IF (l_obc_char) THEN
!FVDM                 vv=v(k,i,j-1)+v(k,i,j)
!                 vv=v(k,i,j-1)
!               ELSE
!                 vv=v(k,i,j-1)
!               END IF
!               IF (vv < 0.0_rsh) THEN
!		   sal(k,i,j)=sal(k,i,j)*(1.0_rsh-dtrelax)+obc_sn(k,i,jmax)*dtrelax
!		   temp(k,i,j)=temp(k,i,j)*(1.0_rsh-dtrelax)+obc_tn(k,i,jmax)*dtrelax
!                 sal(k,i,j)=sal(k,i,j-1)
!                 temp(k,i,j)=temp(k,i,j-1)		 
!               ELSE
!                 IF (l_obc_char) THEN
!                   sal(k,i,j)=2.0_rsh*sal(k,i,j-1)-sal(k,i,j-2)
!                   temp(k,i,j)=2.0_rsh*temp(k,i,j-1)-temp(k,i,j-2)
!                 ELSE
!                   cou=vv*dtm/dy_v(i,j-1)
!FVDM                   sal(k,i,j)=sal(k,i,j)*(1.0_rsh-cou)+cou*sal(k,i,j-1)
!FVDM                   temp(k,i,j)=temp(k,i,j)*(1.0_rsh-cou)+cou*temp(k,i,j-1)
!                   sal(k,i,j)=sal(k,i,j-1)
!                   temp(k,i,j)=temp(k,i,j-1)		   
!                 END IF
!               END IF

#ifdef key_OBC_TEXP
!TEMP  
          dft=tempav(k,i,j-1)-temp(k,i,j-1)
          dfx=temp(k,i,j-1)-temp(k,i,j-2)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad2T(k,i,j-1)+grad2T(k,i+1,j-1)) .gt. 0.) then
            dfy=grad2T(k,i,j-1)
          else
            dfy=grad2T(k,i+1,j-1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 temp(k,i,j)  = (dft*dfx*temp(k,i,j-1)+ cff*tempav(k,i,j)  &
                      -max(cy,0.)*grad2T(k,i,j )       &
                      -min(cy,0.)*grad2T(k,i+1,j))     &
		                              /(cff+cx)			      					      					      
         temp(k,i,j)  =(1.-tau)*temp(k,i,j)+tau*obc_tn(k,i,j)
!	 if ((i==188 .or. i==189) .and. k==1) then
!         print*,'temperature',temp(k,i,j),i,j	 	 
!	 print*,dft,cff	 	 	
!	 print*,dfx,dfy	 
!	 print*,cx,cy	 
!	 print*,'min et max',min(cy,0.)*grad2T(k,i+1,j),max(cy,0.)*grad2T(k,i,j ) 
!	 endif
#endif
	 
#ifdef key_OBC_TEXP
!	 
!SAL
          dft=salav(k,i,j-1)-sal(k,i,j-1)
          dfx=sal(k,i,j-1)-sal(k,i,j-2)
 
          if (dfx*dft .lt. 0.) then
            dft=0.                        ! <-- cancel cx, if inflow
            tau=dtm*obc_coefnud
!	    print*,'IN tau',tau
          else
            tau=dtm/(365.*86400)
!	    print*,'OUT tau',tau
!	    read(*,*)
          endif
	  	    
          if (dft*(grad2S(k,i,j-1)+grad2S(k,i+1,j-1)) .gt. 0.) then
            dfy=grad2S(k,i,j-1)
          else
            dfy=grad2S(k,i+1,j-1)
          endif  
!          dfy = 0	  
          cff=max(dfx*dfx+dfy*dfy, eps)
          cx=dft*dfx
          cy=min(cff,max(dft*dfy,-cff))        

	 sal(k,i,j)  = (dft*dfx*sal(k,i,j-1)+ cff*salav(k,i,j)  &
                      -max(cy,0.)*grad2S(k,i,j )       &
                      -min(cy,0.)*grad2S(k,i+1,j))     &
		                              /(cff+cx)	
         sal(k,i,j)  =(1.-tau)*sal(k,i,j)+tau*obc_sn(k,i,j)
	 
!	 if ((i==188 .or. i==189).and. k==1) then
!         print*,'salinite',sal(k,i,j),i,j
!	 print*,dft,cff	 	 	
!	 print*,dfx,dfy	 
!	 print*,cx,cy	
!	 print*,'min et max',min(cy,0.)*grad2S(k,i+1,j),max(cy,0.)*grad2S(k,i,j ) 
!	 endif
	 	 
#endif
         END DO
       END IF
     ENDIF_MPI

    END DO

     END IF  ! ends test on .not.l_obc_cycl_y

   END IF   ! ends test on l_testcase

OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(sal,temp)

  END SUBROUTINE ts_obc3d_apply

   !!======================================================================

  SUBROUTINE ts_obc3d_apply_agrif

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE ts_obc3d_apply_agrif  ***
   !&E
   !&E ** Purpose : apply open boundary conditions for temperature
   !&E              and salinity field for AGRIF
   !&E
   !&E ** Description : apply external value when water mass enters the domain
   !&E
   !&E ** Called by : stepu, stepv
   !&E
   !&E ** External calls :
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !                       Original code
   !&E       !  2007-05 (V. Garnier) Mise en forme - introduction de obc_* variables
   !&E
   !&E---------------------------------------------------------------------
   !! * Modules used
#ifdef key_agrif

   USE parameters
   USE comvarp2d
   USE agrifcommonid

   USE_MPI toolmpi, ONLY : ex_i_rsh,ex_j_rsh
   USE_MPI toolmpi, ONLY : MPI_EXI_recv,MPI_EXI_send
   USE_MPI toolmpi, ONLY : MPI_EXJ_recv,MPI_EXJ_send

   !! * Arguments

   !! * Local declarations
   REAL(kind=rsh),dimension(kmax,liminm1:limaxp2,ljminm1:ljmaxp2) :: ts_temp
   External :: interp3dsal,interp3dtemp

   !!----------------------------------------------------------------------
   !! * Executable part
   CALL Agrif_Bc_Variable(ts_temp,sal_id,procname=interp3dsal)
   IF_MPI(limin==imin) &
   sal(:,imin+1,ljminm1:ljmaxp2) = ts_temp(:,imin+1,ljminm1:ljmaxp2)
   IF_MPI(limax==imax) &
   sal(:,imax  ,ljminm1:ljmaxp2) = ts_temp(:,imax  ,ljminm1:ljmaxp2)
   IF_MPI(ljmax==jmax) &
   sal(:,liminm1:limaxp2,jmax  ) = ts_temp(:,liminm1:limaxp2,jmax  )
   IF_MPI(ljmin==jmin) &
   sal(:,liminm1:limaxp2,jmin+1) = ts_temp(:,liminm1:limaxp2,jmin+1)
   CALL Agrif_Bc_Variable(ts_temp,temp_id,procname=interp3dtemp)
   IF_MPI(limin==imin) &
   temp(:,imin+1,ljminm1:ljmaxp2) = ts_temp(:,imin+1,ljminm1:ljmaxp2)
   IF_MPI(limax==imax) &
   temp(:,imax  ,ljminm1:ljmaxp2) = ts_temp(:,imax  ,ljminm1:ljmaxp2)
   IF_MPI(ljmax==jmax) &
   temp(:,liminm1:limaxp2,jmax  ) = ts_temp(:,liminm1:limaxp2,jmax  )
   IF_MPI(ljmin==jmin) &
   temp(:,liminm1:limaxp2,jmin+1) = ts_temp(:,liminm1:limaxp2,jmin+1)
OMPMPI barrier
OMPMPI master
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,sal(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_i_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
   CALL_MPI ex_j_rsh(-1,2,kmax,liminm1,limaxp2,ljminm1,ljmaxp2,temp(:,liminm1:limaxp2,ljminm1:ljmaxp2))
OMPMPI end master
OMPMPI barrier
OMPMPI flush(sal,temp)

#endif

   END SUBROUTINE ts_obc3d_apply_agrif

  END MODULE traobc3dapply

#ifdef key_agrif

      subroutine interp3dtemp(tabres,k1,k2,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2,k1,k2
       real tabres(k1:k2,i1:i2,j1:j2)
       integer i,j,k

       do j=j1,j2
       do i=i1,i2
         IF (h0(i,j)+ssh(i,j) < 2.0_rsh*h0fond) THEN
           do k=k1,k2
             tabres(k,i,j) = Agrif_SpecialValue
           enddo
         ELSE
           do k=k1,k2
             tabres(k,i,j) = temp(k,i,j)
           enddo
        END IF
       enddo
       enddo
      return
      end

      subroutine interp3dsal(tabres,k1,k2,i1,i2,j1,j2)
USE parameters
use comvarp3d
use comvars2d
use comvarp2d
implicit none

       integer i1,i2,j1,j2,k1,k2
       real tabres(k1:k2,i1:i2,j1:j2)
       integer i,j,k

       do j=j1,j2
        do i=i1,i2
          IF (h0(i,j)+ssh(i,j) < 2.0_rsh*h0fond) THEN
           do k=k1,k2
             tabres(k,i,j) = Agrif_SpecialValue
           enddo
          ELSE
           do k=k1,k2
             tabres(k,i,j) = sal(k,i,j)
           enddo
          END IF
        enddo
       enddo
      return
      end
#endif
