  SUBROUTINE tssub_eq(xe2,xe3,sal,temp        &
#ifdef key_substance
                                      ,cv_wat &
#endif
                                             )

   !&E---------------------------------------------------------------------
   !&E                 ***  ROUTINE tssub_eq_wcompact_sig  ***
   !&E
   !&E ** Purpose : other possibility of subroutine body for the computation 
   !&E              of transport of salinity, temperature and tracers without fall velocity. 
   !&E
   !&E ** Description : An Euler-Compact scheme is used for depths higher than hm.
   !&E                  For water heights lower than hm, a special computation
   !&E                  is performed in the subroutine tssub_eq_hm.
   !&E                  This subroutine body replaces the default body
   !&E                  when the key key_tssub_wcompact is actived.
   !&E
   !&E ** Called by : stepu,stepv
   !&E
   !&E ** External calls : tssub_advdifx,tssub_advdify,ts_advz_compact,sub_advz_compact                     :
   !&E
   !&E ** Used ij-arrays : sal,temp,cv_wat,h0,xe2,xe3,kz,flx_w2s,
   !&E                     fsalx,fsaly,fdifsal,phisal,
   !&E                     ftempx,ftempy,fdiftemp,phitemp,
   !&E                     fx,fy,phicon,dy_u,dx_v,surf
   !&E                     ftempz,fsalz,fz
   !&E  
   !&E ** Modified variables : sal,temp,cv_wat
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2004-12
   !&E       !  2007-07  (M. Sourisseau)  introduction of biology impact via dcdt term
   !&E       !  2007-09  (V. Garnier) homogeneneity of dx dy...
   !&E       !  2009-04  (A. Mano, V. Garnier) MPI2D
   !&E       !  2009-09  (B. Thouvenin)  dcdt term for contaminant
   !&E       !  2010-06  (V.M. Faure, N. Bousserez, M. Baklouti) coupling with eco3m
   !&E       !  2010-06  (F. Cayocca) coupling with the module of sedimentology
   !&E       !  2010-10  (T. Duhaut) introduction of diffusion along isopycnes
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E  
   !&E---------------------------------------------------------------------

#include "toolcpp.h"

   !! * Modules used
   USE parameters
   USE comvars2d,   ONLY : ig,id,h0,hm,dy_u,dx_v,surf,l_tssubadv_over_smalldepth,halfdt,fwet,fwetp
   USE comvars3d,   ONLY : kz,dsigw,dsigu
#if defined key_siggen || defined key_gencoord
   USE toolgeom,    ONLY : f_hzu,f_hzw
#endif
   USE comfluvert,  ONLY : ftempz,fsalz
#ifdef key_substance
   USE comsubstance,ONLY : fx,fy,fz,nv_adv,nvp,phicon,f90
#endif
#if defined key_biolo || defined key_contaminant || defined key_eco3m
   USE comsubstance,ONLY : dcdt
#endif
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   USE comfluxdo,   ONLY : fdifsal,alloc_fdifadsal,dealloc_fdifadsal
   USE comfluxdo,   ONLY : fsalx,fsaly
   USE cometemp,    ONLY : fdiftemp,alloc_fdifadtemp,dealloc_fdifadtemp
   USE cometemp,    ONLY : ftempx,ftempy,phitemp
#else
   USE comfluxdo,   ONLY : fsalx,fsaly
   USE cometemp,    ONLY : ftempx,ftempy,phitemp
#endif
   USE cometemp,    ONLY : phisal
   USE obcogcm,     ONLY : l_obc_ogcm_relax
#if defined key_sedim
   USE comsubstance,ONLY : flx_s2w,flx_w2s
#endif

   IMPLICIT NONE

   !! * Arguments
   REAL(kind=rsh),DIMENSION(limin:limax,ljmin:ljmax),                    INTENT(in)    :: xe2
   REAL(kind=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),            INTENT(in)    :: xe3
!cval xe2 not ready for tsdiffh
   REAL(kind=rsh),DIMENSION(kmax,liminm1:limaxp2,ljminm1:ljmaxp2),       INTENT(inout) :: sal,temp
#ifdef key_substance
   REAL(kind=rsh),DIMENSION(nb_var,kmax,liminm1:limaxp2,ljminm1:ljmaxp2),INTENT(inout) :: cv_wat
#endif

   !! * Local declarations
   INTEGER                                   :: n,i,j,k,l
   REAL(kind=rsh)                            :: dtm,tfleuve,hz3_u,hz2_u,hz3_w,hz3m1_w,h3

#ifdef key_substance
   REAL(kind=rlg),DIMENSION(kmax,nb_var)     :: y
#endif
   REAL(kind=rlg)                            :: xk,yk
   REAL(kind=rlg),DIMENSION(kmax)            :: a,b,c,ysal,u0,v0,p0,q0,ytemp

   !!----------------------------------------------------------------------
   !! * Executable part

   PRINT_DBG*, 'enter TSSUB_EQ_WCOMPACT_SIG.H routine'

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   CALL alloc_fdifadsal   !allocating fdifsal fdifsal
   CALL alloc_fdifadtemp  !allocating fdiftemp fadtemp
#endif

!$OMP parallel default(none)                               &
!$OMP& PRIVATE(hz2_u,h3,hz3_u,hz3_w,hz3m1_w,i,j,k,l,xk,yk, &
!$OMP&         dtm,p0,q0,u0,v0,a,b,c,                      &
#ifdef key_substance
!$OMP&                               y,                    &
#endif
!$OMP&                                 ysal,ytemp)         &
!$OMP& shared(sal,temp,xe3,phisal,phitemp,hm,ig,id,h0      &
!$OMP& ,kz,dsigw,dsigu,dy_u                                &
!$OMP& ,limin,limax,ljmin,ljmax,ljminp2,ljmaxm1            &
!$OMP& ,liminm1,limaxp2,ljminm1,ljmaxp2,fwet,fwetp           &
!$OMP& ,fsalx,dx_v,fsaly,fsalz,surf,ftempx,ftempy,ftempz   &
#if defined key_siggen || defined key_gencoord
#endif
#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
!$OMP&  ,fdifsal,fdiftemp                                  &
#endif
#ifdef key_substance
!$OMP& ,cv_wat,nvp,nv_adv,fx,fy,fz,phicon,f90              &
#if defined key_biolo || defined key_contaminant || defined key_eco3m
!$OMP& ,dcdt                                               &
#endif
#if defined key_sedim
!$OMP& ,flx_s2w,flx_w2s                                    &
#endif
#endif
!$OMP& ,l_tssubadv_over_smalldepth,xe2,halfdt,l_obc_ogcm_relax)

   dtm=halfdt

   ! calcul des flux horizontaux et verticaux (advectifs et diffusifs)
   !------------------------------------------------------------------

   ! horizontal diffusion
   ! --------------------

#ifdef key_ts_diffh_geo
#ifdef key_MPI_2D
   WRITE(*,*) "tradiff not yet MPIed"
   STOP
#endif
   CALL ts_diffh_geo(sal,temp,xe2) ! estimates diffusion fluxes for salinity and temperature along z=cst
#endif
#ifdef key_ts_diffh_rho
#ifdef key_MPI_2D
   WRITE(*,*) "tradiff not yet MPIed"
   STOP
#endif
   CALL ts_diffh_rho(sal,temp,xe2) ! estimates diffusion fluxes for salinity and temperature along rho=cst
#endif

   ! advection (+ horizontal diffusion if not key_ts_diffh_geo or key_ts_diffh_rho)
   ! ------------------------------------------------------------------------------

#ifdef key_tssub_macho3d

#ifdef key_substance
   CALL tssub_advdif_macho3d(xe3,sal,temp,cv_wat)
#else
   CALL tssub_advdif_macho3d(xe3,sal,temp)
#endif

#else

#ifdef key_substance
   CALL tssub_advdifx(sal,temp,cv_wat)
   CALL tssub_advdify(sal,temp,cv_wat)
#else
   CALL tssub_advdifx(sal,temp)
   CALL tssub_advdify(sal,temp)
#endif
#if ! defined key_offline
   CALL ts_advz_compact(xe3,sal,temp)
#endif
#ifdef key_substance
   CALL sub_advz_compact(xe3,cv_wat)
#endif

#endif

!$OMP DO schedule(runtime)

   DO j=ljminp2,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)

       h3=h0(i,j)+xe3(i,j)
       IF(h3 > hm) THEN

! matrice tridiagonale du modele vertical
!----------------------------------------

!-------------------------
         k=1
!-------------------------
         hz2_u=f_hzu(h0(i,j),xe2(i,j),k,i,j)
         hz3_u=f_hzu(h0(i,j),xe3(i,j),k,i,j)
         hz3_w=f_hzw(h0(i,j),xe3(i,j),k,i,j)
         c(k)=-dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
         b(k)=hz3_u*fwet(i,j)+dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
#ifdef key_sedim
         b(k)=b(k)+dtm*flx_w2s(-1,i,j)/dsigu(k) ! car flux identique pour toute
                                                ! variable dissoute et calcule
                                                ! pour temp(-1) et sal (0)
                                                ! flx_w2s(-1,i,j)=flx_w2s( 0,i,j)
#endif

         a(k)=0.0_rlg

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
         ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)          &
                +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j)  &
                +phisal(k,i,j)/dsigu(k)
         ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)

         ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)         &
                 +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                 +phitemp(k,i,j)/dsigu(k)
         ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)

#else

         ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)        &
                +phisal(k,i,j)/dsigu(k)
         ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)
#if defined key_sedim
         ysal(k)=ysal(k)+dtm*flx_s2w(0,i,j)/dsigu(k)
#endif

         ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                 +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j        ) &
                 +phitemp(k,i,j)/dsigu(k)
         ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)
#if defined key_sedim
         ytemp(k)=ytemp(k)+dtm*flx_s2w(-1,i,j)/dsigu(k)  !! ATTENTION PB dx dy et dt !!!!!
#endif

#endif

   ! key_offline
#endif

         q0(k)=-c(k)/b(k)        ! factorisation LU

#if ! defined key_offline
         u0(k)=ysal(k)/b(k)      ! salinite
         v0(k)=ytemp(k)/b(k)     ! temperature
#endif

!-------------------------
         DO k=2,kmax-1
!-------------------------
           hz2_u=f_hzu(h0(i,j),xe2(i,j),k,i,j)
           hz3_u=f_hzu(h0(i,j),xe3(i,j),k,i,j)
           hz3_w=f_hzw(h0(i,j),xe3(i,j),k,i,j)
           hz3m1_w=f_hzw(h0(i,j),xe3(i,j),k-1,i,j)
           c(k)=-dtm*kz(k,i,j)/(hz3_w*dsigw(k))/dsigu(k)
           a(k)=-dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
           b(k)=hz3_u*fwet(i,j)+dtm*(kz(k,i,j)/(hz3_w*dsigw(k))+kz(k-1,i,j)/(hz3m1_w*dsigw(k-1)))/dsigu(k)

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho

           ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)          &
                  +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j)  &
                  +phisal(k,i,j)/dsigu(k)
           ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)

           ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)         &
                   +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                   +phitemp(k,i,j)/dsigu(k)
           ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)

#else

           ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                  +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)        &
                  +phisal(k,i,j)/dsigu(k)
           ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)

           ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                   +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j)         &
                   +phitemp(k,i,j)/dsigu(k)
           ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)

#endif

   ! key_offline
#endif

           p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU
           q0(k)=-c(k)/p0(k)

#if ! defined key_offline
           u0(k)=(ysal(k)-a(k)*u0(k-1))/p0(k)     ! salinite
           v0(k)=(ytemp(k)-a(k)*v0(k-1))/p0(k)    ! temperature
#endif

!-------------------------
         ENDDO
!-------------------------

!-------------------------
         k=kmax
!-------------------------
         hz2_u=f_hzu(h0(i,j),xe2(i,j),k,i,j)
         hz3_u=f_hzu(h0(i,j),xe3(i,j),k,i,j)
         hz3m1_w=f_hzw(h0(i,j),xe3(i,j),k-1,i,j)
         a(k)=-dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
         b(k)=hz3_u*fwet(i,j)+dtm*kz(k-1,i,j)/(hz3m1_w*dsigw(k-1))/dsigu(k)
         c(k)=0.0_rlg

#if ! defined key_offline

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho

         ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(fsalx(k,i,j)-fsalx(k,i-1,j)         &
                +fsaly(k,i,j)-fsaly(k,i,j-1)+fdifsal(k,i,j))/surf(i,j) &
                +phisal(k,i,j)/dsigu(k)
         ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)

         ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(ftempx(k,i,j)-ftempx(k,i-1,j)         &
                 +ftempy(k,i,j)-ftempy(k,i,j-1)+fdiftemp(k,i,j))/surf(i,j) &
                 +phitemp(k,i,j)/dsigu(k)
         ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)

#else

         ysal(k)=hz2_u*fwetp(i,j)*sal(k,i,j)-(dy_u(i,j)*fsalx(k,i,j)-dy_u(i-1,j)*fsalx(k,i-1,j) &
                +dx_v(i,j)*fsaly(k,i,j)-dx_v(i,j-1)*fsaly(k,i,j-1))/surf(i,j)        &
                +phisal(k,i,j)/dsigu(k)
         ysal(k)=ysal(k)-(fsalz(k,i,j)-fsalz(k-1,i,j))/dsigu(k)

         ytemp(k)=hz2_u*fwetp(i,j)*temp(k,i,j)-(dy_u(i,j)*ftempx(k,i,j)-dy_u(i-1,j)*ftempx(k,i-1,j) &
                 +dx_v(i,j)*ftempy(k,i,j)-dx_v(i,j-1)*ftempy(k,i,j-1))/surf(i,j)         &
                 +phitemp(k,i,j)/dsigu(k)
         ytemp(k)=ytemp(k)-(ftempz(k,i,j)-ftempz(k-1,i,j))/dsigu(k)

#endif

   ! key_offline
#endif

         p0(k)=a(k)*q0(k-1)+b(k)                ! factorisation LU

#if ! defined key_offline
         u0(k)=(ysal(k)-a(k)*u0(k-1))/p0(k)     ! salinite
         v0(k)=(ytemp(k)-a(k)*v0(k-1))/p0(k)    ! temperature

!-------------------------------------------------
!   resolution des systemes lineaires tridiagonaux
!-------------------------------------------------

         xk=u0(kmax)
         sal(kmax,i,j)=xk
         yk=v0(kmax)
         temp(kmax,i,j)=yk
         DO k=kmax-1,1,-1
           xk=q0(k)*xk+u0(k)
           ! MANGA
           sal(k,i,j)=min(max(xk,0.0_rsh),40.0_rsh)
           yk=q0(k)*yk+v0(k)
           ! MANGA
           temp(k,i,j)=min(max(yk,0.0_rsh),30.0_rsh)
         ENDDO

   ! key_offline
#endif

   ! estimation des traceurs
   !------------------------

#ifdef key_substance
         ! attention, pas vitesse de chute car prevu pour var dissoute et fixee do l=nvp+1,nv_adv
         DO l=nvp+1,nv_adv
           DO k=1,kmax
             hz2_u=f_hzu(h0(i,j),xe2(i,j),k,i,j)
             y(k,l)=hz2_u*fwetp(i,j)*cv_wat(l,k,i,j)-(dy_u(i,j)*fx(l,k,i,j)-dy_u(i-1,j)*fx(l,k,i-1,j) &
                                          +dx_v(i,j)*fy(l,k,i,j)-dx_v(i,j-1)*fy(l,k,i,j-1) &
                                          -phicon(l,k,i,j)/dsigu(k))/surf(i,j)
             y(k,l)=y(k,l)-(fz(l,k,i,j)-fz(l,k-1,i,j))/dsigu(k)
#if defined key_biolo || defined key_contaminant || defined key_eco3m
             y(k,l)=y(k,l)+dcdt(l,k,i,j)*dtm*hz2_u
#endif
           ENDDO
#if defined key_sedim
           y(1,l)=y(1,l)+dtm*flx_s2w(l,i,j)/dsigu(1)
#endif
           u0(1)=y(1,l)/b(1)
           DO k=2,kmax
             u0(k)=(y(k,l)-a(k)*u0(k-1))/p0(k)
           ENDDO
           xk=u0(kmax)
           cv_wat(l,kmax,i,j)=f90(l)*(xk)
           DO k=kmax-1,1,-1
             xk=q0(k)*xk+u0(k)
             cv_wat(l,k,i,j)=f90(l)*(xk)
           ENDDO
         ENDDO
#endif

       ENDIF ! fin test sur la hauteur
     ENDDO ! fin boucle en i
   ENDDO ! fin boucle en j

!$OMP END DO

   IF (l_tssubadv_over_smalldepth) CALL tssub_eq_hm(xe2,xe3,sal,temp &
#ifdef key_substance
                                     ,cv_wat           &
#endif
                                     )

!$OMP END PARALLEL

#if defined key_ts_diffh_geo || defined key_ts_diffh_rho
   CALL dealloc_fdifadsal   !deallocating fdifsal fdifsal
   CALL dealloc_fdifadtemp  !deallocating fdiftemp fadtemp
#endif

   PRINT_DBG*, 'exit TSSUB_EQ_WCOMPACT_SIG.H routine'

  END SUBROUTINE tssub_eq
