 MODULE sedimento

#ifdef key_sedim

   !&E==========================================================================
   !&E                   ***  MODULE  sedimento  ***
   !&E
   !&E
   !&E ** Purpose : concerns all subroutines related to sediment dynamics
   !&E              to be used in MARSiAM system
   !&E              version 2 : same algorithms (sand+mud) as for Marennes-Oleron
   !&E 
   !&E ** Description :
   !&E     subroutine sed_alloc          ! allocates variables in sediment 
   !&E     subroutine sed_param          ! defines parameterization of sediment
   !&E     subroutine sed_init           ! initializes the sediment
   !&E     subroutine sed_init_fromfile  ! reads filrepsed where all results of 
   !&E                                   ! sediment dyn. are stored
   !&E     subroutine sed_morphoinit     !
   !&E     subroutine sed_morpho         !
   !&E
   !&E     subroutine sed_skinstress     ! computes the skin stress
   !&E     subroutine sed_settlveloc_ini ! initialisation of settling velocity 
   !&E     subroutine sed_settlveloc     ! settling velocity in the water column
   !&E
   !&E     subroutine sed_erosion        ! computes erosion thickness
   !&E     subroutine sed_comp_tocr_mixsed    ! critical shear stress for erosion (Pa)
   !&E     subroutine sed_comp_tocr_mudsand   ! critical shear stress for erosion (Pa)
   !&E     subroutine sed_erosion_flux   ! erosion flux (kg/m2/s)
   !&E
   !&E     subroutine sed_sandconcextrap ! extrapolation of sand concentration
   !&E     subroutine sed_depflx         ! computation of deposition tendancy
   !&E     subroutine sed_effdep         ! effective deposition
   !&E     subroutine sed_fusion         ! fusion of two first bottom layers
   !&E
   !&E     subroutine sed_consol_diffu   ! consolidation in sediment (si besoin) + diffusion
   !&E     subroutine sed_consolid       ! consolidation in sediment
   !&E     subroutine sed_diffusion      ! diffusion in sediment (not ready)
   !&E     subroutine sed_constitutivrel ! sediment permeability & effective stress
   !&E     subroutine sed_difcoef_int    ! diffusion coef at the bed/water interface
   !&E     subroutine sed_difcoef_sed    ! diffusion coef in the sediment
   !&E
   !&E     subroutine sed_outsaverestart ! save related field for future runs
   !&E
   !&E ** History :
   !&E     !  2009-03  (Pierre Le Hir, Florence Cayocca)
   !&E     !  2010-06  (Florence Cayocca) corrections
   !&E     !  2010-06  (F. Cayocca, V. Garnier) MPI
   !&E     !  2010-07  (B. Thouvenin) taking into account of non constitutive particulate variables
   !&E     !  2011-01 (R. Ramel Alyotech) Ionc library in fortran 90/95, netcdf4 and MPI
   !&E     !  2011-02  (B. Thouvenin) * unique dynamical allocation of raphbx, raphby, frofonx, frofony, rho_bott
   !&E                                * get restart file at a different date
   !&E                                * evolution of diffusion/consolidation
   !&E                                * re-estimate of ndt_part according to the real settling velocity (key_parsub_newdt)
   !&E
   !&E==========================================================================

#include "toolcpp.h"

   !! * Modules used

   USE parameters,  ONLY : lchain,imin,imax,jmin,jmax,ksdmin,ksdmax,rsh,rlg,        &
                           limin,limax,ljmin,ljmax,liminp1,limaxm1,ljminp1,ljmaxm1, &
                           liminm1,ljminm1,limaxp1,ljmaxp1,limaxp2,ljmaxp2,         &
                           liminm2,ljminm2,riosh
   USE ionc4,       ONLY : ionc4_openr,ionc4_read_subxyt,ionc4_read_subzxyt,ionc4_close,  &
                           ionc4_createfile,ionc4_gatt,ionc4_createvar,ionc4_write_sig,&
                           ionc4_vatt_fill,ionc4_torigine,ionc4_write_time,ionc4_write_xyt,&
                           ionc4_write_zxyt,ionc4_sync

   IMPLICIT NONE

   !! * Interface

   INTERFACE sed_erosion
#ifdef key_sedim_mixsed
   MODULE PROCEDURE sed_erosion_mixsed
#endif
#ifdef key_sedim_mudsand
   MODULE PROCEDURE sed_erosion_mudsand
#endif
   END INTERFACE

   INTERFACE sed_effdep
#ifdef key_sedim_mixsed
   MODULE PROCEDURE sed_effdep_mixsed
#endif
#ifdef key_sedim_mudsand
   MODULE PROCEDURE sed_effdep_mudsand
#endif
   END INTERFACE
   
   !! * Accessibility

   ! functions & routines of this module, called outside :
   ! PUBLIC functions                                ! none ?
   PUBLIC sed_effdep,sed_erosion,sed_consol_diffu
   PUBLIC sed_init,sed_param,sed_alloc 
   PUBLIC sed_sandconcextrap,sed_settlveloc,sed_skinstress,sed_outsaverestart
   PUBLIC sed_settlveloc_ini,sed_depflx
   PUBLIC sed_init_fromfile
   PUBLIC sed_morpho,sed_morphoinit

   PRIVATE

   !! * Shared or public module variables

   REAL(KIND=rlg),PUBLIC   :: tstartconsol,  &   ! time beginning consolidation
                              tconsol,       &   ! time of next consolidation step
                              dtconsol           ! time step for consolidation

   LOGICAL,PUBLIC          :: l_fricwave,l_repsed,l_diffused
   LOGICAL,PUBLIC          :: l_consolid
   LOGICAL,PUBLIC          :: l_morphocoupl
   LOGICAL,PUBLIC          :: l_unised
   INTEGER,PUBLIC          :: ksmiuni,ksmauni
   INTEGER,ALLOCATABLE,DIMENSION(:,:),PUBLIC              :: ksmi,ksma

   CHARACTER(len=lchain),PUBLIC   :: filsauvsed,filrepsed,filoutsed
   CHARACTER(len=lchain),PUBLIC   :: name_out_eptot,name_out_nbniv,name_out_dzs,name_out_tenfon,&
                                     name_out_tenfonc,name_out_tenfonw

   CHARACTER(len=19),PUBLIC       :: date_startconsol
   
   REAL(kind=riosh),PUBLIC      :: riog_valid_min_eptot,riog_valid_max_eptot,riog_valid_min_nbniv, &
                                   riog_valid_max_nbniv,riog_valid_min_dzs,riog_valid_max_dzs,     &
                                   riog_valid_min_tenfon,riog_valid_max_tenfon
!cval reste local                                   riog_valid_min_cvsed,riog_valid_max_cvsed
   REAL(KIND=rsh),DIMENSION(:),ALLOCATABLE,PUBLIC         :: diam_sed,cini_sed,ros
   REAL(KIND=rsh),DIMENSION(:,:,:),ALLOCATABLE  ,PUBLIC   :: c_sedtot,poro,dzs       
   REAL(KIND=rsh),DIMENSION(:,:,:),ALLOCATABLE  ,PUBLIC   :: corflux,corfluy       
   REAL(KIND=rsh),DIMENSION(:,:,:,:),ALLOCATABLE,PUBLIC   :: cv_sed
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE,PUBLIC       :: hsed,z0sed
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE,    PUBLIC   :: tenfon,tenfonc,tenfonw

   REAL(KIND=rsh),ALLOCATABLE,DIMENSION(:,:)        :: raphbx,raphby,frofonx,frofony,rho_bott
   REAL(KIND=rsh),DIMENSION(:,:,:),ALLOCATABLE,PUBLIC    :: dzsiin

#ifdef key_parsub_newdt
!  rajout pour avoir ndt_part variable dans le temps et calcule en fonction de ws
   REAL(KIND=rsh),DIMENSION(:),ALLOCATABLE,PUBLIC   :: ws3max
#endif

#ifdef key_sedim_mixsed
   REAL(KIND=rsh),DIMENSION(:),ALLOCATABLE,PUBLIC:: diamstar,ws0,rosmrowsros,          &
                                                    stresscri0,tetacri0,xnielsen,      &
                                                    sigmapsg,stateconsol,permeab
   REAL(KIND=rsh),PUBLIC     :: z0seduni,fricwav,fws2,cseduni,hseduni,                 &
                                sini_sed,tini_sed,dzsmax,dzsmin,                       &
                                csupi,csedmin,cmudcr,cvolmaxsort,cvolmaxmel,  &
                                xperm1,xperm2,xsigma1,xsigma1sg,xsigma2,               &
                                xdifs1,xdifs2,xdifsi1,xdifsi2,epdifi,                  &
                                activlayer,frmudcr2,eroa1,eroa2,eroa3,                 &
                                erob1,erob2,erob3,corfluer1,corfluer2,xeros,excespowr, &
                                sed_tocr_mixsed,sed_eros_flx
#endif
#ifdef key_sedim_mudsand
   REAL(KIND=rsh),PUBLIC     :: frmudcr1,frmudcr2,E0sable,alphavase,alphasable,        &
                                xeros,x1toce,x2toce,z0seduni,cseduni,hseduni,          &
                                sini_sed,tini_sed,fimpls,cimpls,xcvgce,                &
                                fexcs,cexcs,dzsmin,dzsmax,                             &
                                csupi,csmaxmel,csedmin,                                &
                                xdifs1,xdifs2,xdifsi1,xdifsi2,epdifi,poro1,            &
                                xtas11,xtas12,poro2,xtas21,xtas22,poro3,xtas31,xtas32, &
                                fricwav,fws2,sed_tocr_mudsand,sed_sedimvel,sed_eros_flx
   REAL(KIND=rsh),DIMENSION(:,:,:),ALLOCATABLE,PUBLIC    :: poroin
#endif
 
   !! * Private variables
   
   REAL(KIND=rsh),DIMENSION(:,:),ALLOCATABLE     :: epeaui,ustarbot,winsurf
   REAL(KIND=rsh),DIMENSION(:,:,:),ALLOCATABLE   :: fludif,fluconsol
   REAL(KIND=rsh),DIMENSION(:),ALLOCATABLE       :: alp1,alp2,alp3,alp4,alp5,varpwa 
   REAL(KIND=rsh),DIMENSION(imin:imax,jmin:jmax) :: morphox,morphoy,hxi,hyi, &
                                                    hupper,dhcell,tomax,tomoy
   REAL(KIND=rsh)                                :: sed_difint,sed_difsed 
   REAL(KIND=rlg)                                :: dtconsoleff
   REAL(KIND=rsh)                                :: dtconsolinv
   


 CONTAINS
 
   !!===========================================================================
 
  SUBROUTINE sed_alloc
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_alloc  ***
   !&E
   !&E ** Purpose : allocation of arrays relative to sediment
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : main, after call traceur and sed_param 
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from traceur.F90
   !&E       !  2008-10  (F. Cayocca)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE parameters,   ONLY : l_testcase
   USE comvars2d,    ONLY : iscreenlog
   USE comsubstance, ONLY : nv_tot,nv_adv,nv_state,nvp,                     &
                            flx_s2w,flx_w2s,flx_w2s_sum,irk_fil,diam_r,cini_sed_r,      &
                            irkm_var_assoc,ros_r

   !! * Local declarations
   INTEGER               :: iv
   CHARACTER(len=lchain) :: filepc
   CHARACTER(len=19)     :: date_startconsol

   !!--------------------------------------------------------------------------
   !! * Executable part
   
   ALLOCATE(ksmi(limin:limax,ljmin:ljmax))
   ALLOCATE(ksma(limin:limax,ljmin:ljmax))
   ALLOCATE(hsed(limin:limaxp1,ljmin:ljmaxp1))
   ALLOCATE(z0sed(limin:limax,ljmin:ljmax))
   ALLOCATE(epeaui(limin:limax,ljmin:ljmax))
   ALLOCATE(tenfon(limin:limax,ljmin:ljmax))
   ALLOCATE(tenfonc(limin:limax,ljmin:ljmax))
   ALLOCATE(tenfonw(limin:limax,ljmin:ljmax))
   ALLOCATE(ustarbot(limin:limax,ljmin:ljmax))
   ALLOCATE(winsurf(limin:limax,ljmin:ljmax))

   ksmi(limin:limax,ljmin:ljmax)=0
   ksma(limin:limax,ljmin:ljmax)=0
   hsed(limin:limaxp1,ljmin:ljmaxp1)=0.0_rsh
   z0sed(limin:limax,ljmin:ljmax)=0.0_rsh
   epeaui(limin:limax,ljmin:ljmax)=0.0_rsh
   tenfon(limin:limax,ljmin:ljmax)=0.0_rsh
   tenfonc(limin:limax,ljmin:ljmax)=0.0_rsh
   tenfonw(limin:limax,ljmin:ljmax)=0.0_rsh
   ustarbot(limin:limax,ljmin:ljmax)=0.0_rsh
   winsurf(limin:limax,ljmin:ljmax)=0.0_rsh

   ALLOCATE(cv_sed(-1:nv_tot,ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   ALLOCATE(cini_sed(nv_state))
   ALLOCATE(c_sedtot(ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   ALLOCATE(poro(ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   ALLOCATE(dzs(ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   ALLOCATE(dzsiin(ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   ALLOCATE(ros(nvp))
   ALLOCATE(diam_sed(nvp))
   ALLOCATE(flx_s2w(-1:nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(flx_w2s(-1:nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(flx_w2s_sum(-1:nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(corflux(nv_adv,limin:limaxp1,ljmin:ljmax  ))
   ALLOCATE(corfluy(nv_adv,limin:limax  ,ljmin:ljmaxp1))
   ALLOCATE(fludif(-1:nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(fluconsol(-1:nv_adv,limin:limax,ljmin:ljmax))
   ALLOCATE(alp1(nv_adv))
   ALLOCATE(alp2(nv_adv))
   ALLOCATE(alp3(nv_adv))
   ALLOCATE(alp4(nv_adv))
   ALLOCATE(alp5(nv_adv))
   ALLOCATE(varpwa(-1:nv_adv))
   cv_sed(-1:nv_tot,ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
   c_sedtot(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
   poro(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
   dzs(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
   dzsiin(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
   flx_s2w(-1:nv_adv,limin:limax,ljmin:ljmax)=0.0_rsh
   flx_w2s(-1:nv_adv,limin:limax,ljmin:ljmax)=0.0_rsh
   flx_w2s_sum(-1:nv_adv,limin:limax,ljmin:ljmax)=0.0_rsh
   corflux(nv_adv,limin:limaxp1,ljmin:ljmax  )=0.0_rsh
   corfluy(nv_adv,limin:limax  ,ljmin:ljmaxp1)=0.0_rsh
   fludif(-1:nv_adv,limin:limax,ljmin:ljmax)=0.0_rsh
   fluconsol(-1:nv_adv,limin:limax,ljmin:ljmax)=0.0_rsh
   
   ros(1:nvp)=0.0_rsh
   diam_sed(1:nvp)=0.0_rsh
   alp1(1:nv_adv)=0.0_rsh
   alp2(1:nv_adv)=0.0_rsh
   alp3(1:nv_adv)=0.0_rsh
   alp4(1:nv_adv)=0.0_rsh
   alp5(1:nv_adv)=1.0_rsh
   varpwa(-1:nv_adv)=0.0_rsh
   
#ifdef key_sedim_mixsed    
   ALLOCATE(sigmapsg(ksdmin:ksdmax))
   ALLOCATE(stateconsol(ksdmin:ksdmax))
   ALLOCATE(permeab(ksdmin:ksdmax))
   ALLOCATE(rosmrowsros(nvp))
   ALLOCATE(diamstar(nvp))
   ALLOCATE(ws0(nvp))        
   ALLOCATE(stresscri0(nvp))
   ALLOCATE(tetacri0(nvp))
   ALLOCATE(xnielsen(nvp))
#else
   ALLOCATE(poroin(ksdmin:ksdmax,limin:limax,ljmin:ljmax))
   poroin(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=0.0_rsh
#endif   
#ifdef key_parsub_newdt
   ! rajout pour calcul de ndt_part en fonction du temps, calcule en fonction de ws
   ALLOCATE(ws3max(nvp))        
#endif
   
   IF (.NOT.l_testcase) THEN
      DO iv=1,nvp
!        xws(:,iv)=xws_r(:,irk_fil(iv))  ! tjs dans traceur hors key_sedim
!        tocd(iv)=tocd_r(irk_fil(iv))    ! tjs dans traceur hors key_sedim
        ros(iv)=ros_r(irk_fil(iv))
        diam_sed(iv)=diam_r(irk_fil(iv))
      ENDDO
      DO iv=1,nv_state
        cini_sed(iv)=cini_sed_r(irk_fil(iv))
      ENDDO
   ENDIF

   ALLOCATE( rho_bott(limin:limax,ljmin:ljmax) )
   ALLOCATE( raphbx(liminm1:limax,ljmin:ljmax),raphby(limin:limax,ljminm1:ljmax) )
   ALLOCATE( frofonx(liminm1:limax,ljmin:ljmax),frofony(limin:limax,ljminm1:ljmax) )

   PRINT_DBG*, 'FIN SED_ALLOC'

   END SUBROUTINE sed_alloc
 
   !!===========================================================================
 
   SUBROUTINE sed_param
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_param  ***
   !&E
   !&E ** Purpose : reads parameterization of sediment dynamics
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by : ??? init ou main, after initialisation of physical variabless
   !&E                  and before sed_init and sed_alloc
   !&E
   !&E ** External calls : tool_datosec(), tool_sectodat()
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca)
   !&E       !  2009-02  (P. Le Hir)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,    ONLY : iscreenlog,dt
   USE comsubstance, ONLY : nvp,nv_ncp,nvpc,name_var,irk_fil,irkm_var_assoc,  &
                            tdebsubs,date_startsub,xws
   USE parameters,   ONLY : grav

   !! * Local declarations
   LOGICAL               :: l_varassoc
   CHARACTER(len=lchain) :: filepc
   CHARACTER(len=19)     :: tool_sectodat
   INTEGER               :: iv,ivpc,isubs
   REAL(KIND=rlg)        :: tool_datosec

#ifdef key_sedim_mixsed
   NAMELIST/namsedim_mixsed/ l_fricwave,l_morphocoupl,fricwav,l_repsed,       &
                       filrepsed,l_consolid,l_diffused,                 &
                       dtconsol,date_startconsol,                       &
                       l_unised,z0seduni,hseduni,cseduni,               &
                       ksmiuni,ksmauni,sini_sed,tini_sed,               &
                       cvolmaxsort,cvolmaxmel,activlayer,frmudcr2,      &
                       eroa1,eroa2,eroa3,erob1,erob2,erob3,             &
                       corfluer1,corfluer2,csupi,                       &
                       dzsmin,dzsmax,csedmin ,cmudcr,                   &
                       xperm1,xperm2,xsigma1,xsigma2,                   &
                       xdifs1,xdifs2,xdifsi1,xdifsi2,epdifi
#else
   NAMELIST/namsedim_mudsand/ l_fricwave,l_morphocoupl,fricwav, &
                       filrepsed,l_consolid,l_diffused,l_repsed,        &
                       dtconsol,date_startconsol,                       &
                       l_unised,z0seduni,hseduni,cseduni,               &
                       ksmiuni,ksmauni,sini_sed,tini_sed,               &
                       fimpls,xcvgce,frmudcr1,frmudcr2,E0sable,         &
                       alphavase,alphasable,xeros,x1toce,x2toce,        &
                       csupi,csmaxmel,dzsmin,dzsmax,                    &
                       poro1,xtas11,xtas12,poro2,                       &
                       xtas21,xtas22,poro3,xtas31,xtas32,fexcs,         &
                       xdifs1,xdifs2,xdifsi1,xdifsi2,epdifi
#endif

   NAMELIST/namsedoutput/ name_out_eptot,name_out_nbniv,name_out_dzs,name_out_tenfon,&
                          name_out_tenfonc,name_out_tenfonw,&   
                          riog_valid_min_eptot,riog_valid_max_eptot,&
                          riog_valid_min_nbniv,riog_valid_max_nbniv,riog_valid_min_dzs,&
                          riog_valid_max_dzs,riog_valid_min_tenfon,riog_valid_max_tenfon
!cval variable restee locale                          riog_valid_min_cvsed,riog_valid_max_cvsed


   !!--------------------------------------------------------------------------
   !! * Executable part

    filepc='./parasedim.txt'
    OPEN(50,file=filepc,status='old',form='formatted',access='sequential')
#ifdef key_sedim_mixsed
    READ(50,namsedim_mixsed)
    xsigma1sg=xsigma1/grav
#else
    READ(50,namsedim_mudsand)
    cimpls=1.0_rsh-fimpls
    cexcs=1.0_rsh-fexcs
#endif
    fws2=fricwav*0.5_rsh
    READ(50,namsedoutput)
    CLOSE(50)                                  
    ! fin de lecture de la namelist

    WRITE(iscreenlog,*) ' '
    WRITE(iscreenlog,*) '! sedimentology'
    WRITE(iscreenlog,*) '***********************'
  
    ! remind of sediment parameters :
    
#ifdef key_sedim_mixsed
    WRITE(iscreenlog,namsedim_mixsed)
#else
    WRITE(iscreenlog,namsedim_mudsand)
#endif
  
    tstartconsol=tool_datosec(date_startconsol)
    IF(tstartconsol<tdebsubs) THEN
      WRITE(iscreenlog,*) ' '
      WRITE(iscreenlog,*) 'WARNING : Variable date_startconsol is uncorrect. '
      WRITE(iscreenlog,*) 'WARNING : consolidation simulations will start '
      WRITE(iscreenlog,*) 'WARNING : at the begining of substances simulations :' &
                          , date_startsub
    ENDIF
    tstartconsol=MAX(tstartconsol,tdebsubs)
    tconsol=tstartconsol

    PRINT_DBG*, 'FIN SED_PARAM'   
   
  END SUBROUTINE sed_param
 
   !!===========================================================================
 
  SUBROUTINE sed_init
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_init  ***
   !&E
   !&E ** Purpose : initialize the sediment geometry & content
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  main, after reading sed_param
   !&E                 only if not.l_repsed
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from subsedim.F90 (from F. Dufois)
   !&E       !  2009-02  (P. Le Hir, F. Cayocca) modified for new sand/mud sediment 
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nvp,nvpc,nv_state,nv_adv,irk_mod,nv_tot,irkm_var_assoc
   USE comvars2d,    ONLY : ig,id,h0
   USE parameters,   ONLY : valmanq
#ifdef key_sedim_mixsed
   USE parameters,   ONLY : grav
#endif

   !! * Local declarations
   CHARACTER*50   :: filesand,filemud
   INTEGER        :: i,j,k,iv                
   REAL(KIND=rsh) :: somalp,dzsf
   
   !!--------------------------------------------------------------------------
   !! * Executable part

    ! characterization of sediment variables (new parameters)
#ifdef key_sedim_mixsed
    DO iv=1,nvp
      ! here, for simplification, water density = 1000. kg/m3 
      rosmrowsros(iv)=(ros(iv)-1000.0_rsh)/ros(iv)
      diamstar(iv)=diam_sed(iv)*10000.0_rsh*(grav*(ros(iv)/1000.0_rsh-1.0_rsh))**0.33_rsh
      ! according to Soulsby, 1997, and if viscosity = 10-6 m/s :
      ws0(iv)=.000001_rsh*((107.33_rsh+1.049_rsh*diamstar(iv)**3)**0.5_rsh-10.36_rsh)/diam_sed(iv)
      tetacri0(iv)=0.3_rsh/(1.0_rsh+1.2_rsh*diamstar(iv))+0.055_rsh*(1.0_rsh-EXP(-0.02_rsh*diamstar(iv)))
      stresscri0(iv)=tetacri0(iv)*grav*(ros(iv)-1000.0_rsh)*diam_sed(iv)
      xnielsen(iv)=12.0_rsh*1000.0_rsh/grav/(1.0_rsh-1000.0_rsh/ros(iv))
    ENDDO
#endif

    z0sed(limin:limax,ljmin:ljmax)=z0seduni
    ksmi(limin:limax,ljmin:ljmax)=ksmiuni
    ksma(limin:limax,ljmin:ljmax)=ksmauni
    hsed(limin:limaxp1,ljmin:ljmaxp1)=valmanq
    dzs(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=valmanq
    cv_sed(-1:nv_tot,ksdmin:ksdmax,limin:limax,ljmin:ljmax)=valmanq
    c_sedtot(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=valmanq

    IF (l_unised)THEN
   
      ! uniform bed :
      WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) ksmi(limin:limax,ljmin:ljmax)=ksmiuni
      WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) ksma(limin:limax,ljmin:ljmax)=ksmauni
      WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) hsed(limin:limaxp1,ljmin:ljmaxp1)=hseduni
      DO k=ksdmin,ksdmax
        WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) dzs(k,limin:limax,ljmin:ljmax)=0.0_rsh
        WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) c_sedtot(k,limin:limax,ljmin:ljmax)=0.0_rsh
        DO iv=-1,nv_tot
          WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) cv_sed(iv,k,limin:limax,ljmin:ljmax)=0.0_rsh
        ENDDO
      ENDDO
 
      DO j=ljmin,ljmax
        DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
          DO k=ksmi(i,j),ksma(i,j)
              somalp=0.
!VAL2FLO encore une initialisation ?
              c_sedtot(k,i,j)=0.0_rsh
              DO iv=1,nvpc
                ! constitutive variables : cini_sed in % of cseduni
                cv_sed(iv,k,i,j)=cseduni*cini_sed(iv)
                c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
                somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
              ENDDO
              poro(k,i,j)=1.0_rsh-somalp
              DO iv=nvpc+1,nvp
                ! non constitutive variables : cini_sed in conc/kg of sediment
                IF (irkm_var_assoc(iv) >0) THEN
                  cv_sed(iv,k,i,j)=cini_sed(iv)*cv_sed(irkm_var_assoc(iv),k,i,j)
                ELSE
                  cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
                END IF
              ENDDO
              DO iv=nvp+1,nv_state
                ! dissolved variables : cini_sed in conc/m3 of pore water
                ! + fixed variables : cini_sed in conc/m3 of sediment
                cv_sed(iv,k,i,j)=cini_sed(iv)
              ENDDO
              cv_sed(-1,k,i,j)=tini_sed
              cv_sed( 0,k,i,j)=sini_sed
          ENDDO
        ENDDO
      ENDDO
    ELSE

      ! non uniform bed :
      ! -----------------------

      PRINT*, 'NON UNIFORM BED COVERAGE'
      PRINT*, 'you have to read a netcdf file describing for each grid cell'
      PRINT*, 'the values for hsed(i,j),z0sed(i,j),ksmi(i,j),ksma(i,j)'
      PRINT*, 'c_sedtot(i,j),cini_sed(iv,iv=1:nvstate)'
      STOP
      
! variables a declarer
! nom du fichier des numeros de facies a mettre dans le parasedim
! lire dans ce fichier i,j,hsed(i,j),z0sed(i,j),ksmi_file(i,j),ksma_file(i,j)
!                      c_sedtot_file(i,j),cini_sed(iv,iv=1:nvstate)
!
!      ksmi(limin:limax,ljmin:ljmax)=ksmi_file(limin:limax,ljmin:ljmax)
!      DO j=ljmin,ljmax
!        DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
!          DO k=ksmi(i,j),ksma(i,j)
!            c_sedtot(k,i,j)=c_sedtot_file(i,j)
!            somalp=0.
!            DO iv=1,nvpc
!            ! - constitutive variables : cini_sed as percentage of sediment mass
!              cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
!              somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
!            ENDDO
!            poro(k,i,j)=1.-somalp
!            DO iv=nvpc+1,nvp
!            ! - non constitutive variables : cini_sed in conc/kg of sediment
!              cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
!            ENDDO
!            DO iv=nvp+1,nv_adv
!            ! - dissolved variables : cini_sed in conc/m3 of pore water
!             cv_sed(iv,k,i,j)=cini_sed(iv)
!            ENDDO
!            cv_sed(-1,k,i,j)=tini_sed
!            cv_sed( 0,k,i,j)=sini_sed
!          ENDDO
!
!        ENDDO
!      ENDDO


    ENDIF
!
!   FIN COUVERTURE SEDIMENTAIRE NON UNIFORME L_UNISED
!   -------------------------------------------------
!   regular vertical mesh
!   ---------------------

    DO j=ljmin,ljmax
      DO i=limin,limax
        IF(ksma(i,j).NE.0)THEN
          dzsf=hsed(i,j)/float(ksma(i,j)-ksmi(i,j)+1)
          DO k=ksmi(i,j),ksma(i,j)
            dzs(k,i,j)=dzsf
          ENDDO
#ifdef key_sedim_mudsand
          DO k=ksmi(i,j),ksma(i,j)-1
            dzsiin(k,i,j)=2.0_rsh/(dzs(k,i,j)+dzs(k+1,i,j))
          ENDDO
#endif
        ELSE
          dzs(k,i,j)=0.0_rsh
        ENDIF
      ENDDO
    ENDDO

    PRINT_DBG*, 'FIN SED_INIT'

  END SUBROUTINE sed_init

   !!===========================================================================
 
  SUBROUTINE sed_init_fromfile
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_init_fromfile  ***
   !&E
   !&E ** Purpose : manages the fields to be re-initialized in case of the 
   !&E              continuation of a previous run
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  main 
   !&E
   !&E ** External calls : ionc4_openr, ionc4_read_time,
   !&E                     ionc4_read_subxyt, ionc4_read_subzxyt
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from init.F90
   !&E       !  oct.2008 inserted into module_sedimento.F90 by F. Cayocca
   !&E       !  2009-02  modified for new sand/mud sediment (P. Le Hir & F. Cayocca)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nv_adv,nvp,nvpc,flx_w2s,nv_tot,irkm_var_assoc,nv_state
   USE comvars2d,    ONLY : tsauv,t,saverestart_step,ig,id,h0,l_restart_subs
   USE parameters,   ONLY : riosh,grav,valmanq

   !! * Local declarations
   CHARACTER*50     :: filesand,filemud
   CHARACTER*8      :: nomc
   INTEGER          :: i,j,k,iv                
   REAL(KIND=rsh)                                             ::  somalp,dzsf
   REAL(KIND=riosh),DIMENSION(ksdmax,limin:limax,ljmin:ljmax) ::  tzecrits
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)        ::  xind

   !!--------------------------------------------------------------------------   
   !! * Executable part

#ifdef key_sedim_mixsed
     ! characterization of sediment variables (new parameters)
     DO iv=1,nvp
       ! here, for simplification, water density = 1000. kg/m3 
       rosmrowsros(iv)=(ros(iv)-1000.0_rsh)/ros(iv)
       diamstar(iv)=diam_sed(iv)*10000.0_rsh*(grav*(ros(iv)/1000.0_rsh-1.0_rsh))**0.33_rsh
       ! according to Soulsby, 1997, and if viscosity = 10-6 m/s :
       ws0(iv)=.000001_rsh*((107.33_rsh+1.049_rsh*diamstar(iv)**3)**0.5_rsh-10.36_rsh)/diam_sed(iv)
       tetacri0(iv)=0.3_rsh/(1.0_rsh+1.2_rsh*diamstar(iv))+0.055_rsh*(1.0_rsh-EXP(-0.02_rsh*diamstar(iv)))
       stresscri0(iv)=tetacri0(iv)*grav*(ros(iv)-1000.0_rsh)*diam_sed(iv)
       xnielsen(iv)=12.0_rsh*1000.0_rsh/grav/(1.0_rsh-1000.0_rsh/ros(iv))
     ENDDO
#endif

     CALL ionc4_openr(filrepsed,l_out_nc4par=.true.)

     CALL ionc4_read_subxyt(filrepsed,'ksmi',xind,limin,limax,ljmin,ljmax,1,0,0)
     DO j=ljmin,ljmax
        DO i=limin,limax
          ksmi(i,j)=INT(xind(i,j))
        END DO
     END DO

     CALL ionc4_read_subxyt(filrepsed,'ksma',xind,limin,limax,ljmin,ljmax,1,0,0)
     DO j=ljmin,ljmax 
        DO i=limin,limax 
          ksma(i,j)=INT(xind(i,j)) 
        END DO 
     END DO

     CALL ionc4_read_subzxyt(filrepsed,'dzs',tzecrits,limin,limax,ljmin,ljmax,1,ksdmax,1,0,0,1)
     DO j=ljmin,ljmax 
      DO i=limin,limax 
        DO k = 1,ksdmax 
           dzs(k,i,j)=tzecrits(k,i,j)
        END DO 
      END DO 
     END DO

     IF (l_restart_subs) THEN
      DO iv =-1,nv_adv
       IF (IV>0) WRITE(nomc,10) iv
       IF (iv==-1) nomc='concTEM'
       IF (iv==0) nomc='concSAL'
       CALL ionc4_read_subzxyt(filrepsed,nomc,tzecrits,                         &
                           limin,limax,ljmin,ljmax,1,ksdmax,1,0,0,1)
       DO j =ljmin,ljmax
         DO i =limin,limax
           DO k =1,ksdmax
             cv_sed(iv,k,i,j)=tzecrits(k,i,j)
           ENDDO
         ENDDO
       ENDDO
 
       IF (IV>0) WRITE(nomc,20) iv
       IF (iv==-1) nomc='fxw2sTEM'
       IF (iv==0) nomc='fxw2sSAL'
       CALL ionc4_read_subxyt(filrepsed,nomc,xind,limin,limax,ljmin,ljmax,1,0,0)
       DO j = ljmin,ljmax
         DO i = limin,limax
           flx_w2s(iv,i,j) = xind(i,j)
         ENDDO
       ENDDO

      ENDDO
      CALL ionc4_close(filrepsed)

     ELSE

!      reprise des conditions initiales du fichier variable.dat
!      et au lieu du fichier de restart

      cv_sed(-1:nv_tot,ksdmin:ksdmax,limin:limax,ljmin:ljmax)=valmanq
      c_sedtot(ksdmin:ksdmax,limin:limax,ljmin:ljmax)=valmanq
   
      ! uniform bed :
      DO k=ksdmin,ksdmax
        WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) c_sedtot(k,limin:limax,ljmin:ljmax)=0.0_rsh
        DO iv=-1,nv_tot
          WHERE (h0(limin:limax,ljmin:ljmax) /= -valmanq) cv_sed(iv,k,limin:limax,ljmin:ljmax)=0.0_rsh
        ENDDO
      ENDDO
 
      DO j=ljmin,ljmax
        DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
          IF(h0(i,j) /= -valmanq) THEN
            DO k=ksmi(i,j),ksma(i,j)
              somalp=0.
              c_sedtot(k,i,j)=0.0_rsh
              DO iv=1,nvpc
                ! constitutive variables : cini_sed in % of cseduni
                cv_sed(iv,k,i,j)=cseduni*cini_sed(iv)
                c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
                somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
              ENDDO
              poro(k,i,j)=1.0_rsh-somalp
              DO iv=nvpc+1,nvp
                ! non constitutive variables : cini_sed in conc/kg of sediment
                IF (irkm_var_assoc(iv) >0) THEN
                  cv_sed(iv,k,i,j)=cini_sed(iv)*cv_sed(irkm_var_assoc(iv),k,i,j)
                ELSE
                  cv_sed(iv,k,i,j)=cini_sed(iv)*c_sedtot(k,i,j)
                END IF
              ENDDO
              DO iv=nvp+1,nv_state
                ! dissolved variables : cini_sed in conc/m3 of pore water
                ! + fixed variables : cini_sed in conc/m3 of sediment
                cv_sed(iv,k,i,j)=cini_sed(iv)
              ENDDO
              cv_sed(-1,k,i,j)=tini_sed
              cv_sed( 0,k,i,j)=sini_sed
            ENDDO
          ENDIF
        ENDDO
      ENDDO

     ENDIF


     !FLO? tsauv=t+saverestart_step

     DO j=ljmin,ljmax
       DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
         z0sed(i,j)=z0seduni
         IF (h0(i,j) /= -valmanq) THEN
         DO k=ksmi(i,j),ksma(i,j)
           somalp=0.0_rsh
           c_sedtot(k,i,j)=0.0_rsh
           DO iv=1,nvpc
             c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
             somalp=somalp+cv_sed(iv,k,i,j)/ros(iv)
           END DO
           poro(k,i,j)=1.0_rsh-somalp
         END DO
#ifdef key_sedim_mudsand
         DO k=ksmi(i,j),ksma(i,j)-1
            dzsiin(k,i,j)=2.0_rsh/(dzs(k,i,j)+dzs(k+1,i,j))
         ENDDO
#endif
        END IF
       END DO
     END DO
      
   10 FORMAT('concen',i2.2)
   20 FORMAT('flxw2s',i2.2)
      
  END SUBROUTINE sed_init_fromfile

   !!===========================================================================
 
  SUBROUTINE sed_morphoinit
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_morphoinit  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  main 
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  march 2009 (P. Le Hir) enables morphodynamics computation
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,    ONLY : ig,id,hx,hy,h0,h0fond
   USE comvarp2d,    ONLY : ssh
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh

   !! * Local declarations
   INTEGER                  :: i,j,k          
   REAL(KIND=rsh)           :: hultra

   !!--------------------------------------------------------------------------  
   !! * Executable part

      ! morphox(i,j) & morphoy(i,j) indicate whether morphodynamic coupling  
      ! is effective in right or upper side of cell (i,j)
      ! if zero, hsed is ignored on this border
      morphox(liminm1:limax,ljmin  :ljmax)=1.0_rsh
      morphoy(limin  :limax,ljminm1:ljmax)=1.0_rsh

      DO j=ljmin,ljmax
        DO i=liminp1,limaxm1
          hsed(i,j)=0.0_rsh
          DO k=ksmi(i,j),ksma(i,j)
            hsed(i,j)=hsed(i,j)+dzs(k,i,j)
          ENDDO
        ENDDO
        IF_MPI (imin == limin) &
        hsed(imin,j)=hsed(imin+1,j)
        IF_MPI (imax == limax) &
        hsed(imax,j)=hsed(imax-1,j)
      ENDDO
!VAL2FLO florence pourquoi CL en j pas gerees de la meme maniere ?

   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hsed(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hsed(limin:limaxp1,ljmin:ljmaxp1))

      DO j=ljmin,ljmaxm1
        DO i=limin,limaxm1
          hxi(i,j)=hx(i,j)+0.5_rsh*(hsed(i,j)+hsed(i+1,j))*morphox(i,j)
          hyi(i,j)=hy(i,j)+0.5_rsh*(hsed(i,j)+hsed(i,j+1))*morphoy(i,j)
        ENDDO
      ENDDO
      IF_MPI (ljmax==jmax) THEN
      DO i=limin,limaxm1
        hxi(i,jmax)=hx(i,jmax)+0.5_rsh*(hsed(i,jmax)+hsed(i+1,jmax))*morphox(i,jmax)
        hyi(i,jmax)=hy(i,jmax)+hsed(i,jmax)*morphoy(i,jmax)
      ENDDO
      ENDIF_MPI
      IF_MPI (limax == imax) THEN
        DO j=ljmin,ljmaxm1
          hxi(imax,j)=hx(imax,j)+hsed(imax,j)*morphox(imax,j)
          hyi(imax,j)=hy(imax,j)+0.5_rsh*(hsed(imax,j)+hsed(imax,j+1))*morphoy(imax,j)
        ENDDO
      ENDIF_MPI
      IF_MPI (limax == imax .AND. ljmax == jmax) THEN
        hxi(imax,jmax)=hx(imax,jmax)+hsed(imax,jmax)*morphox(imax,jmax)
        hyi(imax,jmax)=hy(imax,jmax)+hsed(imax,jmax)*morphoy(imax,jmax)
      ENDIF_MPI

      ! evaluation de la cote des mailles
      ! ---------------------------------

      DO j=ljminp1,ljmax
        DO i=liminp1,limax
          h0(i,j)=MAX(hx(i,j),hy(i,j),hx(i-1,j),hy(i,j-1))+h0fond
      !    hmoy(i,j)=(hx(i,j)*morphox(i,j)+hx(i-1,j)*morphox(i-1,j)  &
      !        +hy(i,j)*morphoy(i,j)+hy(i,j-1)*morphoy(i,j-1))/ &
      !        (morphox(i,j)+morphox(i-1,j)+  &
      !         morphoy(i,j)+morphoy(i,j-1)+eps)  
          ssh(i,j)=MAX(ssh(i,j),-h0(i,j))
        ENDDO
      ENDDO
      IF_MPI (limin == imin) THEN
        DO j=ljminp1,ljmax
          h0(imin,j)=MAX(hx(imin,j),hy(imin,j),hy(imin,j-1))+h0fond
      !   hmoy(imin,j)=(hx(imin,j)*morphox(imin,j)+hy(imin,j)*morphoy(imin,j)  &
      !            +hy(imin,j-1)*morphoy(imin,j-1))/  &
      !        (morphox(imin,j)+morphoy(imin,j)+morphoy(imin,j-1)+eps)   
          ssh(imin,j)=MAX(ssh(imin,j),-h0(imin,j))
      ENDDO
      ENDIF_MPI
      IF_MPI (ljmin == jmin) THEN
        DO i=liminp1,limax
          h0(i,jmin)=MAX(hx(i,jmin),hy(i,jmin),hx(i-1,jmin))+h0fond
      !   hmoy(i,jmin)=(hx(i,jmin)*morphox(i,j)+hy(i,jmin)*morphoy(i,jmin)  &
      !            +hx(i-1,jmin)*morphox(i-1,jmin))/  &
      !        (morphox(i,jmin)+morphoy(i,jmin)+morphox(i-1,jmin)+eps)        
          ssh(i,jmin)=MAX(ssh(i,jmin),-h0(i,jmin))
        ENDDO
      ENDIF_MPI
      IF_MPI (limin == imin .AND. ljmin == jmin) THEN
        h0(imin,jmin)=MAX(hx(imin,jmin),hy(imin,jmin))+h0fond
      ! hmoy(imin,jmin)=(hx(imin,jmin)*morphox(imin,jmin)+  &
      !      hy(imin,jmin)*morphoy(imin,jmin))/  &
      !      (morphox(imin,jmin)+morphoy(imin,jmin)+eps)
        ssh(imin,jmin)=MAX(ssh(imin,jmin),-h0(imin,jmin))
      ENDIF_MPI
    
#ifdef key_toseelater
!ATTENTION : hupper et dhcell non utilises, a integrer a l avenir pour bien gerer 
!            le decouvrement (coherence avec fwet de SIAM) - voir PLH
      hultra=10000.0_rsh
      DO j=ljminp1,ljmaxm1
        DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
          hupper(i,j)=MIN(hx(i,j)*morphox(i,j)+(1.0_rsh-morphox(i,j))*hultra,         &
                          hx(i-1,j)*morphox(i-1,j)+(1.0_rsh-morphox(i-1,j))*hultra,   &
                          hy(i,j)*morphoy(i,j)+(1.0_rsh-morphoy(i,j))*hultra,         &
                          hy(i,j-1)*morphoy(i,j-1)+(1.0_rsh-morphoy(i,j-1))*hultra)
          IF(hupper(i,j).GE.hultra)hupper(i,j)=-99.99_rsh
          dhcell(i,j)=h0(i,j)-hupper(i,j)
        ENDDO  
      ENDDO        
#endif

      !calcul des htx, hty initiaux (hauteurs totales)
      !===============================================
      !normalement htx et hex , hty et hey sont recalcules en debut de step3dxy/yx
      !do j=jmin,jmax
      !   do i=imin,imax-1
      !     hzx=hx(i,j)+hz1(i,j)
      !     hzxp=hx(i,j)+hz1(i+1,j)
      !    janv05: htx(i,j)=amax1(hzx,hzxp,0.0_rsh)
      !     htx(i,j)=amax1(0.0_rsh,0.5_rsh*(hzx+hzxp))
      !   enddo
      !enddo
      !i=imax
      !do j=jmin,jmax
      !   hzx=hx(i,j)+hz1(i,j)
      !   htx(i,j)=0.5_rsh*(hzx+abs(hzx))
      !enddo
      !do j=jmin,jmax-1
      !   do i=imin,imax
      !     hzy=hy(i,j)+hz1(i,j)
      !     hzyp=hy(i,j)+hz1(i,j+1)
      !    janv05: hty(i,j)=amax1(hzy,hzyp,0.0_rsh)
      !     hty(i,j)=amax1(0.0_rsh,0.5_rsh*(hzy+hzyp))
      !   enddo
      !enddo
      !j=jmax
      !do i=imin,imax
      !   hzy=hy(i,j)+hz1(i,j)
      !   hty(i,j)=0.5_rsh*(hzy+abs(hzy))
      !enddo
      !do j=jmin,jmax
      !  do i=imin,imax
      !    if(htx(i,j).lt.0.0_rsh)write(*,*)'init i j htx',i,j,htx(i,j)
      !    if(hty(i,j).lt.0.0_rsh)write(*,*)'init i j hty',i,j,hty(i,j)
      !  enddo
      !enddo
      ! do i=imin,imax
      ! do j=jmin,jmax
      !   hmoyini(i,j)=hmoy(i,j)
      !   hmoyini(i,j)=hmoy(i,j)
      ! enddo
      !enddo        

    CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
    CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
    CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,h0(liminm2:limaxp2,ljminm2:ljmaxp2))
    CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,h0(liminm2:limaxp2,ljminm2:ljmaxp2))
   ! exchange hupper and dhcell in the future if required
   ! hxi, hyi are not exchanged because neighboring cells are not used


  END SUBROUTINE sed_morphoinit     

   !!===========================================================================
 
  SUBROUTINE sed_morpho
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_morpho  ***
   !&E
   !&E ** Purpose : 
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  step, at the end
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-03  (P. Le Hir) enables morphodynamics computation
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comvars2d,    ONLY : ig,id,hx,hy,h0,h0fond
   USE comvarp2d,    ONLY : ssh
   USE_MPI toolmpi,  ONLY : ex_i_rsh,ex_j_rsh

   !! * Local declarations
   INTEGER                  :: i,j,k
   REAL(KIND=rsh)           :: hultra                
   
   !!--------------------------------------------------------------------------
   !! * Executable part

      PRINT_DBG*, 'DANS SED_MORPHO'
      DO j=ljmin,ljmax
        DO i=liminp1,limaxm1
          hsed(i,j)=0.0_rsh
          DO k=ksmi(i,j),ksma(i,j)
            hsed(i,j)=hsed(i,j)+dzs(k,i,j)
          ENDDO
        ENDDO
        IF_MPI (imin == limin) &
        hsed(imin,j)=hsed(imin+1,j)
        IF_MPI (imax == limax) &
        hsed(imax,j)=hsed(imax-1,j)
      ENDDO
!VAL2FLO florence pourquoi CL en j pas gerees de la meme maniere ?

   CALL_MPI ex_i_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hsed(limin:limaxp1,ljmin:ljmaxp1))
   CALL_MPI ex_j_rsh(0,1,1,limin,limaxp1,ljmin,ljmaxp1,hsed(limin:limaxp1,ljmin:ljmaxp1))
      
      DO j=ljmin,ljmaxm1
      DO i=limin,limaxm1
!valerie revoir morpho coupee Remettre morpho
!        hx(i,j)=hxi(i,j)-0.5_rsh*(hsed(i,j)+hsed(i+1,j))*morphox(i,j)
!        hy(i,j)=hyi(i,j)-0.5_rsh*(hsed(i,j)+hsed(i,j+1))*morphoy(i,j)
!valerie introduction bug dans code cote Florence non ?
!FLO2VAL : routine pas stable, il faut garder les 2. je ne vois pas de quel bug tu parles
        hx(i,j)=hxi(i,j)-MIN(hsed(i,j),hsed(i+1,j))*morphox(i,j)
        hy(i,j)=hyi(i,j)-MIN(hsed(i,j),hsed(i,j+1))*morphoy(i,j)
      ENDDO
      ENDDO

      IF_MPI (ljmax == jmax) THEN
        DO i=limin,limaxm1
          hx(i,jmax)=hxi(i,jmax)-0.5_rsh*(hsed(i,jmax)+hsed(i+1,jmax))*morphox(i,jmax)
          !hx(i,jmax)=hxi(i,jmax)-MIN(hsed(i,jmax),hsed(i+1,jmax))*morphox(i,jmax)
          hy(i,jmax)=hyi(i,jmax)-hsed(i,jmax)*morphoy(i,jmax)
        ENDDO
      ENDIF_MPI
      IF_MPI (limax == imax) THEN
        DO j=ljmin,ljmaxm1
          hx(imax,j)=hxi(imax,j)-hsed(imax,j)*morphox(imax,j)
!          hy(imax,j)=hyi(imax,j)-MIN(hsed(imax,j),hsed(imax,j+1))*morphoy(imax,j)
          hy(imax,j)=hyi(imax,j)-0.5_rsh*(hsed(imax,j)+hsed(imax,j+1))*morphoy(imax,j)
        ENDDO
      ENDIF_MPI
      IF_MPI (imax == limax .AND. jmax == ljmax) THEN
        hx(imax,jmax)=hxi(imax,jmax)-hsed(imax,jmax)*morphox(imax,jmax)
        hy(imax,jmax)=hyi(imax,jmax)-hsed(imax,jmax)*morphoy(imax,jmax)
      ENDIF_MPI

      ! evaluation de la cote des mailles
      !---------------------------------

      DO j=ljminp1,ljmax
        DO i=liminp1,limax
          h0(i,j)=MAX(hx(i,j),hy(i,j),hx(i-1,j),hy(i,j-1))+h0fond
          ssh(i,j)=MAX(ssh(i,j),-h0(i,j))
      !     hmoy(i,j)=(hx(i,j)*morphox(i,j)+hx(i-1,j)*morphox(i-1,j)  &
      !        +hy(i,j)*morphoy(i,j)+hy(i,j-1)*morphoy(i,j-1))/  &
      !        (morphox(i,j)+morphox(i-1,j)+  &
      !         morphoy(i,j)+morphoy(i,j-1)+eps)
        ENDDO
      ENDDO
      

      IF_MPI (limin == imin) THEN
        DO j=ljminp1,ljmax
          h0(imin,j)=MAX(hx(imin,j),hy(imin,j),hy(imin,j-1))+h0fond
          ssh(imin,j)=MAX(ssh(imin,j),-h0(imin,j))
      !   hmoy(imin,j)=(hx(imin,j)*morphox(imin,j)+hy(imin,j)*morphoy(imin,j)  &
      !            +hy(imin,j-1)*morphoy(imin,j-1))/  &
      !        (morphox(imin,j)+morphoy(imin,j)+morphoy(imin,j-1)+eps)   
        ENDDO
      ENDIF_MPI
      IF_MPI (ljmin == jmin) THEN
        DO i=liminp1,limax
          h0(i,jmin)=MAX(hx(i,jmin),hy(i,jmin),hx(i-1,jmin))+h0fond
          ssh(i,jmin)=MAX(ssh(i,jmin),-h0(i,jmin))
      !   hmoy(i,jmin)=(hx(i,jmin)*morphox(i,jmin)+hy(i,jmin)*morphoy(i,jmin)  &
      !            +hx(i-1,jmin)*morphox(i-1,jmin))/  &
      !        (morphox(i,jmin)+morphoy(i,jmin)+morphox(i-1,jmin)+eps)        
        ENDDO
      ENDIF_MPI
      IF_MPI (imin == limin .AND. jmin == ljmin) THEN
        h0(imin,jmin)=MAX(hx(imin,jmin),hy(imin,jmin))+h0fond
        ssh(imin,jmin)=MAX(ssh(imin,jmin),-h0(imin,jmin))
      !   hmoy(imin,jmin)=(hx(imin,jmin)*morphox(imin,jmin)+  &
      !      hy(imin,jmin)*morphoy(imin,jmin))/  &
      !      (morphox(imin,jmin)+morphoy(imin,jmin)+eps)
      ENDIF_MPI


!ATTENTION : hupper et dhcell non utilises, a integrer a l avenir pour bien gerer 
!            le decouvrement (coherence avec fwet de SIAM) - voir PLH

#ifdef key_toseelater
      hultra=10000.0_rsh
      DO j=ljminp1,ljmaxm1
        DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
          hupper(i,j)=MIN(hx(i,j)*morphox(i,j)+(1.0_rsh-morphox(i,j))*hultra,         &
                          hx(i-1,j)*morphox(i-1,j)+(1.0_rsh-morphox(i-1,j))*hultra,   &
                          hy(i,j)*morphoy(i,j)+(1.0_rsh-morphoy(i,j))*hultra,         &
                          hy(i,j-1)*morphoy(i,j-1)+(1.0_rsh-morphoy(i,j-1))*hultra)
          IF(hupper(i,j).GE.hultra)hupper(i,j)=-99.99_rsh
          dhcell(i,j)=h0(i,j)-hupper(i,j)
        ENDDO
      ENDDO
#endif


      ! calcul des htx, hty nouveaux
      ! ----------------------------
      !  normalement htx et hex , hty et hey sont recalculs en dbut de step3dxy/yx
      !   do j=jmin,jmax
      !     do i=imin,imax-1
      !       hzx=hx(i,j)+hz1(i,j)
      !       hzxp=hx(i,j)+hz1(i+1,j)
      !janv05: htx(i,j)=amax1(hzx,hzxp,0.0_rsh)
      !       htx(i,j)=amax1(0.0_rsh,0.5_rsh*(hzx+hzxp))
      !     enddo
      !   enddo
      !   do j=jmin,jmax
      !     i=id(j)
      !     hzx=hx(i,j)+hz1(i,j)
      !     htx(i,j)=0.5_rsh*(hzx+abs(hzx))
      !   enddo
      !   do j=jmin,jmax-1
      !     do i=imin,imax
      !       hzy=hy(i,j)+hz1(i,j)
      !       hzyp=hy(i,j)+hz1(i,j+1)
      !janv05: hty(i,j)=amax1(hzy,hzyp,0.0_rsh)
      !       hty(i,j)=amax1(0.0_rsh,0.5_rsh*(hzy+hzyp))
      !     enddo
      !   enddo
      !   j=jmax
      !   do i=imin,imax
      !     hzy=hy(i,j)+hz1(i,j)
      !     hty(i,j)=0.5_rsh*(hzy+abs(hzy))
      !   enddo

   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,h0(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,h0(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hx(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hy(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,hy(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_i_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))
   CALL_MPI ex_j_rsh(-2,2,1,liminm2,limaxp2,ljminm2,ljmaxp2,ssh(liminm2:limaxp2,ljminm2:ljmaxp2))

  END SUBROUTINE sed_morpho      

   !!==============================================================================

  SUBROUTINE sed_skinstress
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_skinstress  ***
   !&E
   !&E ** Purpose : computes current related bottom shear stress
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  stepu, stepv
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       ! 2011-11 (J. Vareilles) Change in the expression of the bottom stress to be compatible with the 2D/3D
   !&E                              distinctions as is done in the hydrodynamic part
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE parameters,     ONLY : kmax,epsilon,grav,pi,valmanq,exp1
   USE comvarp2d,      ONLY : ssh,u,v
   USE comvars2d,      ONLY : hx,hy
   USE comvarp3d,      ONLY : uz,vz,boz
   USE comvars2d,      ONLY : ig,id,h0,h0fond,hex,hey,htx,hty,hm,hminfrot,hmaxfrot,nbouc
   USE comsubstance,   ONLY : tdebsubs
   USE comvars3d,      ONLY : sig,rhoref,dsigu
#ifdef key_wave
   USE comwave,        ONLY : wave_ubot, wave_tp
#endif
#if defined key_siggen || defined key_gencoord
   USE toolgeom,       ONLY : f_hzu_x,f_hzu_y
#endif
      
   USE_MPI toolmpi,    ONLY : ex_i_rsh,ex_j_rsh

   !! * Local declarations
   INTEGER i,j,k
   REAL(KIND=rsh)                                   :: sigfrot,vit,z,thoule,hhoule
   REAL(KIND=rsh)                                   :: alofwscd,tentot,xwa,coef_a,coef_b,       &
                                                       coef_m,coef_n,coef_p,coef_q,htn
      
   REAL(KIND=rsh),PARAMETER                         :: zfrot=0.010_rsh
   
   REAL(KIND=rsh),PARAMETER                         :: a1=-0.07_rsh, a2=1.87_rsh, a3=-0.34_rsh, a4=-0.12_rsh,   &
                                                       b1=0.27_rsh, b2=0.51_rsh, b3=-0.1_rsh, b4=-0.24_rsh,     &
                                                       cm1=0.72_rsh, cm2=-0.33_rsh, cm3=0.08_rsh, cm4=0.34_rsh, &
                                                       cn1=0.78_rsh, cn2=-0.23_rsh, cn3=0.12_rsh, cn4=-0.12_rsh,&
                                                       p1=-0.75_rsh, p2=0.13_rsh, p3=0.12_rsh, p4=0.02_rsh,     &
                                                       q1=0.89_rsh, q2=0.4_rsh, q3=0.5_rsh, q4=-0.28_rsh,       &     
                                                       hminwave=0.05_rsh

   REAL(KIND=rsh),PARAMETER                         :: a1pa2=a1+a2,a3pa4=a3+a4,                 &
                                                       b1pb2=b1+b2,b3pb4=b3+b4,                 &
                                                       cm1pm2=cm1+cm2,cm3pm4=cm3+cm4,           &
                                                       cn1pn2=cn1+cn2,cn3pn4=cn3+cn4,           &
                                                       p1pp2=p1+p2,p3pp4=p3+p4,                 &
                                                       q1pq2=q1+q2,q3pq4=q3+q4                 

   !!--------------------------------------------------------------------------
   !! * Executable part


   rho_bott(limin:limax,ljmin:ljmax)=rhoref*(1-boz(1,limin:limax,ljmin:ljmax)/grav)
!   cdcurrent=0.16_rsh/(LOG(1.0_rsh/z0seduni))**2

   DO j=ljmin,ljmax
     DO i=limin,limax
       raphbx(i,j)=0.0_rsh
       raphby(i,j)=0.0_rsh
       frofonx(i,j)=0.0_rsh
       frofony(i,j)=0.0_rsh
     ENDDO
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
        IF (htx(i,j).LE.hm) THEN 
           raphbx(i,j)=ABS(u(i,j))/(ABS(u(i,j))+epsilon)
        ELSE 
           raphbx(i,j)=ABS(uz(1,i,j))/(ABS(uz(1,i,j))+epsilon)
        ENDIF 

        IF(hty(i,j).LE.hm) THEN 
           raphby(i,j)=ABS(v(i,j))/(ABS(v(i,j))+epsilon)
        ELSE
           raphby(i,j)=ABS(vz(1,i,j))/(ABS(vz(1,i,j))+epsilon)
        ENDIF 
     ENDDO
   ENDDO

   ! the bottom stress needs to be computed at a fairly "uniform" distance from the bed
   ! throughout the domain. This distance is called zfrot (=0.5), and transcribed into the
   ! corresponding sigma layer
   ! Otherwise, the bottom stress is simply computed in the first layer 

!FLO : rajouter un booleen dans parasedim pour choisir l option
   
   DO j=ljminp1,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
       !sigfrot=zfrot/(htx(i,j)+epsilon)-1
       !k=1
       !DO WHILE (sig(k).le.sigfrot)
       !k=k+1
       !ENDDO
       !k=MIN(k,kmax)
       !IF (i.eq.160 .AND. j.eq.2) PRINT*, k,sig(k),sigfrot,htx(i,j)

       IF(htx(i,j).LE.hm) THEN            !under hm, hydrodynamic 2D calculation
          vit=SQRT(u(i,j)**2+0.0625_rsh*(v(i,j)+v(i+1,j)            &
              +v(i,j-1)+v(i+1,j-1))**2)*u(i,j)
          frofonx(i,j)=0.16_rsh*(LOG(MAX(MIN(htx(i,j),hmaxfrot),hminfrot)/(z0sed(i,j)*exp1)))**(-2)*vit
       ELSE                               !above hm, hydrodynamic 3D calculation
          k=1
#if defined key_siggen || defined key_gencoord
          z=0.5_rsh*f_hzu_x(hx(i,j),ssh(i,j),k,i,j)*dsigu(k)
#else
          z=(1.0_rsh+sig(k))*htx(i,j)
#endif
          vit=SQRT(uz(k,i,j)**2+0.0625_rsh*(vz(k,i,j)+vz(k,i+1,j)            &
               +vz(k,i,j-1)+vz(k,i+1,j-1))**2)*uz(k,i,j)
          frofonx(i,j)=0.16_rsh*(LOG(z/z0sed(i,j)))**(-2)*vit
       ENDIF   
       
       !sigfrot=zfrot/(hty(i,j)+epsilon)-1
       !k=1
       !DO WHILE (sig(k).le.sigfrot)
       !k=k+1
       !ENDDO
       !k=MIN(k,kmax)
      ! IF (i.eq.160 .AND. j.eq.2) PRINT*, 'k',k,sig(k),sigfrot,htx(i,j)
       
       IF(hty(i,j).LE.hm) THEN            !under hm, hydrodynamic 2D calculation
          vit=SQRT(0.0625_rsh*(u(i,j)+u(i,j+1)+u(i-1,j+1)         &
                  +u(i-1,j))**2+v(i,j)**2)*v(i,j)
          frofony(i,j)=0.16_rsh*(LOG(MAX(MIN(hty(i,j),hmaxfrot),hminfrot)/(z0sed(i,j)*exp1)))**(-2)*vit
       ELSE                               !above hm, hydrodynamic 3D calculation
          k=1
#if defined key_siggen || defined key_gencoord
          z=0.5_rsh*f_hzu_y(hy(i,j),ssh(i,j),k,i,j)*dsigu(k)
#else
          z=(1.0_rsh+sig(k))*hty(i,j)
#endif
          vit=SQRT(0.0625_rsh*(uz(k,i,j)+uz(k,i,j+1)+uz(k,i-1,j+1)         &
                           +uz(k,i-1,j))**2+vz(k,i,j)**2)*vz(k,i,j)
          frofony(i,j)=0.16_rsh*(LOG(z/z0sed(i,j)))**(-2)*vit
       ENDIF
     ENDDO
   ENDDO

   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin  ,ljmax,raphbx (liminm1:limax,ljmin  :ljmax))
   CALL_MPI ex_i_rsh(-1,0,1,liminm1,limax,ljmin  ,ljmax,frofonx(liminm1:limax,ljmin  :ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin  ,limax,ljminm1,ljmax,raphby (limin  :limax,ljminm1:ljmax))
   CALL_MPI ex_j_rsh(-1,0,1,limin  ,limax,ljminm1,ljmax,frofony(limin  :limax,ljminm1:ljmax))

    ! calcul de la tension (en N/m2)
    ! ------------------------------

   tenfonc(limin:limax,ljmin:ljmax)=0.0_rsh
   tenfonw(limin:limax,ljmin:ljmax)=0.0_rsh
   tenfon(limin:limax,ljmin:ljmax)=0.0_rsh
 
   DO j=ljminp1,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))
       IF (h0(i,j) /= -valmanq) THEN
         htn=h0(i,j)+ssh(i,j)
         IF(htn.gt.h0fond) THEN
           tenfonc(i,j)=SQRT(((frofonx(i,j)*raphbx(i,j)+frofonx(i-1,j)           &
                     *raphbx(i-1,j))/(raphbx(i,j)                             &
                     +raphbx(i-1,j)+epsilon))**2+((frofony(i,j)*raphby(i,j)   &
                     +frofony(i,j-1)*raphby(i,j-1))/(raphby(i,j)              &
                     +raphby(i,j-1)+epsilon))**2)*rho_bott(i,j)
#ifdef key_wave
            IF (l_fricwave .AND. wave_ubot(i,j)*wave_tp(i,j) > 0.0_rsh) THEN
             fws2=0.5_rsh*1.39_rsh*(wave_ubot(i,j)*wave_tp(i,j)/REAL(2.0_rlg*pi*z0sed(i,j),rsh))**(-0.52_rsh)
            ENDIF
!FLO 11jan2011           tenfonw(i,j)=(rho_bott(i,j)*fws2*wave_ubot(i,j)**2)
            tenfonw(i,j)=(1025._rsh*fws2*wave_ubot(i,j)**2)
            tenfon(i,j)=SQRT(tenfonw(i,j)**2+tenfonc(i,j)**2)
#else
            tenfon(i,j)=tenfonc(i,j) 
#endif
          ENDIF
       ELSE
         tenfonc(i,j)=-valmanq
         tenfonw(i,j)=-valmanq
         tenfon(i,j)=-valmanq
       ENDIF
     ENDDO
   ENDDO

   PRINT_DBG*,'controle tenfon max: ',MAXVAL(tenfonc),MAXVAL(tenfonw),MAXVAL((tenfon))
   PRINT_DBG*,'controle tenfon max: ',MAXVAL(tenfonc),MAXVAL(rho_bott)
   PRINT_DBG*,'controle tenfon max: ',MAXLOC(tenfonc),MAXLOC(rho_bott)

   PRINT_DBG*,'controle tenfon max: ',MAXVAL(tenfon)


   PRINT_DBG*, 'FIN SED_SKINSTRESS'  

  END SUBROUTINE sed_skinstress

   !!==============================================================================

  SUBROUTINE sed_settlveloc_ini

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_settlveloc_ini  ***
   !&E
   !&E ** Purpose :     initializes settling velocity parameters
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :    main
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference : parametric formulation of settling velocity, with hindering effects
   !&E                 from communications INTERCOH 98 et 2000
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from DUFOIS
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       !  2009-02  (P. Le Hir)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : xws,nvp

   !! * Local declarations
   INTEGER iv
   REAL(KIND=rsh)                        :: voleau6,w6
   REAL(KIND=rsh),PARAMETER              :: cend=500.0_rsh,wsend=0.0000001_rsh                   
   
   !!---------------------------------------------------------------------------
   !! * Executable part

      !we assume a variation of Ws according to mes up to cend 
      !for which the (extremely) hindered settling velocity is wsend
      
      !hindered formulation is applied up to cmes=xws(6,iv)
      !the volumic concentration is computed as voleau6:
      !voleau6=1.-xws(4,iv)*xws(6,iv)**xws(5,iv)
      !above xws(6,iv), a sedimentation power law is applied,
      !reaching wsfin when cmes=cfend

      !xws(1,iv) is min value of settling velocity
      !xws(2,iv) is max value of settling velocity
      !xws(3,iv) is concentration for which Ws is max
      !xws(4,iv) is param. beta1 de passage de cmes  la concent.volum.
      !xws(5,iv) is param. beta2 de passage de cmes  la concent.volum.
      !xws(6,iv) is max concentration for which hindering is applied
      !xws(7,iv) is for dependence on salinity (sal.max)
      !xws(8,iv) is for dependence on turbulence (vitmax**2)
      !xws(9,iv) is a new parameter available (e.g. for differentiating vertical advection) 

      DO iv=1,nvp
        IF(xws(1,iv).EQ.0.0_rsh .OR. xws(2,iv).EQ.0.0_rsh) THEN
          alp1(iv)=0.0_rsh
          alp2(iv)=0.0_rsh
          alp3(iv)=0.0_rsh
          alp4(iv)=0.0_rsh
          alp5(iv)=1.0_rsh
          xws(10,iv)=0.0_rsh
          xws(11,iv)=0.0_rsh
          xws(12,iv)=0.0_rsh
          xws(13,iv)=0.0_rsh
        ELSE
          alp1(iv)=2.0_rsh*(xws(2,iv)/(xws(1,iv))-1.0_rsh)/xws(3,iv)
          alp2(iv)=(1.0_rsh-xws(2,iv)/(xws(1,iv)))/xws(3,iv)**2.0_rsh
          alp3(iv)=1.0_rsh/(1.0_rsh-xws(4,iv)*xws(3,iv)**xws(5,iv))
          voleau6=1.0_rsh-xws(4,iv)*xws(6,iv)**xws(5,iv)
          w6=xws(2,iv)*(voleau6/(1.0_rsh-xws(4,iv)*xws(3,iv)**xws(5,iv)))**4.65_rsh
          alp5(iv)=(LOG10(wsend)-LOG10(w6))/(LOG10(cend)-LOG10(xws(6,iv)))
          alp4(iv)=10**(LOG10(w6)-alp5(iv)*LOG10(xws(6,iv)))
          xws(10,iv)=0.0_rsh
          IF(xws(7,iv).GT.0.0_rsh)THEN
            xws(10,iv)=1.0_rsh
            xws(12,iv)=1.0_rsh/xws(7,iv)
          ENDIF

          ! presently, the effect of turbulence follows Van Leussen straightforwardly 
          !xws(11,iv)=0.0_rsh
          !if(xws(8,iv).gt.0.0_rsh)then
          !  xws(11,iv)=1.0_rsh
          !   xws(13,iv)=1.0_rsh/xws(8,iv)
          !endif
        ENDIF
      ENDDO

  END SUBROUTINE sed_settlveloc_ini
      
   !!==============================================================================

  SUBROUTINE sed_settlveloc

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_settlveloc  ***
   !&E
   !&E ** Purpose : settling velocity computation
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  stepu and stepv
   !&E
   !&E ** Reference : Settling velocity computed after Van Leussen (1994)
   !&E                Ws=Ws*(1+aG)/(a+bG**2) with a=0.3 and b=0.09 (Ems estuary)
   !&E                G=sqrt(eps(k,i,j)/nz(k,i,j))
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : xws,ws3,nvp,nvpc,nv_adv,imud1,isand1,isand2,cv_wat
   USE comvars2d,    ONLY : h0fond,h0,ig,id
   USE comvarp2d,    ONLY : ssh,sal
   USE comvars3d,    ONLY : sig,rhoref,nz
   USE parameters,   ONLY : kmax,nb_var
   USE comturb,      ONLY : eps 

   !! * Local declarations
   INTEGER i,j,iv,k,ivpc
   REAL(KIND=rsh)                     :: coeftur,ctursal,voleau,gradvit,cmes,htn
   REAL(KIND=rsh),PARAMETER           :: avanleussen=0.3_rsh,bvanleussen=0.09_rsh

   !!---------------------------------------------------------------------------
   !! * Executable part

#ifdef key_parsub_newdt   
   ! rajoute pour calcul de ndt_part en fonction du temps
      ws3max(:)=0.0_rsh
#endif
      DO j=ljmin,ljmax
        DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
          htn=h0(i,j)+ssh(i,j)
          IF(htn.GT.h0fond)  THEN
            ustarbot(i,j)=(tenfon(i,j)/1000.0_rsh)**0.5_rsh
            DO k=1,kmax
              gradvit=sqrt(eps(k,i,j)/nz(k,i,j))
              cmes=0.0_rsh
              DO ivpc=1,nvpc
                cmes=cmes+cv_wat(ivpc,k,i,j)
              ENDDO
              cmes=MAX(0.0_rsh,cmes)
              DO iv=1,isand2
#ifdef key_sedim_mixsed
                ws3(k,iv,i,j)=ws0(iv)
#elif key_sedim_mudsand
                ws3(k,iv,i,j)=xws(1,iv)
#endif
#ifdef key_parsub_newdt
                ws3max(iv)=ws3(k,iv,i,j)
#endif
              ENDDO
 
              DO iv=imud1,nvp
                !coeftur=1.0_rsh+xws(11,iv)*(-1.0_rsh+xws(13,iv)*min(xws(8,iv),vitmodu+0.1_rsh))
                coeftur=(1.0_rsh+avanleussen*gradvit)/(1.0_rsh+bvanleussen*gradvit**2)
                ctursal=coeftur*(1.0_rsh+xws(10,iv)*(-1.0_rsh+xws(12,iv)*MIN(xws(7,iv),sal(k,i,j))))
                IF(cmes.LE.xws(3,iv))THEN
                  !ws3(k,iv,i,j)=max(xws(1,iv),xws(2,iv)*0.5_rsh*min(2.0_rsh,cmes))
                  !ws3(k,iv,i,j)=max(xws(1,iv),ctursal*ws3(k,iv,i,j))
                  ws3(k,iv,i,j)=ctursal*xws(1,iv)*(1.0_rsh+alp1(iv)*cmes+alp2(iv)*cmes**2)
                ELSEIF(cmes.LE.xws(6,iv))THEN
                  voleau=MAX(0.05_rsh,1.0_rsh-xws(4,iv)*cmes**xws(5,iv))
                  ws3(k,iv,i,j)=ctursal*xws(2,iv)*(voleau*alp3(iv))**4.65_rsh     
                ELSE
                  ws3(k,iv,i,j)=alp4(iv)*cmes**alp5(iv)     
                ENDIF
              ENDDO
              
#ifdef key_parsub_newdt   
              DO iv=imud1,nvp
                if (ws3(k,iv,i,j)>ws3max(iv)) ws3max(iv)=ws3(k,iv,i,j)
              ENDDO
#endif	      
              DO iv=nvp+1,nv_adv
                ws3(k,iv,i,j)=0.0_rsh
              ENDDO
            ENDDO
          ENDIF

        ENDDO
      ENDDO

  END SUBROUTINE sed_settlveloc      

   !!==============================================================================
 
#ifdef key_sedim_mudsand

  SUBROUTINE sed_erosion_mudsand
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_erosion  ***
   !&E
   !&E ** Purpose : computes erosion fluxes
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  stepu, stepv
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from subsedim.F90 (from F. Dufois)
   !&E       !  2008-10  (F. Cayocca) included into module_sedimento
   !&E       !  2008-02  (P. Le Hir) replaced by new "Marennes-Oleron" sand/mud sediment model
   !&E       !  2011-04  (F. Cayocca) introduction of the fraction of wet cell (fwetp)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used  
   USE comsubstance, ONLY : nvpc,nv_adv,nvp,flx_s2w,typart,imud1,imud2

   USE comvars2d,    ONLY : h0,h0fond,dt,ig,id,fwetp
   USE comvarp2d,    ONLY : ssh
   USE parameters,   ONLY : epsilon
   
   !! * Local declarations
   INTEGER i,j,iv,ksmax,ksmaxa,k
   REAL(KIND=rsh) :: dt1,toce,erodab,ero,erosi,dzsa,dtinv
   REAL(KIND=rsh) :: ddzs,dzsam1,poroa,poroam1,dzsisu,somalp,dzpoi
   REAL(kind=rsh) :: cmud,fcr,E0,alpha,E0vase

   !!---------------------------------------------------------------------------
   !! * Executable part

    dtinv=1.0_rsh/REAL(dt,rsh)
    fcr=0.0_rsh
    E0=0.0_rsh
    alpha=0.0_rsh
    E0vase=xeros
    
    DO j=ljminp1,ljmax
      DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))
    
      IF (h0(i,j)+ssh(i,j).GT.h0fond) THEN

        ksmax=ksma(i,j)
        flx_s2w(-1,i,j)=0.0_rsh
        flx_s2w( 0,i,j)=0.0_rsh
        DO iv=1,nv_adv
          flx_s2w(iv,i,j)=0.0_rsh
        ENDDO


        IF(ksmax.GE.ksmi(i,j))THEN

          dt1=REAL(dt,rsh)
          ! epeaui represente l epaisseur d eau interstitielle init_fromfileeree 
          ! par le sediment lors du depot (calcule par depeff precedent) 
          ! ici, epeaui va representer la variation d epaisseur d eau a
          ! prendre en compte dans la maille inferieure de la colonne d eau
          ! epeaui(i,j)=-epeaui(i,j)

          2 CONTINUE

          CALL sed_comp_tocr_mudsand(c_sedtot(ksmax,i,j))
          toce=sed_tocr_mudsand
          IF(tenfon(i,j).GT.toce)THEN

            erodab=c_sedtot(ksmax,i,j)*dzs(ksmax,i,j)
            ! relation de Partheniades:
!fdufois
            cmud=0.0_rsh
            DO iv=imud1,imud2
              cmud=cmud+cv_sed(iv,ksmax,i,j)
            ENDDO
            fcr=(cmud/csupi)
            IF (fcr.GT.frmudcr2) THEN
              E0=E0vase
              alpha=alphavase
            ELSEIF (fcr.LT.frmudcr1) THEN
              E0=E0sable
              alpha=alphasable
            ELSE
              E0=(E0vase-E0sable)/(frmudcr2-frmudcr1)*(fcr-frmudcr1)+E0sable
              alpha=(alphavase-alphasable)/(frmudcr2-frmudcr1)*(fcr-frmudcr1)+alphasable
            ENDIF

            ero=E0*(tenfon(i,j)/toce-1.)**alpha
            !  ero=xeros*(tenfon(i,j)/toce-1.0_rsh)
!fdufois
            erosi=ero*dt1*fwetp(i,j)

          IF(erosi.LT.erodab)THEN
            ddzs=erosi/c_sedtot(ksmax,i,j)
            dzs(ksmax,i,j)=dzs(ksmax,i,j)-ddzs
            !epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
            DO iv=1,nvp
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*cv_sed(iv,ksmax,i,j)
            ENDDO
            flx_s2w(-1,i,j)=flx_s2w(-1,i,j)+ddzs*poro(ksmax,i,j)*cv_sed(-1,ksmax,i,j)
            flx_s2w( 0,i,j)=flx_s2w( 0,i,j)+ddzs*poro(ksmax,i,j)*cv_sed( 0,ksmax,i,j)
            DO iv=nvp+1,nv_adv
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*poro(ksmax,i,j)*cv_sed(iv,ksmax,i,j)
            ENDDO
          ELSE
            ddzs=erodab*fwetp(i,j)/c_sedtot(ksmax,i,j)
            !epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
            DO iv=1,nvp
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*cv_sed(iv,ksmax,i,j)
            ENDDO
            flx_s2w(-1,i,j)=flx_s2w(-1,i,j)+ddzs*poro(ksmax,i,j)*cv_sed(-1,ksmax,i,j)
            flx_s2w( 0,i,j)=flx_s2w( 0,i,j)+ddzs*poro(ksmax,i,j)*cv_sed( 0,ksmax,i,j)
            DO iv=nvp+1,nv_adv
              flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+ddzs*poro(ksmax,i,j)*cv_sed(iv,ksmax,i,j)
            ENDDO
            dt1=dt1-erodab/ero
            ksmax=ksmax-1
            IF(ksmax.GE.ksmi(i,j))THEN
              GOTO 2
            ELSE
              !write(*,*)'a t=',t,i,j,', il n"y a plus de sediment'
              GOTO 120
            ENDIF
          ENDIF
        ENDIF

      ENDIF

      !fusion des 2 couches superf., si dzs(ksmax) < dzsmin
      !---------------------------------------------------
      ! if(ksmax.ne.0)then

      1000 IF(ksmax.GE.2)THEN
      IF(dzs(ksmax,i,j).LE.dzsmin)THEN
        ksmaxa=ksmax
        dzsa=dzs(ksmax,i,j)
        dzsam1=dzs(ksmax-1,i,j)
        poroa=poro(ksmax,i,j)
        poroam1=poro(ksmax-1,i,j)
        ksmax=ksmax-1
        dzs(ksmax,i,j)=dzsa+dzs(ksmax,i,j)
        dzsisu=1.0_rsh/dzs(ksmax,i,j)
        c_sedtot(ksmax,i,j)=0.0_rsh
        somalp=0.0_rsh

        DO iv=1,nvp
          cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*dzsa+cv_sed(iv,ksmax,i,j)*dzsam1)*dzsisu
          c_sedtot(ksmax,i,j)=c_sedtot(ksmax,i,j)+cv_sed(iv,ksmax,i,j)*typart(iv)
            somalp=somalp+cv_sed(iv,ksmax,i,j)*typart(iv)/(ros(iv)+epsilon)
        ENDDO
        poro(ksmax,i,j)=1.0_rsh-somalp
        poroin(ksmax,i,j)=poro(ksmax,i,j)
        dzpoi=dzsisu/poro(ksmax,i,j)            
        cv_sed(-1,ksmax,i,j)=(cv_sed(-1,ksmaxa,i,j)*poroa*dzsa+cv_sed(-1,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        cv_sed( 0,ksmax,i,j)=(cv_sed( 0,ksmaxa,i,j)*poroa*dzsa+cv_sed( 0,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        DO iv=nvp+1,nv_adv
          ! les cs dissous sont en .../m3 d eau interst.
          cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*poroa*dzsa+cv_sed(iv,ksmax,i,j)*poroam1*dzsam1)*dzpoi
        ENDDO
        ! et les variables benthiques ???
        IF(ksmax.GT.ksmi(i,j))THEN
          poroin(ksmax-1,i,j)=0.5_rsh*(poro(ksmax,i,j)+poro(ksmax-1,i,j))
          dzsiin(ksmax-1,i,j)=2.0_rsh/(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
        ENDIF

        GOTO 1000
       ENDIF
      ENDIF

     120 CONTINUE

      DO iv=-1,nv_adv
        ! pour retrouver un flux d erosion en .../m2/s :
        flx_s2w(iv,i,j)=flx_s2w(iv,i,j)*dtinv
      ENDDO
      flx_s2w(-1,i,j)=flx_s2w(-1,i,j)+fwetp(i,j)*(-fludif(-1,i,j)+fluconsol(-1,i,j))
      flx_s2w( 0,i,j)=flx_s2w( 0,i,j)+fwetp(i,j)*(-fludif( 0,i,j)+fluconsol( 0,i,j))
      DO iv=nvp+1,nv_adv
        ! pour les dissoutes, on ajoute le flux diffusif a l interface
        ! et celui qui resulte de l expulsion d eau interst. par tasst:
        flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+fwetp(i,j)*(-fludif(iv,i,j)+fluconsol(iv,i,j))
      ENDDO

      ! reactualisation de ksma 
      ksma(i,j)=ksmax

      ! rectification de l epaisseur de la maille d eau de fond
      ! pour tenir compte de l apport (resp. fuite) d eau interstitielle par
      ! erosion ou tassement (resp. depot)

      !phieau(1,i,j)=(epeaui(i,j)/dt+winsurf(i,j))*surf(i,j) 

      ! attention dans la version med phieau est en m/s

      !phieau(1,i,j)=phieau(1,i,j)+(epeaui(i,j)/dt+winsurf(i,j))*surf(i,j)

      ENDIF
     END DO
   END DO

  END SUBROUTINE sed_erosion_mudsand

#endif
   !!==============================================================================

#ifdef key_sedim_mixsed 
 
  SUBROUTINE sed_erosion_mixsed
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_erosion  ***
   !&E
   !&E ** Purpose : computes erosion fluxes
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  stepu, stepv
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from subsedim.F90 (from F. Dufois)
   !&E       !  2008-10  (F. Cayocca) included into module_sedimento
   !&E       !  2008-02  (P. Le Hir) replaced by new "Marennes-Oleron" sand/mud sediment model
   !&E       !  2011-04  (F. Cayocca) introduction of the fraction of wet cell (fwetp)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nvpc,nv_adv,nvp,flx_s2w,igrav1,igrav2,typart
!   USE comfluxdo,    only : phieau
   USE comvars2d,    ONLY : h0,h0fond,dt,ig,id,fwetp
   USE comvarp2d,    ONLY : ssh
   USE parameters,   ONLY : epsilon

   !! * Local declarations
   INTEGER        ::  i,j,k,iv,ksmax,ksup,ksmaxa
   REAL(KIND=rsh) ::  dt1,toce,cvolgrv,csanmud,dtinv,                          &
                      erodab,ero,erosi,ddzs,dzsa,dzsisu,dzsam1,dflx_s2w,       &
                      cvolp,volerod,profsed,poroa,dzsma1,poroam1

   !!---------------------------------------------------------------------------
   !! * Executable part

   DO j=ljminp1,ljmax
      DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))

        flx_s2w(-1,i,j)=0.0_rsh
        flx_s2w( 0,i,j)=0.0_rsh
        DO iv=1,nv_adv
          flx_s2w(iv,i,j)=0.0_rsh
        ENDDO
        
        IF((h0(i,j)+ssh(i,j)).GT.h0fond) THEN
          ! erosion only occurs if there is water

          ksmax=ksma(i,j)

          IF(ksmax.GE.ksmi(i,j))THEN

            dt1=REAL(dt,rsh)
            dtinv=1.0_rsh/REAL(dt,rsh)
        ! epeaui represente l epaisseur d eau interstitielle recuperee 
        ! par le sediment lors du depot (calcule par depeff precedent) 
        ! ici, epeaui va representer la variation d epaisseur d eau a
        ! prendre en compte dans la maille inferieure de la colonne d eau
        ! epeaui(i,j)=-epeaui(i,j)

            k=ksmax
             
            2 CONTINUE

            CALL sed_comp_tocr_mixsed(k,i,j)
            toce=MAX(0.00005_rsh,sed_tocr_mixsed)
            ! pour ecriture dans SiAM:        tocsur(i,j)=toce
            ! pour ecriture dans SiAM:        erosur(i,j)=xeros
            !if(tenfonmax(i,j).gt.toce)then
            ! a regarder de + pres, cf W de Benoit Waeles
            IF(tenfon(i,j).GT.toce)THEN
              cvolgrv=0.0_rsh
              DO iv=igrav1,igrav2
                cvolgrv=cvolgrv+cv_sed(iv,k,i,j)/ros(iv)
              ENDDO
              csanmud=0.0_rsh
              DO iv=igrav2+1,nvpc
                csanmud=csanmud+cv_sed(iv,k,i,j)
              ENDDO
              erodab=csanmud*dzs(k,i,j)
              CALL sed_comp_eros_flx(tenfon(i,j),toce,c_sedtot(ksmax,i,j))
              ero=sed_eros_flx
              erosi=ero*dt1*fwetp(i,j)
              IF(erosi.LT.erodab)THEN
                ddzs=erosi/csanmud
                dzsa=dzs(k,i,j)
                dzs(k,i,j)=MAX(dzsa-ddzs,dzsa*cvolgrv/cvolmaxsort)
                IF(dzs(k,i,j).LE.0.0_rsh)THEN
                  !WRITE(*,*)'en kij:',k,i,j,dzs(k,i,j),dzsa,ddzs,csanmud,cvolgrv
                  dzs(k,i,j)=epsilon
                ENDIF
                dzsisu=1.0_rsh/dzs(k,i,j)
                !epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
                DO iv=igrav1,igrav2
                  cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*dzsa*dzsisu
                ENDDO
                DO iv=igrav2+1,nvp
                  dflx_s2w=ddzs*cv_sed(iv,k,i,j)
                  flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+dflx_s2w
                  cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsa-dflx_s2w)*dzsisu
                ENDDO
                c_sedtot(k,i,j)=0.0_rsh
                cvolp=0.0_rsh
                DO iv=1,nvpc
                  c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
                  cvolp=cvolp+cv_sed(iv,k,i,j)/ros(iv)
                ENDDO
                poro(k,i,j)=1.0_rsh-cvolp
                !do iv=nvp+1,nv_adv
                !  dflusve=ddzs*poroa*cv_sed(iv,k,i,j)
                !  flusve(iv,i,j)=flusve(iv,i,j)+dflusve
                !  cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsa*poroa-dflusve)*dzsisu/poro(k,i,j) 
                !enddo
              ELSE
                volerod=erodab*fwetp(i,j)/csanmud
                !epeaui(i,j)=epeaui(i,j)+ddzs*poro(ksmax,i,j)
                DO iv=igrav2+1,nvp
                  flx_s2w(iv,i,j)=flx_s2w(iv,i,j)+volerod*cv_sed(iv,k,i,j)
                  cv_sed(iv,k,i,j)=0.0_rsh
                ENDDO
                IF(cvolgrv.LE.0.0_rsh)THEN
                  ksmax=ksmax-1
                  !do iv=nvp+1,nv_adv
                  !flusve(iv,i,j)=flusve(iv,i,j)+volerod*poro(k,i,j)*cv_sed(iv,k,i,j)
                  !enddo
                ELSE
                  dzsa=dzs(k,i,j)
                  dzs(k,i,j)=dzsa*cvolgrv/cvolmaxsort
                  !??? dzs(k,i,j)=dzsa*cvolgrv/cvolmaxmel
                  c_sedtot(k,i,j)=0.0_rsh
                  DO iv=igrav1,igrav2
                    cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*dzsa/dzs(k,i,j)
                    c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
                  ENDDO
                  ! normalement, les concentrations dissoutes dans l eau inter-
                  ! stitielle ne devraient pas changer; pb de conservation ?
                  !poroa=poro(k,i,j)
                  poro(k,i,j)=1.0_rsh-cvolmaxsort
                  !??? poro(k,i,j)=1.0_rsh-cvolmaxmel
                  !do iv=nvp+1,nv_adv
                  !  flusve(iv,i,j)=flusve(iv,i,j)+cv_sed(iv,k,i,j)*(dzsa*poroa   &
                  !                 -cv_sed(iv,k,i,j)*dzs(k,i,j)*poro(k,i,j)
                  !enddo
                ENDIF
              
                ! test on active layer...
                profsed=0.0_rsh
                DO ksup=k,ksmax
                  profsed=profsed+dzs(ksup,i,j)
                ENDDO
                k=k-1
                IF(profsed.LE.activlayer.AND.k.GE.ksmi(i,j))THEN
                  dt1=dt1-erodab/ero
                  GOTO 2
                ENDIF
              ENDIF
            ENDIF
          ENDIF
!crv        ENDIF
      
        ! fusion des 2 couches superf., tant que dzs(ksmax) < dzsmin
        ! ----------------------------------------------------------

 1000     IF(ksmax.GT.ksmi(i,j))THEN
            IF(dzs(ksmax,i,j).LE.dzsmin)THEN
              ksmaxa=ksmax
              dzsa=dzs(ksmax,i,j)
              dzsam1=dzs(ksmax-1,i,j)
              poroa=poro(ksmax,i,j)
              poroam1=poro(ksmax-1,i,j)
              ksmax=ksmax-1
              dzs(ksmax,i,j)=dzsa+dzsam1
              dzsisu=1.0_rsh/dzs(ksmax,i,j)
              c_sedtot(ksmax,i,j)=0.0_rsh
              cvolp=0.0_rsh
              DO iv=1,nvp
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*dzsa             &
                                     +cv_sed(iv,ksmax,i,j)*dzsam1)*dzsisu
                c_sedtot(ksmax,i,j)=c_sedtot(ksmax,i,j)                      &
                                    +cv_sed(iv,ksmax,i,j)*typart(iv)
                cvolp=cvolp+cv_sed(iv,ksmax,i,j)*typart(iv)/ros(iv)
              ENDDO
              poro(ksmax,i,j)=1.0_rsh-cvolp
              !dzpoi=dzsisu/poro(ksmax,i,j)            
              !do iv=nvp+1,nv_adv
              ! les cs dissous sont en .../m3 d eau interst.
              !cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmaxa,i,j)*poroa*dzsa+
              !                      cv_sed(iv,ksmax,i,j)*poroam1*dzsam1)*dzpoi
              !enddo
              GOTO 1000
            ENDIF
          ENDIF

          ! pour retrouver un flux d erosion en .../m2/s :
          DO iv=1,nv_adv
            flx_s2w(iv,i,j)=flx_s2w(iv,i,j)*dtinv
          ENDDO
          !do iv=nvp+1,nv_adv
          !    pour les dissoutes, on ajoute le flux diffusif a l interface
          !    et celui qui resulte de l'expulsion d'eau interst. par tasst:
          !  flusve(iv,i,j)=flusve(iv,i,j)+flutas(iv,i,j)-fludif(iv,i,j)
          !enddo

          ! updating ksma (as ksmax can be modified in the routine)
          ksma(i,j)=ksmax

          ! no correction of water column height (could be !)

        ENDIF ! crv

      END DO
    END DO

   PRINT_DBG*, 'FIN SED_EROSION'

  END SUBROUTINE sed_erosion_mixsed
#endif
   !!==============================================================================

#ifdef key_sedim_mixsed

  SUBROUTINE sed_comp_tocr_mixsed(k,i,j)
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  SUBROUTINE sed_comp_tocr_mixsed  ***
   !&E
   !&E ** Purpose : the critical shear stress for erosion in sediment layer k
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  sed_erosion
   !&E 
   !&E ** External calls : none
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-02  (P. Le Hir) extracted from (SiAM) new "Marennes-Oleron"
   !&E                               sand/mud sediment model
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : igrav1,igrav2,isand1,isand2,imud1,imud2
   USE parameters,   ONLY : epsilon
   !! * Arguments
   INTEGER,INTENT(in) :: k,i,j

   !! * Local declarations
   INTEGER            :: iv
   REAL(KIND=rsh)     :: sommud,somsan,diamsan,critstressan,frvolsan,   &
                         somsedsusp,frvolsangrv,frmudsup,frmudcr1,pinterp,cmudr

   !!---------------------------------------------------------------------------
   !! * Executable part

      sommud=0.0_rsh     
      DO iv=imud1,imud2
        sommud=sommud+cv_sed(iv,k,i,j)
      ENDDO
      somsan=0.0_rsh
      diamsan=0.0_rsh
      critstressan=0.0_rsh
      frvolsan=0.0_rsh      
      DO iv=isand1,isand2
        somsan=somsan+cv_sed(iv,k,i,j)
        diamsan=diamsan+diam_sed(iv)*cv_sed(iv,k,i,j)
        critstressan=critstressan+stresscri0(iv)*cv_sed(iv,k,i,j)
        frvolsan=frvolsan+cv_sed(iv,k,i,j)/ros(iv)
      ENDDO
      somsedsusp=sommud+somsan
      IF(somsedsusp.LE.0.0_rsh)THEN
        ! surficial sediment non transportable in suspension
        sed_tocr_mixsed=1000.0_rsh
      ELSE
      
        diamsan=MAX(diamsan/(somsan+epsilon),diam_sed(isand2))
        critstressan=MAX(critstressan/(somsan+epsilon),stresscri0(isand2))
        frvolsangrv=frvolsan
        DO iv=igrav1,igrav2
          frvolsangrv=frvolsangrv+cv_sed(iv,k,i,j)/ros(iv)
        ENDDO
        frmudsup=sommud/(sommud+somsan)
        frmudcr1=MIN(eroa1*diamsan,frmudcr2)
        IF(frmudsup.LE.frmudcr1)THEN
          sed_tocr_mixsed=critstressan
          !6mars2007,in SiAM          xeros=min(0.27_rsh,1000.0_rsh*diamsab-0.01_rsh)*(1000.0_rsh*tencrisab)**erob1
          xeros=MIN(0.27_rsh,1000.0_rsh*diamsan-0.01_rsh)
          excespowr=erob1
        ELSEIF(frmudsup.LE.frmudcr2)THEN
          pinterp=(frmudcr2-frmudsup)/(frmudcr2-frmudcr1)
          cmudr=sommud/(1.0_rsh-frvolsangrv)
          sed_tocr_mixsed=critstressan*pinterp+(1.0_rsh-pinterp)*eroa2*cmudr**eroa3
          !6mars2007,in SiAM           xeros=pinterp*min(0.27_rsh,1000.0_rsh*diamsab-0.01_rsh)*(1000.0_rsh*tencrisab)
          !6mars2007,in SiAM      &           **erob1+(1.0_rsh-pinterp)*erob2
          xeros=pinterp*MIN(0.27_rsh,1000.0_rsh*diamsan-0.01_rsh)+(1.0_rsh-pinterp)*erob2
          excespowr=pinterp*erob1+(1.0_rsh-pinterp)*erob3 
        ELSE
          cmudr=sommud/(1.0_rsh-frvolsangrv)
          sed_tocr_mixsed=eroa2*cmudr**eroa3
          xeros=erob2
          excespowr=erob3
        ENDIF
      ENDIF
    
  END SUBROUTINE sed_comp_tocr_mixsed
#endif   

!!==============================================================================

#ifdef key_sedim_mudsand

  SUBROUTINE sed_comp_tocr_mudsand(csedim)

   !&E--------------------------------------------------------------------------
   !&E                 ***  SUBROUTINE sed_comp_tocr_mudsand  ***
   !&E
   !&E ** Purpose : compute the critical shear stress of sediment
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by : 
   !&E 
   !&E ** External calls : none
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used

   !! * Arguments
   REAL(KIND=rsh),INTENT(in)  :: csedim

   !! * Local declarations

   !!---------------------------------------------------------------------------
   !! * Executable part

   sed_tocr_mudsand=x1toce*MAX(csedim,csupi)**x2toce
   IF(csedim.GE.1200.0_rsh) sed_tocr_mudsand=20.0_rsh

  END SUBROUTINE sed_comp_tocr_mudsand

#endif   
   
   !!==============================================================================

#ifdef key_sedim_mixsed

  SUBROUTINE sed_comp_eros_flx(tenfo,toce,csurface)
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  SUBROUTINE sed_erosion_flux  ***
   !&E
   !&E ** Purpose : computes erosion flux for given stress, critical stress and
   !&E              surficial sediment concentration
   !&E              uses toce, excespowr and xeros computed by sed_tocr
   !&E
   !&E ** Description : Partheniades or adapted Partheniades
   !&E
   !&E ** Called by :  sed_erosion
   !&E 
   !&E ** External calls : nonee
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-02  (P. Le Hir) extracted from (SiAM) new "Marennes-Oleron"
   !&E                               sand/mud sediment model
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   !! * Arguments
   REAL(KIND=rsh),INTENT(in) :: csurface,tenfo,toce

   !! * Local declarations   
   REAL(KIND=rsh) ::  corfluero

   !!---------------------------------------------------------------------------
   !! * Executable part
   
   corfluero=1.0_rsh+corfluer1*MAX(0.0_rsh,corfluer2-csurface)     
   ! Partheniades :
!   sed_eros_flx=xeros*corfluero*(tenfo/toce-1.0_rsh)**excespowr
!cbene choisir une des formules mais pour comparer mudsand et mixsed il faut
!choisir la formule par defaut
! mettre un booleen
  sed_eros_flx=xeros*corfluero*(tenfo-toce)**excespowr
    
  END SUBROUTINE sed_comp_eros_flx

#endif

   !!==============================================================================

  SUBROUTINE sed_sandconcextrap

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_sandconcextrap  ***
   !&E
   !&E ** Purpose : interpolates sand concentration in the first layer
   !&E              assuming a Rouse profile 
   !&E
   !&E ** Description : tenfon has to be known
   !&E
   !&E ** Called by :  main ou step
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       !  2009-02 (P. Le Hir)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance,   ONLY : ws3,flx_w2s,isand1,isand2,nv_adv
   USE comvarp2d,      ONLY : ssh
   USE comvars2d,      ONLY : h0,hm,ig,id,h0fond
   USE comvars3d,      ONLY : sig,sigw
   USE_MPI toolmpi,    ONLY : ex_i_rsh,ex_j_rsh

   !! * Local declarations
   REAL(KIND=rsh)                       :: altc1,ep,htot,extrap,rouse,som,dzcche,hzed,  &
                                           einstein,alogaltc1sz0,ashmapwrouse
   INTEGER                              :: izz,i,j,ivp
   INTEGER,PARAMETER                    :: nbcche=20
   REAL(KIND=rsh),PARAMETER             :: aref=0.02_rsh
   REAL(KIND=rsh),DIMENSION(1:nbcche+1) :: hzi

   !!---------------------------------------------------------------------------
   !! * Executable part   

   corflux(:,limin:limaxp1,ljmin:ljmax  )=1.0_rsh
   corfluy(:,limin:limax  ,ljmin:ljmaxp1)=1.0_rsh

!cvalerie a verifier avec florence bug corriges mais...
!   DO j=ljminp1,ljmaxm1
!     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
   DO j=ljminp1,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))

          htot=h0(i,j)+ssh(i,j)  
          altc1=(1.0_rsh+sig(1))*htot
          ep=(1.0_rsh+sigw(1))*htot
          IF(altc1.LE.aref .OR. htot.LE.h0fond)THEN
            extrap=1.0_rsh
            ! in this case, flx_w2s(ivp,i,j) is not changed
            DO ivp=isand1,isand2
              corflux(ivp,i,j)=1.0_rsh
              corfluy(ivp,i,j)=1.0_rsh
            ENDDO
          ELSE
            alogaltc1sz0=LOG(altc1/z0sed(i,j))                  
            hzi(1)=ep
            DO izz=1,nbcche
              hzi(izz+1)=hzi(izz)-(hzi(1)-aref)/2.0_rsh**izz
            ENDDO     
            hzi(nbcche+1)=aref
            DO ivp=isand1,isand2
            ! Rouse number depends on settling velocity, then on the sand variable
              rouse=ws3(1,ivp,i,j)                                        &
                    /(0.4_rsh*MAX(ustarbot(i,j),0.8_rsh*ws3(1,ivp,i,j)))

            ! integrale concentration et concentration*vitesse log
            ! dans premiere couche
              som=0.0_rsh
              einstein=0.0_rsh
              DO izz=1,nbcche
                hzed=0.5_rsh*(hzi(izz)+hzi(izz+1))
                dzcche=hzi(izz)-hzi(izz+1) 
                som=som+dzcche*((htot-hzed)/hzed)**rouse
                einstein=einstein+dzcche*((htot-hzed)/hzed)**rouse      &  
                                  *LOG(hzed/z0sed(i,j))
              ENDDO
              ashmapwrouse=(aref/(htot-aref))**rouse 
              som=som*ashmapwrouse
              extrap=(ep-aref)/som 
              
              einstein=einstein*ashmapwrouse*extrap/alogaltc1sz0/ep
              ! the computation of einstein and USE of corflux should be checked
              !??? modif P.LeHir              if(hex(i,j).le.hm)then
              !??? modif P.LeHir              if(hey(i,j).le.hm)then
              IF(htot.LE.hm)THEN
                corflux(ivp,i,j)=einstein*alogaltc1sz0/LOG(htot/(2.718_rsh*z0sed(i,j)))
                corfluy(ivp,i,j)=einstein*alogaltc1sz0/LOG(htot/(2.718_rsh*z0sed(i,j)))
              ELSE
                corflux(ivp,i,j)=einstein
                corfluy(ivp,i,j)=einstein
              ENDIF
              flx_w2s(ivp,i,j)=flx_w2s(ivp,i,j)*extrap              
            ENDDO              
          ENDIF
          
        ENDDO
      ENDDO

   CALL_MPI ex_i_rsh(0,1,nv_adv,limin,limaxp1,ljmin,ljmax  ,corflux(:,limin:limaxp1,ljmin:ljmax  ))
   CALL_MPI ex_j_rsh(0,1,nv_adv,limin,limax  ,ljmin,ljmaxp1,corfluy(:,limin:limax  ,ljmin:ljmaxp1))

      ! corflux are interpolated on mesh edges (in u & v)
      ! -------------------------------------------------
   DO j=ljminp1,ljmaxm1
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j)-1)
          DO ivp=isand1,isand2
            corflux(ivp,i,j)=0.5_rsh*(corflux(ivp,i,j)+corflux(ivp,i+1,j))
            corfluy(ivp,i,j)=0.5_rsh*(corfluy(ivp,i,j)+corfluy(ivp,i,j+1))
           ENDDO
        ENDDO
      ENDDO

   PRINT_DBG*, 'FIN SED_SANDCONCEXTRAP'

  END SUBROUTINE sed_sandconcextrap

   !!==============================================================================
 
  SUBROUTINE sed_depflx
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_depflx  ***
   !&E
   !&E ** Purpose : computes deposition flux trends after sed_erosion and 
   !&E              and sand_concextrap, before advection computations
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  stepu, stepv
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-03  (F. Cayocca) extracted from marsedim.F90 (from F. Dufois)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nvp,nvpc,tocd,flx_w2s,ws3,cv_wat,irkm_var_assoc
   USE comvars2d,    ONLY : ig,id

   !! * Local declarations
   INTEGER                          :: i,j,ivp
   REAL(KIND=rlg)                   :: tocdpe,depo
   REAL(KIND=rlg),PARAMETER         :: epsdepot=1.e-14

   !!---------------------------------------------------------------------------
   !! * Executable part

   DO j=ljminp1,ljmax
     DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))
       ! Benedicte : separation des boucles car pas forcment la meme unite pour les non constitutives
       DO ivp=1,nvpc
         tocdpe=tocd(ivp)+0.00001_rsh
         flx_w2s(ivp,i,j)=ws3(1,ivp,i,j)                             &
                          *MAX(0.0_rsh,1.0_rsh-(tenfon(i,j)/tocdpe)) &  
                          *tocd(ivp)/tocdpe
         depo=flx_w2s(ivp,i,j)*cv_wat(ivp,1,i,j)
         IF(depo.LE.epsdepot)flx_w2s(ivp,i,j)=0.0_rsh
       ENDDO
       DO ivp=nvpc+1,nvp
         tocdpe=tocd(ivp)+0.00001_rsh
         flx_w2s(ivp,i,j)=ws3(1,ivp,i,j)                             &
                          *MAX(0.0_rsh,1.0_rsh-(tenfon(i,j)/tocdpe)) &  
                          *tocd(ivp)/tocdpe
         depo=flx_w2s(ivp,i,j)*cv_wat(ivp,1,i,j)
         IF(irkm_var_assoc(ivp) > 0 ) THEN
          IF (flx_w2s(irkm_var_assoc(ivp),i,j)==0.0_rsh) flx_w2s(ivp,i,j)=0.0_rsh
         ELSE
          IF(depo.LE.epsdepot)flx_w2s(ivp,i,j)=0.0_rsh
         END IF
       ENDDO
     END DO
   END DO
         
  END SUBROUTINE sed_depflx

   !!==============================================================================
 
#ifdef key_sedim_mixsed

  SUBROUTINE sed_effdep_mixsed(xeloc)
    
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_effdep_mixsed  ***
   !&E
   !&E ** Purpose : computes effective deposition
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  step3dxy, step3dyx
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from subsedim.F90 (from F. Dufois)
   !&E       !  2008-10  (F. Cayocca) included into module_sedimento
   !&E       !  2008-02  (P. Le Hir)replaced by new "Marennes-Oleron" sand/mud sediment model
   !&E       !  2010-07  (B. Thouvenin) taking into account of non constitutive particulate variables
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : typart,nv_adv,nvp,nvpc,flx_w2s,flx_w2s_sum,cv_wat, &
                            igrav1,igrav2,isand1,isand2,imud1,imud2,irkm_var_assoc
   USE comvars2d,    ONLY : ig,id,dt,t,h0,h0fond,fwet
   USE parameters,   ONLY : epsilon
   USE comvars3d,    ONLY : dsigu
#if defined key_siggen || defined key_gencoord
  USE toolgeom,     ONLY : f_hzu
#endif

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: xeloc

   !! * Local declarations
   REAL(KIND=rsh),DIMENSION(nvp)  :: flx_w2s_loc
   REAL(KIND=rsh),DIMENSION(nvpc) :: frdep          
   INTEGER                        :: i,j,k,iv,ksmin,ksmax,kl,ivp_assoc
   REAL(KIND=rsh) :: fludep,qdep,vitdepo,ddzs,ddzs1,ddzs2,dzsa,dzsisu,dzsi,dzsaici,      &
                     dzsgrv,dzssan,dzsmud,dzsnew,dzsa2,dzsa3,ddzs3,ddzs4,ddzs5,ddzs6,    &
                     poroa,somalp,somala,cdepiv,porosi,dzpoi,porodep,                    &
                     ddzsici,voldepgrv,voldepsan,masdepmud,                              &
                     sommud,cvolinigrv,cvolinisan,cvolp,cmudr,                           &
                     dvolgrv,dvolsan,dmasmud
   REAL(KIND=rlg),PARAMETER       :: epsdepef=1.e-12,epsd=1.e-16

   !!----------------------------------------------------------------------
   !! * Executable part   
   
    ddzsici=-1000.0_rsh

   DO j=ljmin,ljmax
     DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))
     
      IF(h0(i,j)+xeloc(i,j).GT.h0fond) THEN

        ksmax=ksma(i,j)

        ! updating effective deposition :  (flx_w2s) is implicit in the vertical advection scheme
        ! hence multiplied here by the newly computed concentration
        DO iv=igrav2+1,nvp
         IF(cv_wat(iv,1,i,j) >= 0.0_rsh) THEN
!          flx_w2s_loc(iv)=flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)*fwet(i,j)*REAL(dt,rsh)*0.5_rsh
! modif bene pour prendre le depot integre sur les ndt_part pas de temps
!      deja calcule a chaque sous pas de temps en fonction des cv_wat intermediaire
!       et exprime en Masse/m2 integre sur le demi pas de temps
           flx_w2s_loc(iv)=MAX(flx_w2s_sum(iv,i,j),0.0_rsh)*fwet(i,j)
           flx_w2s_sum(iv,i,j)=0.0_rsh
         ELSE
          flx_w2s_loc(iv)=0.0_rsh
         ENDIF
        ENDDO

        fludep=0.0_rsh
        DO iv=1,nvpc
          fludep=fludep+flx_w2s_loc(iv)
        ENDDO

        voldepgrv=0.0_rsh
        voldepsan=0.0_rsh
        masdepmud=0.0_rsh
              
        IF(fludep.GT.epsdepef)THEN
        ! the relevance of this test should be demonstrated 

          ! case 1:  there is deposition
          ! ****************************

          ! characterising deposition:
          DO iv=igrav1,igrav2
            voldepgrv=voldepgrv+flx_w2s_loc(iv)/ros(iv)
          ENDDO
          DO iv=igrav1,igrav2
            frdep(iv)=flx_w2s_loc(iv)/(voldepgrv+epsd)
          ENDDO
          DO iv=isand1,isand2
            voldepsan=voldepsan+flx_w2s_loc(iv)/ros(iv)
          ENDDO
          DO iv=isand1,isand2
            frdep(iv)=flx_w2s_loc(iv)/(voldepsan+epsd)
          ENDDO
          DO iv=imud1,imud2
            masdepmud=masdepmud+flx_w2s_loc(iv)
          ENDDO
          DO iv=imud1,imud2
            frdep(iv)=flx_w2s_loc(iv)/(masdepmud+epsd)
          ENDDO

          IF(ksmax.GE.ksmi(i,j))THEN
          
            ! case 1.1: sediment already exists
            ! ---------------------------------
            ! characterising previous surficial sediment:

            sommud=0.0_rsh      
            DO iv=imud1,imud2
              sommud=sommud+cv_sed(iv,ksmax,i,j)
            ENDDO
            cvolinigrv=0.0_rsh      
            DO iv=igrav1,igrav2
              cvolinigrv=cvolinigrv+cv_sed(iv,ksmax,i,j)/ros(iv)
            ENDDO
            cvolinisan=0.0_rsh
            DO iv=isand1,isand2
              cvolinisan=cvolinisan+cv_sed(iv,ksmax,i,j)/ros(iv)
            ENDDO
            cmudr=sommud/(1.0_rsh-cvolinisan-cvolinigrv)
            
            IF(cmudr.LE.cmudcr)THEN
            
              ! case 1.1.1: surficial sediment is not consolidated
              ! --------------------------------------------------
              ! mixing deposits with upper layer, until completion:

              dzsa=dzs(ksmax,i,j)

              ! first, gravels deposition :
              ! ---------------------------
              ! dvolgrv represente le volume de graviers qui s insere dans
              ! le sediment existant, en le densifiant
              ! il restera ddzs a deposer a la concentration volumique
              ! cvolmaxsort, dont ddzs2 dans la maille existante, ddzs1 en
              ! une nouvelle couche
              ! attention: a confirmer:
              dvolgrv=MIN(voldepgrv,dzsa*(cvolmaxsort-cvolinigrv),dzsa*(cvolmaxmel-cvolinigrv-cvolinisan))
              dvolgrv=MAX(dvolgrv,0.0_rsh)
              ddzs=(voldepgrv-dvolgrv)/cvolmaxsort
              ddzsici=ddzs
              dzsaici=dzsa
              IF((dzsa+ddzs).LE.dzsmax)THEN
                ddzs1=0.0_rsh
              ELSEIF(dzsa.GE.dzsmax)THEN
                ddzs1=ddzs
              ELSE
                ddzs1=dzsa+ddzs-dzsmax
              ENDIF              
              !ddzs1=max(0.0_rsh,dzsa+ddzs-dzsmax)t
              ddzs2=ddzs-ddzs1
              dzsa2=dzsa+ddzs2
              voldepgrv=ddzs1*cvolmaxsort

              ! sand deposition :
              ! -----------------
              ! attention: a confirmer:
              ! modification du calcul de dvolsan (9 janvier 2012)
              !dvolsan=MIN(voldepsan,dzsa2*cvolmaxsort-dzsa*cvolinisan, &
              !                      dzsa2*cvolmaxmel-dzsa*(cvolinigrv+cvolinisan)-ddzs2*cvolmaxsort)
              ! ajout dvolgrv
              dvolsan=MIN(voldepsan,dzsa2*cvolmaxsort-dzsa*cvolinisan, &
                                    dzsa2*cvolmaxmel-dzsa*(cvolinigrv+cvolinisan)-ddzs2*cvolmaxsort)
              dvolsan=MAX(dvolsan,0.0_rsh)
              ! une partie des sandes dvolsab (avec repartition frdep) 
              ! vient densifier l epaisseur dzsa2,l autre partie vient creer 
              ! une surepaisseur ddzs4 (repartition frdep) et
              ! contribuer a une nouvelle couche pour ddzs3 (repartition frdep)
              ddzs=(voldepsan-dvolsan)/cvolmaxsort
              ! suppression de la ligne suivante (9 janvier 2012)
              !voldepsan=0.0_rsh
              IF((dzsa2+ddzs).LE.dzsmax)THEN
                ddzs3=0.0_rsh
              ELSEIF(dzsa2.GE.dzsmax)THEN
                ddzs3=ddzs
              ELSE
                ddzs3=dzsa2+ddzs-dzsmax
              ENDIF              
              !ddzs3=max(0.0_rsh,dzsa2+ddzs-dzsmax)
              ddzs4=ddzs-ddzs3
              dzsa3=dzsa2+ddzs4
              voldepsan=ddzs3*cvolmaxsort

              ! mud deposition :
              ! ----------------
              ! une partie des vases dmasmud (avec repartition frdep) 
              ! vient densifier l epaisseur dzsa3, l autre partie vient creer 
              ! une surepaisseur ddzs6 (repartition frdep) et
              ! contribuer a une nouvelle couche pour ddzs5 (repartition frdep)
              dmasmud=MIN(masdepmud,(dzsa3-dzsa)*csupi*(1.0_rsh-cvolmaxmel))
              ddzs=(masdepmud-dmasmud)/csupi

              ! suppression ligne suivante (9 janvier 2012)
              !masdepmud=0.0_rsh
              IF((dzsa3+ddzs).LE.dzsmax)THEN
                ddzs5=0.0_rsh
              ELSEIF(dzsa3.GE.dzsmax)THEN
                ddzs5=ddzs
              ELSE
                ddzs5=dzsa3+ddzs-dzsmax
              ENDIF              
              !ddzs5=max(0.0_rsh,dzsa3+ddzs-dzsmax)
              ddzs6=ddzs-ddzs5
              masdepmud=ddzs5*csupi

              dzs(ksmax,i,j)=dzsa3+ddzs6
              dzsi=1.0_rsh/dzs(ksmax,i,j)
              DO iv=igrav1,igrav2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+(dvolgrv    &
                                     +ddzs2*cvolmaxsort)*frdep(iv))*dzsi
              ENDDO
              DO iv=isand1,isand2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+(dvolsan    &
                                     +ddzs4*cvolmaxsort)*frdep(iv))*dzsi
              ENDDO
              DO iv=imud1,imud2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+(dmasmud    &
                                     +ddzs6*csupi)*frdep(iv))*dzsi
              ENDDO
              ! rajout benedicte   ??? A VERIFIER (1.1.1)
              DO iv=nvpc+1,nvp
                ivp_assoc=irkm_var_assoc(iv)
                IF (ivp_assoc == 0) THEN
                   ! its own settling velocity 
                   cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)+flx_w2s_loc(iv)/dzs(ksmax,i,j)
                ELSE IF (ivp_assoc < isand1) THEN
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+            &
                                     flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*    &
                                     (dvolgrv+ddzs2*cvolmaxsort)*frdep(iv))*dzsi
                ELSE IF (ivp_assoc < imud1) THEN
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+            &
                                      flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*   &
                                    (dvolsan+ddzs4*cvolmaxsort)*frdep(iv))*dzsi
                ELSE 
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa             &
                                     +flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*    &
                                     (dmasmud+ddzs6*csupi)*frdep(ivp_assoc))*dzsi
                END IF 
              ENDDO
              k=ksmax
              c_sedtot(k,i,j)=0.0_rsh
              cvolp=0.0_rsh
              ! correction benedicte  ? jusqu a nvpc et non nvp ???
              DO iv=1,nvpc
                c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
                cvolp=cvolp+cv_sed(iv,k,i,j)/ros(iv)
              ENDDO
              poro(k,i,j)=1.0_rsh-cvolp
            ENDIF
          ENDIF
  
          ! case 1.2 or following 1.1: computation of remaining deposits
          ! -----------------------------------------------------------

          dzsgrv=0.0_rsh
          dzssan=0.0_rsh
          dzsmud=0.0_rsh
          ! gravel deposition:
          ! ------------------
          IF(voldepgrv.GE.0.0_rsh)THEN
            dzsgrv=voldepgrv/cvolmaxsort
          ENDIF

          ! sand deposition:
          ! ----------------
          IF(voldepsan.GE.0.0_rsh)THEN
            dvolsan=MIN(voldepsan,dzsgrv*(cvolmaxmel-cvolmaxsort))
            dzssan=(voldepsan-dvolsan)/cvolmaxsort
          ENDIF

          ! mud deposition:
          ! ---------------
          IF(masdepmud.GE.0.0_rsh)THEN
            ! depot de la vase
            dmasmud=MIN(masdepmud,(dzsgrv+dzssan)*csupi*(1.0_rsh-cvolmaxmel))
            dzsmud=(masdepmud-dmasmud)/csupi
          ENDIF
          
          dzsnew=dzsgrv+dzssan+dzsmud

          IF(dzsnew.GT.0.0_rsh)THEN
          
            ! case 1.2 or following 1.1: possible creation of a new layer
            ! ----------------------------------------------------------

            IF(dzsnew >= dzsmin .OR. ksmax == 0)THEN
            
              ! actual constitution of the new layer:
              ! -------------------------------------
              IF(ksmax.EQ.ksdmax)CALL sed_fusion(i,j,ksmax)
              dzsa=0.0_rsh
              ksmax=ksmax+1
              dzs(ksmax,i,j)=dzsnew
              dzsi=1.0_rsh/dzs(ksmax,i,j)
              DO iv=igrav1,igrav2
                cv_sed(iv,ksmax,i,j)=voldepgrv*frdep(iv)*dzsi
              ENDDO
              DO iv=isand1,isand2
                cv_sed(iv,ksmax,i,j)=voldepsan*frdep(iv)*dzsi
              ENDDO
              DO iv=imud1,imud2
                cv_sed(iv,ksmax,i,j)=masdepmud*frdep(iv)*dzsi
              ENDDO
              !!  rajout benedicte A VERIFIER (1.2)
              DO iv=nvpc+1,nvp
                ivp_assoc=irkm_var_assoc(iv)
                IF (ivp_assoc == 0) THEN
                   ! its own settling velocity 
                   cv_sed(iv,ksmax,i,j)=MAX(cv_wat(iv,1,i,j),0.0_rsh)/csupi*masdepmud*dzsi
                ELSE 
                   cv_sed(iv,ksmax,i,j)=flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*  &
                                        cv_sed(ivp_assoc,ksmax,i,j)
                END IF 
              ENDDO
            ELSE
            
              ! as the new layer is too small, back to an increase of the surf. layer:
              ! ----------------------------------------------------------------------
              dzsa=dzs(ksmax,i,j)
              dzs(ksmax,i,j)=dzs(ksmax,i,j)+dzsnew
              dzsi=1.0_rsh/dzs(ksmax,i,j)
              DO iv=igrav1,igrav2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa                 &
                                  +voldepgrv*frdep(iv))*dzsi
              ENDDO
              DO iv=isand1,isand2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa                 &
                                 +voldepsan*frdep(iv))*dzsi
              ENDDO
              DO iv=imud1,imud2
                cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa                 &
                                  +masdepmud*frdep(iv))*dzsi
              ENDDO
              !!  rajout benedicte A VERIFIER (1.3)
              DO iv=nvpc+1,nvp
                ivp_assoc=irkm_var_assoc(iv)
                IF (ivp_assoc == 0) THEN
                   ! its own settling velocity 
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa                 &
                                       +MAX(cv_wat(iv,1,i,j),0.0_rsh)/csupi*masdepmud)*dzsi
                ELSE IF (ivp_assoc < isand1) THEN
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+            &
                                       flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*  &
                                       voldepgrv*frdep(iv))*dzsi
                ELSE IF (ivp_assoc < imud1) THEN
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+            &
                                      flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*    &
                                      voldepsan*frdep(iv))*dzsi
                ELSE 
                   cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa             &
                                      +flx_w2s_loc(iv)/(flx_w2s_loc(ivp_assoc)+epsd)*   &
                                      masdepmud*frdep(ivp_assoc))*dzsi
                END IF 
              ENDDO
            ENDIF
            
            ! computing total concentration and porosity of the surf.layer:
            ! -------------------------------------------------------------
            k=ksmax
            c_sedtot(k,i,j)=0.0_rsh
            cvolp=0.0_rsh
            DO iv=1,nvpc
              c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
              cvolp=cvolp+cv_sed(iv,k,i,j)/ros(iv)
            ENDDO
            poro(k,i,j)=1.0_rsh-cvolp
 
            ! control writing :            
            !if(poro(k,i,j).ge.0.9999_rsh)then
            !  write(*,*)'dans depeff, ',t,'  en kij:',k,i,j
            !  write(*,555)k,dzs(k,i,j),c_sedtot(k,i,j),poro(k,i,j)
            !  write(*,*)dzsa,dzsgrv,dzssan,dzsmud,voldepgrv,voldepsan, &
            !            masdepmud,frdep,(cv_sed(iv,k,i,j),iv=1,4)
            !  write(*,*)igrav1,igrav2,isand1,isand2,imud1,imud2
            !  write(*,*)(fluevs(iv,i,j),iv=1,5)
            !  write(*,*)cvasr,cvascr,ddzs,ddzs1,ddzs2,ddzs3,ddzs4,ddzs5
            !  write(*,*)ddzsici,dvolgrv,dvolsan,cvolinigrv,cvolinisan,dzsaici
            !endif

          ENDIF

          ! computing flx_w2s for dissolved var., to be used in the next time step:
          ! -----------------------------------------------------------------------
          DO iv=nvp+1,nv_adv
            !fluevs(iv,i,j)=fludep*porosi/csupi
            !fluevs est une vitesse de chute virtuelle
            !commune a toutes les variables dissoutes : 
            !a revoir, ventuellement: fluevs(iv,i,j)=porodep*vitdepo
            flx_w2s(iv,i,j)=0.0_rsh
          ENDDO
          !epeaui(i,j)=epeaui(i,j)*porosi

        ELSE

          ! case 2. : no deposition
          ! ***********************  

          ! computing flx_w2s for dissolved var., to be used in the next time step:
          ! -----------------------------------------------------------------------
          DO iv=nvp+1,nv_adv
            flx_w2s(iv,i,j)=0.0_rsh
          ENDDO
          !? epeaui(i,j)=0.0_rsh

          ! accounting for possible deposition of non constitutive particulate variable:
          ! ----------------------------------------------------------------------------
          ! prise en compte des depots eventuels 
          ! de variables particulaires non constitutives:
          IF(ksmax.GE.ksmi(i,j))THEN
            DO iv=nvpc+1,nvp
              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)+flx_w2s_loc(iv)/dzs(ksmax,i,j)
            ENDDO
          ELSE
            DO iv=nvpc+1,nvp
#if defined key_siggen || defined key_gencoord
              cv_wat(iv,1,i,j)=cv_wat(iv,1,i,j)                                        &
                               +flx_w2s_loc(iv)/(dsigu(1)*f_hzu(h0(i,j),xeloc(i,j),k,i,j))
#else
              cv_wat(iv,1,i,j)=cv_wat(iv,1,i,j)                                        &
                               +flx_w2s_loc(iv)/(dsigu(1)*(xeloc(i,j)+h0(i,j)))
#endif
            ENDDO
          ENDIF

        ENDIF

        !GOTO 51
        ! control writing :            
        DO k=ksmi(i,j),ksmax
          IF(poro(k,i,j).GE.0.9999_rsh)THEN
            !WRITE(*,*)'a la fin de depeff, a',t,'  en kij:',k,i,j
            DO kl=ksmax,ksmi(i,j),-1
              !WRITE(*,555)kl,dzs(kl,i,j),c_sedtot(kl,i,j),poro(kl,i,j)
            555 FORMAT('k=',i3,'  dzs ',g8.2,'  csed ',g8.2,'  poro ',g8.2)
            ENDDO
            GOTO 51
          ENDIF
        ENDDO
        51 CONTINUE

        ! updating ksma ( ksmax can be modified in the routine)
        ksma(i,j)=ksmax
        
       END IF ! test on h0+xe
      END DO  ! loop on i
    END DO    ! loop on j

   PRINT_DBG*, 'FIN SED_EFFDEP'   

  END SUBROUTINE sed_effdep_mixsed

#endif

   !!==============================================================================
 
#ifdef key_sedim_mudsand

  SUBROUTINE sed_effdep_mudsand(xeloc)
    
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_effdep  ***
   !&E
   !&E ** Purpose : computes effective deposition
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  step3dxy, step3dyx
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (Pierre Le Hir) extracted from subsedim.F90 (from F. Dufois)
   !&E       !  2008-10  (F. Cayocca) included into module_sedimento by F. Cayocca
   !&E       !  2008-02  (P. Le Hir) replaced by new "Marennes-Oleron" sand/mud sediment model
   !&E       !  2011-11  (V. Garnier) Spatial extension of ssh(liminm2:limaxp2,ljminm2:ljmaxp2)
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used   
#if defined key_contaminant
   USE contareaddat , ONLY : cin_max_re,cf_stochio_re,    &
                             irkm_var_re,nreacmax,nb_var_re,ityp_reac
#endif 
   USE comsubstance,  ONLY : typart,nv_adv,nvp,nvpc,flx_w2s,cv_wat,flx_w2s_sum
   USE comvars2d,     ONLY : h0fond,t,ig,id,h0,dt
   USE comvars3d,     ONLY : dsigu
   USE comvarp2d,     ONLY : sal,temp
   USE parameters,    ONLY : epsilon
#if defined key_siggen || defined key_gencoord
  USE toolgeom,     ONLY : f_hzu
#endif

   !! * Arguments
   REAL(KIND=rsh),DIMENSION(liminm2:limaxp2,ljminm2:ljmaxp2),INTENT(IN) :: xeloc

   !! * Local declarations
   REAL(KIND=rsh),DIMENSION(nvp) :: flx_w2s_loc
   INTEGER        :: i,j,k,iv,ksmin,ksmax,kl,niter
   REAL(KIND=rsh) :: fludep,qdep,vitdepo,ddzs,ddzs1,ddzs2,dzsa,dzsisu,      &
                     poroa,somalp,somala,cdepiv,porosi,dzpoi,porodep,dts2
#if defined key_contaminant 
   INTEGER        :: ivr,ire,itypr
#endif
   !!
   !!----------------------------------------------------------------------
   !! * Executable part

    ddzs=0.0_rsh
    dts2=0.5_rsh*REAL(dt,rsh)

    DO j=ljmin,ljmax
    DO i=MAX0(limin,ig(j)),MIN0(limax,id(j))

      IF(h0(i,j)+xeloc(i,j).GT.h0fond) THEN

        ksmax=ksma(i,j)

        ! reactualisation du depot effectif (flx_w2s) car implicite

        DO iv=1,nvp
!FLOoct2009          flx_w2s(iv,i,j)=flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)
!        flx_w2s_loc(iv)=flx_w2s(iv,i,j)*cv_wat(iv,1,i,j)
!      flx_w2s_loc(iv)=MAX(flx_w2s_loc(iv),0.0_rsh)
! modif bene pour prendre le depot integre sur les ndt_part pas de temps
!      deja calcule a chaque sous pas de temps en fonction des cv_wat intermediaire
!       et exprime en Masse/m2 integre sur le demi pas de temps
         flx_w2s_loc(iv)=MAX(flx_w2s_sum(iv,i,j),0.0_rsh)/dts2
         flx_w2s_sum(iv,i,j)=0.0_rsh
        ENDDO

        fludep=0.0_rsh
        DO iv=1,nvpc
!FLOoct2009          fludep=fludep+flx_w2s(iv,i,j)
          fludep=fludep+flx_w2s_loc(iv)
        ENDDO

      IF(fludep.GT.0.0_rsh)THEN
         ! cas ou il y a depot
         ! *******************
          qdep=fludep*dts2
          vitdepo=fludep/csupi
          ddzs=dts2*vitdepo
          somalp=0.0_rsh
          DO iv=1,nvpc
!FLOoct2009            somalp=somalp+flx_w2s(iv,i,j)/(ros(iv)+epsilon)
            somalp=somalp+flx_w2s_loc(iv)/(ros(iv)+epsilon)
          ENDDO
          porodep=1.0_rsh-somalp/vitdepo
          epeaui(i,j)=ddzs*porodep
          !i.e. ddzs=qdep/csupi
          IF(ksmax.LT.ksmi(i,j))THEN
            ddzs2=ddzs

          ELSE
            ! il existe deja une couche
            ! -------------------------
            ddzs1=MIN(dzsmax-dzs(ksmax,i,j),ddzs)
            ddzs1=0.5_rsh*(ddzs1+ABS(ddzs1))
            IF(ksdmax.GT.1 .AND. c_sedtot(ksmax,i,j).GE.csmaxmel) ddzs1=0.0_rsh 
            ddzs2=ddzs-ddzs1
            IF(ddzs2.LE.dzsmin)THEN
              ddzs1=ddzs
              ddzs2=0.0_rsh
            ENDIF

            ! cas ou il n'y a qu'une couche : on remplit indefiniment
            ! -------------------------------------------------------
            IF(ddzs2>epsilon .AND. ksdmax==1) THEN
              ddzs1=ddzs
              ddzs2=0.0_rsh
            ENDIF
 

            ! melange avec la couche superieure
            ! ---------------------------------
            dzsa=dzs(ksmax,i,j)
            dzs(ksmax,i,j)=dzs(ksmax,i,j)+ddzs1
            dzsisu=1.0_rsh/dzs(ksmax,i,j)
            c_sedtot(ksmax,i,j)=(c_sedtot(ksmax,i,j)*dzsa+csupi*ddzs1)*dzsisu
            poroa=poro(ksmax,i,j)
            somalp=0.0_rsh
            somala=0.0_rsh
            DO iv=1,nvp
!FLOoct2009              cdepiv=flx_w2s(iv,i,j)/vitdepo
              cdepiv=flx_w2s_loc(iv)/vitdepo
              cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*dzsa+cdepiv*ddzs1)*dzsisu
              somala=somala+cdepiv*typart(iv)/(ros(iv)+epsilon)
              somalp=somalp+cv_sed(iv,ksmax,i,j)*typart(iv)/(ros(iv)+epsilon)
            ENDDO
            porosi=1.0_rsh-somala
            poro(ksmax,i,j)=1.0_rsh-somalp
            poroin(ksmax,i,j)=poro(ksmax,i,j)
            IF(ksmax.GT.ksmi(i,j))THEN
             dzsiin(ksmax-1,i,j)=2.0_rsh/(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
             poroin(ksmax-1,i,j)=0.5_rsh*(poro(ksmax-1,i,j)+poro(ksmax,i,j))
            ENDIF
            dzpoi=dzsisu/poro(ksmax,i,j)
            cv_sed(-1,ksmax,i,j)=(cv_sed(-1,ksmax,i,j)*poroa*dzsa+temp(1,i,j)*porosi*ddzs1)*dzpoi
            cv_sed( 0,ksmax,i,j)=(cv_sed( 0,ksmax,i,j)*poroa*dzsa+sal(1,i,j)*porosi*ddzs1)*dzpoi
            DO iv=nvp+1,nv_adv
              cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax,i,j)*poroa*dzsa+cv_wat(iv,1,i,j)*porosi*ddzs1)*dzpoi
              ! attention, le puits dans l eau a pu etre different,   
              ! car flx_w2s calcule dt avant.
              !                 +flx_w2s(iv,i,j)/vitdepo*ddzs1)*dzpoi
              ! a condition que: flx_w2s(iv,i,j) en m/s =cv_wat*vitdepo*porosi
            ENDDO

          ENDIF

          ! poursuite du remplissage
          ! ------------------------ 

          niter=0
777       niter=niter+1
          IF(niter.GE.1000)THEN
            !WRITE(*,*)'probleme iteratif dans depeff'
!FLOoct2009            WRITE(*,778)t,i,j,ksmax,ddzs2,(flx_w2s(iv,i,j),cv_wat(iv,1,i,j),iv=1,nvp)
            WRITE(*,778)t,i,j,ksmax,ddzs2,(flx_w2s_loc(iv),cv_wat(iv,1,i,j),iv=1,nvp)
778         FORMAT('a',f12.0,'s en i,j=',i3,1x,i3,' ksmax=',i3,' ddzs2=',g10.4,'flu+cv',8g9.3)
            STOP
          ENDIF

          IF(ddzs2.GT.0.0_rsh)THEN
            IF(ksmax.EQ.ksdmax)THEN

              ! cas ou on a atteint le nbre max de couches:
              ! fusion des 2 premieres couches (fond) et decalage des autres
              ! ------------------------------------------------------------
              ksmin=ksmi(i,j)
              dzsa=dzs(ksmin,i,j)
              poroa=poro(ksmin,i,j)
              dzs(ksmin,i,j)=dzs(ksmin,i,j)+dzs(ksmin+1,i,j)
              c_sedtot(ksmin,i,j)=0.0_rsh
              somalp=0.0_rsh
              DO iv=1,nvp
                cv_sed(iv,ksmin,i,j)=(cv_sed(iv,ksmin,i,j)*dzsa+cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j))/dzs(ksmin,i,j)
                c_sedtot(ksmin,i,j)=c_sedtot(ksmin,i,j)+cv_sed(iv,ksmin,i,j)*typart(iv)
                somalp=somalp+cv_sed(iv,ksmin,i,j)*typart(iv)/(ros(iv)+epsilon)
              ENDDO
              poro(ksmin,i,j)=1.0_rsh-somalp
              cv_sed(-1,ksmin,i,j)=(cv_sed(-1,ksmin,i,j)*dzsa*poroa                             &
                                   +cv_sed(-1,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
                                  /(dzs(ksmin,i,j)*poro(ksmin,i,j))
              cv_sed( 0,ksmin,i,j)=(cv_sed( 0,ksmin,i,j)*dzsa*poroa                             &
                                   +cv_sed( 0,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
                                  /(dzs(ksmin,i,j)*poro(ksmin,i,j))
              DO iv=nvp+1,nv_adv
                cv_sed(iv,ksmin,i,j)=(cv_sed(iv,ksmin,i,j)*dzsa*poroa                             &
                                     +cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))  &
                                    /(dzs(ksmin,i,j)*poro(ksmin,i,j))
              ENDDO
              poroin(ksmin,i,j)=0.5_rsh*(poro(ksmin,i,j)+poro(ksmin+2,i,j))
              dzsiin(ksmin,i,j)=2.0_rsh/(dzs(ksmin,i,j)+dzs(ksmin+2,i,j))
              DO k=ksmin+1,ksmax-1
                dzs(k,i,j)=dzs(k+1,i,j)
                dzsiin(k,i,j)=dzsiin(k+1,i,j)
                c_sedtot(k,i,j)=c_sedtot(k+1,i,j)
                poro(k,i,j)=poro(k+1,i,j)
                poroin(k,i,j)=poroin(k+1,i,j)
                !do iv=1,nvar
                DO iv=-1,nv_adv
                  cv_sed(iv,k,i,j)=cv_sed(iv,k+1,i,j)
                ENDDO
              ENDDO
              ksmax=ksmax-1

            ENDIF

            ! creation d une nouvelle couche
            ! ------------------------------
            ksmax=ksmax+1
            dzs(ksmax,i,j)=MIN(dzsmax,ddzs2)
            ddzs2=ddzs2-dzs(ksmax,i,j)
            IF(ddzs2.LE.dzsmin)THEN
              dzs(ksmax,i,j)=ddzs2+dzs(ksmax,i,j)
              ddzs2=0.0_rsh
            ENDIF
    
            dzsisu=1.0_rsh/dzs(ksmax,i,j)
            !? dzsiin(ksmax,i,j)=2.0_rsh*dzsisu
            c_sedtot(ksmax,i,j)=csupi
            DO iv=1,nvp
!FLOoct2009              cv_sed(iv,ksmax,i,j)=flx_w2s(iv,i,j)/vitdepo
              cv_sed(iv,ksmax,i,j)=flx_w2s_loc(iv)/vitdepo
            ENDDO
            poro(ksmax,i,j)=porodep
            poroin(ksmax,i,j)=poro(ksmax,i,j)
            IF (ksmax.GT.ksmi(i,j))THEN
              poroin(ksmax-1,i,j)=0.5_rsh*(poro(ksmax-1,i,j)+poro(ksmax,i,j))
              dzsiin(ksmax-1,i,j)=2.0_rsh/(dzs(ksmax,i,j)+dzs(ksmax-1,i,j))
            ENDIF

            cv_sed(-1,ksmax,i,j)=temp(1,i,j)
            cv_sed( 0,ksmax,i,j)=sal(1,i,j)
            DO iv=nvp+1,nv_adv
              cv_sed(iv,ksmax,i,j)=cv_wat(iv,1,i,j)
            ENDDO

            GOTO 777
          ENDIF

          ! calcul du flx_w2s des var. diss. pour le pas de temps suivant:
          flx_w2s(-1,i,j)=porodep*vitdepo
          flx_w2s( 0,i,j)=porodep*vitdepo
          DO iv=nvp+1,nv_adv
            !flx_w2s(iv,i,j)=fludep*porosi/csupi
            !flx_w2s est une vitesse de chute virtuelle
            !commune a toutes les variables dissoutes : 
            flx_w2s(iv,i,j)=porodep*vitdepo
          ENDDO

        ELSE

          ! cas ou pas de depot
          ! *******************  
          ! calcul du flx_w2s des var. diss. pour le pas de temps suivant:
          flx_w2s(-1,i,j)=0.0_rsh
          flx_w2s( 0,i,j)=0.0_rsh
          !do iv=nvp+1,nvar
          DO iv=nvp+1,nv_adv
            flx_w2s(iv,i,j)=0.0_rsh
          ENDDO
          epeaui(i,j)=0.0_rsh

          ! prise en compte des depots eventuels 
          ! de variables particulaires non constitutives:
          !if(ksmax.ge.ksmi(i,j).and.dzs(ksmax,i,j).ge.dzsmin)then
          IF(ksmax.GE.1)THEN
            DO iv=nvpc+1,nvp
!FLOoct2009              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)+flx_w2s(iv,i,j)*dts2/dzs(ksmax,i,j)
              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)+flx_w2s_loc(iv)*dts2/dzs(ksmax,i,j)
            ENDDO
          ELSE
            DO iv=nvpc+1,nvp
#if defined key_siggen || defined key_gencoord
              cv_wat(iv,1,i,j)=cv_wat(iv,1,i,j) &
                               +flx_w2s_loc(iv)*dts2/(dsigu(1)*f_hzu(h0(i,j),xeloc(i,j),k,i,j))
#else
              cv_wat(iv,1,i,j)=cv_wat(iv,1,i,j) &
                               +flx_w2s_loc(iv)*dts2/(dsigu(1)*(xeloc(i,j)+h0(i,j)))
#endif
            ENDDO
          ENDIF

        ENDIF ! test on fludep

        ! reactualisation de ksma ( ksmax a pu etre modifie dans la routine)
        ksma(i,j)=ksmax

#ifdef key_contaminant
!      DO ire=1,nreacmax
!        itypr=ityp_reac(ire)
!        IF(itypr.EQ.1) THEN
!          iv=irkm_var_re(ire,1) 
!          IF (SUM(cv_sed(iv,:,i,j)).NE.0.) THEN  
!            DO k=ksdmin,ksma(i,j)
!              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*(1+cf_stochio_re(ire,1)*cin_max_re(ire)*dts2)
!            ENDDO
!          ENDIF
!        ENDIF  
!      ENDDO
#endif

      ENDIF ! test on h0+xe

      END DO ! loop on i
    END DO ! loop on j

  END SUBROUTINE sed_effdep_mudsand

#endif
   !!==============================================================================
 
#ifdef key_sedim_mixsed

  SUBROUTINE sed_fusion(i,j,ksmaxhere)
 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_fusion  ***
   !&E
   !&E ** Purpose : makes fusion of the 2 first layers of the sediment (at the bottom)
   !&E              when the max. number of layers is reached
   !&E              and updating of the sediment column
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  sed_effdep
   !&E 
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-03 (P. Le Hir) reproduces s_fusion in new "Marennes-Oleron" sand/mud sediment model
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nv_adv,nvp,typart

   !! * Arguments
   INTEGER,INTENT(IN)     ::  i,j
   INTEGER,INTENT(INOUT)  ::  ksmaxhere

   !! * Local declarations
   INTEGER        ::  k,iv,ksmin
   REAL(KIND=rsh) ::  dzsa,poroa,cvolp,dzspwi               
   
   !!----------------------------------------------------------------------
   !! * Executable part
      
   ksmin=ksmi(i,j)
   dzsa=dzs(ksmin,i,j)
   poroa=poro(ksmin,i,j)
   dzs(ksmin,i,j)=dzs(ksmin,i,j)+dzs(ksmin+1,i,j)
   cvolp=0.0_rsh
   c_sedtot(ksmin,i,j)=0.0_rsh
   DO iv=1,nvp
     cv_sed(iv,ksmin,i,j)=(cv_sed(iv,ksmin,i,j)*dzsa+cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j))/dzs(ksmin,i,j)
     c_sedtot(ksmin,i,j)=c_sedtot(ksmin,i,j)+cv_sed(iv,ksmin,i,j)*typart(iv)
     cvolp=cvolp+cv_sed(iv,ksmin,i,j)*typart(iv)/ros(iv)
   ENDDO
   poro(ksmin,i,j)=1.0_rsh-cvolp
   dzspwi=1.0_rsh/(dzs(ksmin,i,j)*poro(ksmin,i,j))
   DO iv=nvp+1,nv_adv
    ! cv_sed(iv,ksmin,i,j)=dzspwi*(cv_sed(iv,ksmin,i,j)*dzsa*poroa &
    !                             +cv_sed(iv,ksmin+1,i,j)*dzs(ksmin+1,i,j)*poro(ksmin+1,i,j))
      ! correction bene
      cv_sed(iv,ksmin,i,j)=0.0_rsh
   ENDDO
   DO k=ksmin+1,ksmaxhere-1
     dzs(k,i,j)=dzs(k+1,i,j)
     poro(k,i,j)=poro(k+1,i,j)
     c_sedtot(k,i,j)=c_sedtot(k+1,i,j)
     DO iv=1,nv_adv
       cv_sed(iv,k,i,j)=cv_sed(iv,k+1,i,j)
     ENDDO
   ENDDO
   ksmaxhere=ksmaxhere-1

  END SUBROUTINE sed_fusion
      
#endif

!!==============================================================================
   SUBROUTINE sed_consol_diffu
! 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_consol_diffu ***
   !&E
   !&E ** Purpose : call subroutines of consolidation and/or diffusion if needed
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  step
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2010-09  (B. Thouvenin) 
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
  
   USE comvars2d,    ONLY : h0,ig,id
   USE comvarp2d,    ONLY : ssh
   USE comvars2d,    ONLY : t
   USE comsubstance, ONLY : nvp,nv_adv

   !! * Local declarations
   LOGICAL                                   :: l_consol
   INTEGER                                   :: i,j,k,iv
   REAL(KIND=rsh)                            :: dzsa,volpwa
   REAL(KIND=rsh),DIMENSION(ksdmin  :ksdmax)   :: poroa
   REAL(KIND=rsh),DIMENSION(ksdmin-1:ksdmax)    :: winter
   !!----------------------------------------------------------------------
   !! * Executable part
   

   IF (l_consolid .AND. t.GE.tstartconsol .AND. t.GE.tconsol) then
     dtconsoleff=dtconsol+(t-tconsol)
     tconsol=t+dtconsol
     dtconsolinv=1.0_rsh/REAL(dtconsoleff,rsh)
     l_consol=.true.
   ELSE
     l_consol=.false.
   END IF
   PRINT_DBG*, 'DEBUT DE DIF et CONSOL '
     
     DO j=ljminp1,ljmax
      DO i=MAX0(limin,ig(j)+1),MIN0(limax,id(j))

        IF (h0(i,j)+ssh(i,j).GT.0.0_rsh) THEN

          IF (ksma(i,j).GE.ksmi(i,j))THEN

           ! memorisation des porosites, du volume d eau interstitielle
           ! et de l epaisseur de la couche de surface
           ! ----------------------------------------------------------
           DO k=ksmi(i,j),ksma(i,j)
             poroa(k)=poro(k,i,j)
           END DO
           dzsa=dzs(ksma(i,j),i,j)
           volpwa=poro(ksma(i,j),i,j)*dzsa
           varpwa(-1)=volpwa*cv_sed(-1,ksma(i,j),i,j)
           varpwa( 0)=volpwa*cv_sed( 0,ksma(i,j),i,j)
           DO iv=nvp+1,nv_adv
             varpwa(iv)=volpwa*cv_sed(iv,ksma(i,j),i,j)
           END DO
           winter(:)=0.0_rsh
           
           IF (l_consol) THEN

#ifdef key_sedim_mudsand
             CALL sed_consolid_mudsand(i,j,volpwa,poroa,winter)
             IF(l_diffused) CALL sed_diffusion_mudsand(i,j,poroa,winter)
#else
             CALL sed_consolid_mixsed(i,j)
! not ready        IF(l_diffused) CALL sed_diffusion_mixsed(i,j)
#endif

    
           ELSE
    
#ifdef key_sedim_mudsand
             IF(l_diffused) CALL sed_diffusion_mudsand(i,j,poroa,winter)
#else
! not ready        IF(l_diffused) CALL sed_diffusion_mixsed(i,j)
#endif
           END IF
          END IF
        END IF
    
      END DO
     END DO
        
  END SUBROUTINE sed_consol_diffu

   !!==============================================================================

#ifdef key_sedim_mixsed

   SUBROUTINE sed_consolid_mixsed(i,j)
! 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_consolid  ***
   !&E
   !&E ** Purpose : computes sediment consolidation and manages sediment layers
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  consolid_diffu
   !&E
   !&E ** External calls : sed_constitutivrel
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       !  2009-02  (P. Le Hir) new version from SiAM "sable-vase"
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
  
   USE comvars2d,    ONLY : ig,id
   USE comsubstance, ONLY : typart,nvpc,nvp,nv_adv,isand1,isand2,imud1,imud2
   USE comvars2d,    ONLY : t

   !! Arguments 
   INTEGER, intent(IN)                       :: i,j
   
   !! * Local declarations
   INTEGER                                   :: ksmin,ksmax,iv,k,ksmaxa,kp,kl
   REAL(KIND=rsh),DIMENSION(ksdmin:ksdmax)   :: loadograv,bb,cc,bbsan,ccsan,cseda 
   REAL(KIND=rsh)                            :: dtrest,dtiter,dtsrow,sigmadjge,   &
                                                somdeltaro,halfmaslayer,somcsan,   &
                                                somcmud,cvolp,rapcsed,csednew,     &
                                                dzsa,dzsam1,dzsisu    
                                                   
   REAL(KIND=rsh),PARAMETER                  :: dtminconsol=30.0_rsh,              &
                                                cmaxsan=1200.0_rsh,                &
                                                cmaxmud=300.0_rsh

   !!----------------------------------------------------------------------
   !! * Executable part
   

        ksmax=ksma(i,j)
        ksmin=ksmi(i,j)

        dtrest=REAL(dtconsoleff,rsh)
  100   CONTINUE
        
        IF(ksmax.GT.ksmin)THEN
  
          ! computation of permeability, effective stress and hydraulic load
          ! ----------------------------------------------------------------
          CALL sed_constitutivrel(i,j)        
        
          dtiter=MIN(dtrest,dtminconsol)
          DO k=ksmin,ksmax
            cseda(k)=c_sedtot(k,i,j)
          ENDDO
          dtsrow=dtiter/1000.0_rsh

          sigmadjge=0.0_rsh
          DO k=ksmax,ksmin,-1
            somdeltaro=0.0_rsh
            DO iv=1,nvpc
              somdeltaro=somdeltaro+cv_sed(iv,k,i,j)*rosmrowsros(iv)
            ENDDO
            halfmaslayer=.5*dzs(k,i,j)*somdeltaro
            sigmadjge=sigmadjge+halfmaslayer
            loadograv(k)=MAX(0.0_rsh,sigmadjge-sigmapsg(k))*stateconsol(k)
            sigmadjge=sigmadjge+halfmaslayer
          ENDDO

          ! computing new concentrations from mass conservation
          ! ---------------------------------------------------       
          k=ksmin
          !cc(k)=dtsrow*min(permeab(k),permeab(k+1))*2.0_rsh ! has been tested
          cc(k)=dtsrow*(permeab(k+1)+permeab(k))               &
                *MIN(0.0_rsh,loadograv(k+1)-loadograv(k))       &
                /(dzs(k+1,i,j)+dzs(k,i,j))/dzs(k,i,j)
          somcsan=0.0_rsh
          DO iv=isand1,isand2
            somcsan=somcsan+cv_sed(iv,k,i,j)
          ENDDO
          somcmud=0.0_rsh
          DO iv=imud1,imud2
            somcmud=somcmud+cv_sed(iv,k,i,j)
          ENDDO
          ccsan(k)=-dtiter*(MAX(0.0_rsh,1.0_rsh-somcsan/cmaxsan  &
                   -somcmud/cmaxmud))**4.65_rsh/dzs(k,i,j)
          bb(k)=0.0_rsh 
          bbsan(k)=0.0_rsh  
            
          DO k=ksmin+1,ksmax-1
            !cc(k)=dtsrow*min(permeab(k),permeab(k+1))*2.0_rsh ! has been tested
            cc(k)=dtsrow*(permeab(k+1)+permeab(k))               &
                  *MIN(0.0_rsh,loadograv(k+1)-loadograv(k))       &
                  /(dzs(k+1,i,j)+dzs(k,i,j))/dzs(k,i,j)
            somcsan=0.0_rsh
            DO iv=isand1,isand2
              somcsan=somcsan+cv_sed(iv,k,i,j)
            ENDDO
            somcmud=0.0_rsh
            DO iv=imud1,imud2
              somcmud=somcmud+cv_sed(iv,k,i,j)
            ENDDO
            ccsan(k)=-dtiter*(MAX(0.0_rsh,1.0_rsh-somcsan/cmaxsan &
                     -somcmud/cmaxmud))**4.65_rsh/dzs(k,i,j)
                   
            !bb(k)=1.0_rsh-dtsrow*min(permeab(k),permeab(k-1))*2.0_rsh ! has been tested
            bb(k)=-dtsrow*(permeab(k)+permeab(k-1))              &
                *MIN(0.0_rsh,loadograv(k)-loadograv(k-1))         &
                /(dzs(k,i,j)+dzs(k-1,i,j))/dzs(k,i,j)
            somcsan=0.0_rsh
            DO iv=isand1,isand2
              somcsan=somcsan+cv_sed(iv,k-1,i,j)
            ENDDO
            somcmud=0.0_rsh
            DO iv=imud1,imud2
              somcmud=somcmud+cv_sed(iv,k-1,i,j)
            ENDDO
            bbsan(k)=dtiter*(MAX(0.0_rsh,1.0_rsh-somcsan/cmaxsan &
                     -somcmud/cmaxmud))**4.65_rsh/dzs(k,i,j)
          ENDDO
        
          k=ksmax
          !bb(k)=1.0_rsh-dtsrow*min(permeab(k),permeab(k-1))*2.0_rsh ! has been tested
          bb(k)=-dtsrow*(permeab(k)+permeab(k-1))              &
                *MIN(0.0_rsh,loadograv(k)-loadograv(k-1))          &
                /(dzs(k,i,j)+dzs(k-1,i,j))/dzs(k,i,j)
          somcsan=0.0_rsh
          DO iv=isand1,isand2
            somcsan=somcsan+cv_sed(iv,k-1,i,j)
          ENDDO
          somcmud=0.0_rsh
          DO iv=imud1,imud2
            somcmud=somcmud+cv_sed(iv,k-1,i,j)
          ENDDO
          bbsan(k)=dtiter*(MAX(0.0_rsh,1.0_rsh-somcsan/cmaxsan-somcmud/cmaxmud))**4.65_rsh/dzs(k,i,j)
    
          DO iv=1,nvpc
            IF(iv.GE.isand1.AND.iv.LE.isand2)THEN
              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)                               &
                                   /(1.0_rsh+MAX(bb(ksmax),bbsan(ksmax)*ws0(iv)))
              DO k=ksmax-1,ksmin,-1
                cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)-MIN(cc(k),ws0(iv)*               &
                                 ccsan(k))*cv_sed(iv,k+1,i,j))                        &
                                 /(1.0_rsh+MAX(bb(k),bbsan(k)*ws0(iv)))
              ENDDO
            ELSE
              cv_sed(iv,ksmax,i,j)=cv_sed(iv,ksmax,i,j)/(1.0_rsh+bb(ksmax))
              DO k=ksmax-1,ksmin,-1
                cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)-cc(k)*cv_sed(iv,k+1,i,j))/(1.0_rsh+bb(k))
              ENDDO
            ENDIF
          ENDDO

          ! computing new concentrations/porosities :
          DO k=ksmin,ksmax
            c_sedtot(k,i,j)=0.0_rsh
            cvolp=0.0_rsh
            DO iv=1,nvpc
              c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
              cvolp=cvolp+cv_sed(iv,k,i,j)/ros(iv)
            ENDDO
            poro(k,i,j)=1.0_rsh-cvolp
          
            ! if(porosity too small (<30%), it is increased until (1.0_rsh-cvolmaxmel):
            IF(poro(k,i,j).LE.0.3_rsh)THEN
              poro(k,i,j)=1.0_rsh-cvolmaxmel
              csednew=ros(1)*cvolmaxmel
              rapcsed=csednew/c_sedtot(k,i,j)
              DO iv=1,nvpc
                cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*rapcsed
              ! question  benedicte :: que se passe t il pour les autres ivp=nvpc+1  nvp ??????
              ENDDO
              dzs(k,i,j)=dzs(k,i,j)/rapcsed
            ENDIF 
          ENDDO
        
          !do k=ksmin,ksmax     ! replaced by a test only in surface layer :
          !if(c_sedtot(k,i,j).lt.cseda(k).or.k.eq.ksmax)then
          k=ksmax
          dzsa=dzs(k,i,j)
          dzs(k,i,j)=c_sedtot(k,i,j)*dzsa/cseda(k)
          cvolp=0.0_rsh
          c_sedtot(k,i,j)=0.0_rsh
          DO iv=1,nvpc
           ! question  benedicte :: que se passe t il pour les autres ivp=nvpc+1  nvp ??????
            cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*dzsa/dzs(k,i,j)
            c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
            cvolp=cvolp+cv_sed(iv,k,i,j)/ros(iv)
          ENDDO
          poro(k,i,j)=1.0_rsh-cvolp
          !endif
          IF(poro(k,i,j).LE.0.3_rsh)THEN
            poro(k,i,j)=1.0_rsh-cvolmaxmel
            csednew=ros(1)*cvolmaxmel
            rapcsed=csednew/c_sedtot(k,i,j)
            DO iv=1,nvpc
              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*rapcsed
            ENDDO
            dzs(k,i,j)=dzs(k,i,j)/rapcsed
          ENDIF 
 
          ksmaxa=ksmax
          DO k=ksmaxa,ksmin+1,-1
            IF(c_sedtot(k,i,j).LE.csedmin.OR.dzs(k,i,j).LE.dzsmin)THEN
              ! fusion with layer below
              dzsa=dzs(k,i,j)
              dzsam1=dzs(k-1,i,j)
              dzs(k-1,i,j)=(dzsam1*c_sedtot(k-1,i,j)+c_sedtot(k,i,j)*dzsa)   &
                           /c_sedtot(k-1,i,j)
              dzsisu=1.0_rsh/dzs(k-1,i,j)
              c_sedtot(k-1,i,j)=0.0_rsh
              cvolp=0.0_rsh
              DO iv=1,nvp
                cv_sed(iv,k-1,i,j)=(cv_sed(iv,k,i,j)*dzsa+                           &
                                cv_sed(iv,k-1,i,j)*dzsam1)*dzsisu
                c_sedtot(k-1,i,j)=c_sedtot(k-1,i,j)                          &
                                  +cv_sed(iv,k-1,i,j)*typart(iv)
                cvolp=cvolp+cv_sed(iv,k-1,i,j)*typart(iv)/ros(iv)
              ENDDO
              poro(k-1,i,j)=1.0_rsh-cvolp
              DO kp=k,ksmax-1
                dzs(kp,i,j)=dzs(kp+1,i,j)
                DO iv=1,nvp
                  cv_sed(iv,kp,i,j)=cv_sed(iv,kp+1,i,j)
                ENDDO
                c_sedtot(kp,i,j)=c_sedtot(kp+1,i,j)
                poro(kp,i,j)=poro(kp+1,i,j)
              ENDDO
              ksmax=ksmax-1
            ENDIF
          ENDDO
          DO k=ksmin,ksmax
            IF(poro(k,i,j).GE.0.9999_rsh)THEN
              !WRITE(*,*)'a la fin de tass, a ',t
              DO kl=ksmax,ksmin,-1
                !WRITE(*,555)kl,dzs(kl,i,j),c_sedtot(kl,i,j),poro(kl,i,j)
 555            FORMAT('k=',i3,'  dzs ',g8.2,'  csed ',g8.2,'  poro ',g8.2)
              ENDDO
              GOTO 51
            ENDIF
          ENDDO

 51       CONTINUE
        
          ! Exchange flux at the water/sediment interface
          ! ---------------------------------------------          
          DO iv=nvp+1,nv_adv
            !fluconsol is flux resulting from consolidation, >0 upwards
            !presently non operational
            fluconsol(iv,i,j)=0.0_rsh
          ENDDO

          ! updating ksma (ksmax can be modified in the routine)
          ksma(i,j)=ksmax

          dtrest=dtrest-dtiter
          IF(dtrest.GE.1.0_rsh)GOTO 100

        ENDIF  
      

   PRINT_DBG*, 'FIN SED_CONSOLDIF'   

  END SUBROUTINE sed_consolid_mixsed

#endif
    
   !!==============================================================================

#ifdef key_sedim_mudsand

  SUBROUTINE sed_consolid_mudsand(i,j,volpwa,poroa,winter)

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_consolid  ***
   !&E
   !&E ** Purpose : computes sediment consolidation and manages sediment layers
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  consol_diffu
   !&E
   !&E ** External calls : sed_constitutivrel
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from init.F90
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       !  2009-02  (P. Le Hir) new version from SiAM "sable-vase"
   !&E
   !&E--------------------------------------------------------------------------

   !! * Modules used
   USE comvars2d,    ONLY : h0
   USE comvarp2d,    ONLY : ssh
   USE comsubstance, ONLY : nvp,nvpc,nv_adv
   USE parameters,   ONLY : epsilon

   !! Arguments 
   INTEGER, intent(IN)                       :: i,j
   REAL(KIND=rsh), intent(INOUT)             :: volpwa
   REAL(KIND=rsh),DIMENSION(ksdmin  :ksdmax), intent(INOUT)   :: poroa
   REAL(KIND=rsh),DIMENSION(ksdmin-1:ksdmax), intent(INOUT)   :: winter

   !! * Local declarations
   INTEGER                                   :: ksmin,ksmax,iv,k,it
   REAL(KIND=rsh),DIMENSION(ksdmin  :ksdmax) :: ws
   REAL(KIND=rsh) :: dzsa,dtrest,dzsan,dt1,dt1s
   REAL(KIND=rsh) :: dtsdzs,dzsam1,dvol,dzsisu,somalp
   REAL(KIND=rsh),PARAMETER                  :: dtminconsol=10.0_rsh

   !!----------------------------------------------------------------------
   !! * Executable part


        ksmin=ksmi(i,j)
        ksmax=ksma(i,j)

        ! 1- calcul du tassement
        ! **********************

        ! actuellement, la vitesse de sedimentation est supposee 
        ! identique pour toutes les particules

        dtrest=REAL(dtconsoleff,rsh)
        it=0
  1     CONTINUE
        it=it+1
        dzsan=dzs(ksmax,i,j)
        ! modif du 1-2-99:
        dt1=dtrest
        IF(ksmax.GE.2)THEN

          ! cas ou il y a plus d une couche :
          DO k=ksmin,ksmax-1
            CALL sed_comp_sedvel(poroin(k,i,j))
            ws(k)=sed_sedimvel
            dt1=MIN(xcvgce*dzs(k,i,j)/(ws(k)+1.e-16),dt1)
          ENDDO
          k=ksmax
          CALL sed_comp_sedvel(poroin(k,i,j))
          ws(k)=sed_sedimvel
          dt1s=MIN(xcvgce*dzs(k,i,j),dzsan-dzsmin)/(ws(k)+1.e-16)
          dt1=MIN(dt1,MAX(0.0_rsh,dt1s))
          !if(dt1.le.0.0_rsh)write(*,*)'dt1',dt1,i,j,k,ws(ksmax),dzsan,it
          dzs(ksmax,i,j)=dzsan-dt1*ws(ksmax)

          k=ksmin
          dtsdzs=dt1/dzs(k,i,j)
          DO iv=1,nvp
            !cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(ws(k)*fexcs &
            !                *cv_sed(iv,k+1,i,j)+ws(k)*cexcs*cv_sed(iv,k,i,j))
            ! excentrement total impose pour gagner du temps:
            cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*ws(k)*cv_sed(iv,k+1,i,j)
          ENDDO
          DO k=ksmin+1,ksmax-1
            dtsdzs=dt1/dzs(k,i,j)
            DO iv=1,nvp
             !cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(ws(k)*fexcs &    
             !                *cv_sed(iv,k+1,i,j)+(ws(k)*cexcs-ws(k-1)*fexcs) &
             !                *cv_sed(iv,k,i,j))-ws(k-1)*cexcs*cv_sed(iv,k-1,i,j))
              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)+dtsdzs*(-ws(k-1)*cv_sed(iv,k,i,j)+ws(k)*cv_sed(iv,k+1,i,j))
            ENDDO
          ENDDO
          k=ksmax
          DO iv=1,nvp
            !cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsan-dt1*ws(k-1)* &
            !                 (fexcs*cv_sed(iv,k,i,j)+cexcs*cv_sed(iv,k-1,i,j)))/dzs(k,i,j)
            cv_sed(iv,k,i,j)=(cv_sed(iv,k,i,j)*dzsan-dt1*ws(k-1)*cv_sed(iv,k,i,j))/dzs(k,i,j)
          ENDDO

          IF(dzs(ksmax,i,j).LE.dzsmin)THEN
            ! fusion des 2 couches de surface           
            dzsan=dzs(ksmax,i,j)
            dzsam1=dzs(ksmax-1,i,j)
            ! reactualisation de dzsa et volpwa:
            dzsa=dzsa+dzsam1
            dvol=poroa(ksmax-1)*dzsam1
            volpwa=volpwa+dvol
            varpwa(-1)=varpwa(-1)+dvol*cv_sed(-1,ksmax-1,i,j)
            varpwa( 0)=varpwa( 0)+dvol*cv_sed(0,ksmax-1,i,j)
            DO iv=nvp+1,nv_adv
                varpwa(iv)=varpwa(iv)+dvol*cv_sed(iv,ksmax-1,i,j)
            ENDDO
            ksmax=ksmax-1
            dzs(ksmax,i,j)=dzs(ksmax,i,j)+dzsan
            dzsisu=1.0_rsh/dzs(ksmax,i,j)
            DO iv=1,nvp
              cv_sed(iv,ksmax,i,j)=(cv_sed(iv,ksmax+1,i,j)*dzsan+cv_sed(iv,ksmax,i,j)*dzsam1)*dzsisu
            ENDDO
          ENDIF

        ELSE
          ! cas o il n y a qu une couche :
          !write(*,*)'attention: le sediment n"a qu"une couche !'
          k=ksmin
          IF(dzs(k,i,j).GT.dzsmin)THEN
            CALL sed_comp_sedvel(poro(k,i,j))
            ws(k)=sed_sedimvel
            dt1=MIN(xcvgce*dzs(k,i,j)/(ws(k)+1.e-16),dt1)
            dzs(k,i,j)=dzsan-dt1*ws(k)
            IF(dzs(k,i,j).LE.dzsmin)THEN
              dzs(k,i,j)=dzsmin
              dt1=dtrest
            ENDIF            
            dzsisu=1.0_rsh/dzs(k,i,j)
            DO iv=1,nvp
              cv_sed(iv,k,i,j)=cv_sed(iv,k,i,j)*dzsan*dzsisu
            ENDDO
          ENDIF
        ENDIF

        ! calcul de la densite (kg/m3) et de la porosite du sediment
        ! ----------------------------------------------------------
        DO k=ksmin,ksmax
          c_sedtot(k,i,j)=0.0_rsh
          somalp=0.0_rsh
          DO iv=1,nvpc
            c_sedtot(k,i,j)=c_sedtot(k,i,j)+cv_sed(iv,k,i,j)
            somalp=somalp+cv_sed(iv,k,i,j)/(ros(iv)+epsilon)
          ENDDO
          poro(k,i,j)=1.0_rsh-somalp
        ENDDO
        poroin(ksmax,i,j)=poro(ksmax,i,j)
        IF(ksmax.GE.2)THEN
          DO k=ksmin,ksmax-1
              poroin(k,i,j)=0.5_rsh*(poro(k,i,j)+poro(k+1,i,j))
              dzsiin(k,i,j)=2.0_rsh/(dzs(k,i,j)+dzs(k+1,i,j))
          ENDDO
        ENDIF

        dtrest=dtrest-dt1
        IF(dtrest.GT.dtminconsol)THEN
          IF(it.LE.200)THEN
            GOTO 1
          ELSE
           !WRITE(*,555)it,i,j,dt1,ksmax,dzs(ksmax,i,j),poro(ksmax,i,j)
 555       FORMAT('tassdif iter',i3,' en i,j=',2i3,' dt1=',g8.2,' ksmax',i3,' dzs',g8.2,'po',f4.2)
          ENDIF
        ENDIF

        ! 2- calcul des vitesses du fluide interstitiel
        ! *********************************************
        ! considerees >0 vers le haut, centree a l interface superieure

        winter(0)=0.0_rsh
        DO k=ksmin,ksmax-1
            winter(k)=winter(k-1)+(poroa(k)-poro(k,i,j))*dzs(k,i,j)*dtconsolinv
        ENDDO
        k=ksmax
        winter(k)=winter(k-1)+(volpwa-poro(k,i,j)*dzs(k,i,j))*dtconsolinv
        winsurf(i,j)=winter(k)
        DO iv=nvp+1,nv_adv
          fluconsol(iv,i,j)=winter(ksmax)*cv_sed(iv,ksmax,i,j)
        ENDDO

!
      !  Reactualisation de ksma (ksmax a pu etre modifie)
!
        ksma(i,j)=ksmax
!
!
!

   PRINT_DBG*, 'FIN SED_CONSOLID'   

  END SUBROUTINE sed_consolid_mudsand

#endif

!!==============================================================================
#ifdef key_sedim_mixsed

   SUBROUTINE sed_diffusion_mixsed(i,j)
! 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_diffusion ***
   !&E
   !&E ** Purpose : computes diffusion of dissolved substances into the sediment  (not ready)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  step
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from init.F90
   !&E       !  2010-09  (B. Thouvenin) extracted from consolid_mixsed
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
  
   USE comvars2d,    ONLY : ig,id
   USE comsubstance, ONLY : typart,nvpc,nvp,nv_adv,isand1,isand2,imud1,imud2
   USE comvars2d,    ONLY : t

   INTEGER, intent(IN)                       :: i,j

   !! * Local declarations
   INTEGER                                   :: ksmin,ksmax,iv,k

   !!----------------------------------------------------------------------
   !! * Executable part

    ksmax=ksma(i,j)
    ksmin=ksmi(i,j)

        
        IF(ksmax > ksmin)THEN
          ! Exchange flux at the water/sediment interface
          ! ---------------------------------------------
          DO iv=nvp+1,nv_adv
            !fludif is diffusion flux at the interface, >0 downwards
            !presently non operational
            fludif(iv,i,j)=0.0_rsh
          ENDDO

        END IF
    
  END SUBROUTINE sed_diffusion_mixsed

#endif

!!==============================================================================
#ifdef key_sedim_mudsand

  SUBROUTINE sed_diffusion_mudsand(i,j,poroa,winter)

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_diffusion  ***
   !&E
   !&E ** Purpose :  computes diffusion of dissolved substances into the sediment  (not ready)
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  step (if not l_consolid) or sed_consolid (if l_consolid)
   !&E
   !&E ** External calls : sed_constitutivrel
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2008-09  (P. Le Hir) extracted from init.F90
   !&E       !  2010-09  (B. Thouvenin) extracted from consolid_mudsand
   !&E
   !&E--------------------------------------------------------------------------

   !! * Modules used
   USE comvars2d,    ONLY : h0,h0fond,halfdt,nbouc
   USE comvarp2d,    ONLY : sal,temp,ssh
   USE comsubstance, ONLY : nvp,nv_adv,cv_wat
   USE parameters,   ONLY : epsilon
   
   !! Argument
   INTEGER, intent(IN)                       :: i,j
   REAL(KIND=rsh),DIMENSION(ksdmin  :ksdmax), intent(IN)   :: poroa
   REAL(KIND=rsh),DIMENSION(ksdmin-1:ksdmax), intent(IN)    :: winter
   

   !! * Local declarations
   INTEGER                                   :: ksmin,ksmax,iv,k
   REAL(KIND=rsh),DIMENSION(ksdmin  :ksdmax) :: disvi,aa,bb,cc,dd
   REAL(KIND=rsh)                            :: hcrit,dzssdt,halfdtinv
   REAL(KIND=rsh),DIMENSION(-1:nv_adv)       :: fludif_loc

   !!----------------------------------------------------------------------
   !! * Executable part
   
   disvi(:)=0.0_rsh
   fludif_loc(:)=0.0_rsh
   halfdtinv=1.0_rsh/REAL(halfdt,rsh)

        ksmax=ksma(i,j)
        ksmin=ksmi(i,j)

        ! 3- calcul des concentrations des var. dissoutes ds sediment
        ! ***********************************************************
        DO iv=nvp+1,nv_adv
      !    la boucle sur les variables positionne ici permet de faire
      !    varier les coef. de diffusion en fonction des variables
!
          k=ksmax
!c          CALL sed_difcoef_int(poro(k,i,j))
!c          disvi(k)=sed_difint
!c          disvi(k)=xdifsi1*poro(k,i,j)**xdifsi2
            disvi(k)=xdifsi1/(dzs(k,i,j)*0.5_rsh+epdifi)
            hcrit=MAX(0.0_rsh,ssh(i,j)+h0(i,j)-h0fond)
            disvi(k)=disvi(k)*hcrit/(hcrit+epsilon)
!gain            disve(k)=disvi(k)*cimpls
!gain            disvi(k)=disvi(k)*fimpls
!
!c        accroissement de la diffusion a l interface si charriage:
!c        if(iopcoh.ne.1)then
!c          efchar=xdifs3*teta**.5*max(0.0_rsh,teta-tetac)
!c          disve(k)=disve(k)*efchar
!c          disvi(k)=disvi(k)*efchar
!c        endif
!
          IF(ksmax.GT.ksmin)THEN
!
            !cas ou il y a plusieurs couches:
            !--------------------------------
!
            !calcul des vitesses de diffusion
!
            DO k=ksmin,ksmax-1
!gain         CALL sed_difcoef_sed(poro(k,i,j))
!gain         disvi(k)=sed_difsed*dzsiin(k,i,j)
!gain         disvi(k)=xdifs1*poroin(k,i,j)**xdifs2*dzsiin(k,i,j)
              disvi(k)=xdifs1*dzsiin(k,i,j)
!gain         disve(k)=disvi(k)*cimpls
!gain         disvi(k)=disvi(k)*fimpls
            ENDDO
!
            !calcul d advection-diffusion 
!
            !coefficients de la matrice
!
            !calcul implicite pour l advection, partiellement implicite
            !(fimpls) pour la diffusion; 
            !flux advectifs partielt decentres amont (fexcs), totalt en surf
            k=ksmin
            dzssdt=dzs(k,i,j)*halfdtinv
            aa(k)=0.0_rsh
            bb(k)=dzssdt*poro(k,i,j)+disvi(k)+winter(k)*fexcs
            cc(k)=-disvi(k)+winter(k)*cexcs
!gain            dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)+disve(k)*
!gain     $            (cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
            dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)
            DO k=ksmin+1,ksmax-1
              dzssdt=dzs(k,i,j)*halfdtinv
              aa(k)=-disvi(k-1)-winter(k-1)*fexcs
              bb(k)=dzssdt*poro(k,i,j)-winter(k-1)*cexcs             &
                   +winter(k)*fexcs+disvi(k-1)+disvi(k)
              cc(k)=-disvi(k)+winter(k)*cexcs
              dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)
!gain     $              +disve(k)*(cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
!gain     $              -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
            ENDDO
            k=ksmax
            aa(k)=-disvi(k-1)-winter(k-1)*fexcs
            bb(k)=dzs(k,i,j)*poro(k,i,j)*halfdtinv                     &
                +disvi(k-1)+disvi(k)-winter(k-1)*cexcs+winter(k)
            cc(k)=0.0_rsh
!gain            fludif_loc(iv)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            fludif_loc(iv)=disvi(k)*cv_wat(iv,1,i,j)
            dd(k)=varpwa(iv)*halfdtinv+fludif_loc(iv)
!gain     $            -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
!
            !resolution du systeme tridiagonal
!
            dd(1)=dd(1)/bb(1)
            DO k=ksmin+1,ksmax
              bb(k)=bb(k)-aa(k)*cc(k-1)/bb(k-1)
              dd(k)=(dd(k)-aa(k)*dd(k-1))/bb(k)
            ENDDO
            cv_sed(iv,ksmax,i,j)=dd(ksmax)
            DO k=ksmax-1,ksmin,-1
              cv_sed(iv,k,i,j)=dd(k)-cc(k)*cv_sed(iv,k+1,i,j)/bb(k)
            ENDDO
!
          ELSE
!
            !cas ou il n y a qu une couche (ksmax=ksmin):
            !--------------------------------------------       
            k=ksmax        
!gain       fludif_loc(iv)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            fludif_loc(iv)=disvi(k)*cv_wat(iv,1,i,j)
            cv_sed(iv,k,i,j)=(varpwa(iv)+fludif_loc(iv)*REAL(halfdt,rsh))           &
              /(dzs(k,i,j)*poro(k,i,j)+(winter(k)+disvi(k))*REAL(halfdt,rsh)+epsilon)
          ENDIF
!
      !  flux d echange a l interface eau/sediment:
      !  ------------------------------------------
      !  fludif est le flux par diffusion a l interface, >0 vers le bas
          fludif_loc(iv)=fludif_loc(iv)-disvi(ksmax)*cv_sed(iv,ksmax,i,j)
      !  fluconsol est le flux par tassement a l interface, >0 vers le haut
      !  si pas de consolidation a ce moment winter=0 => pas de flux de consolidation
          fluconsol(iv,i,j)=winter(ksmax)*cv_sed(iv,ksmax,i,j)

        ENDDO                         ! fin boucle sur iv

      !  3bis- calcul des concentrations de la temperature (-1) et salinite (0) 
      !     ds sediment
      !  ***********************************************************
!
        DO iv=-1,0
      !    la boucle sur les variables positionne ici permet de faire
      !    varier les coef. de diffusion en fonction des variables
!
          k=ksmax
!c          CALL sed_difcoef_int(poro(k,i,j))
!c          disvi(k)=sed_difcoef_int
!c          disvi(k)=sed_difint
!c          disvi(k)=xdifsi1*poro(k,i,j)**xdifsi2
            disvi(k)=xdifsi1 /(dzs(k,i,j)*0.5_rsh+epdifi)
            hcrit=MAX(0.0_rsh,ssh(i,j)+h0(i,j)-h0fond)
            disvi(k)=disvi(k)*hcrit/(hcrit+epsilon)
!gain            disve(k)=disvi(k)*cimpls
!gain            disvi(k)=disvi(k)*fimpls
!
!c        accroissement de la diffusion a l interface si charriage:
!c        if(iopcoh.ne.1)then
!c          efchar=xdifs3*teta**.5*max(0.0_rsh,teta-tetac)
!c          disve(k)=disve(k)*efchar
!c          disvi(k)=disvi(k)*efchar
!c        endif
!
          IF(ksmax.GT.ksmin)THEN
!
            !cas ou il y a plusieurs couches:
            !--------------------------------
!
            !calcul des vitesses de diffusion
!
            DO k=ksmin,ksmax-1
!gain         disvi(k)=sed_difsed(poroin(k,i,j))*dzsiin(k,i,j)
!gain         CALL sed_difcoef_sed(poro(k,i,j))
!gain         disvi(k)=sed_difsed*dzsiin(k,i,j)
!gain         disvi(k)=xdifs1*poroin(k,i,j)**xdifs2*dzsiin(k,i,j)
              disvi(k)=xdifs1*dzsiin(k,i,j)
!gain         disve(k)=disvi(k)*cimpls
!gain         disvi(k)=disvi(k)*fimpls
            ENDDO
!
            !calcul d advection-diffusion 
!
            !coefficients de la matrice
!
            !calcul implicite pour l advection, partiellement implicite
            !(fimpls) pour la diffusion; 
            !flux advectifs partielt decentres amont (fexcs), totalt en surf
            k=ksmin
            dzssdt=dzs(k,i,j)*halfdtinv
            aa(k)=0.0_rsh
            bb(k)=dzssdt*poro(k,i,j)+disvi(k)+winter(k)*fexcs
            cc(k)=-disvi(k)+winter(k)*cexcs
!gain            dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)+disve(k)*
!gain     $            (cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
            dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)
            DO k=ksmin+1,ksmax-1
              dzssdt=dzs(k,i,j)*halfdtinv
              aa(k)=-disvi(k-1)-winter(k-1)*fexcs
              bb(k)=dzssdt*poro(k,i,j)-winter(k-1)*cexcs             &
                   +winter(k)*fexcs+disvi(k-1)+disvi(k)
              cc(k)=-disvi(k)+winter(k)*cexcs
              dd(k)=dzssdt*poroa(k)*cv_sed(iv,k,i,j)
!gain     $              +disve(k)*(cv_sed(iv,k+1,i,j)-cv_sed(iv,k,i,j))
!gain     $              -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
            ENDDO
            k=ksmax
            aa(k)=-disvi(k-1)-winter(k-1)*fexcs
            bb(k)=dzs(k,i,j)*poro(k,i,j)*halfdtinv                      &
                 +disvi(k-1)+disvi(k)-winter(k-1)*cexcs+winter(k)
            cc(k)=0.0_rsh
!gain            fludif_loc(iv)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            IF (iv==-1) fludif_loc(iv)=disvi(k)*temp(1,i,j)
            IF (iv== 0) fludif_loc(iv)=disvi(k)*sal(1,i,j) 
            dd(k)=varpwa(iv)*halfdtinv+fludif_loc(iv)
!gain     $            -disve(k-1)*(cv_sed(iv,k,i,j)-cv_sed(iv,k-1,i,j))
!
            !resolution du systeme tridiagonal
!
            dd(1)=dd(1)/bb(1)
            DO k=ksmin+1,ksmax
              bb(k)=bb(k)-aa(k)*cc(k-1)/bb(k-1)
              dd(k)=(dd(k)-aa(k)*dd(k-1))/bb(k)
            ENDDO
            cv_sed(iv,ksmax,i,j)=dd(ksmax)
            DO k=ksmax-1,ksmin,-1
              cv_sed(iv,k,i,j)=dd(k)-cc(k)*cv_sed(iv,k+1,i,j)/bb(k)
            ENDDO
!
          ELSE
!
            !cas ou il n y a qu une couche (ksmax=ksmin):
            !--------------------------------------------       
            k=ksmax        
!gain       fludif_loc(iv)=(disve(k)+disvi(k))*cv_wat(iv,1,i,j)
!gain     $                     -disve(k)*cv_sed(iv,k,i,j)
            IF (iv==-1) fludif_loc(iv)=disvi(k)*temp(1,i,j)
            IF (iv== 0) fludif_loc(iv)=disvi(k)*sal(1,i,j)
            cv_sed(iv,k,i,j)=(varpwa(iv)+fludif_loc(iv)*REAL(halfdt,rsh))           &
              /(dzs(k,i,j)*poro(k,i,j)+(winter(k)+disvi(k))*REAL(halfdt,rsh)+epsilon)
          ENDIF
!
      !  flux d echange a l interface eau/sediment:
      !  ------------------------------------------
      !  fludif est le flux par diffusion a l interface, >0 vers le bas
          fludif_loc(iv)=fludif_loc(iv)-disvi(ksmax)*cv_sed(iv,ksmax,i,j)
      !  fluconsol est le flux par tassement a l interface, >0 vers le haut
      !  si pas de consolidation a ce moment winter=0 => pas de flux de consolidation
          fluconsol(iv,i,j)=winter(ksmax)*cv_sed(iv,ksmax,i,j)

        ENDDO                         ! fin boucle sur temp et salinite
 999 continue
     IF(MOD(nbouc,2) == 1)THEN
       fludif(:,i,j)=fludif_loc(:)
     ELSE 
       fludif(:,i,j)=(fludif(:,i,j)+fludif_loc(:))*0.5_rsh
     ENDIF

   PRINT_DBG*, 'FIN SED_CONSOLDIF'   

  END SUBROUTINE sed_diffusion_mudsand

#endif

!!==============================================================================
#ifdef key_sedim_mixsed

      SUBROUTINE sed_constitutivrel(i,j)
! 
   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_constitutivrel  ***
   !&E
   !&E ** Purpose : applies constitutive relationships for permeability, 
   !&E              effective stess
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  consoldif
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-02  new version from SiAM"sande-vase" by P. Le Hir
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   USE comsubstance, ONLY : nvpc,igrav1,isand2
   USE parameters,   ONLY : grav
   
   !! * Arguments
   INTEGER,INTENT(IN) :: i,j

   !! * Local declarations
   INTEGER                        :: k,iv,icond
   REAL(KIND=rsh),DIMENSION(nvpc) :: cvol
   REAL(KIND=rsh)                 :: cvolsed,cvolsangrv,fracvolcum,diarepres,       &
                                     voidratio,diamstarrep,wsoulsby,somcmud,crelmud 

   !! * executable statement
   !!   --------------------
        
   sigmapsg=0.0_rsh

   DO k=ksmi(i,j),ksma(i,j)

     !  representative diametre for permeability (D15 or D20) :
     cvolsed=0.0_rsh
     DO iv=igrav1,isand2
       cvol(iv)=cv_sed(iv,k,i,j)/ros(iv)
       cvolsed=cvolsed+cvol(iv)
     ENDDO
     cvolsangrv=cvolsed
     DO iv=isand2+1,nvpc
       cvol(iv)=cv_sed(iv,k,i,j)/ros(iv)
       cvolsed=cvolsed+cvol(iv)
     ENDDO
     fracvolcum=0.0_rsh
     diarepres=0.0_rsh
     !  if(cvolsed.le.0.000001_rsh)then
     !    write(*,*)k,i,j,'cvolsed=0',(cv_sed(iv,k,i,j),iv=1,5)
     !  endif
     DO iv=nvpc,1,-1
       fracvolcum=fracvolcum+cvol(iv)/(cvolsed+0.000001_rsh)
       diarepres=diarepres+cvol(iv)*diam_sed(iv)
       IF (fracvolcum.GE.0.2_rsh) THEN
         diarepres=diarepres/(cvolsed*fracvolcum)
         !is is assumed that ros is the same for any iv ...
         diamstarrep=diarepres*10000.0_rsh*(grav*(ros(iv)/1000.0_rsh-1.0_rsh))**0.33_rsh
         GOTO 100
       ENDIF
     ENDDO
 100    CONTINUE

     !  permeability :
     voidratio=poro(k,i,j)/(1.0000001_rsh-poro(k,i,j))
     voidratio=MAX(voidratio,0.0_rsh)
     permeab(k)=xperm1*diarepres*diarepres*voidratio**xperm2

     !  minimalisation of permeab by settling velocity :
     IF (diarepres.GE.0.00005_rsh) THEN
       !    non cohesive settling velocity, Soulsby (1997) :
       wsoulsby=.000001_rsh*((107.33_rsh+1.049_rsh*diamstarrep**3)**0.5_rsh-10.36_rsh)/diarepres
       permeab(k)=MIN(permeab(k),(1.0_rsh+voidratio)*wsoulsby/1.65_rsh)
     ELSE
       !    temporary security :
       permeab(k)=MIN(permeab(k),(1.0_rsh+voidratio)*0.0001_rsh)
     ENDIF

     !  effective stress :
     icond=1
     DO iv=igrav1,isand2
       icond=icond*MAX(0.0_rsh,100.0_rsh*(cvolmaxsort-cvol(iv)-.02_rsh))
     ENDDO
     IF(cvolsed.LE.cvolmaxmel.AND.icond.NE.0)THEN
       !    relative concentration of mud :
       somcmud=0.0_rsh
       DO iv=isand2+1,nvpc
         somcmud=somcmud+cv_sed(iv,k,i,j)
       ENDDO
       crelmud=somcmud/(1.0_rsh-cvolsangrv)
       sigmapsg(k)=xsigma1sg*crelmud**xsigma2
       stateconsol(k)=1.0_rsh
     ELSE
       !    in this case, the effective stress is maximum, 
       !    but porewater overpressure is zero :
       stateconsol(k)=0.0_rsh
     ENDIF
   ENDDO

      !settling velocity of the water/sediment interface (abandoned ?):
      !wsurf=permeab(ksma(i,j))*(1.0000001_rsh-poro(ksma(i,j),i,j))*rosmrowsros(imud1)
   
  END SUBROUTINE sed_constitutivrel
#endif

!!==============================================================================

      SUBROUTINE sed_difcoef_int(porosi)

   !&E--------------------------------------------------------------------------
   !&E                 ***  SUBROUTINE sed_difcoef_int  ***
   !&E
   !&E ** Purpose : computes diffusion coefficient at the water/sediment interface
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  normally sed_diffusion
   !&E 
   !&E ** External calls : nonee
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-02  extracted from (SiAM) by F. Cayocca and P. Le Hir
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   !!   ------------

   !! * Local declarations
   !!   ------------------
   
      REAL(KIND=rsh), INTENT(in) ::  porosi
         
   !! * Executable part
   !!   ---------------

      sed_difint=xdifsi1*porosi**xdifsi2

      END SUBROUTINE sed_difcoef_int

!!==============================================================================

      SUBROUTINE sed_difcoef_sed(porosi)

   !&E--------------------------------------------------------------------------
   !&E                 ***  SUBROUTINE sed_difcoef_sed  ***
   !&E
   !&E ** Purpose : computes diffusion coefficient in sediment 
   !&E
   !&E ** Description : 
   !&E
   !&E ** Called by :  normally sed_diffusion
   !&E 
   !&E ** External calls : nonee
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E       !  2009-02  extracted from (SiAM) by F. Cayocca and P. Le Hir
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   !!   ------------

   !! * Local declarations
   !!   ------------------
   
      REAL(KIND=rsh), INTENT(in) ::  porosi
         
   !! * Executable part
   !!   ---------------

      sed_difsed=xdifs1*porosi**xdifs2

  END SUBROUTINE sed_difcoef_sed

!!==============================================================================

  SUBROUTINE sed_outsaverestart

   !&E--------------------------------------------------------------------------
   !&E                 ***  ROUTINE sed_outsaverestart  ***
   !&E
   !&E ** Purpose : saves sediment-related field for future runs
   !&E
   !&E ** Description :
   !&E
   !&E ** Called by :  step 
   !&E
   !&E ** External calls : 
   !&E
   !&E ** Reference :
   !&E
   !&E ** History :
   !&E
   !&E       !  2008-10  (F. Cayocca) inserted into module_sedimento.F90
   !&E       !  2009-02  (P. Le Hir) modified declarations
   !
   !objet    :  gestion de la reprise de calcul a partir d un etat
         !           enregistre dans un fichier (netcdf ou binaire acces direct)
         !           lors d un run anterieur
   !
   !     appelant         : stepu
         !                :
   !     appeles          : ionc4_createfile, ionc4_torigine, 
   !     appeles          : ionc4_createvar, ionc4_createvar
   !     appeles          : ionc4_createvar, ionc4_write_time
   !     appeles          : ionc4_write_xy, ionc4_write_xyt, ionc4_write_zxyt
         !                :
   !     canal d ecriture :
         !                :
   !     canal de lecture : 26, filrep
         !                :
   !     cle cpp          : key_notra
         !                :
   !     variables in     : t, u, uz, xu, v, vz, xv, sal, temp, tz
   !     variables in     : ect, boz, nz, kz, wz, vish_xe, vish_phi
   !     variables in     : filsauvsed, lon2d,lat2d, ntra
   !     variables in     : cdate, h0,sig
         !                :
   !     variables out    : 
         !                :
   !     variables inout  : tsauv
         !                :
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used
   !!   ------------
   
   USE comsubstance, ONLY :  nv_adv,flx_s2w,flx_w2s
   USE parameters,   ONLY :  riosh,kmax,rg_valmanq_io
   USE comvars2d,    ONLY :  lon2d_g,lat2d_g,lon2d,lat2d,l_saverestart_bydate,date_ref, &
                             t,tsauv,cdate,saverestart_step,name_out_h0,name_out_xe
   USE_MPI toolmpi,  ONLY : ADD_TAG
   USE output,       ONLY : l_out_nc4par

   !! * Local declarations
   !!   ------------------

   CHARACTER(LEN=8)      :: nomc
   CHARACTER(LEN=13)     :: prefsauv
   CHARACTER(LEN=lchain) :: chain
   INTEGER               :: i,j,k,l,iv,ifois
   REAL(KIND=riosh),DIMENSION(limin:limax,ljmin:ljmax)          :: xind
   REAL(KIND=riosh),DIMENSION(ksdmin:ksdmax,limin:limax,ljmin:ljmax)   :: tlits
   REAL(KIND=riosh),DIMENSION(0:ksdmax+1)                       :: sigsed

   DATA ifois /0/
   
   !! * Executable part
   !!   ---------------

   PRINT_DBG*, 'enter sed_outsaverestart'

   ! initialisation of sigsed
   DO k=ksdmin,ksdmax
      sigsed(k)=k
   END DO
   sigsed(0)=0
   sigsed(ksdmax+1)=0
   
   ! creation
   ! ********

   IF (ifois.EQ.0 .AND. .NOT.(l_saverestart_bydate)) THEN
     ifois=1
     filsauvsed='./save_sedim.nc'
     IF_MPI(.NOT. l_out_nc4par) CALL ADD_TAG(filsauvsed,LEN(filsauvsed))
     IF ( l_out_nc4par ) THEN
        CALL ionc4_createfile(filsauvsed,lon2d_g,lat2d_g,imin,imax,1,          &
                              jmin,jmax,1,ksdmin,ksdmax,1,ksdmax,0,l_out_nc4par = .TRUE.)
     ELSE
        CALL ionc4_createfile(filsauvsed,lon2d,lat2d,limin,limax,1,          &
                              ljmin,ljmax,1,ksdmin,ksdmax,1,ksdmax,0)
     ENDIF

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
     CALL ionc4_gatt(filsauvsed,'global_imin',imin)
     CALL ionc4_gatt(filsauvsed,'global_imax',imax)
     CALL ionc4_gatt(filsauvsed,'global_jmin',jmin)
     CALL ionc4_gatt(filsauvsed,'global_jmax',jmax)

   ! indispensable pour concatenation pas terrible a revoir
     CALL ionc4_createvar(filsauvsed,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable',&
#else
                          standard_name= 'ocean_sigma_variable',&
#endif
                          l_auxcoordinate=.false.,&
                          valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io,&
                          dims= 'z', l_out_nc4par = l_out_nc4par)
     CALL ionc4_write_sig(filsauvsed,'SIG',sigsed,ksdmax)

     CALL ionc4_torigine(filsauvsed,date_ref)
     CALL ionc4_createvar(filsauvsed,'ksmi','s.o','ksmi',fill_value=rg_valmanq_io, &
                          dims='xyt',valid_min=0.0_riosh,valid_max=10.0_riosh,l_out_nc4par=l_out_nc4par)
     CALL ionc4_createvar(filsauvsed,'ksma','s.o','ksma',fill_value=rg_valmanq_io, &
                          dims='xyt',valid_min=0.0_riosh,valid_max=1000.0_riosh,l_out_nc4par=l_out_nc4par)
     CALL ionc4_createvar(filsauvsed,'dzs','m','dzs',fill_value=rg_valmanq_io, &
                          dims='zxyt',valid_min=0.0_riosh,valid_max=100.0_riosh,l_out_nc4par=l_out_nc4par)
     DO iv=-1,nv_adv     ! pour T et S
       IF (IV>0) WRITE(nomc,10) iv
       IF (iv==-1) nomc='concTEM'
       IF (iv==0) nomc='concSAL'
       CALL ionc4_createvar(filsauvsed,nomc,'u.s.i','concentration ',&
                            fill_value=rg_valmanq_io,dims='zxyt',l_out_nc4par=l_out_nc4par)
       IF (IV>0) WRITE(nomc,20) iv
       IF (iv==-1) nomc='fxw2sTEM'
       IF (iv==0) nomc='fxw2sSAL'
       CALL ionc4_createvar(filsauvsed,nomc,'u.s.i','flx_w2s ',&
                            fill_value=rg_valmanq_io,dims='xyt',l_out_nc4par=l_out_nc4par)
     END DO 
   ENDIF

   IF (l_saverestart_bydate.AND.t.GE.tsauv) THEN
     prefsauv='./save_sedim_'
     filsauvsed=prefsauv//cdate(7:10)//cdate(4:5)//  &
                    cdate(1:2)//cdate(12:13)//cdate(15:16)//cdate(18:19)//'.nc'
     IF_MPI(.NOT. l_out_nc4par) CALL ADD_TAG(filsauvsed,LEN(filsauvsed))
     IF ( l_out_nc4par ) THEN
        CALL ionc4_createfile(filsauvsed,lon2d_g,lat2d_g,imin,imax,1,          &
                              jmin,jmax,1,ksdmin,ksdmax,1,ksdmax,0,l_out_nc4par = .TRUE.)
     ELSE
        CALL ionc4_createfile(filsauvsed,lon2d,lat2d,limin,limax,1,          &
                              ljmin,ljmax,1,ksdmin,ksdmax,1,ksdmax,0)
     ENDIF

   ! save attribute of the spatial extension of the domain
   ! -----------------------------------------------------
     CALL ionc4_gatt(filsauvsed,'global_imin',imin)
     CALL ionc4_gatt(filsauvsed,'global_imax',imax)
     CALL ionc4_gatt(filsauvsed,'global_jmin',jmin)
     CALL ionc4_gatt(filsauvsed,'global_jmax',jmax)

   ! indispensable pour concatenation pas terrible a revoir
     CALL ionc4_createvar(filsauvsed,'SIG',' ','sigma variable',&
#if defined key_siggen || defined key_gencoord
                          standard_name= 'ocean_s_variable',&
#else
                          standard_name= 'ocean_sigma_variable',&
#endif
                          l_auxcoordinate=.false.,&
                          valid_min= -1.0_riosh,valid_max= 0.0_riosh,fill_value= rg_valmanq_io,&
                          dims= 'z', l_out_nc4par = l_out_nc4par)
     CALL ionc4_write_sig(filsauvsed,'SIG',sigsed,ksdmax)

     CALL ionc4_torigine(filsauvsed,date_ref)
     CALL ionc4_createvar(filsauvsed,'ksmi','s.o','ksmi',fill_value=rg_valmanq_io, &
                          dims='xyt',valid_min= 0.0_riosh,valid_max= 10.0_riosh,l_out_nc4par=l_out_nc4par)
     CALL ionc4_createvar(filsauvsed,'ksma','s.o','ksma',fill_value=rg_valmanq_io, &
                          dims='xyt',valid_min= 0.0_riosh,valid_max= 1000.0_riosh,l_out_nc4par=l_out_nc4par)
     CALL ionc4_createvar(filsauvsed,'dzs','m','dzs',fill_value=rg_valmanq_io, &
                          dims='zxyt',valid_min= 0.0_riosh,valid_max= 100.0_riosh,l_out_nc4par=l_out_nc4par)
     DO iv=-1,nv_adv     ! pour T et S
       IF (IV>0) WRITE(nomc,10) iv
       IF (iv==-1) nomc='concTEM'
       IF (iv==0) nomc='concSAL'
       CALL ionc4_createvar(filsauvsed,nomc,'u.s.i','concentration ',&
                            fill_value=rg_valmanq_io,dims='zxyt',l_out_nc4par=l_out_nc4par)
       IF (IV>0) WRITE(nomc,20) iv
       IF (iv==-1) nomc='fxw2sTEM'
       IF (iv==0) nomc='fxw2sSAL'
       CALL ionc4_createvar(filsauvsed,nomc,'u.s.i','flx_w2s ',&
                            fill_value=rg_valmanq_io,dims='xyt',l_out_nc4par=l_out_nc4par)
     END DO 
   ENDIF

   ! ecriture
   ! ********

   IF (t.GE.tsauv) THEN
     CALL ionc4_write_time(filsauvsed,1,t)
     DO j=ljmin,ljmax
       DO i=limin,limax
         xind(i,j)=REAL(ksmi(i,j))
       END DO
     END DO
     CALL ionc4_write_xyt(filsauvsed,'ksmi',xind,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)

     DO j=ljmin,ljmax
       DO i=limin,limax
         xind(i,j)=REAL(ksma(i,j))
       END DO
     END DO
     CALL ionc4_write_xyt(filsauvsed,'ksma',xind,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)
 
     DO j = ljmin,ljmax
       DO i = limin,limax
         DO k = 1,ksdmax
           tlits(k,i,j) = dzs(k,i,j)
         END DO
       END DO
     END DO
     CALL ionc4_write_zxyt(filsauvsed,'dzs',tlits,limin,limax,ljmin,ljmax,ksdmax,1,l_out_nc4par=l_out_nc4par)

     DO iv=-1,nv_adv     ! pour T et S
       IF (IV>0) WRITE(nomc,10) iv
       IF (iv==-1) nomc='concTEM'
       IF (iv==0) nomc='concSAL'
       DO j = ljmin,ljmax
         DO i = limin,limax
           DO k = 1,ksdmax
             tlits(k,i,j) = cv_sed(iv,k,i,j)
           ENDDO
         ENDDO
       ENDDO
       CALL ionc4_write_zxyt(filsauvsed,nomc,tlits,limin,limax,ljmin,ljmax,ksdmax,1,l_out_nc4par=l_out_nc4par)
       IF (IV>0) WRITE(nomc,20) iv
       IF (iv==-1) nomc='fxw2sTEM'
       IF (iv==0) nomc='fxw2sSAL'
       DO j = ljmin,ljmax
         DO i = limin,limax
           xind(i,j) = flx_w2s(iv,i,j)
         ENDDO
       ENDDO
       CALL ionc4_write_xyt(filsauvsed,nomc,xind,limin,limax,ljmin,ljmax,1,l_out_nc4par=l_out_nc4par)       
     ENDDO
     tsauv=t+saverestart_step

     IF (l_saverestart_bydate) THEN
       CALL ionc4_close(filsauvsed)
     ELSE
       ! To write the data on the disk and not loose data in case of run crash
       CALL ionc4_sync(filsauvsed)
     ENDIF

   ENDIF

   10 FORMAT('concen',i2.2)
   20 FORMAT('flxw2s',i2.2)

   PRINT_DBG*, 'exit sed_outsaverestart'

  END SUBROUTINE sed_outsaverestart

  !!===========================================================================
 

#ifdef key_sedim_mudsand
!
      SUBROUTINE sed_comp_sedvel(porosi)
!
!     appele par consoldiff
!     calcule les vitesses de tassement dans le sediment
! **********************************************************************
   !&E
   !&E--------------------------------------------------------------------------
   !! * Modules used

   !! * Arguments
   REAL(KIND=rsh), INTENT(IN) :: porosi

   !! * Local declarations

   !!---------------------------------------------------------------------------
   !! * Executable part

   IF (porosi >= poro1) THEN
     sed_sedimvel=xtas11*MIN(porosi,0.98_rsh)**xtas12
   ELSEIF(porosi >= poro2)THEN
     sed_sedimvel=xtas21*porosi**xtas22
   ELSEIF(porosi >= poro3)THEN
     sed_sedimvel=xtas31*porosi**xtas32
   ELSE
     sed_sedimvel=0.0_rsh
   ENDIF

  END SUBROUTINE sed_comp_sedvel

#endif

!!===========================================================================================

#endif

END MODULE sedimento
